
  /*\
   *  $Header: app.c,v 5.0 90/01/10 06:52:53 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define DEBUG_VAR	appDebug
#include "utils.h"
#include "appstr.h"
#include "ashash.h"

/***====================================================================***/
/***                         DEBUGGING FUNCTIONS                        ***/
/***====================================================================***/

char *
appText(pApp)
AppPtr	pApp;
{

    if ((pApp==NullApp)||(pApp->name==NullStringToken))	return("<NullApp>");
    return(stText(pApp->name));
}

/***====================================================================***/

void
appDebugPrint(pApp)
AppPtr	pApp;
{
Opaque		lstate;
StringToken	key;
AppStatePtr	pState;

    if (pApp!=NullApp) {
	uDebug("app %s (0x%x)\n",stText(pApp->name),pApp);
	uDebugIndent(1);
	uDebug("pRealState=	%s (0x%x)\n",asText(pApp->pRealState),
							pApp->pRealState);
	uDebug("pDfltState=	%s (0x%x)\n",asText(pApp->pDfltState),
							pApp->pDfltState);
	uDebug("permanent=	%s\n",booleanText(pApp->permanent));
	uDebug("outputPriv=	0x%x\n",pApp->outputPriv);
	uDebug("updateFunc=	0x%x\n",pApp->updateFunc);
	uDebug("roomStates:\n");
	uDebugIndent(1);
	lIterator(pApp->roomStates,key,pState,lstate) {
	    uDebug("%s:	%s (0x%x)\n",stText(key),asText(pState),pState);
	}
	lEndIterator(pApp->roomStates,lstate);
	uDebugIndent(-2);
    }
    else {
	uDebug("<NullApp>\n");
    }
    return;
}

/***====================================================================***/
/***              FUNCTIONS TO CREATE AND DESTROY APPS                  ***/
/***====================================================================***/

AppPtr
appCreate(name,pRealState,pDfltState)
StringToken	name;
AppStatePtr  	pRealState;
AppStatePtr	pDfltState;
{
AppPtr	pApp;
    
    uENTRY3("appCreate(%s,%s,%s)\n",stText(name),asText(pRealState),
						 asText(pDfltState));

    if (pRealState==NullAppState)	pRealState=	asDefaultReal;
    if (pDfltState==NullAppState)	pDfltState=	asDefaultProfile;
    if ((name==NullStringToken)||(!asLegalProfile(pDfltState))||
				 (!asLegalReal(pRealState))) {
	uRETURN(NullApp);
    }

    pApp=	uTypedAlloc(AppRec);
    if (pApp!=NullApp) {
	pApp->name=		name;
	pApp->pRealState=	asDupFully(pRealState);
	pApp->pDfltState=	asDupFully(pDfltState);
	pApp->roomStates= 	lCreate((unsigned)11, NullListSortFunc,
							genIntegerKeyType,
						     	asGenDataType);
	pApp->permanent=	False;
	pApp->updateFunc=	NULL;
	pApp->outputPriv=	NULL;
	if ((pApp->pRealState==NullAppState)||(pApp->pDfltState==NullAppState)||
	    (pApp->roomStates==NullList)) {
	    (void)appDestroy(pApp);
	    pApp= NullApp;
	}
    }
    uRETURN(pApp);
}

/***====================================================================***/

void
appDestroy(pApp)
AppPtr	pApp;
{
   uENTRY1("appDestroy(%s)\n",appText(pApp));
   if (pApp!=NullApp) {
	pApp->name= NullStringToken;
	if (pApp->roomStates!=NullList) {
	    (void)lDestroy(pApp->roomStates);
	    pApp->roomStates= NullList;
	}
	if (pApp->pRealState!=NullAppState) {
	    (void)asDestroy(pApp->pRealState);
	    pApp->pRealState=	NullAppState;
	}
	if (pApp->pDfltState!=NullAppState) {
	    (void)asDestroy(pApp->pDfltState);
	    pApp->pDfltState=	NullAppState;
	}
	uFree((Opaque)pApp);
   }
   uVOIDRETURN;
}

/***====================================================================***/
/***       FUNCTIONS TO CHANGE OR QUERY THE DEFAULT STATE OF AN APP     ***/
/***====================================================================***/

AppStatePtr
appSetDefault(pApp,pNewDflt,isRealState)
AppPtr		pApp;
AppStatePtr	pNewDflt;
Boolean		isRealState;
{
AppStatePtr	pState;

    uENTRY3("appSetDefault(%s,%s,%s)\n",appText(pApp),asText(pNewDflt),
						booleanText(isRealState));
    if ((pApp==NullApp)||(isRealState&&(!asLegalReal(pNewDflt)))||
			 ((!isRealState)&&(!asLegalProfile(pNewDflt)))) {
	uRETURN(NullAppState);
    }
    pState=	pApp->pDfltState;
    if (isRealState)	asCopyRealToProfile(pNewDflt,pState);
    else		asCopyFully(pNewDflt,pState);
    uRETURN(pState);
}

/***====================================================================***/

AppStatePtr
appGetDefault(pApp)
AppPtr	pApp;
{
    uENTRY1("appGetDefault(%s)\n",appText(pApp));
    if (pApp!=NullApp) {
	uRETURN(pApp->pDfltState);
    }
    uRETURN(NullAppState);
}

/***====================================================================***/
/***       FUNCTIONS TO CHANGE OR QUERY THE CURRENT STATE OF AN APP     ***/
/***====================================================================***/

AppStatePtr
appSetCurrent(pApp,pNewReal,isRealState)
AppPtr		pApp;
AppStatePtr	pNewReal;
Boolean		isRealState;
{
AppStatePtr	pState;

    uENTRY3("appSetCurrent(%s,%s,%s)\n",appText(pApp),asText(pNewReal),
						booleanText(isRealState));
    if ((pApp==NullApp)||(isRealState&&(!asLegalReal(pNewReal)))||
			 ((!isRealState)&&(!asLegalProfile(pNewReal)))) {
	uRETURN(NullAppState);
    }
    pState=	pApp->pRealState;
    if (isRealState) {
	asCopyFully(pNewReal,pState);
    }
    else {
	asCopyProfileToReal(pNewReal,pState);
	if (pApp->updateFunc) {
	    (*pApp->updateFunc)(pApp,pState,pApp->outputPriv);
	}
    }
    uRETURN(pState);
}

/***====================================================================***/

AppStatePtr
appGetCurrent(pApp)
AppPtr	pApp;
{
    uENTRY1("appGetCurrent(%s)\n",appText(pApp));
    if (pApp!=NullApp) {
	uRETURN(pApp->pRealState);
    }
    uRETURN(NullAppState);
}

/***====================================================================***/
/***           FUNCTIONS TO MUCK WITH THE LOCAL STATE OF AN APP         ***/
/***====================================================================***/

int
appNLocalStates(pApp)
AppPtr	pApp;
{
int	n= 0;
    uENTRY1("appNLocalStates(%s)\n",appText(pApp));
    if (pApp!=NullApp) {
	n=	lNItems(pApp->roomStates);
    }
    uRETURN(n);
}

/***====================================================================***/

AppStatePtr
appSetRoomState(pApp,roomName,pNew,isRealState)
AppPtr		pApp;
StringToken	roomName;
AppStatePtr	pNew;
Boolean		isRealState;
{
AppStatePtr	pState;

    uENTRY4("appSetRoomState(%s,%s,%s,%s)\n",appText(pApp),stText(roomName),
					asText(pNew),booleanText(isRealState));
    if ((pApp==NullApp)||(roomName==NullStringToken)||
	(isRealState&&(!asLegalReal(pNew)))||
	((!isRealState)&&(!asLegalProfile(pNew)))) {
	uRETURN(NullAppState);
    }
    (void)lLookup(pApp->roomStates,(GenKey)roomName,(GenData *)&pState);
    if (pState==NullAppState) {
	pState=	asDupFully(pApp->pDfltState);
	(void)lStore(pApp->roomStates,(GenKey)roomName,(GenData)pState);
    }
    if (pState!=NullAppState) {
	if (isRealState)	asCopyRealToProfile(pNew,pState);
	else			asCopyFully(pNew,pState);
    }
    uRETURN(pState);
}

/***====================================================================***/

Boolean
appRemoveRoomState(pApp, roomName)
AppPtr		  pApp;
StringToken	  roomName;
{
Boolean	ok;

    uENTRY2("appGetRoomState(%s,%s)\n",appText(pApp),stText(roomName));
    if ((pApp!=NullApp)&&(roomName!=NullStringToken)) {
	ok= lRemove(pApp->roomStates,(GenKey)roomName);
	uRETURN(ok);
    }
    uRETURN(False);
}

/***====================================================================***/

AppStatePtr
appGetRoomState(pApp, roomName)
AppPtr		  pApp;
StringToken	  roomName;
{
AppStatePtr	 pState=	NullAppState;

    uENTRY2("appGetRoomState(%s,%s)\n",appText(pApp),stText(roomName));
    if ((pApp==NullApp)||(roomName==NullStringToken)) {
	uRETURN(NullAppState);
    }
    (void)lLookup(pApp->roomStates,(GenKey)roomName,(GenData *)&pState);
    uRETURN(pState);
}

/***====================================================================***/

Boolean
appRemoveAllRoomStates(pApp)
AppPtr	pApp;
{
    uENTRY1("appRemoveAllRoomStates(%s)\n",appText(pApp));
    if (pApp!=NullApp) {
	lRemoveAll(pApp->roomStates);
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/
/***      FUNCTIONS FOR DEALING WITH THE UPDATE FUNCTION OF AN APP      ***/
/***====================================================================***/

void
appSetUpdateFunc(pApp, updateFunc)
AppPtr		pApp;
AppUpdateFunc	updateFunc;
{
    uENTRY2("appSetUpdateFunc(%s,0x%x)\n",appText(pApp),updateFunc);
    if (pApp) {
	pApp->updateFunc=	updateFunc;
    }
    uVOIDRETURN;
}

/***====================================================================***/

AppUpdateFunc
appGetUpdateFunc(pApp)
AppPtr		pApp;
{
    uENTRY1("appGetUpdateFunc(%s)\n",appText(pApp));
    if (pApp!=NullApp) {
	uRETURN(pApp->updateFunc);
    }
    uRETURN((AppUpdateFunc)NULL);
}


/***====================================================================***/
/***           FUNCTIONS TO GET/SET THE VARIOUS PRIVATE FIELDS          ***/
/***====================================================================***/


StringToken
appName(pApp)
AppPtr	pApp;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"appName(%s)\n",appText(pApp));
    if (pApp==NullApp) {
	uFLAG_RETURN(NullStringToken);
    }
    uFLAG_RETURN(pApp->name);
}

/***====================================================================***/

Boolean
appIsActive(pApp)
AppPtr	pApp;
{
WinState	winState;

    uFLAG_ENTRY1(LOW_ENTRY_BIT,"appIsActive(%s)\n",appText(pApp));
    if (pApp!=NullApp) {
	asValues(pApp->pRealState,&winState,(int *)NULL,(int *)NULL,
					 (unsigned *)NULL,(unsigned *)NULL);
	uFLAG_RETURN(winState!=asInactive);
    }
    uFLAG_RETURN(False);
}

/***====================================================================***/

Boolean
appIsPermanent(pApp)
AppPtr	pApp;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"appIsPermanent(%s)\n",appText(pApp));
    if (pApp!=NullApp) {
	uFLAG_RETURN(pApp->permanent);
    }
    uFLAG_RETURN(False);
}

/***====================================================================***/

Boolean
appSetPermanence(pApp,permanent)
AppPtr	pApp;
Boolean	permanent;
{
    uFLAG_ENTRY2(LOW_ENTRY_BIT,"appSetPermanence(%s,%s)\n",appText(pApp),
						booleanText(permanent));
    if (pApp!=NullApp) {
	pApp->permanent=	permanent;
	uFLAG_RETURN(True);
    }
    uFLAG_RETURN(False);
}

/***====================================================================***/

void
appSetOutputPriv(pApp, pPriv)
AppPtr	pApp;
Opaque	pPriv;
{
    uFLAG_ENTRY2(LOW_ENTRY_BIT,"appSetOutputPriv(%s,0x%x)\n",appText(pApp),
								pPriv);
    if (pApp!=NullApp) {
	pApp->outputPriv= pPriv;
    }
    uFLAG_VOIDRETURN;
}

/***====================================================================***/

Opaque
appGetOutputPriv(pApp)
AppPtr	pApp;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"appGetOutputPriv(%s)\n",appText(pApp));
    if (pApp!=NullApp) {
	uFLAG_RETURN(pApp->outputPriv);
    }
    uFLAG_RETURN((Opaque)NULL);
}

/***====================================================================***/

typedef struct _AppIterArg {
	AppIterFunc	pFunc;
	AppPtr		pApp;
	Opaque		arg;
} AppIterArgRec, *AppIterArgPtr;

static Boolean
_appIterator(keyIn,dataIn,argIn)
GenKey		keyIn;
GenData	dataIn;
Opaque		argIn;
{
StringToken	key=	(StringToken)keyIn;
AppStatePtr	pState=	(AppStatePtr)dataIn;
AppIterArgPtr	pArg=	(AppIterArgPtr)argIn;
Boolean	ok;

    uENTRY3("_appIterator(%s,%s,0x%x)\n",stText(key),asText(pState),pArg);
    ok= (*pArg->pFunc)(pArg->pApp,key,pState,pArg->arg);
    uRETURN(ok);
}

void
appIterate(pApp, pFunc, arg)
AppPtr		pApp;
AppIterFunc	pFunc;
Opaque		arg;
{
    uENTRY3("appIterate(%s,0x%x,0x%x)\n",appText(pApp), pFunc, arg);
    if ((pApp!=NullApp)&&(pFunc!=NULL)) {
	AppIterArgRec	iarg;
	iarg.pFunc=	pFunc;
	iarg.pApp=	pApp;
	iarg.arg=	arg;
	(void)lIterate(pApp->roomStates,_appIterator,(Opaque)&iarg);
    }
    uVOIDRETURN;
}

/***====================================================================***/

Boolean	
_appInitIter(pApp, ppIState)
AppPtr	 pApp;
Opaque	*ppIState;
{
Boolean	ok;

    uENTRY2("_appInitIter(%s,0x%x)\n",appText(pApp), ppIState);
    if (pApp!=NullApp) {
	ok= _lInitIter(pApp->roomStates,ppIState);
	uRETURN(ok);
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean	
_appIterNext(pApp, ppIState, pRoomName, ppState)
AppPtr		 pApp;
Opaque		*ppIState;
StringToken	*pRoomName;
AppStatePtr	*ppState;
{
Boolean	ok;
    uENTRY4("_appIterNext(%s,0x%x,0x%x,0x%x)\n",appText(pApp),ppIState,
							pRoomName,ppState);
    ok= _lIterNext(pApp->roomStates,ppIState,(GenKey *)pRoomName,
						(GenData *)ppState);
    uRETURN(ok);
}

/***====================================================================***/

void
appEndIterator(pApp,pIState)
AppPtr	pApp;
Opaque	pIState;
{
    uENTRY2("_appEndIterator(%s,0x%x)\n",appText(pApp),pIState);
    if (pApp!=NullApp) 
	lEndIterator(pApp->roomStates,pIState);
    uVOIDRETURN;
}
