#ifndef APP_H
#define APP_H 1


  /*\
   *  $Header: app.h,v 5.0 90/01/10 06:52:57 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

  /*\

  \*/

	/*\
	 *  Methods for manipulating an App.  This package was written to
	 *  support an implementation of rooms under X, but might be useful
	 *  for other applications that want to keep track of the state of
	 *  a bunch of windows.  In particular, this package knows nothing
	 *  about X;  it uses a function pointer supplied by rooms to 
	 *  request changes to application states.  
	 *  For the rest of this description: "rooms" is assumed to mean 
	 *  "whatever program contains this package,"   "application" refers 
	 *  to any window being tracked or managed by rooms, and "App" refers
	 *  to an instance of the class implemented by this package that 
	 *  represents an application.   An "external" change is a change in
	 *  the state of an application that is not in response to a request
	 *  made by rooms,  an "internal" change is a change that occurs in
	 *  response to a request by rooms.   Note the distinction between
	 *  "application" and "App" -- an App may represent (serially) multiple
	 *  applications.
	 *
	 *  An App represents the real state and a set of possible desired 
	 *  states of a window.  An App keeps track of its current state 
	 *  (a <real> state), its default state (a <profile> state), and an 
	 *  optional set of named room states (each is a <profile> state and 
	 *  a StringToken name).
	 *  Each room state describes the state the App window should take 
	 *  in the room with the associated name.   The App window should
	 *  use the default state in any room that doesn't have an explicit
	 *  state.    
	 *
	 *  The structures that represent the current, default, and each of 
	 *  the room states are guaranteed not to change across the life
	 *  of an App (*not* application) with one exception:  room states 
	 *  that are explicitly destroyed should not be used again.   If a 
	 *  room state with the same name is created later, it will *not* use 
	 *  the same structure.   Rooms or any other code that uses this
	 *  package should take considerable care to make sure that all 
	 *  references to destroyed room states are removed.
	 *
	 *  Rooms provides an "Update" function for each App.   The 
	 *  Update function is called when an application's current state 
	 *  changes internally.   The Update function should (try to) update 
	 *  the screen to match the internal change.
	 *
	 *  The App class provides several fields (and methods to get and 
	 *  set them) for the convenience of other packages that use App's,
	 *  but does not directly interpret their contents.  These private
	 *  values are:
	 *	name -- The name of the associated program or window.   The
	 *		App class only stores and reports the name of the
	 *		application.   It does not use the name.  It is not
	 *		an error to allow multiple Apps with the same name.
	 *	active -- An active App has a Current window state that is
	 *		not "asInactive."   Active Apps should correspond
	 *		to some window that is realized on the user's screen.
	 *		Inactive Apps describe the desired states of an 
	 *		application should it ever be invoked.
	 *	permanence -- a comment about the expected lifetime of the 
	 *		App.  The structures describing a permanent App
	 *		should not be destroyed when the program (or window) 
	 *		it represents exits or is destroyed.   If a another
	 *		window or program with the same name is created later, 
	 *		the *same* App should be used to represent it.  A 
	 *		transient App should be destroyed when the program (or 
	 *		window)	it represents exits or is destroyed.  The App
	 *		Class does *not* automatically destroy inactive
	 *		transient Apps.
	 *	output priv -- a place for the packages using Apps to store
	 *		any per-App information they need.  No implied 
	 *		semantics.
	 * 
	 *  DEBUGGING FUNCTIONS --
	 *  char *
	 *  appText(pApp)
	 *  AppPtr	pApp;
	 *	Return the name of the application, suitable for debugging.
	 *	appText is guaranteed to never return NullString.   The
	 *	return string is stored in a static buffer that must *not*
	 *	be freed, and may be clobbered by subsequent calls to appText.
	 *
	 *  appDebugPrint(pApp)
	 *  AppPtr	pApp;
	 *	Prints a bunch of useful debugging information about "pApp"
	 *	using uDebug (which usually writes to stderr).
	 *
	 *  FUNCTIONS TO CREATE AND DESTROY APPS --
	 *  AppPtr
	 *  appCreate(name,pRealState,pDfltState)
	 *  StringToken	name;
	 *  AppStatePtr	pRealState;
	 *  AppStatePtr	pDfltState;
	 *	Creates an application with the specified name, current
	 *	state and default state.   If pRealState or pDfltState
	 *	are NullAppState, appCreate uses an appropriate default
	 *	for each (normally [ inactive ] and [ iconic ]).  Returns
	 *	a pointer to the newly created App, or False if name is
	 *	NullStringToken, pRealState is not a legal <real> state,
	 *	<pDfltState> is not a legal <profile> state, or there is
	 *	an allocation failure.
	 *
	 *  appDestroy(pApp)
	 *  AppPtr	pApp;
	 *	Destroys "pApp" and frees all memory associated with it.
	 *	It's not a good idea to try to use an App after it has been
	 *	destroyed, so clean up those pointers.
	 *
	 *  FUNCTIONS TO CHANGE OR QUERY THE DEFAULT STATE OF AN APP --
	 *  AppStatePtr
	 *  appSetDefault(pApp,pNewDflt,isRealState)
	 *  AppPtr	pApp;
	 *  AppStatePtr	pNewDflt;
	 *  Boolean	isRealState.
	 *	Changes the default state of "pApp" to "pNewDflt."   Copies
	 *	values from "pNewDflt" into the existing default state, so 
	 *	previously stored references to the default state of the 
	 *	App remain valid, and the calling function can free pNewDflt.
	 *	If "isRealState" is True, "pNewDflt" is assumed to be a 
	 *	<real> state, and only fields that are already defined in 
	 *	the default state of the App and are not bolted will be
	 *	updated.   If "isRealState" is False, "pNewDflt" is assumed
	 *	to be a <profile> state, the current fields of the default
	 *	state will be clobbered.   See the descriptions of
	 *	asCopyRealToProfile and asCopyFully  in appstate.h for a more
	 *	complete description.   appSetDefault returns a pointer to
	 *	the default state of the App, or NullAppState if "pApp" is
	 *	NullApp, or if "isRealState" lies (i.e. "isRealState" is True
	 *	and "pNewDflt" is not a legal <real> state, or if "isRealState"
	 *	is False, and "pNewDflt" is not a legal <profile> state.  If
	 *	pNewDflt is NullAppState, a suitable default (usually [ iconic])
	 *	will be substituted.
	 *  
	 *  AppStatePtr
	 *  appGetDefault(pApp)
	 *  AppPtr	pApp;
	 *	Gets the default state of pApp.  All calls to "appGetDefault"
	 *	for a specific App are guaranteed to return the same value
	 *	across the lifetime of the App, so this return value can be
	 *	cached safely.  Returns NullAppState if pApp is NullApp.
	 *
	 *  FUNCTIONS TO CHANGE OR QUERY THE CURRENT STATE OF AN APP --
	 *  AppStatePtr
	 *  appSetCurrent(pApp,pNewReal,isRealState)
	 *  AppPtr	pApp;
	 *  AppStatePtr	pNewDflt;
	 *  Boolean	isRealState;
	 *	Changes the current state of "pApp" to "pNewReal."  Also
	 *	copies values from pNewReal into the existing current state,
	 *	so prior references to the current state of the App remain
	 *	valid.  "isRealState" indicates the type of "pNewReal," much
	 *	like the "isRealState" argument to appSetDefault.  This time
	 *	see the documentation for asCopyFully (real -> real) or
	 *	asCopyProfileToReal (profile->real) in appstate.h for a more
	 *	complete description of the semantics.   If "pNewReal" is
	 *	a <real> state, appSetCurrent assumes the change reflects an 
	 *	external change and does *not* call the update function.  If 
	 *	"pNewReal" is a <profile> state, appSetCurrent assumes the
	 *	change is internal and *does* call the update function.
	 *	Returns a pointer to the current state of "pApp," or 
	 *	NullAppState if pApp is NullApp, or if "pNewReal" is not
	 *	legal according to "isRealState."
	 *	
	 *  AppStatePtr
	 *  appGetCurrent(pApp)
	 *  AppPtr	pApp;
	 *	Gets the current state of pApp.  All calls to "appGetCurrent"
	 *	for a specific App are guaranteed to return a pointer to the 
	 *	same instance across the lifetime of the App, so this return 
	 *	value can be cached safely.  Returns NullAppState if pApp 
	 *	is NullApp.
	 *
	 *  FUNCTIONS TO MUCK WITH THE LOCAL STATES OF AN APP --
	 *  unsigned
	 *  appNLocalStates(pApp)
	 *  AppPtr	pApp;
	 *	Returns the number of rooms with explicitly defined states
	 *	for "pApp."   If pApp is NullApp, returns 0.
	 *
	 *  AppStatePtr
	 *  appSetRoomState(pApp,roomName,pNew,isRealState)
	 *  AppPtr	pApp;
	 *  StringToken	roomName;
	 *  AppStatePtr	pNew;
	 *  Boolean	isRealState;
	 *	Changes the state of "pApp" in the room named "roomName" to
	 *	be "pNew."   If "pApp" already has a state local to "roomName,"
	 *	pNew is copied into the existing room state.   If "pApp" does
	 *	not have a state local to "roomName," appSetRoom creates a
	 *	duplicate of the default state of "pApp" and copies "pNew"
	 *	into it (so bolts in the default state of an application are
	 *	effective).  As with appSetDefault and appSetCurrent, 
	 *	"isRealState" indicates the type of "pNew."  Returns a pointer
	 *	to the local state of "pApp" in "roomName," which is guaranteed
	 *	to persist until it is explicitly removed or "pApp" is 
	 *	destroyed.  Returns NullAppState if "pApp" is NullApp, 
	 *	"roomName" is NullStringToken, or "pNew" is illegal.
	 *
	 *  Boolean
	 *  appRemoveRoomState(pApp,roomName)
	 *  AppPtr	pApp;
	 *  StringToken	roomName;
	 *	Removes the local state for pApp in the room named "roomName."
	 *	The previous local state is freed, so outstanding references
	 *	should be cleaned up.  Returns True if the local state is 
	 *	removed, False if "pApp" is NullApp, "roomName" is 
	 *	NullStringToken, or "pApp" does not have a local state for 
	 *	"roomName."
	 *
	 *  AppStatePtr
	 *  appGetRoomState(pApp,roomName)
	 *  AppPtr	pApp;
	 *  StringToken	roomName;
	 *	Gets the local state of "pApp" in "roomName."  All calls 
	 *	to "appGetRoomState" for a specific App and roomName are 
	 *	guaranteed to return a pointer to the same instance until
	 *	the local state is explicitly removed or the App is destroyed,
	 *	so the returned value may be cached safely.
	 *	Returns NullAppState if "pApp" is NullApp, "roomName" is
	 *	NullStringToken, or "pApp" has no local state for "roomName."
	 *
	 *  Boolean
	 *  appRemoveAllRoomStates(pApp)
	 *  AppPtr	pApp;
	 *	Removes the local states for all rooms in pApp.  Frees all 
	 *	local states, so outstanding references should be cleaned
	 *	up.  Returns True if successful, False if "pApp" is NullApp.
	 *
	 *  FUNCTIONS FOR DEALING WITH THE UPDATE FUNCTION OF AN APP --
	 *  void
	 *  appSetUpdateFunc(pApp,pFunc)
	 *  AppPtr		pApp;
	 *  AppUpdateFunc	pFunc;
	 *	Change the update func of "pApp" to "pFunc."   The update
	 *	function is called whenever an internal change to the
	 *	current state of an App is requested (intuited, whatever).
	 *	No return value.
	 *
	 *  AppUpdateFunc
	 *  appGetUpdateFunc(pApp)
	 *  AppPtr	pApp;
	 *	Returns a pointer to the update func of "pApp," or NULL
	 *	if "pApp" is NullApp.
	 *
	 *  GET/SET THE VARIOUS PRIVATE FIELDS --
	 *  StringToken
	 *  appName(pApp)
	 *  AppPtr	pApp;
	 *	Returns the name of "pApp," or NullStringToken if pApp is
	 *	NullApp.
	 *
	 *  Boolean
	 *  appIsActive(pApp)
	 *  AppPtr	pApp;
	 *	Returns True if the window state of the current state of 
	 *	"pApp" is anything but asInactive; returns False if the 
	 *	window state is asInactive, or "pApp" is NullApp.
	 *
	 *  Boolean
	 *  appIsPermanent(pApp)
	 *  AppPtr	pApp;
	 *	Returns True if "pApp" is permanent, False if "pApp" is
	 *	transient or NullApp.
	 *
	 *  Boolean
	 *  appSetPermanence(pApp, permanent )
	 *  AppPtr	pApp;
	 *  Boolean	permanent;
	 *	Sets permanence of "pApp" to "permanent."  Returns True for
	 *	success, False if pApp is NullApp.
	 *  
	 *  void
	 *  appSetOutputPriv(pApp,priv)
	 *  AppPtr	pApp;
	 *  Opaque	priv;
	 *	Sets the output private field of "pApp" to "priv."  No
	 *	return value.
	 *
	 *  Opaque
	 *  appGetOutputPriv(pApp)
	 *  AppPtr	pApp;
	 *	Returns the output private field of "pApp," or NULL if
	 *	pApp is NullApp.
	 *
	 *  OTHER USEFUL FUNCTIONS --
	 *  void
	 *  appIterate(pApp,pFunc,arg)
	 *  AppPtr	pApp;
	 *  AppIterFunc	pFunc;
	 *  Opaque	arg;
	 *	Iterates across all local states of "pApp," calling "*pFunc"
	 *	on each in turn with "(pApp,roomName,pLocalState,arg)."
	 *	Stops iterating if pFunc returns False.  No return value.
	 *
	 *  appIterator(pApp,roomName,pState,istate)	<Statement>
	 *	CLU-like (I'll do time in purgatory for that) iterator.
	 *	Executes <Statement> (which can be a block statement) for
	 *	each local state of "pApp," setting roomName and pState
	 *	to the appropriate value for each state in turn.  This iterator
	 *	keeps its state in a structure pointed to by "istate" --
	 *	don't muck with it, and don't store anything you care about
	 *	in it before you use the iterator.  Also, be absolutely
	 *	certain you call appEndIterator(pApp,istate) after the loop.
	 *	You can nest iterators.   This is smoke and mirrors, but
	 *	useful, so be careful.  For example:
	 *	    appIterator(pApp,rName,pState,ignoreMe) {
	 *		uInformation("app %s has state %s in room %s\n",
	 *				appText(pApp),asText(pState),
	 *				stText(rName));
	 *	    }
	 *	    appEndIterator(pApp,ignoreMe);
	 *	displays all of the local states in pApp.
	 *	
	\*/

#include "appstate.h"
#include "strtbl.h"

typedef struct _AppRec 		*AppPtr;

typedef	Boolean	(*AppUpdateFunc)(/* pApp, pNewState, pPriv */);
typedef	Boolean	(*AppIterFunc)(/* pApp, roomName, pState, arg */);

#define	NullApp			((AppPtr)NULL)

extern	unsigned int	appDebug;

extern	char		*appText();
extern	void		 appDebugPrint(/* pApp */);

extern	AppPtr	 	 appCreate(/*name,pRealState,pDfltState*/);
extern	void	 	 appDestroy(/*pApp*/);

extern	AppStatePtr	 appSetDefault(/* pApp, pNewDflt, isRealState */);
extern	AppStatePtr	 appGetDefault(/* pApp */);

extern	AppStatePtr	 appSetCurrent(/* pApp, pNewReal, isRealState */);
extern	AppStatePtr	 appGetCurrent(/* pApp */);

extern	int		 appNLocalStates(/* pApp */);
extern	AppStatePtr	 appSetRoomState(/* pApp, roomName, pNew, isReal */);
extern	Boolean		 appRemoveRoomState(/* pApp, roomName */);
extern	AppStatePtr	 appGetRoomState(/* pApp, roomName */);
extern	Boolean	 	 appRemoveAllRoomStates(/* pApp */);

extern	void	 	 appSetUpdateFunc(/* pApp, updateFunc */);
extern	AppUpdateFunc 	 appGetUpdateFunc(/* pApp */);

extern	StringToken	 appName(/* pApp */);
extern	Boolean		 appIsActive(/* pApp */);
extern	Boolean		 appIsPermanent(/* pApp */);
extern	Boolean		 appSetPermanence(/* pApp, permanent */);
extern	void	 	 appSetOutputPriv(/* pApp, pPriv */);
extern	Opaque	 	 appGetOutputPriv(/* pApp */);

extern	void		 appIterate(/* pApp, pFunc, arg */);

extern	Boolean		 _appInitIter(/* pApp, ppIState */);
extern	Boolean		 _appIterNext(/* pApp, pIState, pRoomName, ppState */);
extern	void		 appEndIterator(/* pApp, pIState */);

#define	appIterator(pa,k,d,pi) \
	if (_appInitIter(pa,&pi)) \
	    while (_appIterNext(pa,&pi,&k,&d))

#endif /* APP_H */
