
  /*\
   *  $Header: apps.c,v 5.2 90/01/11 13:41:55 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define	DEBUG_VAR appsDebug
#include "utils.h"
#include "hash.h"
#include "strtbl.h"
#include "rooms.h"
#include "apps.h"
#include "profile.h"

/***====================================================================***/

HashTablePtr	pAllApps=	NullHashTable;
HashTablePtr	pActiveApps=	NullHashTable;

char		*AppInstanceFormat=	"%s-%d";

/***====================================================================***/

void
DebugPrintApps(name)
char	*name;
{
AppPtr	pApp;
Opaque	astate;

    if (pAllApps!=NullHashTable) {
	if (name==NullString) {
	    uDebug("All Apps:\n");
	    uDebugIndent(1);
	    AppsIterator(AllApps,pApp,astate) {
		uDebug("%s (0x%x)\n",appText(pApp),pApp);
		appDebugPrint(pApp);
	    }
	    AppsEndIterator(astate);
	    uDebugIndent(-1);
	    uDebug("ActiveApps:\n");
	    uDebugIndent(1);
	    AppsIterator(ActiveApps,pApp,astate) {
		uDebug("%s (0x%x)\n",appText(pApp),pApp);
	    }
	    AppsEndIterator(astate);
	    uDebugIndent(-1);
	}
	else {
	    StringToken	token;
	    if (stGetTokenIfExists(name,&token)) {
		AppPtr  pApp= (AppPtr)htLookup(pAllApps,(GenKey)token);
		if (pApp!=NullApp) {
		    uDebug("%s (0x%x)\n",appText(pApp),pApp);
		    appDebugPrint(pApp);
		}
		else uDebug("App %s doesn't exist\n",stText(token));
	    }
	    else uDebug("token %s undefined\n",uStringText(name));
	}
    }
    else {
	uDebug("No apps defined\n");
    }
    return;
}

/***====================================================================***/

AppPtr
LookupApp(name)
StringToken	name;
{
AppPtr	pApp=	NullApp;

    uENTRY1("LookupApp(%s)\n",stText(name));
    if (pAllApps!=NullHashTable) {
	pApp=	(AppPtr)htLookup(pAllApps,(GenKey)name);
    }
    uRETURN(pApp);
}

/***====================================================================***/

AppPtr
LookupActiveApp(name)
StringToken	name;
{
AppPtr	pApp=	NullApp;

    uENTRY1("LookupActiveApp(%s)\n",stText(name));
    if (pActiveApps!=NullHashTable) {
	pApp=	(AppPtr)htLookup(pActiveApps,(GenKey)name);
    }
    uRETURN(pApp);
}

/***====================================================================***/

AppPtr
GetApp(name,onlyInactive)
StringToken	name;
Boolean		onlyInactive;
{
AppPtr		pApp;
char	*oldName;
char	*tmpBuf=	NullString;
int	count=		1;
Boolean	done;

    uENTRY2("GetApp(%s,%s)\n",stText(name),booleanText(onlyInactive));

    if (pAllApps==NullHashTable) {
	pAllApps=	htCreate((unsigned)71,genIntegerKeyType,
						genPointerDataType);
    }
    if ((pAllApps==NullHashTable)||(name==NullStringToken)) {
	uRETURN(NullApp);
    }

    done=	False;
    do {
        pApp=	(AppPtr)htLookup(pAllApps,(GenKey)name);
	if (pApp!=NullApp) {
	    if (onlyInactive&&appIsActive(pApp)) {
		if (tmpBuf==NullString) {
		    oldName=	stGetString(name);
		    tmpBuf=	(char *)uTmpAlloc(strlen(oldName)+24);
		    if (tmpBuf==NullString) {
			uRETURN(NullApp);
		    }
		}
		sprintf(tmpBuf,AppInstanceFormat,oldName,count++);
		name=	stGetToken(tmpBuf);
	    }
	    else {
		done=	True;
	    }
	}
	else {
	    pApp=	appCreate(name,NullAppState,NullAppState);
	    if (pApp!=NullApp) {
		(void)htAdd(pAllApps,(GenKey)name,(GenData)pApp);
		(void)profileForApp(name);
	    }
	    else {
		uWarning("Couldn't create app %s\n",stText(name));
	    }
	    done=	True;
	}
    } while (!done);

    if (tmpBuf!=NullString) {
	uTmpFree(NullString);
    }
    uRETURN(pApp);
}

/***====================================================================***/

/* ARGSUSED */
static Boolean
_CreateAppRooms(pApp,roomName,pState,pArg)
AppPtr		pApp;
StringToken	roomName;
AppStatePtr	pState;
Opaque		pArg;
{
    uENTRY4("_CreateAppRooms(%s,%s,%s,0x%x)\n",appText(pApp),stText(roomName),
						asText(pState),pArg);
    (void)CreateRoom(roomName);
    uRETURN(True);
}

/***====================================================================***/

Boolean
AppActivated(pApp, pRealState)
AppPtr		pApp;
AppStatePtr	pRealState;
{
Boolean	permanent;
Opaque	rstate;
RoomPtr	pRoom;

    uENTRY2("AppActivated(%s,%s)\n",appText(pApp),asText(pRealState));
    if ((pApp==NullApp)||(!asLegalReal(pRealState))||
	(asWinState(pRealState)==asInactive)||
	(pAllApps==NullHashTable)) {
	uRETURN(False);
    }
    if (appIsActive(pApp)) {
	uRETURN(False);
    }

    if (pActiveApps==NullHashTable) {
	pActiveApps=htCreate((unsigned)51,genIntegerKeyType,genPointerDataType);
	if (pActiveApps==NullHashTable) {
	    uRETURN(False);
	}
    }

    if (permanent=appIsPermanent(pApp)) {
	(void)appIterate(pApp,_CreateAppRooms,(Opaque)NULL);
    }
    (void)appSetCurrent(pApp,pRealState,True);
    if (!permanent) {
	RoomPtr		pRoom=	GetCurrentRoom();
	AppStatePtr	pState;	
	if (asWinState(pRealState)==asNormal) {
	    pState= appSetDefault(pApp,pRealState,False);
	    if (pRoom!=NullRoom) {
		appSetRoomState(pApp,roomName(pRoom),pRealState,False);
	    }
	    if (pState!=NullAppState) {
		asChangeSelected(pState,asWinStateValue,asIconic,0,0,
						(unsigned)0,(unsigned)0);
	    }
	}
	else if (asWinState(pRealState)==asIconic) {
	    pState= appSetDefault(pApp,pRealState,False);
	}
	else if (asWinState(pRealState)==asWithdrawn) {
	    pState= asDupFully(pRealState);
	    if (pState!=NullAppState) {
		asChangeSelected(pState,asWinStateValue,asIconic,0,0,
						(unsigned)0,(unsigned)0);
		appSetDefault(pApp,pState,False);
		asDestroy(pState);
	    }
	}
    }
    (void)htAdd(pActiveApps,(GenKey)appName(pApp),(GenData)pApp);

    RoomsIterator(pRoom,rstate) {
	roomManageApp(pRoom,pApp);
    }
    RoomsEndIterator(rstate);
    uRETURN(True);
}

/***====================================================================***/

Boolean
AppDeactivated(pApp)
AppPtr		pApp;
{
Opaque		rstate;
RoomPtr		pRoom;
AppPtr		pOldApp;

    uENTRY1("_AppDeactivated(%s)\n",appText(pApp));
    if ((pApp==NullApp)||(pActiveApps==NullHashTable)) {
	uRETURN(False);
    }
    if (!appIsActive(pApp)) {
	uRETURN(False);
    }
    (void)appSetCurrent(pApp,asDefaultReal,True);

    RoomsIterator(pRoom,rstate) {
	roomUnmanageApp(pRoom,pApp);
    }
    RoomsEndIterator(rstate);

    pOldApp=	(AppPtr)htRemove(pActiveApps,(GenKey)appName(pApp));
    if (pOldApp!=pApp) {
	uInternalError("deactivated %s(0x%x), expected %s(0x%x)\n",
					appText(pOldApp),pOldApp,
					appText(pApp),pApp);
	uRETURN(False);
    }
    if (!appIsPermanent(pApp)) {
	pOldApp= (AppPtr)htRemove(pAllApps,(GenKey)appName(pApp));
	if (pOldApp!=pApp) {
	    uInternalError("destroyed %s(0x%x), expected %s(0x%x)\n",
					appText(pOldApp),pOldApp,
					appText(pApp),pApp);
	}
	profileForAppDestroy(appName(pApp));
	appDestroy(pApp);
    }
    uRETURN(True);
}

/***====================================================================***/

Boolean
SetAppRoomState(pApp, pRoom, pNewState)
AppPtr		pApp;
RoomPtr		pRoom;
AppStatePtr	pNewState;
{

    uENTRY3("SetAppRoomState(%s,%s,%s)\n",appText(pApp),roomText(pRoom),
					asText(pNewState));
    if (pRoom==NullRoom)
	pRoom=	GetCurrentRoom();
    if ((pApp==NullApp)||(pNewState==NullAppState)||(pRoom==NullRoom)||
	(!asLegalReal(pNewState))) {
	uRETURN(False);
    }
    if (!appIsActive(pApp)) {
	if(appSetRoomState(pApp,roomName(pRoom),pNewState,False)==NullAppState){
	    uRETURN(False);
	}
    }
    else {
	Boolean	ok= roomChangeAppLocalState(pRoom,pApp,pNewState);
	uRETURN(ok);
    }
    uRETURN(True);
}

/***====================================================================***/

Boolean
SetAppPermanence(pApp,permanent)
AppPtr	pApp;
Boolean	permanent;
{
Boolean	ok= True;

    uENTRY2("SetAppPermanence(%s,%s)\n",appText(pApp),booleanText(permanent));
    if (pApp==NullApp) {
	uRETURN(False);
    }
    if ((permanent)&&(!appIsPermanent(pApp))) {
	ok= profileForApp(appName(pApp));
	ok= appSetPermanence(pApp,True)&&ok;
    }
    else if ((!permanent)&&(appIsPermanent(pApp))) {
	ok= appSetPermanence(pApp,False);
    }
    uRETURN(ok);
}

/***====================================================================***/

Boolean
AppStateChanged(pApp, pNewState)
AppPtr		pApp;
AppStatePtr	pNewState;
{
RoomPtr		pRoom;

    uENTRY2("AppStateChanged(%s,%s)\n",appText(pApp),asText(pNewState));
    if ((pApp==NullApp)||(pNewState==NullAppState)||
			 (asWinState(pNewState)==asInactive)) {
	uRETURN(False);
    }
    if (!appIsActive(pApp)) {
	uRETURN(False);
    }

    if (appSetCurrent(pApp,pNewState,True)!=NullAppState) {
	if (asWinState(pNewState)!=asWithdrawn) {
	    pRoom=	GetCurrentRoom();
	    if (pRoom!=NullRoom) {
		(void)roomAppRealStateChanged(pRoom,pApp,pNewState);
	    }
	}
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

static	AppsIterFunc	appsIterFunc;

static Boolean
_AppsIterator(row,keyIn,dataIn,arg)
unsigned	row;
GenKey		keyIn;
GenData	dataIn;
Opaque		arg;
{
StringToken	key=	(StringToken)keyIn;
AppPtr		pApp=	(AppPtr)dataIn;
Boolean	ok;

    uENTRY4("_AppsIterator(%d,%s,%s,0x%x)\n",row,stText(key),appText(pApp),arg);
    ok= (*appsIterFunc)(arg,pApp);
    uRETURN(ok);
}

void
AppsIterate(func,arg)
AppsIterFunc	func;
Opaque		arg;
{
    uENTRY2("AppsIterate(0x%x,0x%x)\n",func,arg);
    if ((func==NULL)||(pAllApps==NullHashTable)) {
	uVOIDRETURN;
    }
    appsIterFunc=	func;
    (void)htIterate(pAllApps,_AppsIterator,arg);
    uVOIDRETURN;
}

void
ActiveAppsIterate(func,arg)
AppsIterFunc	func;
Opaque		arg;
{
    uENTRY2("AppsIterate(0x%x,0x%x)\n",func,arg);
    if ((func==NULL)||(pActiveApps==NullHashTable)) {
	uVOIDRETURN;
    }
    appsIterFunc=	func;
    (void)htIterate(pActiveApps,_AppsIterator,arg);
    uVOIDRETURN;
}

/***====================================================================***/

struct appsIterState {
	HashTablePtr	pTbl;
	Opaque		tblState;
} iState;

Boolean	
_AppsInitIter(which,pOutState)
unsigned	which;
Opaque		*pOutState;
{
Boolean	ok=	False;
HashTablePtr	pTbl;
Opaque		tblState;
struct	appsIterState	*pState;

    uENTRY2("_AppsInitIter(%d,0x%x)\n",which,pOutState);
    if ((which==ActiveApps)&&(pActiveApps!=NullHashTable)) {
	pTbl= pActiveApps;
	ok= _htInitIter(pActiveApps,&tblState);
    }
    else if ((which==AllApps)&&(pAllApps!=NullHashTable)) {
	pTbl= pAllApps;
	ok= _htInitIter(pAllApps,&tblState);
    }
    if (ok) {
	if (iState.pTbl==NullHashTable)	
	     pState=	&iState;
	else pState=	uTypedAlloc(struct appsIterState);
	if (pState!=(struct appsIterState *) NULL) {
	    pState->pTbl=	pTbl;
	    pState->tblState=	tblState;
	    *pOutState=	(Opaque)pState;
	}
	else ok= False;
    }
    if (!ok)
	*pOutState=	(Opaque)NULL;
    uRETURN(ok);
}

/***====================================================================***/

Boolean	
_AppsIterNext(pInState, ppApp)
Opaque	 pInState;
AppPtr	*ppApp;
{
Boolean	ok= False;
GenKey	key;
GenData	data;
struct appsIterState	*pState= (struct appsIterState *)pInState;

    uENTRY2("_AppsIterNext(0x%x,0x%x)\n",pInState,ppApp);
    if (pInState!=(Opaque)NULL) {
	ok=	_htIterNext(pState->pTbl,pState->tblState,&key,&data);
	if (ok) {
	    *ppApp=		(AppPtr)data;
	}
	else *ppApp=	NullApp;
    }
    uRETURN(ok);
}

/***====================================================================***/

void
AppsEndIterator(state)
Opaque	state;
{
struct appsIterState	*pState= (struct appsIterState *)state;

    uENTRY1("AppsEndIterator(0x%x)\n",state);
    if (state!=(Opaque)NULL) {
	htEndIterator(pState->pTbl,pState->tblState);
	pState->pTbl=	NullHashTable;
	pState->tblState=	(Opaque)NULL;
	if (pState!=&iState) {
	    (void)uFree((Opaque)pState);
	}
    }
    uVOIDRETURN;
}
