#ifndef APPS_H
#define APPS_H 1

  /*\
   *  $Header: apps.h,v 5.0 90/01/10 06:53:05 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

  /*\

  \*/
	/*\
	 * Operations on a group of Apps (that live in a group of Rooms).
	 * This package implements a group of Apps and their interactions 
	 * with rooms.   There is a fair amount of interdepency between
	 * this package and rooms.c.
	 *
	 * This package handles name collisions between Apps, App lifetimes
	 * (i.e. automatically destroying transient Apps when they exit, and
	 * matching up permanent Apps with new windows), and figures out 
	 * which Room local state (if any) to change in response to an 
	 * external change on a window.   It makes sure that all of the
	 * active Apps (and none of the inactive Apps) are realized in all
	 * rooms.
	\*/

#include "app.h"

typedef	Boolean	(*AppsIterFunc)(/* arg, pApp */);

extern	unsigned int	appsDebug;

extern	AppPtr		LookupApp(/* name */);
extern	AppPtr		LookupActiveApp(/* name */);

extern	AppPtr		GetApp(/*name,onlyInactive*/);
extern	Boolean		AppActivated(/* pApp, pRealState */);
extern	Boolean		AppDeactivated(/*pApp*/);
extern	Boolean		AppStateChanged(/* pApp, pNewState */);

extern	Boolean		SetAppRoomState(/* pApp, pRoom, pNewState */);
extern	Boolean		SetAppPermanence(/* pApp, permanent */);

extern	void		AppsIterate(/* func, arg */);
extern	void		ActiveAppsIterate(/* func, arg */);

extern	Boolean		_AppsInitIter(/* pState */);
extern	Boolean		_AppsIterNext(/* pState, ppApp */);
extern	void		AppsEndIterator(/* state */);

#define	ActiveApps	((unsigned)1)
#define	AllApps		((unsigned)2)

#define	AppsIterator(w,pa,s)	\
	if (_AppsInitIter((w),&(s))) \
	    while (_AppsIterNext((s),&(pa)))
#endif /* APPS_H */
