
  /*\
   *  $Header: appstate.c,v 5.0 90/01/10 06:53:09 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define	DEBUG_VAR	asDebug
#include "utils.h"
#include "appstatestr.h"

/***====================================================================***/

static	unsigned	asScreenWidth=	AS_DFLT_SCREEN_WIDTH;
static	unsigned	asScreenHeight=	AS_DFLT_SCREEN_HEIGHT;
static	AppStateRec	asDefaultRealRec=	{ 
	asAllValues, asNoValue, asNoValue, asInactive, 0, 0, 0, 0
};
static	AppStateRec	asDefaultProfileRec=	{ 
	asWinStateValue, asNoValue, asNoValue, asIconic, 0, 0, 0, 0
};
	AppStatePtr	asDefaultReal=		&asDefaultRealRec;
	AppStatePtr	asDefaultProfile=	&asDefaultProfileRec;

/***====================================================================***/
/***                      DEBUGGING FUNCTIONS                           ***/
/***====================================================================***/

char *
asWinStateText(winState)
WinState	winState;
{
    switch (winState) {
	case asInactive:	return("inactive");
	case asWithdrawn:	return("withdrawn");
	case asNormal:		return("normal");
	case asIconic:		return("iconic");
    }
    return("unknown");
}

/***====================================================================***/

char *
asText(pState)
AppStatePtr	pState;
{
static int	bufIndex;
static char	asBuf[2][64];
static char	xBuf[12],yBuf[12],widthBuf[12],heightBuf[12];
static char	*stateBuf;
char  *buf;

    if ((pState==NullAppState)||(!ASHasAny(pState)))
	return("[ ]");

    buf= asBuf[bufIndex];
    bufIndex=	!bufIndex;

    if (ASHasWinState(pState))	
	 stateBuf=	asWinStateText(pState->winState);
    else stateBuf=	"";

    if (ASHasWholeGeom(pState)) {
	sprintf(buf,"[ %s%s =%dx%d%c%d%c%d ",stateBuf,
			(ASHasWinState(pState)&&
			 ASWinStateBolted(pState)?"!":""),
			pState->width,pState->height,
			(ASHasNegativeX(pState)?'-':'+'),pState->x,
			(ASHasNegativeY(pState)?'-':'+'),pState->y);
    }
    else if (ASHasAnyGeom(pState)) {
	if (ASHasWidth(pState))	sprintf(widthBuf,"%d",pState->width);
	else 				widthBuf[0]= '\0';

	if (ASHasHeight(pState))	sprintf(heightBuf,"x%d",pState->height);
	else 				heightBuf[0]= '\0';

	if (ASHasX(pState)) {
	    if (ASHasNegativeX(pState))	sprintf(xBuf,"-%d",pState->x);
	    else			sprintf(xBuf,"+%d",pState->x);
	}
	else if (ASHasY(pState))	sprintf(xBuf,"+");
	else 				xBuf[0]= '\0';

	if (ASHasY(pState)) {
	    if (ASHasNegativeY(pState)) sprintf(yBuf,"-%d",pState->y);
	    else			 sprintf(yBuf,"+%d",pState->y);
	}
	else				yBuf[0]= '\0';

	sprintf(buf,"[ %s%s =%s%s%s%s ",stateBuf,
					((ASWinStateBolted(pState)&&
					  ASHasWinState(pState))?"!":""),
					widthBuf,heightBuf,xBuf,yBuf);
    }
    else {
	sprintf(buf,"[ %s%s ",stateBuf,((ASWinStateBolted(pState)&&
					 ASHasWinState(pState))?"!":""));
    }
    if (ASHasAnyGeom(pState)&&ASAnyGeomBolted(pState)) {
	char	bolts[12];
	int	i=	0;
	bolts[i++]=	' ';
	if (ASXBolted(pState))	bolts[i++]=	'x';
	if (ASYBolted(pState))	bolts[i++]=	'y';
	if (ASWidthBolted(pState))	bolts[i++]=	'w';
	if (ASHeightBolted(pState))	bolts[i++]=	'h';
	bolts[i++]=	'!';
	bolts[i++]=	NUL;
	strcat(buf,bolts);
    }
    strcat(buf," ]");
    return(buf);
}

/***====================================================================***/
/***   FUNCTIONS FOR DEALING WITH NEGATIVE SPECIFICATIONS OF X AND Y    ***/
/***====================================================================***/

void
asSetScreenSize(width,height)
unsigned	width;
unsigned	height;
{
    uFLAG_ENTRY2(LOW_ENTRY_BIT,"asSetScreenSize(%d,%d)\n",width,height);
    asScreenWidth=	width;
    asScreenHeight=	height;
    uFLAG_VOIDRETURN;
}

/***====================================================================***/

Boolean
asChangeProfileFlags(pReal,pProf,newFlags)
AppStatePtr		pReal;
AppStatePtr		pProf;
unsigned	newFlags;
{
int		x,y;
unsigned	w,h;

    uENTRY3("asChangeProfileFlags(%s,%s,0x%x)\n",pReal,pProf,newFlags);
    if ((!ASLegalRealState(pReal))||(pProf==NullAppState)||
	(!ASLegalProfileState(pProf))) {
	uRETURN(False);
    }

    if ((newFlags&asAllValues)==asNoValue)
	pProf->flags=	0;
    else if (pProf->flags!=newFlags) {
	if (!ASHasWinState(pProf))
	    pProf->winState=	pReal->winState;
	w=	(ASHasWidth(pProf)?	pProf->width:	pReal->width);
	h=	(ASHasHeight(pProf)?	pProf->height:	pReal->height);
	if (ASHasX(pProf)) {
	    if (ASHasNegativeX(pProf))	x= asScreenWidth-pProf->x-w;
	    else 			x= pProf->x;
	}
	else	x=	pReal->x;
	if (ASHasY(pProf)) {
	    if (ASHasNegativeY(pProf))	y= asScreenHeight-pProf->y-h;
	    else 			y= pProf->y;
	}
	else	y=	pReal->y;

	pProf->flags=	newFlags;
	if (ASHasNegativeX(pProf))	pProf->x=	asScreenWidth-x-w;
	else				pProf->x=	x;
	if (ASHasNegativeY(pProf))	pProf->y=	asScreenHeight-y-h;
	else				pProf->y=	y;
	pProf->width=	w;
	pProf->height=	h;
    }
    uRETURN(True);
}

/***====================================================================***/

Boolean
asGetScreenValues(pReal,pProf,pX,pY,pWidth,pHeight)
AppStatePtr		pReal,pProf;
int		*pX,*pY;
unsigned	*pWidth,*pHeight;
{
int		x,y;
unsigned	w,h;

    uENTRY6("asGetScreenValues(%s,%s,0x%x,0x%x,0x%x,0x%x)\n",
				asText(pReal),asText(pProf),
				pX,pY,pWidth,pHeight);
    if (ASLegalRealState(pReal)&&ASLegalProfileState(pProf)) {
	w=	(ASHasWidth(pProf)?	pProf->width:	pReal->width);
	h=	(ASHasHeight(pProf)?	pProf->height:	pReal->height);
	if (ASHasX(pProf)) {
	    if (ASHasNegativeX(pProf))	x= asScreenWidth-pProf->x-w;
	    else 			x= pProf->x;
	}
	else	x=	pReal->x;
	if (ASHasY(pProf)) {
	    if (ASHasNegativeY(pProf))	y= asScreenHeight-pProf->y-h;
	    else 			y= pProf->y;
	}
	else	y=	pReal->y;
	if (pX!=(int *)NULL)		*pX=		x;
	if (pY!=(int *)NULL)		*pY=		y;
	if (pWidth!=(unsigned *)NULL)	*pWidth=	w;
	if (pHeight!=(unsigned *)NULL)	*pHeight=	h;
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/
/***       FUNCTIONS TO CREATE AND DESTROY APPLICATION STATES           ***/
/***====================================================================***/

AppStatePtr
asCreate(flags, winState, x, y, width, height)
unsigned	flags;
WinState	winState;
int		x,y;
unsigned 	width,height;
{
AppStatePtr	pState;

    uENTRY6("asCreate(0x%x,%s,%d,%d,%d,%d)\n",flags,asWinStateText(winState),
						x,y,width,height);
    pState=	uTypedAlloc(AppStateRec);
    if (pState!=NullAppState) {
	pState->flags=		(flags&asLegalFlags);
	pState->bolts=		asNoValue;
	pState->looseBolts=	asNoValue;
	pState->winState=	winState;
	pState->x=		x;
	pState->y=		y;
	pState->width=		width;
	pState->height=		height;
    }
    uRETURN(pState);
}

/***====================================================================***/

AppStatePtr
asDupFully(pState)
AppStatePtr	pState;
{
AppStatePtr	pNew;

    uENTRY1("asDupFully(%s)\n",asText(pState));
    pNew=	uTypedAlloc(AppStateRec);

    if (pNew!=NullAppState) {
	if (pState==NullAppState) {
	    pNew->flags=	asNoValue;
	}
	else {
	    *pNew=	*pState;
	}
    }
    uRETURN(pNew);
}

/***====================================================================***/

AppStatePtr
asDupReal(pReal,flags)
AppStatePtr		pReal;
unsigned	flags;
{
AppStatePtr		pNew;

    uENTRY2("asDupReal(%s,0x%x)\n",asText(pReal),flags);
    if (!ASLegalRealState(pReal)) {
	uRETURN(NullAppState);
    }
    pNew=	uTypedAlloc(AppStateRec);
    if (pNew!=NullAppState) {
	*pNew=	*pReal;
	pNew->flags=	flags;
	if (ASHasNegativeX(pNew))
	    pNew->x= asScreenWidth-pNew->x-pNew->width;
	if (ASHasNegativeY(pNew))
	    pNew->y= asScreenHeight-pNew->y-pNew->height;
    }
    uRETURN(pNew);
}

/***====================================================================***/

void
asDestroy(pState)
AppStatePtr	pState;
{
    uENTRY1("asDestroy(%s)\n",asText(pState));
    if (pState!=NullAppState) {
	(void)uFree((Opaque)pState);
    }
    uVOIDRETURN;
}

/***====================================================================***/
/***     FUNCTIONS TO COPY, CONVERT, AND MODIFY APPLICATION STATES      ***/
/***====================================================================***/

Boolean
asCopyFully(pFrom, pInto)
AppStatePtr	pFrom, pInto;
{
    uENTRY2("asCopyFully(%s,%s)\n",asText(pFrom),asText(pInto));
    if (pInto==NullAppState) {
	uRETURN(False);
    }
    else if (pFrom==NullAppState) {
	pInto->flags=	asNoValue;
    }
    else *pInto= *pFrom;
    uRETURN(True);
}

/***====================================================================***/

Boolean
asCopySelected(pFrom, pInto, flags)
AppStatePtr	pFrom, pInto;
unsigned	flags;
{
    uENTRY3("asCopySelected(%s,%s,0x%x)\n",asText(pFrom),asText(pInto),flags);
    if (pInto==NullAppState) {
	uRETURN(False);
    }
    if (pFrom!=NullAppState) {
	flags&=	pFrom->flags;
	if (flags&asWinStateValue) {
	    pInto->winState=	pFrom->winState;
	    pInto->flags|=	asWinStateValue;
	}
	if (flags&asXValue) {
	    pInto->flags|=	asXValue;
	    pInto->x=	pFrom->x;
	    if (flags&asXNegative) {
		if (ASHasNegativeX(pFrom))	pInto->flags|=	asXNegative;
		else				pInto->flags&=	~asXNegative;
	    }
	}
	if (flags&asYValue) {
	    pInto->flags|=	asYValue;
	    pInto->y=	pFrom->y;
	    if (flags&asYNegative) {
		if (ASHasNegativeY(pFrom))	pInto->flags|=	asYNegative;
		else				pInto->flags&=	~asYNegative;
	    }
	}
	if (flags&asWidthValue) {
	    pInto->width=	pFrom->width;
	    pInto->flags|=	asWidthValue;
	}
	if (flags&asHeightValue) {
	    pInto->height=	pFrom->height;
	    pInto->flags|=	asHeightValue;
	}
    }
    uRETURN(True);
}

/***====================================================================***/

Boolean
asCopyRealToProfile(pRealFrom,pProfInto)
AppStatePtr	pRealFrom,pProfInto;
{
    uENTRY2("asCopyRealToProfile(%s,%s)\n",asText(pRealFrom),
						asText(pProfInto));
    if ((!ASLegalRealState(pRealFrom))||(pProfInto==NullAppState)||
	(!ASLegalProfileState(pProfInto))) {
	uRETURN(False);
    }
    if ((!ASWinStateBolted(pProfInto))&&
	(pRealFrom->winState!=asInactive)&&(pRealFrom->winState!=asWithdrawn)) {
	pProfInto->winState=pRealFrom->winState;
    }
    if (!ASWidthBolted(pProfInto))
	pProfInto->width=	pRealFrom->width;
    if (!ASHeightBolted(pProfInto))
	pProfInto->height=	pRealFrom->height;
    if ((!ASXBolted(pProfInto))&&(ASHasX(pProfInto))) {
	int	w= (ASHasWidth(pProfInto)?pProfInto->width:pRealFrom->width);
	if (ASHasNegativeX(pProfInto))
	     pProfInto->x=	asScreenWidth-w-pRealFrom->x;
	else pProfInto->x=	pRealFrom->x;
    }
    if ((!ASYBolted(pProfInto))&&(ASHasY(pProfInto))) {
	int	h= (ASHasHeight(pProfInto)?pProfInto->height:pRealFrom->height);
	if (ASHasNegativeY(pProfInto))
	     pProfInto->y=	asScreenHeight-h-pRealFrom->y;
	else pProfInto->y=	pRealFrom->y;
    }
    uRETURN(True);
}

/***====================================================================***/

Boolean
asCopyProfileToReal(pProfFrom,pRealInto)
AppStatePtr	pProfFrom,pRealInto;
{
    uENTRY2("asCopyProfileToReal(%s,%s)\n",asText(pProfFrom),
						asText(pRealInto));
    if ((!ASLegalProfileState(pProfFrom))||(!ASLegalRealState(pRealInto))) {
	uRETURN(False);
    }
    if (ASHasWinState(pProfFrom))
	pRealInto->winState=	pProfFrom->winState;
    if (ASHasWidth(pProfFrom))	pRealInto->width=  pProfFrom->width;
    if (ASHasHeight(pProfFrom))	pRealInto->height= pProfFrom->height;
    if (ASHasX(pProfFrom)) {
	if (ASHasNegativeX(pProfFrom)) 
	     pRealInto->x= asScreenWidth-pProfFrom->x-pRealInto->width;
	else pRealInto->x= pProfFrom->x;
    }
    if (ASHasY(pProfFrom)) {
	if (ASHasNegativeY(pProfFrom)) 
	     pRealInto->y= asScreenHeight-pProfFrom->y-pRealInto->height;
	else pRealInto->y= pProfFrom->y;
    }
    uRETURN(True);
}

/***====================================================================***/

Boolean
asChangeSelected(pState, which, winState, x, y, width, height)
register AppStatePtr pState;
unsigned	which;
WinState	winState;
int		x,y;
unsigned	width,height;
{
    uENTRY7("asChangeSelected(%s,0x%x,%s,%d,%d,%d,%d)\n",asText(pState),which,
						asWinStateText(winState),
						x,y,width,height);
    if (pState==NullAppState) {
	uRETURN(False);
    }

    if (which&asWinStateValue)	pState->winState=	winState;
    if (which&asXValue) {
	pState->x=		x;
	if (which&asXNegative)	pState->flags|=	asXNegative;
	else			pState->flags&=	(~asXNegative);
    }
    if (which&asYValue)	{
	pState->y=		y;
	if (which&asYNegative)	pState->flags|=	asYNegative;
	else			pState->flags&=	(~asYNegative);
    }
    if (which&asWidthValue)	pState->width=	width;
    if (which&asHeightValue)	pState->height=	height;
    pState->flags|=	(which&asAllValues);
    uRETURN(True);
}

/***====================================================================***/

Boolean
asChangeAll(pState, flags, winState, x, y, width, height)
register AppStatePtr pState;
unsigned	flags;
WinState	winState;
int		x,y;
unsigned	width,height;
{
    uENTRY7("asChangeAll(%s,0x%x,%s,%d,%d,%d,%d)\n",asText(pState),flags,
						asWinStateText(winState),
						x,y,width,height);
    if (pState==NullAppState) {
	uRETURN(False);
    }

    pState->flags=	(flags&asLegalFlags);
    pState->winState=	winState;
    pState->x=		x;
    pState->y=		y;
    pState->width=	width;
    pState->height=	height;
    uRETURN(True);
}

/***====================================================================***/

Boolean
asBolt(pState,which)
AppStatePtr	pState;
unsigned	which;
{
    uENTRY2("asBolt(%s,0x%x)\n",asText(pState),which);
    if (pState!=NullAppState) {
	pState->bolts|=		(which&asAllValues);
	pState->looseBolts&=	~(which&asAllValues);
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
asUnbolt(pState,which)
AppStatePtr	pState;
unsigned	which;
{
    uENTRY2("asUnbolt(%s,0x%x)\n",asText(pState),which);
    if (pState!=NullAppState) {
	pState->bolts&=		~(which&asAllValues);
	pState->looseBolts&=	~(which&asAllValues);
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
asLoosenBolts(pState,which)
AppStatePtr	pState;
unsigned	which;
{
    uENTRY2("asLoosenBolt(%s,0x%x)\n",asText(pState),which);
    if (pState!=NullAppState) {
	which&=	pState->bolts;
	pState->bolts&=		~(which&asAllValues);
	pState->looseBolts|=	 (which&asAllValues);
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
asTightenBolts(pState,which)
AppStatePtr	pState;
unsigned	which;
{
    uENTRY2("asTightenBolts(%s,0x%x)\n",asText(pState),which);
    if (pState!=NullAppState) {
	which&=	pState->looseBolts;
	pState->bolts|=		(which&asAllValues);
	pState->looseBolts&=	~(which&asAllValues);
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
asSetBolts(pState,which)
AppStatePtr	pState;
unsigned	which;
{
    uENTRY2("asSetBolts(%s,0x%x)\n",asText(pState),which);
    if (pState!=NullAppState) {
	pState->bolts=		which&asAllValues;
	pState->looseBolts&=	~(which&asAllValues);
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
asSetLooseBolts(pState,which)
AppStatePtr	pState;
unsigned	which;
{
    uENTRY2("asSetLooseBolts(%s,0x%x)\n",asText(pState),which);
    which&=	asAllValues;
    if (pState!=NullAppState) {
	pState->bolts|=		pState->looseBolts;
	pState->looseBolts=	which;
	pState->bolts&=		~which;
    }
    uRETURN(False);
}

/***====================================================================***/
/***       FUNCTIONS TO GET INFORMATION ABOUT APPLICATION STATES        ***/
/***====================================================================***/

unsigned	
asValues(pState, pWinState, pX, pY, pWidth, pHeight)
register AppStatePtr pState;
WinState	*pWinState;
int		*pX,*pY;
unsigned	*pWidth,*pHeight;
{
unsigned	flags;

    uFLAG_ENTRY6(LOW_ENTRY_BIT,"asValues(%s,0x%x,0x%x,0x%x,0x%x,0x%x)\n",
				asText(pState),pWinState,pX,pY,pWidth,pHeight);
    if (pState==NullAppState) {
	uFLAG_RETURN(asNoValue);
    }

    flags=	pState->flags;
    if ((pWinState!=(WinState *)NULL)&&(ASHasWinState(pState)))
	*pWinState=	pState->winState;
    if ((pX!=(int *)NULL)&&ASHasX(pState)) {
	if (ASHasWidth(pState)&&(ASHasNegativeX(pState))) {
	     *pX=	asScreenWidth-pState->x-pState->width;
	     flags&=	(~asXNegative);
	}
	else *pX=	  pState->x;
    }
    if ((pY!=(int *)NULL)&&ASHasY(pState)) {
	if (ASHasHeight(pState)&&(ASHasNegativeY(pState))) {
	     *pY=	asScreenHeight-pState->y-pState->height;
	     flags&=	(~asYNegative);
	}
	else *pY=	  pState->y;
    }
    if ((pWidth!=(unsigned *)NULL)&&ASHasWidth(pState)) 
	*pWidth=  pState->width;
    if ((pHeight!=(unsigned *)NULL)&&ASHasHeight(pState))
	*pHeight= pState->height;
    uFLAG_RETURN(flags);
}

/***====================================================================***/

WinState	
asWinState(pState)
AppStatePtr	 pState;
{

    uFLAG_ENTRY1(LOW_ENTRY_BIT,"asWinState(%s)\n",asText(pState));
    if ((pState!=NullAppState)&&(ASHasWinState(pState))) {
	uFLAG_RETURN(pState->winState);
    }
    uFLAG_RETURN(asInactive);
}

/***====================================================================***/

unsigned
asFlags(pState)
AppStatePtr	pState;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"asFlags(%s)\n",asText(pState));
    if (pState!=NullAppState) {
	uFLAG_RETURN(pState->flags);
    }
    uFLAG_RETURN(asNoValue);
}

/***====================================================================***/

unsigned
asBolts(pState)
AppStatePtr	pState;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"asBolts(%s)\n",asText(pState));
    if (pState!=NullAppState) {
	uFLAG_RETURN(pState->bolts);
    }
    uFLAG_RETURN(asNoValue);
}

/***====================================================================***/

unsigned
asLooseBolts(pState)
AppStatePtr	pState;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"asLooseBolts(%s)\n",asText(pState));
    if (pState!=NullAppState) {
	uFLAG_RETURN(pState->looseBolts);
    }
    uFLAG_RETURN(asNoValue);
}

/***====================================================================***/
/***                 FUNCTIONS TO COMPARE GEOMETRIES                    ***/
/***====================================================================***/

Boolean
asRealMatchesProfile(pReal,pProf)
AppStatePtr	pReal;
AppStatePtr	pProf;
{
Boolean	match;

    uFLAG_ENTRY2(LOW_ENTRY_BIT,"asRealMatchesProfile(%s,%s)\n",asText(pReal),
								asText(pProf));
    match= asSelectedRealMatchProfile(pReal,pProf,asAllValues);
    uFLAG_RETURN(match);
}

/***====================================================================***/

Boolean
asSelectedRealMatchProfile(pReal,pProf,which)
AppStatePtr	pReal;
AppStatePtr	pProf;
unsigned	which;
{
Boolean	match;

    uENTRY3("asSelectedRealMatchProfile(%s,%s,0x%x)\n",asText(pReal),
						asText(pProf),which);
    if ((!ASLegalRealState(pReal))||(!ASLegalProfileState(pProf))) {
	uRETURN(False);
    }
    which&=	pProf->flags;
    match= True;
    if (pProf!=NullAppState) {
	if ((which&asWinStateValue)&&
	    (pReal->winState!=asInactive)&&(pReal->winState!=asWithdrawn)&&
	    (pReal->winState!=pProf->winState))
	    match= False;
	else if (((which&asWidthValue)&&(pProf->width!=pReal->width))||
		 ((which&asHeightValue)&&(pProf->height!=pReal->height)))
	    match= False;
	else {
	    if (which&asXValue) {
		if (ASHasNegativeX(pProf)) {
		    int	w= (ASHasWidth(pProf)?pProf->width:pReal->width);
		    if ((asScreenWidth-pProf->x-w)!=pReal->x)
			match=	False;
		}
		else if (pProf->x!=pReal->x)
		    match= False;
	    }
	    if (match&&(which&asYValue)) {
		if (ASHasNegativeY(pProf)) {
		    int	h= (ASHasHeight(pProf)?pProf->height:pReal->height);
		    if ((asScreenHeight-pProf->y-h)!=pReal->y)
			match=	False;
		}
		else if (pProf->y!=pReal->y)
		    match= False;
	    }
	}
    }
    uRETURN(match);
}

/***====================================================================***/

Boolean
asProfileCorrectWRTReal(pProf,pReal)
AppStatePtr	pProf;
AppStatePtr	pReal;
{
Boolean	match;

    uFLAG_ENTRY2(LOW_ENTRY_BIT,"asProfileCorrectWRTReal(%s,%s)\n",asText(pProf),
								asText(pReal));
    match= asSelectedProfileCorrectWRTReal(pProf,pReal,asAllValues);
    uFLAG_RETURN(match);
}

/***====================================================================***/

Boolean
asSelectedProfileCorrectWRTReal(pProf,pReal,which)
AppStatePtr	pProf;
AppStatePtr	pReal;
unsigned	which;
{
Boolean	match;

    uENTRY3("asSelectedProfileCorrectWRTReal(%s,%s,0x%x)\n",asText(pProf),
							asText(pReal),which);
    if (!ASLegalRealState(pReal)) {
	uRETURN(False);
    }
    match= True;
    which&=	pProf->flags;
    if (pProf!=NullAppState) {
	if ((which&asWinStateValue)&&(!ASWinStateBolted(pProf))&&
	    (pReal->winState!=asInactive)&&(pReal->winState!=asWithdrawn)&&
	    (pReal->winState!=pProf->winState))
	    match= False;
	else if (((which&asWidthValue)&&(!ASWidthBolted(pProf))&&
				     (pProf->width!=pReal->width))||
		 ((which&asHeightValue)&&(!ASHeightBolted(pProf))&&
				     (pProf->height!=pReal->height)))
	    match= False;
	else if ((which&asXValue)&&(!ASXBolted(pProf))) {
	    if (ASHasNegativeX(pProf)) {
		int	w= (ASHasWidth(pProf)?pProf->width:pReal->width);
		if ((asScreenWidth-pProf->x-w)!=pReal->x)
		    match=	False;
	    }
	    else if (pProf->x!=pReal->x)
		match= False;
	}
	else if ((which&asYValue)&&(!ASYBolted(pProf))) {
	    if (ASHasNegativeY(pProf)) {
		int	h= (ASHasHeight(pProf)?pProf->height:pReal->height);
		if ((asScreenHeight-pProf->y-h)!=pReal->y)
		    match=	False;
	    }
	    else if (pProf->y!=pReal->y)
		match= False;
	}
    }
    uRETURN(match);
}

/***====================================================================***/

Boolean
asFullySame(pState1,pState2)
AppStatePtr	pState1;
AppStatePtr	pState2;
{
Boolean	match=	True;

    uFLAG_ENTRY2(LOW_ENTRY_BIT,"asFullySame(%s,%s)\n",asText(pState1),
							asText(pState2));
    match= asSelectedSame(pState1,pState2,asAllValues);
    uFLAG_RETURN(match);
}

/***====================================================================***/

Boolean
asSelectedSame(pState1,pState2,which)
AppStatePtr	pState1;
AppStatePtr	pState2;
unsigned	which;
{
Boolean	match=	True;

    uENTRY3("asSelectedSame(%s,%s,0x%x)\n",asText(pState1),asText(pState2),
								which);
    if (pState1!=pState2) { 
	if ((pState1==NullAppState)||(pState2==NullAppState))
	    match=	False;
	else if ((asFlags(pState1)&which)!=(asFlags(pState2)&which))
	    match=	False;
	else {
	    which&= asFlags(pState1);
	    if (((which&asWinStateValue)&&
		(pState1->winState!=pState2->winState))||
		 ((which&asXValue)&&(pState1->x!=pState2->x))||
	         ((which&asYValue)&&(pState1->y!=pState2->y))||
		 ((which&asWidthValue)&&(pState1->width!=pState2->width))||
		 ((which&asHeightValue)&&(pState1->height!=pState2->height)))
	    match=	False;
	}
    }
    uRETURN(match);
}

/***====================================================================***/
/***    FUNCTIONS TO DETERMINE OTHER USEFUL THINGS ABOUT GEOMETRIES     ***/
/***====================================================================***/

Boolean
asLegalReal(pState)
AppStatePtr	pState;
{
Boolean	ok;

    uFLAG_ENTRY1(LOW_ENTRY_BIT,"asLegalReal(%s)\n",asText(pState));
    ok= ASLegalRealState(pState);
    uFLAG_RETURN(ok);
}

/***====================================================================***/

Boolean
asLegalProfile(pState)
AppStatePtr	pState;
{
Boolean	ok;

    uFLAG_ENTRY1(LOW_ENTRY_BIT,"asLegalProfile(%s)\n",asText(pState));
    ok= ASLegalProfileState(pState);
    uFLAG_RETURN(ok);
}
/***====================================================================***/

Boolean
asDontCare(pState)
AppStatePtr	pState;
{
Boolean	ok;

    uFLAG_ENTRY1(LOW_ENTRY_BIT,"asDontCare(%s)\n",asText(pState));
    ok= ASDontCareState(pState);
    uFLAG_RETURN(ok);
}
