#ifndef APPSTATE_H
#define APPSTATE_H 1

  /*\
   *  $Header: appstate.h,v 5.0 90/01/10 06:53:13 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

  /*\

  \*/

	/*\
	 * A positive specification of X or Y means that the defined value
	 * refers to the left or top edge of a window.  A negative X or Y 
	 * specification means that the defined value refers to the right 
	 * or bottom edge of the window.
	 *
	 *    A number of these functions distinguish between <real> AppStates
	 * and <profile> AppStates: 
	 *    <real> AppStates must be fully specified with positive X and Y.  
	 * NullAppState is not a legal <real> AppState.
	 *    <profile> AppStates may be partially specified and may have 
	 * negative X and Y.  <profile> AppStates may not have window state 
	 * Inactive or Withdrawn.  NullAppState is a legal <profile> state 
	 * with no values specified for any field.
	 *    It is an error to try to change any values in NullAppState.
	 *    By convention, the variable "pReal" should always refer to a 
	 * <real> AppState and "pProf" should always refer to a <profile> 
	 * AppState.
	 *
	 *     Values in a <profile> AppState may be "bolted."  A bolted value
	 * is not affected by values in a <real> state.   i.e.  Copying a
	 * <real> state into a bolted state does not change any bolted values.
	 *
	 *    Bolts may be "loose."   A loose bolt has no effect, but may be
	 * rebolted.   Rebolting does not affect values that do not have loose
	 * bolts.
	 *
	 * PUBLIC PACKAGE GLOBALS --
	 * unsigned	asDebug;	[ default: 0 ]
	 *	Debugging flags.  The only flag currently used is ENTER (0x10)
	 *	defined in ../utils/utils.h, which tracks function entry and
	 *	exit.
	 *
	 * AppStatePtr	asDefaultReal;	  [ default: [ inactive =0x0+0+0] ]
	 * AppStatePtr	asDefaultProfile; [ default: [ iconic ] ]
	 *	Default values for <real> and <profile> states respectively.
	 *
	 * DEBUGGING FUNCTIONS --
	 * char	*
	 * asWinStateText(winState)
	 * WinState	winState;
	 *	Returns a string containing a description of "winState."
	 *	This string is stored in a static buffer so it must *not* 
	 *	be freed and may be clobbered by subsequent calls to 
	 *	asWinStateText.
	 *
	 * char	*
	 * asText(pState)
	 * AppStatePtr	 pState;
	 *	Returns a string containing a description  of "pState."   
	 *	This string is a static buffer so it must *not* be freed 
	 *	and may be clobbered by subsequent calls to asText.
	 *
	 * FUNCTIONS FOR DEALING WITH NEGATIVE SPECIFICATIONS OF X AND Y --
	 * void	
	 * asSetScreenSize(width,height)
	 * unsigned	width,height;
	 *	Set the screen size for future conversions to and from negative
	 *	specifications of X and Y.
	 *
	 * Boolean
	 * asChangeProfileFlags(pReal,pProf,pNewFlags)
	 * AppStatePtr	pReal,pProf;
	 * unsigned	pNewFlags;
	 *	Change the specification of "pProf" to contain all of (and only)
	 *	the fields defined in pNewFlags, taking any missing values from
	 *	pReal.   "asChangeProfileFlags" will compute negative X and
	 *	Y specifications if defined in "pNewFlags."   
	 *	asChangeProfileFlags ignores bolts.
	 *	Returns True if it succeeds, False if pReal is not a legal 
	 *	<real> state or pProf is NullAppState or not a legal <profile> 
	 *	state.
	 *
	 * Boolean
	 * asGetScreenValues(pReal, pProf, pWinState, pX, pY, pWidth, pHeight)
	 * AppStatePtr	 pReal, pProf;
	 * WinState	*pWinState;		// RETURN
	 * int		*pX,*pY;		// RETURN
	 * unsigned	*pWidth, *pHeight;	// RETURN
	 *	Computes the positive screen coordinates of "pProf" and returns
	 *	the values in the return arguments pWinState, pX, pY, pWidth 
	 *	and pHeight.  Any of the return arguments may be NULL without 
	 *	error if the calling function does not use the value of that 
	 *	field.
	 *	Returns True for success, False if pReal is not a legal <real>
	 *	state or pProf is not a legal <profile> state.
	 *
	 * FUNCTIONS TO CREATE AND DESTROY APPLICATION STATES --
	 * AppStatePtr
	 * asCreate(flags, winState, x, y, width, height)
	 * unsigned	flags;
	 * WinState	winState;
	 * int		x,y;
	 * unsigned	width,height;
	 *	Creates an AppState with the specified values.  Does *not*
	 *	compute negative X or Y.  Returns NullAppState on allocation 
	 *	failure.
	 *
	 * AppStatePtr
	 * asDupFully(pState)
	 *	Creates a faithful (i.e. unmodified) duplicate of "pState."  
	 *	All of fields and bolts that are set in "pState" will be set 
	 *	to the same values in the newly created AppState.   
	 *	Returns NullAppState on	allocation failure.  
	 *	If "pState" is NullAppState, "asDupFully" returns a 
	 *	AppStatePtr with no values defined; it does *not* return 
	 *	NullAppState.
	 *
	 * AppStatePtr
	 * asDupReal(pReal, flags)
	 * AppStatePtr	pReal;
	 * unsigned	flags;
	 *	Creates a duplicate of "pReal" (which *must* be a legal real
	 *	AppState).  The fields specified in "flags" will be set to
	 *	the appropriate values from "pReal."   Correct values for
	 *	negative X and Y values *are* calculated.   
	 *	Use this function to create a new partial or negative 
	 *	specification from a real window state.   
	 *	Returns NullAppState on allocation failure or if "pReal" 
	 *	is not a legal <real> AppState.
	 *
	 * void	
	 * asDestroy(pState)
	 * AppStatePtr	pState;
	 *	Destroys "pState."   Attempts to use an AppState after it
	 *	has been destroyed may be disastrous.  No return value.
	 *

	 *
	 * FUNCTIONS TO COPY, CONVERT, AND MODIFY APPLICATION STATES --
	 * Boolean
	 * asCopyFully(pFrom, pInto)
	 * AppStatePtr	pFrom, pInto;
	 *	Faithfully (i.e. without conversion) copies "pFrom" into 
	 *	"pInto."   Sets all of the fields and bolts that are defined 
	 *	in "pFrom" to their same values in "pInto."   Former values 
	 *	of "pInto" are lost.   
	 *	If "pFrom" is NullAppState, "pInto" is changed to have 
	 *	no values defined, but the copy is successful.  
	 *	Returns True for a successful copy, False if "pInto" is 
	 *	NullAppState.
	 *
	 * Boolean
	 * asCopySelected(pFrom, pInto, flags)
	 * AppStatePtr	pFrom,pInto;
	 * unsigned	flags;
	 *	Copies selected values from "pFrom" into "pInto."  Only
	 *	copies those values that are defined in both "pFrom" and 
	 *	"flags."    Does not compute values for negative X or Y 
	 *	specifications in pFrom, but does copy the XNegative and
	 *	YNegative flags if set in "flags."    
	 *	Returns True if the copy is successful, False if any of 
	 *	the fields specified in "flags" are not defined in "pFrom."
	 *
	 * Boolean
	 * asCopyRealToProfile(pRealFrom, pProfInto)
	 * AppStatePtr	pRealFrom, pProfInto;
	 *	Copies values from "pRealFrom" into "pProfInto."  Only
	 *	copies those values that are specified in "pProfInto," and
	 *	are not bolted.  Computes correct values for negative X or
	 *	Y specifications in "pProfInto."   
	 *	Returns True for a successful copy, False if pRealFrom is 
	 *	not a legal <real> state, pProfInto is not a legal <profile> 
	 *	state, or pProfInto is NullAppState.
	 *
	 * Boolean
	 * asCopyProfileToReal(pProfFrom, pRealInto)
	 * AppStatePtr	pProfFrom,pRealInto;
	 *	Copies values from "pProfFrom" into "pRealInto."  Only
	 *	copies those values that are defined in "pProfFrom."
	 *	Computes correct values for negative X or Y specifications in
	 *	pProfFrom.   
	 *	Returns True if the copy is successful, False if pProfFrom 
	 *	is not a legal <profile> state or pRealInto is not a legal 
	 *	<real> state.
	 *
	 * Boolean
	 * asChangeSelected(pState, flags, winState, x, y, width, height)
	 * AppStatePtr	pState;
	 * unsigned	flags;
	 * WinState	winState;
	 * int		x,y;
	 * unsigned	width,height;
	 *	Changes selected values in "pState."  Changes only values 
	 *	specified in "flags."   Changes settings for X or Y Negative 
	 *	if both the Value *and* negative fields are set in "flags," 
	 *	but does *not* compute values for x or y.   Values not 
	 *	specified in "flags" are unchanged.  
	 *	Returns True for success, False if "pState" is NullAppState.
	 *
	 * Boolean	
	 * asChangeAll(pState, flags, winState, x, y, width, height )
	 * AppStatePtr	pState;
	 * unsigned	flags;
	 * WinState	winState;
	 * int		x,y;
	 * unsigned	width,height;
	 *	Changes all values in "pState" to those specified in "flags."
	 *	Does affect XNegative and YNegative, but does not compute
	 *	values for X and Y (assumes it is called with correctly
	 *	computed values).  Old values from "pState" are lost.  
	 *	Returns	True for suceessful change, False if "pState" 
	 *	is NullAppState.
	 *
	 * Boolean
	 * asBolt(pState,which)
	 * AppStatePtr	pState;
	 * unsigned	which;
	 *	Bolts the fields specified by "which" in "pState."   If
	 *	any of the specified fields have loose bolts, those bolts
	 *	are tightened.   Fields that are already bolted are not
	 *	affected.
	 *	Returns True for success, False if "pState" is NullAppState.
	 *
	 * Boolean
	 * asUnbolt(pState,which)
	 * AppStatePtr	pState;
	 * unsigned	which;
	 *	Unbolts the fields specified by "which" in "pState."   If
	 *	any of the specified fields have loose bolts, those bolts
	 *	are fully removed.   Fields without bolts are unaffected.
	 *	Returns True for success, False if "pState" is NullAppState.
	 *
	 * Boolean
	 * asLoosenBolts(pState,which)
	 * AppStatePtr	pState;
	 * unsigned	which;
	 *	Loosens (temporarily unbolts) the fields specified by "which"
	 *	in pState.   Fields without bolts or with loose bolts are 
	 *	unaffected.
	 *	Returns True for success, False if "pState" is NullAppState.
	 *
	 * Boolean
	 * asTightenBolts(pState,which)
	 * AppStatePtr	pState;
	 * unsigned	which;
	 *	Tightens (re-bolts fields that were temporarily unbolted) the
	 *	fields specified by "which" in "pState."   Fields without
	 *	loose bolts are unaffected.
	 *	Returns True for success, False if "pState" is NullAppState.
	 *
	 * Boolean
	 * asSetBolts(pState,which)
	 * AppStatePtr	pState;
	 * unsigned	which;
	 *	Bolts only the fields specified by "which" in "pState."   If
	 *	any of the specified fields have loose bolts, those bolts
	 *	are tightened.   Fields that are previously bolted that are
	 *	are not specified in "which" are unbolted.
	 *	Returns True for success, False if "pState" is NullAppState.
	 *
	 * Boolean
	 * asSetLooseBolts(pState,which)
	 * AppStatePtr	pState;
	 * unsigned	which;
	 *	Sets the loose bolts in "pState" to be only the fields 
	 *	specified by "which."   Any of the specified fields which are
	 *	bolted are loosened.   Tightens the bolts of any fields that 
	 *	are not specified in "which" but have loose bolts.
	 *	Returns True for success, False if "pState" is NullAppState.
	 *
	 * FUNCTIONS TO GET INFORMATION ABOUT APPLICATON STATES --
	 * unsigned
	 * asValues(pState, pWinState, pX, pY, pWidth, pHeight)
	 * AppStatePtr	 pState;
	 * WinState	*pWinState;
	 * int		*pX,*pY;		// RETURN 
	 * unsigned	*pWidth,*pHeight;	// RETURN
	 * 	Use asValues to find out the value of any of the components
	 *	of a AppState.   Returns the fields defined and values of 
	 *	XNegative and YNegative for "pState".  Stores the values of
	 *	any defined fields in the return arguments "pWinState," "pX,"
	 *	"pY," "pWidth," and "pHeight."   If x or y has a negative 
	 *	specification and width or height is defined, asValues will
	 *	calculate the positive (left or top) value and will 
	 *	*not* return asXNegative or asYNegative.   If the x or y
	 *	specification is negative but width or height is not defined,
	 *	the right or top side value is returned along wth asXNegative
	 *	(asYNegative). 
	 *	Any of the return arguments may	be NULL, and only those values 
	 *	defined in the AppState will be changed.  
	 *	Returns asNoValue if "pState" is NullAppState.
	 *
	 * WinState
	 * asWinState(pState)
	 * AppStatePtr	 pState;
	 *	Returns in the window state of "pState," or asInactive if 
	 *	"pState" is NullAppState, or "pState" has no defined
	 *	value for window state.  
	 *
	 * unsigned
	 * asFlags(pState)
	 * AppStatePtr	pState;
	 *	Returns the fields defined and values for XNegative and
	 *	YNegative of "pState."   Returns asNoValue if "pState" is 
	 *	NullAppState.
	 *
	 * unsigned
	 * asBolts(pState)
	 * AppStatePtr	pState;
	 *	Returns a mask of the fields that are currently bolted in 
	 *	"pState."  Does not report loose bolts.
	 *	Returns asNoValue if "pState" is NullAppState.
	 *
	 * unsigned
	 * asLooseBolts(pState)
	 * AppStatePtr	pState;
	 *	Returns a mask of the fields that are currently temporarily
	 *	unbolted in "pState."   Does not return fields that are
	 *	fully bolted or unbolted.
	 *	Returns asNoValue if "pState" is NullAppState.
	 *
	 * FUNCTIONS TO COMPARE APPLICATION STATES --
	 * Boolean
	 * asProfileCorrectWRToReal(pProf, pReal)
	 * AppStatePtr	pProf,pReal;
	 *	    Returns True if "pProf" is correct with respect to  
	 *	"pReal."  
	 *	    A <profile> AppState is correct WRT a <real> AppState 
	 *	IFF all of the values that are defined in the <profile> 
	 *	AppState and are *not* bolted match the corresponding values in 
	 *	the <real> AppState  (i.e. undefined or bolted values in the 
	 *	<profile> AppState are always correct WRT the corresponding 
	 *	value in the <real> AppState).  
	 *	    Window states asWithdrawn and asInactive in a <real> 
	 *	AppState match any state in a <profile> AppState.  
	 *	    Values for X and Y match if they refer to the same point 
	 *	on the screen.   Values for width and height must be the 
	 *	same if defined.  For example:  If the screen is 1000x1000, 
	 *	the <profile> AppStates [ Normal =-100-100 ] and  
	 *	[ =+800+800 ] both match the <real> AppState 
	 *	[ Normal =100x100+800+800 ].
	 *	    Use this function to determine if the <profile> AppState 
	 *	"pProf" must be modified to match a change in the corresponding
	 *	<real> AppState "pReal."
	 *	   NOTE:  asProfileCorrectWRTReal and asRealMatchesProfile are
	 *	*NOT* symmetric.
	 *
	 * Boolean
	 * asSelectedProfileCorrectWRToReal(pProf, pReal, which)
	 * AppStatePtr	pProf,pReal;
	 * unsigned	which;
	 *	    Like asProfileCorrectWRTToReal, but only compares
	 *	fields specified in "which."
	 *
	 * Boolean
	 * asRealMatchesProfile(pReal,pProf)
	 * AppStatePtr	pReal,pProf;
	 *	Returns True if "pReal" matches "pProf."
	 *	A <real> AppState matches a <profile> AppState IFF all of the
	 *	values in the <real> AppState match those elements of the 
	 *	corresponding <profile> AppState that are defined.   Undefined
	 *	values in the profile AppState always match the corresponding
	 *	value in the the <real> AppState.   Bolts in the <profile>
	 *	AppState do not affect matches.
	 *	Window states and values for x, y, width, and height match as
	 *	they do for asProfileCorrectWRTReal.
	 *	Use this function to determine if the <real> AppState "pReal"
	 *	must be changed to correspond to the <profile> AppState
	 *	"pProf."
	 *	   NOTE:  asRealMatchesProfile and asProfileCorrectWRTReal are
	 *	*NOT* symmetric.
	 *
	 * Boolean
	 * asSelectedRealMatchProfile(pReal,pProf,which)
	 * AppStatePtr	pReal,pProf;
	 * unsigned	which;
	 *	Like asRealMatchesProfile, but only compares fields specified
	 *	in "which."
	 *
	 * Boolean
	 * asFullySame(pState1, pState2)
	 * AppStatePtr	pState1,pState2;
	 *	Returns True if "pState1" and "pState2" have the same fields
	 *	and bolts defined, the same values for XNegative and 
	 *	YNegative, and all fields have the same value.
	 *
	 * Boolean
	 * asSelectedSame(pState1, pState2, which)
	 * AppStatePtr	pState1,pState2;
	 * unsigned	which;
	 *	Returns True if all of the fields and bolts specified in 
	 *	"which" have the same values in "pState1" and "pState2." 
	 *	Does *not* compute positive values of X and Y.
	 * 
	 * FUNCTIONS TO DETERMINE OTHER USEFUL THINGS ABOUT APPLICATION STATES
	 * Boolean
	 * asLegalReal(pState)
	 * AppStatePtr	pState;
	 *	Returns True if "pState" is a legal <real> AppState, False 
	 *	otherwise.  NullAppState is not a legal <real> AppState.
	 *
	 * Boolean
	 * asLegalProfile(pState)
	 * AppStatePtr	pState;
	 *	Returns True if "pState" is a legal <profile> AppState,
	 *	False otherwise.  NullAppState is a legal <profile> AppState.
	 *
	 * Boolean	asDontCare(pState)
	 * AppStatePtr	pState;
	 *	Returns True if "pState" is NullAppState or if none of the 
	 *	values in pState are defined.
	 * 
	\*/

/*\

\*/

typedef struct _AppStateRec	*AppStatePtr;
#define	NullAppState		((AppStatePtr)NULL)

	/*\
	 * All of these flags except asWinStateValue must be the same as 
	 * the corresponding value in X11/Xutil.h.
	 * asWinStateValue must not conflict with any of the flags in 
	 * X11/Xutil.h
	 *
	 * these flags are used to determine which fields of an AppState
	 * are defined, and whether the X and Y values are positive or
	 * negative specifications.
	\*/

#define asNoValue	((unsigned)0x0000)
#define asXValue  	((unsigned)0x0001)
#define asYValue	((unsigned)0x0002)
#define asWidthValue  	((unsigned)0x0004)
#define asHeightValue 	((unsigned)0x0008)

#define	asWinStateValue	((unsigned)0x0100)

#define	asPosValues	(asXValue|asYValue)
#define	asSizeValues	(asWidthValue|asHeightValue)
#define	asGeomValues	(asPosValues|asSizeValues)
#define asAllValues 	(asGeomValues|asWinStateValue)

#define	asXNegative	((unsigned)0x0010)
#define	asYNegative	((unsigned)0x0020)
#define	asSignFlags	(asXNegative|asYNegative)

#define	asLegalFlags	(asAllValues|asSignFlags)

	/*\
	 * Most of these values are *not* arbitrary.   asWithdrawn,
	 * asIconic, and asNormal *must* correspond to the ICCCM.
	 * The other values should not conflict with the ICCCM.
	\*/
typedef	int	WinState;

#define	asInactive	((WinState)-1)
#define	asWithdrawn	((WinState)0)
#define	asNormal	((WinState)1)
#define	asIconic	((WinState)3)

extern	unsigned int	 asDebug;
extern  AppStatePtr	 asDefaultReal;
extern	AppStatePtr	 asDefaultProfile;

extern	char		*asWinStateText(/* pState */);
extern	char		*asText(/* pState */);

extern	void		asSetScreenSize( /*width, height*/ );
extern	Boolean		asChangeProfileFlags(/* pReal, pProf, pNewFlags */);
extern	Boolean	 	asGetScreenValues(/*pReal,pProf,pWState,pX,pY,pW,pH*/);

extern	AppStatePtr	asCreate(/* flags, winState, x, y, width, height */);
extern	AppStatePtr	asDupFully(/* pState */);
extern	AppStatePtr	asDupReal(/* pReal, flags*/);
extern	void	 	asDestroy(/* pState */);

extern	Boolean	 	asCopyFully(/* pOldAppState, pNewAppState */);
extern	Boolean		asCopySelected(/*pFrom, pInto, flags*/);
extern	Boolean	 	asCopyRealToProfile(/* pReal, pProf */);
extern	Boolean	 	asCopyProfileToReal(/* pProf, pReal */);

extern	Boolean	 	asChangeSelected(/*pState,flags,winState,x,y,w,h*/);
extern	Boolean	 	asChangeAll(/*pState,flags,winState,x,y,width,height*/);

extern	Boolean		asBolt(/*pState,which*/);
extern	Boolean		asUnbolt(/*pState,which*/);
extern	Boolean		asLoosenBolts(/*pState,which*/);
extern	Boolean		asTightenBolts(/*pState,which*/);
extern	Boolean		asSetBolts(/*pState,which*/);
extern	Boolean		asSetLooseBolts(/*pState,which*/);

extern	unsigned 	asValues(/* pState, pWState,pX, pY, pWidth, pHeight */);
extern	unsigned 	asFlags(/*pState*/);
extern	unsigned	asBolts(/*pState*/);
extern	unsigned	asLooseBolts(/*pState*/);

extern	Boolean	 	asProfileCorrectWRTReal(/* pProf, pReal */);
extern	Boolean	 	asSelectedProfileCorrectWRTReal(/*pProf,pReal,which*/);
extern	Boolean	 	asRealMatchesProfile(/* pReal, pProf */);
extern	Boolean	 	asSelectedRealMatchProfile(/* pReal, pProf, which */);
extern	Boolean	 	asFullySame(/* pState1, pState2 */);
extern	Boolean	 	asSelectedSame(/* pState1, pState2, which */);

extern	Boolean	 	asLegalReal(/* pState */);
extern	Boolean	 	asLegalProfile(/* pState */);
extern	Boolean	 	asDontCare(/* pState */);

#endif /* APPSTATE_H */
