
  /*\
   *  $Header: nameprof.c,v 5.0 90/01/10 06:53:30 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define DEBUG_VAR	nameprofDebug
#include "utils.h"
#include "strtbl.h"
#include "list.h"
#include "nameprof.h"

/***====================================================================***/

	StringToken	nprDefaultName=	NullStringToken;

static	ListPtr		nprFunctions=	NullList;
static	NameResolveFunc	nprResolver=	NullNameResolveFunc;	
static	ChangeNamedFunc	nprChanger=	NullChangeNamedFunc;	

/***====================================================================***/

void
nprSetResolver(pFunc)
NameResolveFunc	pFunc;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"nprSetResolver(0x%x)\n",pFunc);
    nprResolver=	pFunc;
    uFLAG_VOIDRETURN;
}

/***====================================================================***/

void
nprSetChanger(pFunc)
ChangeNamedFunc	pFunc;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"nprSetChanger(0x%x)\n",pFunc);
    nprChanger=	pFunc;
    uFLAG_VOIDRETURN;
}

/***====================================================================***/

Boolean
nprGetName(funcName,osPriv,ppName,ppClass)
StringToken	  funcName;
Opaque		  osPriv;
char		**ppName;
char		**ppClass;
{
char		*name=	NullString;
char		*class=	NullString;
NPStatementPtr	pStmt;

    uENTRY4("profileGetName(%s,0x%x,0x%x,0x%x)\n",stText(funcName),osPriv,
								ppName,ppClass);
    if (ppName!=(char **)NULL)	*ppName=	(char *)NULL;
    if (ppClass!=(char **)NULL)	*ppClass=	(char *)NULL;
    if (funcName==NullStringToken)	funcName=	nprDefaultName;

    if (funcName==NullStringToken) {
	uRETURN(False);
    }

    pStmt=	nprLookupFunction(funcName);
    if (pStmt!=NullNPStatement) {
	if (npEvaluate(pStmt,&name,&class,nprResolver,nprChanger,osPriv)) {
	    if		(ppName!=(char **)NULL)	*ppName=	name;
	    else if 	(name!=NullString)	(void)uFree((Opaque)name);

	    if		(ppClass!=(char **)NULL) *ppClass=	class;
	    else if	(class!=NullString)	 (void)uFree((Opaque)class);
	    uRETURN(True);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean	
nprAddFunction(name, pStmt, merge)
StringToken	name;
NPStatementPtr	pStmt;
Boolean		merge;
{
NPStatementPtr	pOld;
Boolean		found;

    uENTRY3("nprAddFunction(%s,0x%x,%s)\n",stText(name),pStmt,
							booleanText(merge));
    if (name==NullStringToken)	name=	nprDefaultName;
    if ((name==NullStringToken)||(pStmt==NullNPStatement)) {
	uRETURN(False);
    }
    if (nprFunctions==NullList) {
	nprFunctions= lCreate((unsigned)7,NullListSortFunc,genIntegerKeyType,
							  genPointerDataType);
	if (nprFunctions==NullList) {
	    uInternalError("allocation failure in nprAddFunction\n");
	    uRETURN(False);
	}
    }

    found=	(lLookup(nprFunctions,(GenKey)name,(GenData *)&pOld)&&
		 (pOld!=NullNPStatement));
    merge=	merge&&found;
    if (merge)	npAppendStatement(pOld,pStmt);
    else {
	lStore(nprFunctions,(GenKey)name,(GenData)pStmt);
	if (found)
	    npDestroyStatement(pOld);
    }
    uRETURN(True);
}

/***====================================================================***/

NPStatementPtr
nprRemoveFunction(name, destroy)
StringToken	name;
Boolean		destroy;
{
NPStatementPtr	pOld;

    uENTRY2("nprRemoveFunction(%s,%s)\n",stText(name),booleanText(destroy));
    if ((nprFunctions==NullList)||(name==NullStringToken)) {
	uRETURN(NullNPStatement);
    }
    if (lLookup(nprFunctions,(GenKey)name,(GenData *)&pOld)&&
	(pOld!=NullNPStatement)) {
	lRemove(nprFunctions,(GenKey)name);
	if (destroy) {
	    npDestroyStatement(pOld);
	    pOld=	NullNPStatement;
	}
    }
    else pOld=	NullNPStatement;
    uRETURN(pOld);
}

/***====================================================================***/

NPStatementPtr
nprLookupFunction(name)
StringToken	name;
{
NPStatementPtr	pStmt;

    uENTRY1("nprLookupFunction(%s)\n",stText(name));
    if (name==NullStringToken)	name=	nprDefaultName;
    if ((nprFunctions==NullList)||(name==NullStringToken)) {
	uRETURN(NullNPStatement);
    }
    if (lLookup(nprFunctions,(GenKey)name,(GenData *)&pStmt)) {
	uRETURN(pStmt);
    }
    uRETURN(NullNPStatement);
}
