
  /*\
   *  $Header: names.c,v 5.0 90/01/10 06:53:38 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define DEBUG_VAR	namesDebug
#include "utils.h"
#include <ctype.h>

#include "namesstr.h"

int	namesIndentSize=	4;
char	*npJoinText=		"";

/***====================================================================***/

/* ARGSUSED */
void
npDestroyStatement(pStmt)
NPStatementPtr	pStmt;
{
    uWarning("memory leak.  npDestroyStatement not implemented yet\n");
    return;
}

/***====================================================================***/

static	void
npIndent(file,level)
FILE	*file;
int	level;
{
    uENTRY2("npIndent(0x%x,%d)\n",file,level);
    level*=	namesIndentSize;
    while (level>0) {
	putc(' ',file);
	level--;
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
npWriteBlock(file,indent,pBlock)
FILE		*file;
int		indent;
NPStatementPtr	pBlock;
{
Boolean	needBraces;

    uENTRY3("npWriteBlock(0x%x,%d,0%x)\n",file,indent,pBlock);
    needBraces=	pBlock->pNext!=NullNPStatement;
    if (needBraces) {
	fprintf(file,"{\n");
	npWriteStatement(file,indent+1,pBlock);
	npIndent(file,indent);
	fprintf(file,"}\n");
    }
    else {
	npWriteStatement(file,indent,pBlock);
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
npWriteStringExpr(file,pStringExpr)
FILE		*file;
NPStringExprPtr	 pStringExpr;
{
extern	void	npWriteBoolExpr();

    uENTRY2("npWriteStringExpr(0x%x,0%x)\n",file,pStringExpr);
    if (pStringExpr->type==NPStringConstant) {
	fprintf(file,"\"%s\"",pStringExpr->u.constant);
    }
    else if (pStringExpr->type==NPIdentifier) {
	fprintf(file,"%s",pStringExpr->u.constant);
    }
    else if (pStringExpr->type==NPPropertyName) {
	fprintf(file,"<%s>",pStringExpr->u.constant);
    }
    else if (pStringExpr->type==NPStringConcat) {
	npWriteStringExpr( file, pStringExpr->u.concat.pLeft );
	fprintf(file," + ");
	npWriteStringExpr( file, pStringExpr->u.concat.pRight );
    }
    else if (pStringExpr->type==NPStringConditional) {
	fprintf(file, "( ");
	npWriteBoolExpr(file, pStringExpr->u.conditional.pCondition);
	fprintf(file, " ? ");
	npWriteStringExpr(file, pStringExpr->u.conditional.pTrue);
	if (pStringExpr->u.conditional.pFalse!=NullNPStringExpr) {
	    fprintf(file, " : ");
	    npWriteStringExpr(file, pStringExpr->u.conditional.pFalse);
	}
	fprintf(file, " ) ");
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
npWriteBoolExpr(file,pBoolExpr)
FILE		*file;
NPBoolExprPtr	 pBoolExpr;
{
    uENTRY2("npWriteBoolExpr(0x%x,0%x)\n",file,pBoolExpr);
    if (pBoolExpr->invert)	fprintf(file,"! ");
    npWriteStringExpr(file,pBoolExpr->left);
    if (pBoolExpr->right!=NullNPStringExpr) {
	fprintf(file," == ");
	npWriteStringExpr(file,pBoolExpr->right);
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
npWriteCases(file,indent,pCases)
FILE		*file;
int		indent;
NPCasePtr	pCases;
{
    uENTRY3("npWriteCases(0x%x,%d,0x%x)\n",file,indent,pCases);
    while (pCases!=NullNPCase) {
	npIndent(file,indent);
	fprintf(file,"case ");
	npWriteStringExpr(file,pCases->pattern);
	fprintf(file,": ");
	npWriteBlock(file,indent,pCases->statement);
	pCases= pCases->pNext;
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
npWriteStatement(file,indent,pStatement)
FILE		*file;
int		 indent;
NPStatementPtr	 pStatement;
{

    uENTRY3("npWriteStatement(0x%x,%d,0x%x)\n",file,indent,pStatement);
    while (pStatement!=NullNPStatement) {
	npIndent(file,indent);
	switch (pStatement->what) {
	    case NPIfStatement:
		fprintf(file,"if ( ");
		npWriteBoolExpr(file,pStatement->u.ifS.condition);
		fprintf(file," ) ");
		npWriteBlock(file,indent,pStatement->u.ifS.thenClause);
		if (pStatement->u.ifS.elseClause!=NullNPStatement) {
		    putc('\n',file);
		    npIndent(file,indent);
		    fprintf(file,"else ");
		    npWriteBlock(file,indent,pStatement->u.ifS.elseClause);
		}
		break;
	    case NPSwitchStatement:
		fprintf(file,"switch ( ");
		npWriteStringExpr(file,pStatement->u.switchS.string);
		npIndent(file,indent);
		fprintf(file," ) {\n");
		npWriteCases(file,indent+1,pStatement->u.switchS.cases);
		if (pStatement->u.switchS.dfltCase!=NullNPStatement) {
		    npIndent(file,indent+1);
		    fprintf(file,"default: ");
		    npWriteBlock(file,indent+1,pStatement->u.switchS.dfltCase);
		}
		npIndent(file,indent);
		fprintf(file,"}\n");
		break;
	    case NPUseStatement:
		fprintf(file,"use ");
		npWriteStringExpr(file,pStatement->u.pUseExpr);
		fprintf(file,"\n");
		break;
	    case NPClassStatement:
		fprintf(file,"class ");
		npWriteStringExpr(file,pStatement->u.pClassExpr);
		fprintf(file,"\n");
		break;
	    case NPAssignStatement:
		npWriteStringExpr(file,pStatement->u.assignS.pLHS);
		fprintf(file," %s ",pStatement->u.assignS.append?"+=":"=");
		npWriteStringExpr(file,pStatement->u.assignS.pValue);
		fprintf(file,"\n");
		break;
	    case NPIgnoreStatement:
		fprintf(file,"ignore\n");
		break;
	    case NPComment:
		fprintf(file,"# %s\n",uStringText(pStatement->u.comment));
		break;
	}
	pStatement=	pStatement->pNext;
    }

    uVOIDRETURN;
}

/***====================================================================***/

NPStringExprPtr
npStringTerminal(type,strValue)
unsigned	type;
char		*strValue;
{
NPStringExprPtr	pExpr= NullNPStringExpr;

    uENTRY2("npStringTerminal(0x%x,%s)\n",type,uStringText(strValue));
    if (strValue!=NullString) {
	pExpr=	uTypedAlloc(NPStringExprRec);
	if (pExpr!=NullNPStringExpr) {
	    pExpr->type=		type;
	    pExpr->u.constant=		strValue;
	}
	else {
	    uWarning("Allocation failure in npStringTerminal\n");
	}
    }
    uRETURN(pExpr);
}

/***====================================================================***/

NPStringExprPtr	
npStringConcat(pLeft, pRight)
NPStringExprPtr	pLeft;
NPStringExprPtr	pRight;
{
NPStringExprPtr	pExpr;

    uENTRY2("npStringConcat(0x%x,0x%x)\n",pLeft,pRight);
    if ((pLeft==NullNPStringExpr)||(pRight==NullNPStringExpr)) {
	uRETURN(NullNPStringExpr);
    }
    pExpr=	uTypedAlloc(NPStringExprRec);
    if (pExpr!=NullNPStringExpr) {
	pExpr->type=		NPStringConcat;
	pExpr->u.concat.pLeft=	pLeft;
	pExpr->u.concat.pRight=	pRight;
    }
    uRETURN(pExpr);
}

/***====================================================================***/

NPStringExprPtr	
npStringConditional(pCond, pTrue, pFalse)
NPBoolExprPtr	pCond;
NPStringExprPtr	pTrue;
NPStringExprPtr	pFalse;
{
NPStringExprPtr	pExpr;

    uENTRY3("npStringConditional(0x%x,0x%x,0x%x)\n",pCond,pTrue,pFalse);
    if ((pCond==NullNPBoolExpr)||(pTrue==NullNPStringExpr)) {
	uRETURN(NullNPStringExpr);
    }
    pExpr=	uTypedAlloc(NPStringExprRec);
    if (pExpr!=NullNPStringExpr) {
	pExpr->type=			NPStringConditional;
	pExpr->u.conditional.pCondition=	pCond;
	pExpr->u.conditional.pTrue=		pTrue;
	pExpr->u.conditional.pFalse=		pFalse;
    }
    uRETURN(pExpr);
}

/***====================================================================***/

NPBoolExprPtr
npUnaryExpr(invert, pStrExpr)
Boolean		invert;
NPStringExprPtr	pStrExpr;
{
NPBoolExprPtr	pExpr= NullNPBoolExpr;

    uENTRY2("npUnaryExpr(%s,0x%x)\n",booleanText(invert),pStrExpr);
    if (pStrExpr!=NullNPStringExpr) {
	pExpr=	uTypedAlloc(NPBoolExprRec);
	if (pExpr!=NullNPBoolExpr) {
	    pExpr->invert=	invert;
	    pExpr->left=	pStrExpr;
	    pExpr->right=	NullNPStringExpr;
	}
    }
    uRETURN(pExpr);
}

/***====================================================================***/

NPBoolExprPtr
npEquals(pLeft, pRight)
NPStringExprPtr	pLeft;
NPStringExprPtr	pRight;
{
NPBoolExprPtr	pExpr= NullNPBoolExpr;

    uENTRY2("npEquals(%s,0x%x)\n",pLeft,pRight);
    if ((pLeft!=NullNPStringExpr)&&(pRight!=NullNPStringExpr)) {
	pExpr=	uTypedAlloc(NPBoolExprRec);
	if (pExpr!=NullNPBoolExpr) {
	    pExpr->invert=	False;
	    pExpr->left=	pLeft;
	    pExpr->right=	pRight;
	}
    }
    uRETURN(pExpr);
}

/***====================================================================***/

NPStatementPtr
npIfStatement( pCond, thenClause, elseClause )
NPBoolExprPtr	pCond;
NPStatementPtr	thenClause;
NPStatementPtr	elseClause;
{
NPStatementPtr		pStatement= NullNPStatement;

    uENTRY3("npIfStatement(0x%x,0x%x,0x%x)\n",pCond,thenClause,elseClause);
    if ((pCond!=NullNPBoolExpr)&&(thenClause!=NullNPStatement)) {
	pStatement=	uTypedAlloc(NPStatementRec);
	if (pStatement!=NullNPStatement) {
	    pStatement->what=			NPIfStatement;
	    pStatement->u.ifS.condition=	pCond;
	    pStatement->u.ifS.thenClause=	thenClause;
	    pStatement->u.ifS.elseClause=	elseClause;
	    pStatement->pNext=			NullNPStatement;
	}
    }
    uRETURN(pStatement);
}

/***====================================================================***/

NPCasePtr
npCase( pPattern, pStatement )
NPStringExprPtr	pPattern;
NPStatementPtr	pStatement;
{
NPCasePtr	pCase= NullNPCase;

    uENTRY2("npCase(0x%x,0x%x)\n",pPattern,pStatement);
    pCase= uTypedAlloc(NPCaseRec);
    if (pCase!=NullNPCase) {
	pCase->pattern=	pPattern;
	pCase->statement=	pStatement;
	pCase->pNext=		NullNPCase;
    }
    uRETURN(pCase);
}

/***====================================================================***/

void
npAppendCase( pOnto, pAdded )
NPCasePtr	pOnto;
NPCasePtr	pAdded;
{
NPCasePtr	pNext;

    uENTRY2("npAppendCase(0x%x,0x%x)\n",pOnto,pAdded);
    if ((pOnto!=NullNPCase)&&(pAdded!=NullNPCase)) {
	pNext=	pOnto;
	while (pNext->pNext!=NullNPCase) {
	    pNext= pNext->pNext;
	}
	pNext->pNext=	pAdded;
    }
    uVOIDRETURN;
}

/***====================================================================***/

NPStatementPtr
npSwitchStatement( pStrExpr, pCases, pDfltCase )
NPStringExprPtr	pStrExpr;
NPCasePtr	pCases;
NPStatementPtr	pDfltCase;
{
NPStatementPtr		pStatement= NullNPStatement;

    uENTRY3("npSwitchStatement(0x%x,0x%x,0x%x)\n",pStrExpr,pCases,
							pDfltCase);
    if ((pStrExpr!=NullNPStringExpr)&&(pCases!=NullNPCase)) {
	pStatement=	uTypedAlloc(NPStatementRec);
	if (pStatement!=NullNPStatement) {
	    pStatement->what=			NPSwitchStatement;
	    pStatement->u.switchS.string=	pStrExpr;
	    pStatement->u.switchS.cases=	pCases;
	    pStatement->u.switchS.dfltCase=	pDfltCase;
	    pStatement->pNext=			NullNPStatement;
	}
    }
    uRETURN(pStatement);
}

/***====================================================================***/

NPStatementPtr
npUseStatement( pNameExpr )
NPStringExprPtr	pNameExpr;
{
NPStatementPtr		pStatement= NullNPStatement;

    uENTRY1("npUseStatement(0x%x)\n",pNameExpr);
    if (pNameExpr!=NullNPStringExpr) {
	pStatement=	uTypedAlloc(NPStatementRec);
	if (pStatement!=NullNPStatement) {
	    pStatement->what=		NPUseStatement;
	    pStatement->u.pUseExpr=	pNameExpr;
	    pStatement->pNext=		NullNPStatement;
	}
    }
    uRETURN(pStatement);
}

/***====================================================================***/

NPStatementPtr
npAssignStatement( pLHSExpr, pValueExpr, append )
NPStringExprPtr	pLHSExpr;
NPStringExprPtr	pValueExpr;
Boolean		append;
{
NPStatementPtr		pStatement= NullNPStatement;

    uENTRY3("npAssignStatement(0x%x,0x%x,%s)\n",pLHSExpr,pValueExpr,
							booleanText(append));
    if ((pLHSExpr!=NullNPStringExpr)&&(pValueExpr!=NullNPStringExpr)) {
	pStatement=	uTypedAlloc(NPStatementRec);
	if (pStatement!=NullNPStatement) {
	    pStatement->what=			NPAssignStatement;
	    pStatement->u.assignS.append=	append;
	    pStatement->u.assignS.pLHS=		pLHSExpr;
	    pStatement->u.assignS.pValue=	pValueExpr;
	    pStatement->pNext=			NullNPStatement;
	}
    }
    uRETURN(pStatement);
}

/***====================================================================***/

NPStatementPtr
npClassStatement( pClassExpr )
NPStringExprPtr	pClassExpr;
{
NPStatementPtr		pStatement= NullNPStatement;

    uENTRY1("npClassStatement(0x%x)\n",pClassExpr);
    if (pClassExpr!=NullNPStringExpr) {
	pStatement=	uTypedAlloc(NPStatementRec);
	if (pStatement!=NullNPStatement) {
	    pStatement->what=		NPClassStatement;
	    pStatement->u.pClassExpr=	pClassExpr;
	    pStatement->pNext=		NullNPStatement;
	}
    }
    uRETURN(pStatement);
}

/***====================================================================***/

NPStatementPtr
npComment(str)
char	*str;
{
NPStatementPtr		pStatement= NullNPStatement;

    uENTRY1("npComment(%s)\n",uStringText(str));
    pStatement=	uTypedAlloc(NPStatementRec);
    if (pStatement!=NullNPStatement) {
	pStatement->what=			NPComment;
	pStatement->u.comment=			str;
	pStatement->pNext=			NullNPStatement;
    }
    uRETURN(pStatement);
}

/***====================================================================***/

NPStatementPtr
npIgnoreStatement()
{
NPStatementPtr		pStatement= NullNPStatement;

    uENTRY("npIgnoreStatement()\n");
    pStatement=	uTypedAlloc(NPStatementRec);
    if (pStatement!=NullNPStatement) {
	pStatement->what=			NPIgnoreStatement;
	pStatement->pNext=			NullNPStatement;
    }
    uRETURN(pStatement);
}

/***====================================================================***/

void
npInsertStatement( pAfter, pAdded )
NPStatementPtr	pAfter;
NPStatementPtr	pAdded;
{
NPStatementPtr	pNext;

    uENTRY2("npInsertStatement(0x%x,0x%x)\n",pAfter,pAdded);
    if ((pAfter!=NullNPStatement)&&(pAdded!=NullNPStatement)) {
	pNext=	pAfter->pNext;
	pAfter->pNext=	pAdded;
	if (pNext!=NullNPStatement) {
	    while (pAdded->pNext!=NullNPStatement) {
		pAdded= pAdded->pNext;
	    }
	    pAdded->pNext=	pNext;
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
npAppendStatement( pOnto, pAdded )
NPStatementPtr	pOnto;
NPStatementPtr	pAdded;
{
NPStatementPtr	pNext;

    uENTRY2("npInsertStatement(0x%x,0x%x)\n",pOnto,pAdded);
    if ((pOnto!=NullNPStatement)&&(pAdded!=NullNPStatement)) {
	pNext=	pOnto;
	while (pNext->pNext!=NullNPStatement) {
	    pNext= pNext->pNext;
	}
	pNext->pNext=	pAdded;
    }
    uVOIDRETURN;
}

/***====================================================================***/

extern	void	_npEvalStatement(/* */);

static void
_npSetMatch(pState,match)
NPStatePtr	 pState;
char		*match;
{
    uENTRY2("_npSetMatch(0x%x,%s)\n",pState,uStringText(match));
    if (pState->lastMatchString!=NullString) 
	(void)uFree((Opaque)pState->lastMatchString);
    pState->lastMatchString=	match;
    if (match==NullString)	pState->lastMatchSubStrings.nMatches=	0;
    else {
	pState->lastMatchSubStrings.nMatches= 1;
	pState->lastMatchSubStrings.matchLengths[0]= strlen(match);
	pState->lastMatchSubStrings.matchStrings[0]= match;
    }
    uVOIDRETURN;
}

/***====================================================================***/

static char *
_npConstructName(rule,pState)
char		*rule;
NPStatePtr	 pState;
{
char		buf[MAX_NAME_LEN+1];
char		*match,*rtrn;
int		matchLen,matchNum;
int		len;
RXMatchesPtr	pMatches;

    uENTRY2("_npConstructName(%s,0x%x)\n",uStringText(rule),pState);
    len=	0;
    pMatches=	&pState->lastMatchSubStrings;
    if (rule!=NullString) {
	while ((*rule)&&(len<MAX_NAME_LEN)) {
	    if ((*rule=='\\')&&isdigit((*(rule+1)))) {
		matchNum=	*(rule+1)-'0'-1;
		rule+= 2;
		if (matchNum<pMatches->nMatches) {
		    matchLen=	pMatches->matchLengths[matchNum];
		    match=	pMatches->matchStrings[matchNum];
		    while ((matchLen>0)&&(len<MAX_NAME_LEN)) {
			buf[len]=	*match;
			len++;	match++;	matchLen--;
		    }
		}
	    }
	    else {
		buf[len]=	*rule;
		len++;	rule++;
	    }
	}
	buf[len]=	NUL;
    }
    if (len>0)	rtrn= uStringDup(buf);
    else	rtrn= NullString;
    uRETURN(rtrn);
}

/***====================================================================***/

extern	Boolean	_npEvalStringExpr();
extern	Boolean	_npEvalBoolExpr();

static Boolean
_npEvalStringConcat(pLeft,pRight,pState,ppValue)
NPStringExprPtr	pLeft;
NPStringExprPtr	pRight;
NPStatePtr	pState;
char		**ppValue;
{
char	 *str,*left,*right;
Boolean	 leftOk,rightOk;
Boolean	 allOk;

    uENTRY4("_npEvalStringConcat(0x%x,0x%x,0x%x,0x%x)\n",pLeft,pRight,pState,
								ppValue);

    str= left= right=	NullString;
    allOk=	True;

    leftOk= 	_npEvalStringExpr(pLeft,pState,&left);
    rightOk=	_npEvalStringExpr(pRight,pState,&right);
    if (leftOk&&rightOk) {
	str= (char *)uAlloc((unsigned)(strlen(left)+strlen(npJoinText)+
							strlen(right)+1));
	if (str!=NullString) {
	    sprintf(str,"%s%s%s",left,npJoinText,right);
	}
	else  {
	    uWarning("Allocation failure in _npEvalStringConcat\n");
	    allOk=	False;
	}
	(void)uFree((Opaque)left);	(void)uFree((Opaque)right);
    }
    else if (leftOk)	str=	left;
    else if (rightOk)	str=	right;
    else		allOk=	False;
    if (ppValue!=(char **)NULL)
	*ppValue=	str;
    uRETURN(allOk);
}

/***====================================================================***/

static Boolean
_npEvalStringConditional(pCond,pTrue,pFalse,pState,ppValue)
NPBoolExprPtr	pCond;
NPStringExprPtr	pTrue;
NPStringExprPtr	pFalse;
NPStatePtr	pState;
char		**ppValue;
{
Boolean	 	 ok;

    uENTRY5("_npEvalStringConditional(0x%x,0x%x,0x%x,0x%x)\n",pCond,
						pTrue,pFalse,pState,ppValue);

    if (_npEvalBoolExpr(pCond,pState)) {
	char	*rule;
	if (_npEvalStringExpr(pTrue,pState,&rule)) {
	    *ppValue=	_npConstructName(rule,pState);
	    if (rule!=NullString) 
		(void)uFree((Opaque)rule);
	    ok=	True;
	}
	else {
	    _npSetMatch(pState,NullString);
	    ok= False;
	}
    }
    else {
	_npSetMatch(pState,NullString);
	ok= _npEvalStringExpr(pFalse,pState,ppValue);
    }
    uRETURN(ok);
}

/***====================================================================***/

Boolean
_npEvalStringExpr(pExpr,pState,ppValue)
NPStringExprPtr	pExpr;
NPStatePtr	pState;
char		**ppValue;
{
Boolean	exists=	True;
char	*str= NullString;

    uENTRY3("_npEvalStringExpr(0x%x,0x%x,0x%x)\n",pExpr,pState,ppValue);
    switch (pExpr->type) {
	case NPStringConstant:
	    str=	uStringDup(pExpr->u.constant);
	    break;
	case NPIdentifier:
	case NPPropertyName:
	    if (pState->ResolveFunc) {
		exists= (*pState->ResolveFunc)(pState->OSPrivate,pExpr->type,
						pExpr->u.constant,&str);
	    }
	    else exists=	False;
	    break;
	case NPStringConcat:
	    exists=	_npEvalStringConcat(pExpr->u.concat.pLeft,
						pExpr->u.concat.pRight,
						pState,&str);
	    break;
	case NPStringConditional:
	    exists= _npEvalStringConditional(pExpr->u.conditional.pCondition,
						pExpr->u.conditional.pTrue,
						pExpr->u.conditional.pFalse,
						pState,&str);
	    break;
	    
    }
    if (ppValue)	*ppValue=	str;
    uRETURN(exists);
}

/***====================================================================***/

Boolean
_npEvalBoolExpr(pExpr,pState)
NPBoolExprPtr	pExpr;
NPStatePtr	pState;
{
Boolean	ok;
char	*leftStr= NullString;
char	*rightStr= NullString;

    uENTRY2("_npEvalBoolExpr(0x%x,0x%x)\n",pExpr,pState);
    ok=	_npEvalStringExpr(pExpr->left,pState,&leftStr);
    if (pExpr->right!=NullNPStringExpr) {
	if (leftStr==NullString)	ok=	False;
	ok= ok&&_npEvalStringExpr(pExpr->right,pState,&rightStr)&&
	    (rightStr!=NullString);
	ok= ok&&regexMatch(rightStr,leftStr,&pState->lastMatchSubStrings);
	if ((!ok)&&(pState->lastMatchString!=NullString)) {
	    (void)_npSetMatch(pState,NullString);
	}
	else leftStr=	NullString;
    }
    else if ((leftStr!=NullString)&&(strlen(leftStr)>0)) {
	(void)_npSetMatch(pState,leftStr);
	leftStr=	NullString;
    }
    else ok=	False;
    if (leftStr!=NullString)	uFree((Opaque)leftStr);
    if (rightStr!=NullString)	uFree((Opaque)rightStr);
    if (pExpr->invert)	ok= !ok;
    uRETURN(ok);
}

/***====================================================================***/

static void
_npEvalAssignStatement(pAssign,pState)
NPAssignStatementPtr	pAssign;
NPStatePtr		pState;
{
char	*value=	NullString;

    uENTRY2("_npEvalAssignStatement(0x%x,0x%x)\n",pAssign,pState);
    if (_npEvalStringExpr(pAssign->pValue,pState,&value)) {
	if (pState->ChangeFunc) {
	    (*pState->ChangeFunc)(pState->OSPrivate,
				pAssign->pLHS->type,pAssign->pLHS->u.constant,
				value,pAssign->append);
	}
    }
    if (value!=NullString)	(void)uFree((Opaque)value);
    uVOIDRETURN;
}

/***====================================================================***/

static void
_npEvalUseStatement(pUseExpr,pState)
NPStringExprPtr		pUseExpr;
NPStatePtr		pState;
{
char	*rule= NullString;

    uENTRY2("_npEvalUseStatement(0x%x,0x%x)\n",pUseExpr,pState);
    if (_npEvalStringExpr(pUseExpr,pState,&rule)) {
	pState->resultName=	_npConstructName(rule,pState);
	pState->done=		True;
	if (rule!=NullString) 
	    (void)uFree((Opaque)rule);
    }
    uVOIDRETURN;
}

/***====================================================================***/

static void
_npEvalClassStatement(pClassExpr,pState)
NPStringExprPtr		pClassExpr;
NPStatePtr		pState;
{
char	*rule=		NullString;
char	*result=	NullString;

    uENTRY2("_npEvalClassStatement(0x%x,0x%x)\n",pClassExpr,pState);
    if (_npEvalStringExpr(pClassExpr,pState,&rule)) {
	result=	_npConstructName(rule,pState);
	if (result!=NullString) {
	   if  (pState->resultClass!=NullString) {
		(void)uFree((Opaque)pState->resultClass);
	   }
	   pState->resultClass=	result;
	}
	if (rule!=NullString) 
	    (void)uFree((Opaque)rule);
    }
    uVOIDRETURN;
}

/***====================================================================***/

static void
_npEvalCase(str,pCase,pState)
char		*str;
NPCasePtr	 pCase;
NPStatePtr	 pState;
{
char		*pattern;
Boolean		 ok= True;

    uENTRY3("_npEvalCase(%s,0x%x,0x%x)\n",uStringText(str),pCase,pState);
    if (str!=NullString) {
	if (pCase->pattern!=NullNPStringExpr) {
	    _npSetMatch(pState,NullString);
	    ok= _npEvalStringExpr(pCase->pattern,pState,&pattern)&&
		(pattern!=NullString);
	    ok= ok&&regexMatch(pattern,str,&pState->lastMatchSubStrings);
	    if (pattern!=NullString)
		(void)uFree((Opaque)pattern);
	}
	if (ok)
	    (void)_npEvalStatement(pCase->statement,pState);
    }
    uVOIDRETURN;
}

/***====================================================================***/

static void
_npEvalSwitchStatement(pSwitch,pState)
NPSwitchStatementPtr	pSwitch;
NPStatePtr		pState;
{
char		*str= NullString;
NPCasePtr	 pCase;

    uENTRY2("_npEvalSwitchStatement(0x%x,0x%x)\n",pSwitch,pState);
    if (_npEvalStringExpr(pSwitch->string,pState,&str)) {
	pCase=	pSwitch->cases;
	while ((pCase!=NullNPCase)&&(!pState->done)) {
	    (void)_npEvalCase(str,pCase,pState);
	    pCase=	pCase->pNext;
	}
	if ((!pState->done)&&(pSwitch->dfltCase!=NullNPStatement)) {
	    (void)_npSetMatch(pState,str);
	    (void)_npEvalStatement(pSwitch->dfltCase,pState);
	}
    }
    if (str!=NullString) {
	(void)uFree((Opaque)str);
    }
    (void)_npSetMatch(pState,NullString);
    uVOIDRETURN;
}

/***====================================================================***/

static void
_npEvalIfStatement(pIf,pState)
NPIfStatementPtr	pIf;
NPStatePtr		pState;
{
    uENTRY2("_npEvalIfStatement(0x%x,0x%x)\n",pIf,pState);
    if (_npEvalBoolExpr(pIf->condition,pState))
	_npEvalStatement(pIf->thenClause,pState);
    else if (pIf->elseClause!=NullNPStatement)
	_npEvalStatement(pIf->elseClause,pState);
    _npSetMatch(pState,NullString);
    uVOIDRETURN;
}

/***====================================================================***/

void
_npEvalStatement(pStatement,pState)
NPStatementPtr	pStatement;
NPStatePtr	pState;
{
    uENTRY2("_npEvalStatement(0x%x,0x%x)\n",pStatement,pState);
    while ((pStatement!=NullNPStatement)&&(!pState->done)) {
	switch (pStatement->what) {
	    case NPIfStatement:
		 _npEvalIfStatement(&pStatement->u.ifS,pState);
		 break;
	    case NPSwitchStatement:
		 _npEvalSwitchStatement(&pStatement->u.switchS,pState);
		 break;
	    case NPUseStatement:
		 _npEvalUseStatement(pStatement->u.pUseExpr,pState);
		 break;
	    case NPClassStatement:
		 _npEvalClassStatement(pStatement->u.pClassExpr,pState);
		 break;
	    case NPAssignStatement:
		 _npEvalAssignStatement(&pStatement->u.assignS,pState);
		 break;
	    case NPIgnoreStatement:
		 pState->done=		True;
		 pState->resultName=	NullString;
		 pState->resultClass=	NullString;
		 break;
	    case NPComment:
		 break;
	}
	pStatement=	pStatement->pNext;
    }
    uVOIDRETURN;
}

/***====================================================================***/

Boolean
npEvalBoolExpr(pExpr,ppMatch,ResolveFunc,ChangeFunc,OSPrivate)
NPBoolExprPtr	  pExpr;
char		**ppMatch;
NameResolveFunc	  ResolveFunc;
ChangeNamedFunc	  ChangeFunc;
Opaque		  OSPrivate;
{
NPStateRec	  state;
Boolean		  ok;

    uENTRY5("npEvalBoolExpr(0x%x,0x%x,0x%x,0x%x,0x%x)\n",pExpr,ppMatch,
					ResolveFunc,ChangeFunc,OSPrivate);
    state.done=			False;
    state.resultName=		NullString;
    state.resultClass=		NullString;
    state.lastMatchString=	NullString;
    bzero(&state.lastMatchString,sizeof(RXMatchesRec));
    state.ResolveFunc=		ResolveFunc;
    state.ChangeFunc=		ChangeFunc;
    state.OSPrivate=		OSPrivate;
    ok=	_npEvalBoolExpr(pExpr,&state);
    if (ok&&(ppMatch!=(char **)NULL)) {
	*ppMatch=	state.lastMatchString;
    }
    else _npSetMatch(&state,NullString);
    uRETURN(ok);
}

/***====================================================================***/

Boolean
npEvalStringExpr(pExpr,ppRtrn,ResolveFunc,ChangeFunc,OSPrivate)
NPStringExprPtr	  pExpr;
char		**ppRtrn;
NameResolveFunc	  ResolveFunc;
ChangeNamedFunc	  ChangeFunc;
Opaque		  OSPrivate;
{
NPStateRec	  state;
Boolean		  ok;

    uENTRY5("npEvalStringExpr(0x%x,0x%x,0x%x,0x%x,0x%x)\n",pExpr,ppRtrn,
					ResolveFunc,ChangeFunc,OSPrivate);
    state.done=			False;
    state.resultName=		NullString;
    state.resultClass=		NullString;
    state.lastMatchString=	NullString;
    bzero(&state.lastMatchString,sizeof(RXMatchesRec));
    state.ResolveFunc=		ResolveFunc;
    state.ChangeFunc=		ChangeFunc;
    state.OSPrivate=		OSPrivate;
    ok= _npEvalStringExpr(pExpr,&state,ppRtrn);
    _npSetMatch(&state,NullString);
    uRETURN(ok);
}

/***====================================================================***/

Boolean
npEvaluate(pStatement,ppName,ppClass,ResolveFunc,ChangeFunc,OSPrivate)
NPStatementPtr	  pStatement;
char		**ppName;
char		**ppClass;
NameResolveFunc	  ResolveFunc;
ChangeNamedFunc	  ChangeFunc;
Opaque		  OSPrivate;
{
NPStateRec	  state;

    uENTRY6("npEvaluate(0x%x,0x%x,0x%x,0x%x,0x%x,0x%x)\n",pStatement,
			ppName,ppClass,ResolveFunc,ChangeFunc,OSPrivate);
    state.done=			False;
    state.resultName=		NullString;
    state.resultClass=		NullString;
    state.lastMatchString=	NullString;
    bzero(&state.lastMatchString,sizeof(RXMatchesRec));
    state.ResolveFunc=		ResolveFunc;
    state.ChangeFunc=		ChangeFunc;
    state.OSPrivate=		OSPrivate;
    (void)_npEvalStatement(pStatement,&state);
    if (ppName)	 {
	*ppName= state.resultName;
    }
    else if (state.resultName!=NullString) {
	uFree((Opaque)state.resultName);
    }
    _npSetMatch(&state,NullString);
    if (ppClass) {
	*ppClass= state.resultClass;
    }
    else if (state.resultClass!=NullString) {
	uFree((Opaque)state.resultClass);
    }
    uRETURN(state.done);
}

