#ifndef NAMES_H
#define	NAMES_H

  /*\
   *  $Header: names.h,v 5.0 90/01/10 06:53:42 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

  /*\

  \*/

	/*\
	 * This package implements an interpreter for a small language
	 * that is used in xrooms to determine a unique name for a structure
	 * (a window in xrooms).  This language assumes that there is a list 
	 * of named properties associated with the structure we are using,
	 * but doesn't know anything about reading those properties directly.
	 * Instead, it calls a PropLookup function to find the value of a 
	 * property on a window.
	\*/
typedef	struct	_NPStateRec		*NPStatePtr;
typedef	struct	_NPBoolExprRec		*NPBoolExprPtr;
typedef	struct	_NPStringExprRec	*NPStringExprPtr;
typedef struct	_NPCaseRec		*NPCasePtr;
typedef	struct	_NPStatementRec		*NPStatementPtr;

#define	NullNPState		((NPStatePtr)NULL)
#define	NullNPBoolExpr		((NPBoolExprPtr)NULL)
#define	NullNPStringExpr	((NPStringExprPtr)NULL)
#define	NullNPCase		((NPCasePtr)NULL)
#define	NullNPStatement		((NPStatementPtr)NULL)

	/*\
	 * Legal types of string expression
	\*/
#define	NPStringConstant	((unsigned)1)
#define	NPPropertyName		((unsigned)2)
#define	NPIdentifier		((unsigned)3)
#define	NPStringConcat		((unsigned)4)
#define	NPStringConditional	((unsigned)5)

	/*\
	 * Functions to resolve and change string values provided
	 * by the OS layer
	\*/
typedef	Boolean	(*NameResolveFunc)(/*osPriv,what,name,pRtrn*/);
typedef	Boolean	(*ChangeNamedFunc)(/*osPriv,what,name,value,append*/);
#define	NullNameResolveFunc	((NameResolveFunc)NULL)
#define	NullChangeNamedFunc	((ChangeNamedFunc)NULL)

extern	unsigned	 namesDebug;
extern	char		*npJoinText;

extern	void		 npDestroyStatement(/* pStmt */);

extern	void		 npWriteStatement(/* file, indent, pStatement*/);

extern	NPStringExprPtr	 npStringTerminal(/*type,strValue*/);
extern	NPStringExprPtr	 npStringConcat(/* pLeft, pRight */);
extern	NPStringExprPtr	 npStringConditional(/* pCond, pTrue, pFalse */);

extern	NPBoolExprPtr	 npUnaryExpr(/* invert, strExpr */);
extern	NPBoolExprPtr	 npEquals(/* pLeft, pRight */);

extern	NPStatementPtr	 npIfStatement(/* cond, then, else */);
extern	NPCasePtr	 npCase(/* pattern, statement */);
extern	NPStatementPtr	 npSwitchStatement(/* str, choices, dfltCase */);
extern	NPStatementPtr	 npUseStatement(/* name */);
extern	NPStatementPtr	 npClassStatement(/*  class */);
extern	NPStatementPtr	 npAssignStatement(/* lhsExpr, valueExpr, append */);
extern	NPStatementPtr	 npIgnoreStatement();
extern	NPStatementPtr	 npComment(/* str */);
extern	void		 npInsertStatement(/* after, added */);
extern	void		 npAppendStatement(/* onto, added */);
extern	void		 npAppendCase(/* onto, added */);

extern	Boolean	npEvalStringExpr(/*pExpr,ppRtrn,LookupFunc,ChangeFunc,priv*/);
extern	Boolean	npEvalBoolExpr(/*pStmt,ppMatch,LookupFunc,ChangeFunc,priv*/);
extern	Boolean	npEvaluate(/*pStmt,ppName,ppClass,LookupFunc,ChangeFunc,priv*/);

#define	MAX_NAME_LEN	1024

#endif /* NAMES_H */
