
  /*\
   *  $Header: profile.c,v 5.0 90/01/10 06:53:48 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define	DEBUG_VAR	profileDebug
#include "utils.h"
#include "apps.h"
#include "rooms.h"
#include "profilestr.h"
#include "profscan.h"
#include "names.h"
#include "nameprof.h"

/***====================================================================***/

static	ProfilesPtr	pNameProfiles=	NullProfiles;
static	ProfilesPtr	pRoomProfiles=	NullProfiles;
static	ProfilesPtr	pAppProfiles=	NullProfiles;
static	CommentPtr	pPendingComments=	NullComment;

/***====================================================================***/

static CommentPtr 
_profileConcatComments(pFirst,pSecond)
CommentPtr	pFirst,pSecond;
{
CommentPtr	pTmp;

    uENTRY2("_profileConcatComments(0x%x,0x%x)\n",pFirst,pSecond);
    if (pFirst!=NullComment) {
	for (pTmp=pFirst;pTmp->pNext!=NullComment;pTmp= pTmp->pNext) {
	    /* do nothing */
	}
	pTmp->pNext=	pSecond;
    }
    else pFirst=	pSecond;
    uRETURN(pFirst);
}

/***====================================================================***/

static void 
_profileDestroyComments(pComments)
CommentPtr	pComments;
{
CommentPtr	pNext;

    uENTRY1("_profileDestroyComments(0x%x)\n",pComments);
    while (pComments!=NullComment) {
	if (pComments->text!=NullString)
	    (void)uFree((Opaque)pComments->text);
	pNext=	pComments->pNext;
	(void)uFree((Opaque)pComments);
	pComments=	pNext;
    }
    uVOIDRETURN;
}

/***====================================================================***/

static ProfilesPtr
_profilesCreate(initSize)
unsigned	initSize;
{
ProfilesPtr	pProfs;

    uENTRY1("_profilesCreate(%d)\n",initSize);
    pProfs=	uTypedAlloc(ProfilesRec);
    if (pProfs!=NullProfiles) {
	pProfs->sizeProfiles=	initSize;
	pProfs->nProfiles=	0;
	pProfs->pProfiles=	uTypedCalloc(initSize,ProfileRec);
    }
    uRETURN(pProfs);
}

/***====================================================================***/

static void
_profilesAddName(pProfs,name)
ProfilesPtr	pProfs;
StringToken	name;
{
int		i;
Boolean		found=	False;

    uENTRY2("_profilesAddName(0x%x,%s)\n",pProfs,stText(name));
    if (pProfs!=NullProfiles) {
	for (i=0;(i<pProfs->nProfiles);i++) {
	    found=  stEqual(name,pProfs->pProfiles[i].name);
	    if (found) break;
	}
	if (!found) {
	    if (pProfs->nProfiles==pProfs->sizeProfiles) {
		pProfs->sizeProfiles*= 2;
		pProfs->pProfiles=	(ProfilePtr)
		    uRealloc((Opaque)pProfs->pProfiles,
			(unsigned)sizeof(ProfileRec)*pProfs->sizeProfiles);
	    }
	    pProfs->pProfiles[pProfs->nProfiles].name=	    name;
	    pProfs->pProfiles[pProfs->nProfiles].pComments= pPendingComments;
	    pProfs->nProfiles++;
	    pPendingComments= NullComment;
	}
	else {
	    pProfs->pProfiles[i].pComments=	_profileConcatComments(
					pProfs->pProfiles[i].pComments,
					pPendingComments);
	    pPendingComments= NullComment;
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
_profilesRemoveName(pProfs,name)
ProfilesPtr	pProfs;
StringToken	name;
{
int		i,j;

    uENTRY2("_profilesRemoveName(0x%x,%s)\n",pProfs,stText(name));
    if (pProfs!=NullProfiles) {
	for (i=0;(i<pProfs->nProfiles);i++) {
	    if (stEqual(name,pProfs->pProfiles[i].name)) {
		if (pProfs->pProfiles[i].pComments!=NullComment)
		    _profileDestroyComments(pProfs->pProfiles[i].pComments);
		for (j=i;j<pProfs->nProfiles;j++) {
		    pProfs->pProfiles[j]= pProfs->pProfiles[j+1];
		}
		pProfs->nProfiles--;
		break;
	    }
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

Boolean
profileForNames(funcName)
StringToken	funcName;
{
    uENTRY1("profileForNames(%s)\n",stText(funcName));
    if (funcName!=NullStringToken) {
	if (pNameProfiles==NullProfiles) 
	    pNameProfiles=	_profilesCreate((unsigned)5);
	if (pNameProfiles!=NullProfiles) {
	    _profilesAddName(pNameProfiles,funcName);
	    uRETURN(True);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
profileForRoom(roomName)
StringToken	roomName;
{
    uENTRY1("profileForRoom(%s)\n",stText(roomName));
    if (roomName!=NullStringToken) {
	if (pRoomProfiles==NullProfiles) 
	    pRoomProfiles=	_profilesCreate((unsigned)10);
	if (pRoomProfiles!=NullProfiles) {
	    _profilesAddName(pRoomProfiles,roomName);
	    uRETURN(True);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
profileForApp(appName)
StringToken	appName;
{
    uENTRY1("profileForApp(%s)\n",stText(appName));
    if (appName!=NullStringToken) {
	if (pAppProfiles==NullProfiles) 
	    pAppProfiles=	_profilesCreate((unsigned)30);
	if (pAppProfiles!=NullProfiles) {
	    _profilesAddName(pAppProfiles,appName);
	    uRETURN(True);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

void
profileForNameDestroy(funcName)
StringToken	funcName;
{
    uENTRY1("profileForNameDestroy(%s)\n",stText(funcName));
    if ((funcName!=NullStringToken)&&(pNameProfiles!=NullProfiles)) {
	_profilesRemoveName(pNameProfiles,funcName);
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
profileForRoomDestroy(roomName)
StringToken	roomName;
{
    uENTRY1("profileForRoomDestroy(%s)\n",stText(roomName));
    if ((roomName!=NullStringToken)&&(pRoomProfiles!=NullProfiles)) {
	_profilesRemoveName(pRoomProfiles,roomName);
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
profileForAppDestroy(appName)
StringToken	appName;
{
    uENTRY1("profileForAppDestroy(%s)\n",stText(appName));
    if ((appName!=NullStringToken)&&(pAppProfiles!=NullProfiles)) {
	_profilesRemoveName(pAppProfiles,appName);
    }
    uVOIDRETURN;
}

/***====================================================================***/

Boolean
profileAddComment(comment)
char	*comment;
{
CommentPtr	pComment;

    uENTRY1("profileAddComment(%s)\n",uStringText(comment));
    if (comment!=NullString) {
	pComment=	uTypedAlloc(CommentRec);
	if (pComment!=NullComment) {
	    pComment->text=		comment;
	    pComment->pNext=	NullComment;
	    pPendingComments= _profileConcatComments(pPendingComments,pComment);
	    uRETURN(True);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

CommentPtr
profileGetComments()
{
CommentPtr	pComment=	pPendingComments;

    uENTRY("profileGetComments()\n");
    pPendingComments=	NullComment;
    uRETURN(pComment);
}

/***====================================================================***/

Boolean	
profileRead(pScan)
ScannablePtr	pScan;
{
extern	ScannablePtr	pYYLexScan;

    uENTRY1("profileRead(0x%x)\n",pScan);
    if (pScan!=NullScannable) {
	pYYLexScan=	pScan;
	(void)yyparse();
	pYYLexScan=	NullScannable;
    }
    uRETURN(True);
}

/***====================================================================***/

static void
_profileIndent(pFile,n)
FILE	*pFile;
int	n;
{
    do {
	putc(' ',pFile);
    } while (--n>0);
}

/***====================================================================***/

static Boolean
_profileWriteAppState(pApp,roomName,pState,argIn)
AppPtr		pApp;
StringToken	roomName;
AppStatePtr	pState;
Opaque		argIn;
{
FILE	*pFile=	(FILE *)argIn;
char	*name;
int	 len;

    uENTRY4("_profileWriteAppState(%s,%s,%s,0%x)\n",appText(pApp),
					stText(roomName),asText(pState),
					argIn);
    name= stGetString(roomName);
    if (name!=NullString) {
	 _profileIndent(pFile,PROFILE_NAME_COL);
	 fprintf(pFile,"\"%s\"",name);	len= PROFILE_NAME_COL+strlen(name)+2;
	 _profileIndent(pFile,PROFILE_STATE_COL-len);
	 fprintf(pFile,"%s\n",asText(pState));
    }
    uRETURN(True);
}

static Boolean
_profileWriteApp(pFile,pApp,writeStates)
FILE		*pFile;
AppPtr		 pApp;
Boolean		 writeStates;
{
AppStatePtr	pState;
int		len= 0;

    uENTRY3("_profileWriteApp(0x%x,%s,%s)\n",pFile,appText(pApp),
						booleanText(writeStates));

    if (pApp!=NullApp) {
	pState=	appGetDefault(pApp);

	if (!appIsPermanent(pApp)) {
	    fprintf(pFile,"transient "); len+= strlen("transient ");
	}
	fprintf(pFile,"application "); len+= strlen("application ");
	_profileIndent(pFile,PROFILE_NAME_COL-len);
	if (PROFILE_NAME_COL>len)	len+= (PROFILE_NAME_COL-len);
	fprintf(pFile,"\"%s\"",appText(pApp)); len+= strlen(appText(pApp))+2;
	if (pState!=NullAppState) {
	    _profileIndent(pFile,PROFILE_STATE_COL-len);
	    fprintf(pFile,"%s",asText(pState));
	}
	if ((writeStates)&&(appNLocalStates(pApp)>0)) {
	    fprintf(pFile," {\n");
	    appIterate(pApp,_profileWriteAppState,(Opaque)pFile);
	    fprintf(pFile,"}");
	}
	fprintf(pFile,"\n");
	fflush(pFile);
    }
    uRETURN(True);
}

/***====================================================================***/

struct	pIterArg {
	FILE		*pFile;
	StringToken	 roomName;
	RoomPtr		 pRoom;
	Boolean		 written;
	Boolean		 writeTransient;
};


static void
_profileWriteRoomHeader(pFile,pRoom)
FILE	*pFile;
RoomPtr	pRoom;
{
unsigned	vis;

    uENTRY2("_profileWriteRoomHeader(0x%x,%s)\n",pFile,roomText(pRoom));
    fprintf(pFile,"room ");
    _profileIndent(pFile,PROFILE_NAME_COL-strlen("room "));
    fprintf(pFile,"\"%s\"",roomText(pRoom));
    _profileIndent(pFile,PROFILE_STATE_COL-
				PROFILE_NAME_COL-strlen(roomText(pRoom))-2);
    if ( (vis=roomGetVisibility(pRoom))!=roomDefaultVisibility) {
	if	(vis==0)		fprintf(pFile,"hidden");
	else if	(vis==roomVisible)	fprintf(pFile,"visible");
	else {
	    fprintf(pFile,"visible ");
	    if		(vis&roomAlways)	fprintf(pFile,"always");
	    else {
		if	(vis&roomWhenActive)	fprintf(pFile,"whenActive ");
		if	(vis&roomWhenNonEmpty)	fprintf(pFile,"whenNonEmpty");
	    }
	}
    }
    else {
	fprintf(pFile,"default");
    }
    uVOIDRETURN;
}

static Boolean
_profileWriteRoomState(argIn,pApp)
Opaque	argIn;
AppPtr	pApp;
{
struct	pIterArg	*pArg= (struct pIterArg *)argIn;
AppStatePtr		 pState;
char			*str;

    uENTRY2("_profileWriteRoomState(0x%x,%s)\n",argIn,appText(pApp));
    if ((!pArg->writeTransient)&&(!appIsPermanent(pApp))) {
	uRETURN(True);
    }
    pState=	appGetRoomState(pApp,pArg->roomName);
    if (pState!=NullAppState) {
	if (!pArg->written) {
	    _profileWriteRoomHeader(pArg->pFile,pArg->pRoom);
	    fprintf(pArg->pFile," {\n");
	    pArg->written=	True;
	}
	str=	appText(pApp);
	if (str!=NullString) {
	    int	len;
	    _profileIndent(pArg->pFile,PROFILE_NAME_COL);
	    fprintf(pArg->pFile,"\"%s\"",str);	
	    len= PROFILE_NAME_COL+strlen(str)+2;
	    _profileIndent(pArg->pFile,PROFILE_STATE_COL-len);
	    fprintf(pArg->pFile,"%s\n",asText(pState));
	}
    } uRETURN(True);
}

static Boolean
_profileWriteRoom(pFile,pRoom,writeStates,writeTransient)
FILE		*pFile;
RoomPtr	 	 pRoom;
Boolean		 writeStates;
Boolean		 writeTransient;
{
StringToken	name;
Boolean	written= False;
struct pIterArg	iarg;

    uENTRY4("_profileWriteRoom(0x%x,%s,%s,%s)\n",pFile,roomText(pRoom),
						booleanText(writeStates),
						booleanText(writeTransient));
    name=	roomName(pRoom);
    if ((pRoom!=NullRoom)&&(writeStates)) {
	iarg.pFile=		pFile;
	iarg.roomName=		name;
	iarg.pRoom=		pRoom;
	iarg.writeTransient=	writeTransient;
	iarg.written=		False;
	AppsIterate(_profileWriteRoomState,(Opaque)&iarg);
	written=	iarg.written;
    }
    if (!written) {
	_profileWriteRoomHeader(pFile,pRoom);
	fprintf(pFile,"\n");
    }
    else {
	fprintf(pFile,"}\n");
    }
    uRETURN(True);
}

/***====================================================================***/

static void
_profileWriteComment(pFile,pComment)
FILE		*pFile;
CommentPtr	pComment;
{
    uENTRY2("_profileWriteComment(0x%x,0x%x)\n",pFile,pComment);
    while (pComment!=NullComment) {
	fprintf(pFile,"#%s\n",uStringText(pComment->text));
	pComment= pComment->pNext;
    }
    uVOIDRETURN;
}

/***====================================================================***/

Boolean	
profileWrite(pFile,writeRooms)
FILE	*pFile;
Boolean	writeRooms;
{
int		i;
ProfilePtr	pProf;

    uENTRY2("profileWrite(0x%x,%s)\n",pFile,booleanText(writeRooms));
    if (pFile!=NULL) {
	if (pNameProfiles!=NullProfiles) {
	    NPStatementPtr	pFunc;

	    pProf=	pNameProfiles->pProfiles;
	    for (i=pNameProfiles->nProfiles;i>0;i--,pProf++) {
		pFunc=	nprLookupFunction(pProf->name);
		if (pFunc!=NullNPStatement) {
		    if (pProf->pComments!=NullComment)
			_profileWriteComment(pFile,pProf->pComments);
		    fprintf(pFile,"names \"%s\" {\n",stText(pProf->name));
		    npWriteStatement(pFile,1,pFunc);
		    fprintf(pFile,"}\n");
		}
	    }
	}

	if (pAppProfiles!=NullProfiles) {
	    AppPtr	pApp;

	    pProf=	pAppProfiles->pProfiles;
	    for (i=pAppProfiles->nProfiles;i>0;i--,pProf++) {
		pApp=	LookupApp(pProf->name);
		if ((pApp!=NullApp)&&(appIsPermanent(pApp))) {
		    if (pProf->pComments!=NullComment)
			_profileWriteComment(pFile,pProf->pComments);
		    _profileWriteApp(pFile,pApp,!writeRooms);
		}
	    }
	}
	if (pRoomProfiles!=NullProfiles) {
	    RoomPtr	pRoom;

	    pProf=	pRoomProfiles->pProfiles;
	    for (i=pRoomProfiles->nProfiles;i>0;i--,pProf++) {
		pRoom=	LookupRoom(pProf->name);
		if (pRoom!=NullRoom) {
		    if (pProf->pComments!=NullComment)
			_profileWriteComment(pFile,pProf->pComments);
		    _profileWriteRoom(pFile,pRoom,writeRooms,False);
		}
	    }
	}
	if (pPendingComments)
	    _profileWriteComment(pFile,pPendingComments);
	uRETURN(True);
    }
    uRETURN(False);
}


