%{
  /*\
   *  $Header: profparser.y,v 5.0 90/01/10 06:54:03 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/
%}
%token	APPLICATION	 1
	ROOM		 2
	NAMES		 3

	LBRACE		 10
	RBRACE		 11
	LBRACKET	 12
	RBRACKET	 13
	LPAREN		 14
	RPAREN		 15

	BOLT		20
	QUESTION	21
	EQUALS		22
	GETS		23
	PLUS		24
	SEMI		25
	COLON		26

	DEFAULT		30
	ICONIC		31
	NORMAL		32

	IF		200
	ELSE		201
	SWITCH		202
	CASE		203
	USE		204
	CLASS		205
	IGNORE		206

	VISIBLE		40
	HIDDEN		41
	ALWAYS		42
	WHEN_ACTIVE	43
	WHEN_NONEMPTY	44

	TRANSIENT	50
	PERMANENT	51

	PROPERTY	100
	STRING		101
	IDENT		102
	GEOMETRY	103
	COMMENT		104
	END_OF_FILE	 0
%{
#define	DEBUG_VAR	parserDebug
#include "utils.h"
#include "profparser.h"
#include "nameprof.h"
%}
%start	ProfileFile
%union	{
	AppStatePtr	 pState;
	ParserStatePtr	 pNamedState;
	NPBoolExprPtr	 pBoolExpr;
	NPStringExprPtr	 pStringExpr;
	NPCasePtr	 pCase;
	NPStatementPtr	 pStatement;
	StringToken	 token;
	Boolean		 bool;
	unsigned	 uVal;
}
%type <pState>		OptStateSpec StateSpec
%type <pNamedState>	OptProfileBody ListOfStates State OptWinState OptGeom
%type <pBoolExpr>	BooleanExpr
%type <pStringExpr>	StringExpr StringTerminal AssignLHS
%type <pCase>		ListOfCases Case 
%type <pStatement>	ListOfNameSpec NameSpec IfStatement SwitchStatement 
			ClassStatement UseStatement IgnoreStatement 
			AssignStatement BlockStatement OptDefaultCase
%type <token>		String OptString
%type <bool>		Profile NameProfile RoomProfile AppProfile 
			OptBolt Profiles Permanence
%type <uVal>		OptBolts OptVisibilities Visibilities 
%%
ProfileFile		:	Profiles END_OF_FILE	{ return(True); 
							  /* NOTREACHED */}
			|	END_OF_FILE
			;

Profiles		:	Profiles Profile	{ $$= True; }
			|	Profile			{ $$= True; }
			;

Profile			:	RoomProfile 
				{ $$= $1; }
			|	AppProfile  
				{ $$= $1; }
			|	NameProfile 
				{ $$= $1; }
			|	COMMENT
				{ $$= profileAddComment(parserUseString()); } 
			;

RoomProfile		:	ROOM String OptVisibilities
				    OptProfileBody
				{ $$= parserRoomProfile($2,$3,$4); }
			;

AppProfile		:	Permanence APPLICATION String OptStateSpec 
				    OptProfileBody
				{ $$= parserAppProfile($3,$4,$5,$1); }
			;

Permanence		:	TRANSIENT		{ $$= True; }
			|	PERMANENT		{ $$= False; }
			|				{ $$= False; }
			;

OptVisibilities		:	VISIBLE	Visibilities	{ $$= $2; }
			|	Visibilities		{ $$= $1; }
			|	VISIBLE			{ $$=  roomVisible; }
			|	HIDDEN			{ $$=  0; }
			|	{ $$= roomDefaultVisibility; }
			;

Visibilities		:	ALWAYS		{ $$= roomAlways; }
			|	DEFAULT		{ $$= roomDefaultVisibility; }
			|	WHEN_ACTIVE
				{ $$= roomWhenActive; }
			|	WHEN_NONEMPTY
				{ $$= roomWhenNonEmpty; }
			|	WHEN_ACTIVE WHEN_NONEMPTY
				{ $$= roomWhenActive|roomWhenNonEmpty; }
			|	WHEN_NONEMPTY WHEN_ACTIVE
				{ $$= roomWhenActive|roomWhenNonEmpty; }
			;

OptProfileBody		:	LBRACE
				    ListOfStates
				RBRACE
				{ $$= $2; }
			|	{ $$= NullParserState; }
			;

OptStateSpec		:	StateSpec	{ $$= $1; }
			|			{ $$= NullAppState; }
			;

ListOfStates		:	ListOfStates	State
				{ $$= parserAppendState($1,$2); }
			|	State
				{ $$= $1; }
			;

State			:	String	StateSpec
				{ $$= parserCreateNamedState($1,$2); }
			;

StateSpec		:	LBRACKET OptWinState OptGeom RBRACKET
				{ $$= parserCreateState( $2, $3 ); }
			;

NameProfile		:	NAMES OptString BlockStatement
				{ 
				    StringToken	name=	($2?$2:nprDefaultName);
				    if (name!=NullStringToken) {
					nprAddFunction(name,$3,False);
					profileForNames(name);
					$$=True;
				    }
				    else $$= False;
				}
			;

ListOfNameSpec		:	ListOfNameSpec	NameSpec
				{
				    npAppendStatement($1,$2);
				    $$= $1;
				}
			|	NameSpec
				{ $$= $1; }
			;

NameSpec		:	IfStatement      	 { $$= $1; }
			|	SwitchStatement	 	 { $$= $1; }
			|	ClassStatement	 	 { $$= $1; }
			|	UseStatement	 	 { $$= $1; }
			|	IgnoreStatement	 	 { $$= $1; }
			|	AssignStatement		 { $$= $1; }
			|	BlockStatement	 	 { $$= $1; }
			|	COMMENT	
				{ $$= npComment(parserUseString()); }
			;

IfStatement		:	IF LPAREN BooleanExpr RPAREN NameSpec 
							ELSE NameSpec
				{ $$= npIfStatement( $3, $5, $7 ); }
			|	IF LPAREN BooleanExpr RPAREN NameSpec
				{ $$= npIfStatement( $3, $5, NullNPStatement);}
			;

SwitchStatement		:	SWITCH LPAREN StringExpr RPAREN LBRACE
				     ListOfCases
				     OptDefaultCase
				RBRACE
				{ $$= npSwitchStatement($3, $6, $7); }
			;


ListOfCases		:	ListOfCases Case
				{ 
				    npAppendCase($1,$2);
				    $$= $1;
				}
			|	Case
				{ $$= $1; }
			;

Case			:	CASE StringExpr COLON NameSpec
				{ $$= npCase( $2, $4 ); }
			;

OptDefaultCase		:	DEFAULT COLON NameSpec
				{ $$= $3; }
			|	{ $$= NullNPStatement; }
			;

ClassStatement		:	CLASS StringExpr
				{ $$= npClassStatement($2); }
			;

UseStatement		:	USE StringExpr
				{ $$= npUseStatement($2); }
			;

IgnoreStatement		:	IGNORE
				{ $$= npIgnoreStatement(); }
			;

AssignStatement		:	AssignLHS GETS StringExpr
				{ $$= npAssignStatement($1,$3,False); }
			|	AssignLHS PLUS GETS StringExpr
				{ $$= npAssignStatement($1,$4,True); }
			;

AssignLHS		:	PROPERTY
				{ $$= npStringTerminal(NPPropertyName,
						   parserUseString()); }
			|	IDENT
				{ $$= npStringTerminal(NPIdentifier,
						   parserUseString()); }
			;

BlockStatement		:	LBRACE ListOfNameSpec RBRACE
				{ $$= $2; }
			;

String			:	STRING
				{ $$= stGetToken(parserUseString()); }
			;

OptString		:	String		{ $$= $1; }
			|			{ $$= NullStringToken; }
			;

OptWinState		:	ICONIC OptBolt
				{$$= parserCreateWinState(asIconic,$2);}
			|	NORMAL OptBolt
				{$$= parserCreateWinState(asNormal,$2);}
			|
			        {$$= NullParserState; }
			;

OptGeom			:	GEOMETRY OptBolts
				{$$= parserCreateGeom(parserUseGeom(),$2);}
			|
				{$$= parserCreateGeom(NullAppState,asNoValue); }
			;

OptBolts		:	IDENT BOLT
				{ $$= parseBolts(parserUseString()); }
			|	{ $$= asNoValue; }
			;

OptBolt			:	BOLT	{ $$= True;  }
			|		{ $$= False; }
			;

BooleanExpr		:	BOLT StringExpr
				{ $$= npUnaryExpr(True,$2); }
			|	StringExpr
				{ $$= npUnaryExpr(False,$1); }
			|	StringExpr EQUALS StringExpr
				{ $$= npEquals($1,$3); }
			;

StringExpr		:	StringExpr PLUS StringTerminal 
				{ $$= npStringConcat($1,$3); }
			|	LPAREN BooleanExpr QUESTION StringExpr 
						COLON 	 StringExpr RPAREN
				{ $$= npStringConditional($2,$4,$6); }
			|	LPAREN BooleanExpr QUESTION StringExpr RPAREN
				{ $$= npStringConditional($2,$4,
							NullNPStringExpr); }
			|	StringTerminal
				{ $$= $1; }
			;

StringTerminal		:	STRING
				{ $$= npStringTerminal(NPStringConstant,
						   parserUseString()); }
			|	PROPERTY
				{ $$= npStringTerminal(NPPropertyName,
						   parserUseString()); }
			|	IDENT
				{ $$= npStringTerminal(NPIdentifier,
						   parserUseString()); }
			;

%%
ScannablePtr	pYYLexScan;

void
yyerror(s)
char	*s;
{
    (void)uError("%s\n",s);
    uInformation("line %d\n",scanLineNum(pYYLexScan));
    return;
}
Boolean
yywrap()
{
   return(True);
}
int
yylex()
{
int	token;

   if (pYYLexScan==NullScannable)	token=	TOK_END;
   else					token=	scanToken(pYYLexScan);
   return(token);
}

char *
parserUseString()
{
char	*str= NullString;

    uENTRY("parserUseString()\n");
    if (pYYLexScan!=NullScannable) {
	str= scanUseString(pYYLexScan);
    }
    uRETURN(str);
}

AppStatePtr
parserUseGeom()
{
AppStatePtr	 pGeomState= NullAppState;

    uENTRY("parserUseGeom()\n");
    if (pYYLexScan!=NullScannable) {
	pGeomState= scanUseState(pYYLexScan);
    }
    uRETURN(pGeomState);
}

/***====================================================================***/

static void
_applyStateToRoom(pPState,pRoom)
ParserStatePtr	pPState;
RoomPtr		pRoom;
{
AppPtr		pApp;

    uENTRY2("_applyStateToRoom(0x%x,%s)\n",pPState,roomText(pRoom));
    if (pPState->u.state.name!=NullStringToken) {
	pApp=	GetApp(pPState->u.state.name,False);
	if (pApp!=NullApp) {
	    (void)appSetRoomState(pApp,roomName(pRoom),
					 pPState->u.state.pState,False);
	    if (appIsActive(pApp)) {
		(void)roomCheckAppStates(pRoom,pApp);
	    }
	}
    }
    else {
	uWarning("Default state for room %s ignored\n",roomText(pRoom));
    }
    uVOIDRETURN;
}

/***====================================================================***/

Boolean
parserRoomProfile(name, vis, pStates)
StringToken	name;
unsigned	vis;
ParserStatePtr	pStates;
{
ParserStatePtr	pNext;
RoomPtr		pRoom= NullRoom;

    uENTRY3("parserRoomProfile(%s,0x%x,0x%x)\n",stText(name),vis,pStates);
    if (name!=NullStringToken) {
	pRoom=	LookupRoom(name);
	if (pRoom==NullRoom) {
	    pRoom=	CreateRoom(name);
	}
	roomSetVisibility(pRoom,vis,True);
	while (pStates!=NullParserState) {
	    pNext=	pStates->pNext;
	    if (pRoom!=NullRoom) 
		_applyStateToRoom(pStates,pRoom);
	    asDestroy(pStates->u.state.pState);
	    pStates->u.state.pState=	NullAppState;
	    pStates->pNext=		NullParserState;
	    (void)uFree((Opaque)pStates);
	    pStates=	pNext;
	}
    }
    if (pRoom==NullRoom) {
	uRETURN(False);
    }
    uRETURN(True);
}

/***====================================================================***/

static void
_applyStateToApp(pPState,pApp)
ParserStatePtr	pPState;
AppPtr		pApp;
{
    uENTRY2("_applyStateToApp(0x%x,%s)\n",pPState,appText(pApp));
    if (pPState->u.state.name!=NullStringToken) {
	(void)appSetRoomState(pApp,pPState->u.state.name,	
					pPState->u.state.pState,False);
    }
    else {
	(void)appSetDefault(pApp,pPState->u.state.pState,False);
    }
    uVOIDRETURN;
}

Boolean
parserAppProfile(name, pDfltState, pStates, transient)
StringToken	name;
AppStatePtr	pDfltState;
ParserStatePtr	pStates;
Boolean		transient;
{
ParserStatePtr	pNext;
RoomPtr		pRoom;
AppPtr		pApp;
Opaque		rstate;

    uENTRY4("parserAppProfile(%s,%s,0x%x,%s)\n",stText(name),asText(pDfltState),
						pStates,booleanText(transient));
    
    if (name!=NullStringToken) {
	pApp=	GetApp(name,False);
	appRemoveAllRoomStates(pApp);
	while (pStates!=NullParserState) {
	    pNext=	pStates->pNext;
	    if (pApp!=NullApp) 
		_applyStateToApp(pStates,pApp);
	    asDestroy(pStates->u.state.pState);
	    pStates->u.state.pState=	NullAppState;
	    pStates->pNext=		NullParserState;
	    (void)uFree((Opaque)pStates);
	    pStates=	pNext;
	}
	if (pDfltState) {
	    if (pApp!=NullApp) {
		(void)appSetDefault(pApp,pDfltState,False);
	    }
	    asDestroy(pDfltState);
	}
    }

    if (appIsActive(pApp)) {
	RoomsIterator(pRoom,rstate) {
	    roomCheckAppStates(pRoom,pApp);
	}
	RoomsEndIterator(rstate);
    }

    if (pApp!=NullApp) {
	SetAppPermanence(pApp,!transient);
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

AppStatePtr
parserCreateState(pPWinState, pPGeom)
ParserStatePtr	pPWinState;
ParserStatePtr	pPGeom;
{
AppStatePtr	pState;

    uENTRY2("parserCreateState(0x%x,0x%x)\n",pPWinState,pPGeom);
    if (pPGeom!=NullParserState) {
	pState=	pPGeom->u.pGeomState;
	uFree((Opaque)pPGeom);

	if (pPWinState!=NullParserState) {
	    asChangeSelected(pState,asWinStateValue,pPWinState->u.ws.winState,
						0,0,(unsigned)0,(unsigned)0);
	    if (pPWinState->u.ws.bolted)
		asBolt(pState,asWinStateValue);
	    uFree((Opaque)pPWinState);
	}
    }
    uRETURN(pState);
}

/***====================================================================***/

ParserStatePtr
parserAppendState(pState1, pState2)
ParserStatePtr	pState1,pState2;
{
ParserStatePtr	pEnd;

    uENTRY2("parserAppendState(0x%x,0x%x)\n",pState1,pState2);
    if (pState1==NullParserState) {
	uRETURN(pState2);
    }
    if (pState2!=NullParserState) {
	pEnd=	pState1;
	while (pEnd->pNext!=NullParserState) {
	    pEnd= pEnd->pNext;
	}
	pEnd->pNext=	pState2;
    }
    uRETURN(pState1);
}

/***====================================================================***/

ParserStatePtr
parserCreateNamedState(name, pState)
StringToken	name;
AppStatePtr	pState;
{
ParserStatePtr	pPState=	NullParserState;

    uENTRY2("parserCreateNamedState(%s,%s)\n",stText(name),asText(pState));
    pPState=	uTypedAlloc(ParserStateRec);
    if (pPState!=NullParserState) {
	bzero(pPState,sizeof(ParserStateRec));
	pPState->u.state.name=		name;
	pPState->u.state.pState=	pState;
    }
    uRETURN(pPState);
}

/***====================================================================***/

ParserStatePtr
parserCreateWinState(winState, bolted)
WinState	winState;
Boolean		bolted;
{
ParserStatePtr	pPState=	NullParserState;

    uENTRY2("parserCreateWinState(%s,%s)\n",asWinStateText(winState),
						booleanText(bolted));
    pPState=	uTypedAlloc(ParserStateRec);
    if (pPState!=NullParserState) {
	bzero(pPState,sizeof(ParserStateRec));
	pPState->u.ws.winState=	winState;
	pPState->u.ws.bolted=	bolted;
    }
    uRETURN(pPState);
}

/***====================================================================***/

ParserStatePtr
parserCreateGeom(pState, bolts)
AppStatePtr	pState;
unsigned	bolts;
{
ParserStatePtr	pPState=	NullParserState;

    uENTRY2("parserCreateGeom(%s,0x%x)\n",asText(pState),bolts);
    pPState=	uTypedAlloc(ParserStateRec);
    if (pPState!=NullParserState) {
	bzero(pPState,sizeof(ParserStateRec));
	if (pState==NullAppState) {
	    pState=	asDupFully(asDefaultProfile);
	}
	pPState->u.pGeomState=	pState;
	if (bolts!=asNoValue)
	    asBolt(pState,bolts&asAllValues&asFlags(pState));
    }
    uRETURN(pPState);
}

/***====================================================================***/

unsigned
parseBolts(str)
char	*str;
{
unsigned	bolts=	asNoValue;

    uENTRY1("parseBolts(%s)\n",uStringText(str));
    if (str!=NullString) {
	while (*str!=NUL) {
	    if		(*str=='x')	bolts|=	asXValue;
	    else if 	(*str=='y')	bolts|= asYValue;
	    else if	(*str=='w')	bolts|= asWidthValue;
	    else if	(*str=='h')	bolts|= asHeightValue;
	    else if	(*str=='s')	bolts|= asWinStateValue;
	    else {
		uWarning("unknown character '%c' in bolt specification\n",*str);
		uAction("ignored.\n");
	    }
	    str++;
	}
    }
    uRETURN(bolts);
}
