
  /*\
   *  $Header: profscan.c,v 5.0 90/01/10 06:54:08 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#define	DEBUG_VAR scanDebug
#include "utils.h"
#include "appstate.h"
#include "profile.h"
#include "scanstr.h"
#include <ctype.h>

/***====================================================================***/

#define	scanChar(ps,c)	{\
    if ((ps)->flags&scannableEnd)		(c)=EOF;\
    else if ((ps)->flags&scannableString) {\
	if (*(ps)->stream.string!='\0') {\
	    (c)= *(ps)->stream.string++;\
	}\
	else (ps)->flags|= (scannableEnd);\
    }\
    else if ((ps)->flags&scannableFile) {\
	(c)=	getc((ps)->stream.file);\
	if ((c)==EOF)	(ps)->flags|= (scannableEnd);\
    }\
    else (c)= EOF;\
    if ((c)=='\n')	(ps)->lineNum++;\
}

#define	scanPutBackChar(ps,c)	{\
    if ((!((ps)->flags&scannableEnd))&&((c)!=EOF)) {\
	if ((ps)->flags&scannableString) {\
	    (ps)->stream.string--;\
	    *(ps)->stream.string= (c);\
	}\
	else if ((ps)->flags&scannableFile) {\
	    ungetc((c),(ps)->stream.file);\
	}\
    }\
    else (c)= EOF;\
}

/***====================================================================***/

char *
scannerText(token)
unsigned	token;
{
   switch (token) {
	case TOK_END:		return("END");
	case TOK_APPLICATION:	return("APPLICATION");
	case TOK_ROOM:		return("ROOM");
	case TOK_NAMES:		return("NAMES");

	case TOK_LBRACE:	return("LBRACE");
	case TOK_RBRACE:	return("RBRACE");
	case TOK_LBRACKET:	return("LBRACKET");
	case TOK_RBRACKET:	return("RBRACKET");
	case TOK_LPAREN:	return("LPAREN");
	case TOK_RPAREN:	return("RPAREN");

	case TOK_BOLT:		return("BOLT");
	case TOK_QUESTION:	return("QUESTION");
	case TOK_EQUALS:	return("EQUALS");
	case TOK_GETS:		return("GETS");
	case TOK_PLUS:		return("PLUS");
	case TOK_SEMI:		return("SEMI");
	case TOK_COLON:		return("COLON");

	case TOK_DEFAULT:	return("DEFAULT");
	case TOK_ICONIC:	return("ICONIC");
	case TOK_NORMAL:	return("NORMAL");

	case TOK_IF:		return("IF");
	case TOK_ELSE:		return("ELSE");
	case TOK_SWITCH:	return("SWITCH");
	case TOK_CASE:		return("CASE");
	case TOK_USE:		return("USE");
	case TOK_CLASS:		return("CLASS");
	case TOK_IGNORE:	return("IGNORE");

	case TOK_VISIBLE:	return("VISIBLE");
	case TOK_HIDDEN:	return("HIDDEN");
	case TOK_ALWAYS:	return("ALWAYS");
	case TOK_WHEN_ACTIVE:	return("WHENACTIVE");
	case TOK_WHEN_NONEMPTY:	return("WHENNONEMPTY");

	case TOK_TRANSIENT:	return("TRANSIENT");
	case TOK_PERMANENT:	return("PERMANENT");

	case TOK_PROPERTY:	return("PROPERTY");
	case TOK_STRING:	return("STRING");
	case TOK_IDENT:		return("IDENT");
	case TOK_GEOMETRY:	return("GEOMETRY");
	case TOK_COMMENT:	return("COMMENT");
   }
   return("WEIRD");
}

/***====================================================================***/

char *
scanUseString(pScan)
ScannablePtr	pScan;
{
char	*str= NullString;

    uENTRY1("scanUseString(0x%x)\n",pScan);
    if (pScan!=NullScannable) {
	str=		pScan->string;
	pScan->string=	NullString;
    }
    uRETURN(str);
}

/***====================================================================***/

void
scanSetString(pScan,str)
ScannablePtr	pScan;
char		*str;
{
    uENTRY2("scanSetString(0x%x,%s)\n",pScan,uStringText(str));
    if (pScan!=NullScannable) {
	if (pScan->string!=NullString)
	    (void)uFree((Opaque)pScan->string);
	if (str!=NullString)	pScan->string=	uStringDup(str);
	else			pScan->string=	NullString;
    }
    uVOIDRETURN;
}

/***====================================================================***/

AppStatePtr	
scanUseState(pScan)
ScannablePtr	pScan;
{
AppStatePtr	pState=	NullAppState;

    uENTRY1("scanUseState(0x%x)\n",pScan);
    if (pScan!=NullScannable) {
	pState=		pScan->pState;
	pScan->pState=	NullAppState;
    }
    uRETURN(pState);
}

/***====================================================================***/

void
scanSetState(pScan,flags,x,y,w,h)
ScannablePtr	pScan;
unsigned	flags;
int		x,y;
unsigned	w,h;
{
    uENTRY6("scanSetGeom(0x%x,0x%x,%d,%d,%d,%d)\n",pScan,flags,x,y,w,h);
    if (pScan!=NullScannable) {
	if (flags!=asNoValue) {
	    if (pScan->pState!=NullAppState) {
		asChangeAll(pScan->pState,flags,asInactive,x,y,w,h);
	    }
	    else {
		pScan->pState=	asCreate(flags,asInactive,x,y,w,h);
	    }
	}
	else if (pScan->pState!=NullAppState) {
	   asDestroy(pScan->pState);
	   pScan->pState=	NullAppState;
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

ScannablePtr
scannableOpen(type, source)
unsigned	type;
Opaque		source;
{
ScannablePtr	pScan;

    uENTRY2("scannableOpen(0x%x,0x%x)\n",type,source);
    pScan=	uTypedAlloc(ScannableRec);
    if (pScan!=NullScannable) {
	if (type==scannableString) {
	    if (source!=(Opaque)NULL) {
		pScan->flags=		type;
		pScan->stream.string=	(char *)source;
	    }
	    else goto BAILOUT;
	}
	else if (type==scannableFile) {
	    if (source!=(Opaque)NULL) {
		pScan->flags=		type;
		pScan->stream.file=	(FILE *)source;
	    }
	    else goto BAILOUT;
	}
	else if (type==scannableFileName) {
	    if (source!=(Opaque)NULL) {
		pScan->flags=		scannableFile|scannableFileName;
		pScan->stream.file=	fopen((char *)source,"r");
		if (pScan->stream.file==NULL)
		    goto BAILOUT;
	    }
	    else goto BAILOUT;
	}
	pScan->lineNum=		0;
	pScan->lookingAhead=	False;
	pScan->token=		TOK_END;
	pScan->string=		NullString;
	pScan->pState=		NullAppState;
    }
    uRETURN(pScan);
BAILOUT:
    (void)uFree((Opaque)pScan);
    uRETURN(NullScannable);
}

/***====================================================================***/

void	
scannableClose(pScan)
ScannablePtr	pScan;
{
    uENTRY1("scannableClose(0x%x)\n",pScan);
    if (pScan!=NullScannable) {
	if (pScan->flags&scannableFileName) {
	    fclose(pScan->stream.file);
	}
	scanSetString(pScan,NullString);
	scanSetState(pScan,asNoValue,0,0,(unsigned)0,(unsigned)0);
	(void)uFree((Opaque)pScan);
    }
    uVOIDRETURN;
}

/***====================================================================***/

static	int
scanFillComment(pScan)
ScannablePtr	pScan;
{
char	stringBuf[SCANNER_STRING_LEN];
int	index,ch;

    uENTRY1("scanFillComment(0x%x)\n",pScan);
    index=	0;
    scanChar(pScan,ch);
    while ((ch!=EOF)&&(ch!='\n')&&(index<SCANNER_STRING_LEN-1)) {
	if (ch!=EOF) {
	    stringBuf[index++]=	ch;
	    scanChar(pScan,ch)
	}
    }
    stringBuf[index++]= '\0';
    while ((ch!='\n')&&(ch!=EOF)) {
	scanChar(pScan,ch);
    }
    scanSetString(pScan,stringBuf);
    uRETURN(TOK_COMMENT);
}

/***====================================================================***/

static	int
scanFillString(pScan)
ScannablePtr	pScan;
{
char	stringBuf[SCANNER_STRING_LEN];
int	index,ch;

    uENTRY1("scanFillString(0x%x)\n",pScan);
    index=	0;
    scanChar(pScan,ch);
    while ((ch!=EOF)&&(ch!='\"')&&(index<SCANNER_STRING_LEN-1)) {
	if (ch=='\\') {
	    scanChar(pScan,ch);
	    if		(ch=='n')	ch=	'\n';
	    else if	(ch=='t')	ch=	'\t';
	    else if ((ch!='\"')&&(ch!='\\')) {
		scanPutBackChar(pScan,ch);
		ch= '\\';
	    }
	}
	if (ch!=EOF) {
	    stringBuf[index++]=	ch;
	    scanChar(pScan,ch)
	}
    }
    stringBuf[index++]= '\0';
    while ((ch!='\"')&&(ch!=EOF)) {
	scanChar(pScan,ch);
    }
    scanSetString(pScan,stringBuf);
    uRETURN(TOK_STRING);
}

/***====================================================================***/

static	int
scanFillProperty(pScan)
ScannablePtr	pScan;
{
char	stringBuf[SCANNER_STRING_LEN];
int	index,ch;

    uENTRY1("scanFillProperty(0x%x)\n",pScan);
    index=	0;
    do {
	scanChar(pScan,ch);
    } while ((ch!=EOF)&&(isspace(ch)));
    while ((ch!=EOF)&&(ch!='>')&&(!isspace(ch))&&(index<SCANNER_STRING_LEN-1)) {
	if (ch!=EOF) {
	    stringBuf[index++]=	ch;
	    scanChar(pScan,ch)
	}
    }
    stringBuf[index++]= '\0';
    if (ch!='>') {
	static char *where;
	uWarning("Missing end of property '>' on line %d\n",pScan->lineNum);
	if 	(ch==EOF)	where= "EOF";
	else if (ch=='\n')	where= "end of line";
	else if (isspace(ch))	where= "whitespace";
	else			where= "the 256th character";
	uAction("assuming property ends at %s\n",where);
	uAction("property name will be \"%s\"\n",stringBuf);
    }
    while ((ch!='>')&&(ch!=EOF)) {
	scanChar(pScan,ch);
    }
    scanSetString(pScan,stringBuf);
    uRETURN(TOK_PROPERTY);
}

/***====================================================================***/

static int
scanFillWord(pScan,firstCh)
ScannablePtr	pScan;
int		firstCh;
{
char	wordBuf[SCANNER_STRING_LEN];
int	index,ch;

    uENTRY2("scanFillWord(0x%x,%c)\n",pScan,firstCh);
    index=	0;
    ch=	firstCh;
    while ((ch!=EOF)&&(isalpha(ch))&&(index<SCANNER_STRING_LEN-1)) {
	if (isupper(ch))	ch=	tolower(ch);
	wordBuf[index++]=	ch;
	scanChar(pScan,ch);
    }
    wordBuf[index++]= '\0';
    if ((ch!=EOF)&&(!isspace(ch))) {
	scanPutBackChar(pScan,ch);
    }
    if (uStringEqual(wordBuf,"application"))	{ uRETURN(TOK_APPLICATION); }
    if (uStringEqual(wordBuf,"room"))		{ uRETURN(TOK_ROOM); }
    if (uStringEqual(wordBuf,"names"))		{ uRETURN(TOK_NAMES); }

    if (uStringEqual(wordBuf,"default"))	{ uRETURN(TOK_DEFAULT); }
    if (uStringEqual(wordBuf,"iconic"))		{ uRETURN(TOK_ICONIC); }
    if (uStringEqual(wordBuf,"normal"))		{ uRETURN(TOK_NORMAL); }

    if (uStringEqual(wordBuf,"if"))		{ uRETURN(TOK_IF); }
    if (uStringEqual(wordBuf,"else"))		{ uRETURN(TOK_ELSE); }
    if (uStringEqual(wordBuf,"switch"))		{ uRETURN(TOK_SWITCH); }
    if (uStringEqual(wordBuf,"case"))		{ uRETURN(TOK_CASE); }
    if (uStringEqual(wordBuf,"use"))		{ uRETURN(TOK_USE); }
    if (uStringEqual(wordBuf,"class"))		{ uRETURN(TOK_CLASS); }
    if (uStringEqual(wordBuf,"ignore"))		{ uRETURN(TOK_IGNORE); }

    if (uStringEqual(wordBuf,"visible"))	{ uRETURN(TOK_VISIBLE); }
    if (uStringEqual(wordBuf,"hidden"))		{ uRETURN(TOK_HIDDEN); }
    if (uStringEqual(wordBuf,"always"))		{ uRETURN(TOK_ALWAYS); }
    if (uStringEqual(wordBuf,"whenactive"))	{ uRETURN(TOK_WHEN_ACTIVE); }
    if (uStringEqual(wordBuf,"whennonempty"))	{ uRETURN(TOK_WHEN_NONEMPTY); }

    if (uStringEqual(wordBuf,"transient"))	{ uRETURN(TOK_TRANSIENT); }
    if (uStringEqual(wordBuf,"permanent"))	{ uRETURN(TOK_PERMANENT); }
    scanSetString(pScan,wordBuf);
    uRETURN(TOK_IDENT);
}

/***====================================================================***/

static int
scanFillGeometry(pScan)
ScannablePtr	pScan;
{
char		geomBuf[SCANNER_STRING_LEN];
int		index,ch;
int		x,y;
unsigned	width,height;
unsigned	rtrn;

    uENTRY1("scanFillGeometry(0x%x)\n",pScan);
    index=	0;
    scanChar(pScan,ch);
    if (ch=='=') {
	uRETURN(TOK_EQUALS);
    }
    else if (isalpha(ch)||(isspace(ch))) {
	scanPutBackChar(pScan,ch);
	uRETURN(TOK_GETS);
    }
    while ((ch!=EOF)&&(index<SCANNER_STRING_LEN-1)&&
	   (isdigit(ch)||(ch=='+')||(ch=='-')||(ch=='x'))) {
	geomBuf[index++]=	ch;
	scanChar(pScan,ch);
    }
    geomBuf[index++]= '\0';
    if ((ch!=EOF)&&(!isspace(ch))) {
	scanPutBackChar(pScan,ch);
    }

    x= y= width= height= 0;
    rtrn= XParseGeometry(geomBuf,&x,&y,&width,&height);
    if (!(rtrn&AllValues)) {
	uWarning("Illegal geometry specification \"%s\" on line %d\n",
						geomBuf,pScan->lineNum);
	return(TOK_END);
    }
    if (rtrn&asXNegative)	x=	-x;
    if (rtrn&asYNegative)	y=	-y;
    scanSetState(pScan,rtrn,x,y,width,height);
    uRETURN(TOK_GEOMETRY);
}

/***====================================================================***/

int
scanPeek(pScan)
ScannablePtr	pScan;
{
int	token= TOK_END;

    uENTRY1("scanPeek(0x%x)\n",pScan);
    if (pScan!=NullScannable) {
	token=	scanToken(pScan);
	pScan->lookingAhead=	True;
    }
    uRETURN(token);
}

/***====================================================================***/

int
scanToken(pScan)
ScannablePtr	pScan;
{
int	ch,token;

    uENTRY1("scanToken(0x%x)\n",pScan);
    if ((pScan==NullScannable)||(pScan->flags&scannableEnd)) {
	uRETURN(TOK_END);
    }

    if (pScan->lookingAhead) {
	pScan->lookingAhead=	False;
	uRETURN(pScan->token);
    }

    do {
	scanChar(pScan,ch);
    } while ((ch!=EOF)&&(isspace(ch)));
    if (ch==EOF) 		{ token=	TOK_END; 	}
    else if (ch=='{')		{ token= TOK_LBRACE; 		}
    else if (ch=='}')		{ token= TOK_RBRACE;		}
    else if (ch=='[')		{ token= TOK_LBRACKET;		}
    else if (ch==']')		{ token= TOK_RBRACKET;		}
    else if (ch=='(')		{ token= TOK_LPAREN;		}
    else if (ch==')')		{ token= TOK_RPAREN;		}
    else if (ch=='!')		{ token= TOK_BOLT;		}
    else if (ch=='?')		{ token= TOK_QUESTION;		}
    else if (ch=='+')		{ token= TOK_PLUS;		}
    else if (ch==';')		{ token= TOK_SEMI;		}
    else if (ch==':')		{ token= TOK_COLON;		}
    else if (ch=='<')		{ token= scanFillProperty(pScan);	}
    else if (ch=='\"')		{ token= scanFillString(pScan);		}
    else if (ch=='=')		{ token= scanFillGeometry(pScan);	}
    else if (ch=='#')		{ token= scanFillComment(pScan);	}
    else if (isalpha(ch))	{ token= scanFillWord(pScan,ch);	}
    else {
	pScan->flags|= scannableEnd;
	uWarning("Unrecognized char '%c' on line %d\n",ch,pScan->lineNum);
	token= TOK_END;
    }
    pScan->token=		token;
    pScan->lookingAhead=	False;
    uDEBUG1(DEBUG_TOKENS,"scanned: %s\n",scannerText((unsigned)token));
    uRETURN(token);
}

/***====================================================================***/

int
scanLineNum(pScan)
ScannablePtr	pScan;
{
    uENTRY1("scanLineNum(0x%x)\n",pScan);
    if (pScan!=NullScannable) {
	uRETURN(pScan->lineNum);
    }
    uRETURN(0);
}
