#ifndef PROFSCAN_H
#define	PROFSCAN_H 1

  /*\
   *  $Header: profscan.h,v 5.0 90/01/10 06:54:12 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

  /*\

  \*/

	/*\
	 *  Implements a "scannable object" class.
	 *  A Scannable traverses a string, named file, or file pointer
	 *  generating a stream of tokens until it reaches the end of the 
	 *  stream or encounters an illegal character.  An illegal character
	 *  is a character that is not contained in a token and is not a prefix 
	 *  for one of the legal tokens.
	 *
	 *  A scannableString ends at the first NUL it encounters, a
	 *  scannableFile, or scannableFileName ends at EOF. 
	 *
	 *  Most tokens represent keywords or punctuation marks, and have
	 *  no associated data.
	 *  The tokens: TOK_PROPERTY,TOK_STRING,TOK_IDENT, and TOK_COMMENT
	 *	also have an associated string value which may be retrieved
	 *	using "scanUseString."
	 *  The token TOK_GEOMETRY has an associated AppState, which may be
	 *	retrieved using "scanUseState."
	 *	
	 *  PUBLIC GLOBAL VARIABLES --
	 *  unsigned	scanDebug;
	 *	A mask of debugging flags.  The only flags currently used
	 *	are ENTER (0x10, defined in ../utils/utils.h) which tracks
	 *	function entry and exit, and DEBUG_TOKENS (0x08) which prints
	 *	out the value of each token as it is scanned.
	 *
	 *  FUNCTIONS TO OPEN AND CLOSE A SCANNABLE --
	 *  ScannablePtr
	 *  scannableOpen( type, source )
	 *  unsigned	type;
	 *  Opaque	source;	// (char *) or (FILE *)
	 *	Opens the "source" for scanning, and returns a pointer to
	 *	the new Scannable.   If "type" is scannableString, "source"
	 *	is expected to contain a (char *) pointer to the string to
	 *	be scanned.   If "type" is scannableFile, "source" should
	 *	contain a (FILE *) pointer to the file to be scanned; this
	 *	(FILE *) should already be open.   If "type" is 
	 *	scannableFileName, "source" should contain a (char *) pointer
	 *	to the name of the file to be scanned; scannableOpen will
	 *	try to open the named file.
	 *	Returns a pointer to the newly created Scannable, or 
	 *	NullScannable if "source" is NULL, "type" is not a legal
	 *	value, or "type" is scannableFileName and scannableOpen
	 *	cannot open the named file.
	 *  
	 *  void
	 *  scannableClose( pScan )
	 *  ScannablePtr	pScan;
	 *	Frees any memory associated with "pScan."  If "pScan" is
	 *	of type scannableFileName, the associated file is closed
	 *	before "pScan" is freed.   If "pScan" is of type 
	 *	scannableFile or scannableString, the calling function is
	 *	expected to finalize the associated file or string.  No
	 *	return value.
	 * 
	 *  FUNCTIONS TO GET TOKENS FROM A SCANNABLE --
	 *  int
	 *  scanPeek(pScan)
	 *  ScannablePtr	pScan;
	 *	Peeks at the next token in the input stream.  The stream 
	 *	pointer is not moved, so successive Peeks will return the
	 *	same value.  Returns the next token or TOK_END if "pScan"
	 *	is NullScannable.
	 *
	 *  int
	 *  scanToken(pScan)
	 *  ScannablePtr	pScan;
	 *	Returns the next token in the input stream.  The stream
	 *	pointer is moved, so successive Scans may return different
	 *	values.   Returns TOK_END if "pScan" is NullScannable.
	 *
	 *  FUNCTIONS TO RETRIEVE VALUES ASSOCIATED WITH TOKENS --
	 *  char *
	 *  scanUseString(pScan)
	 *  ScannablePtr	pScan;
	 *	Returns the string value associated with the most recently
	 *	scanned STRING, PROPERTY, COMMENT, or IDENT in "pScan." 
	 *	The returned string is allocated and should be freed by 
	 *	the calling function when it is no longer needed.   
	 *	The "most recent string" field is cleared by this function, 
	 *	so subsequent calls will not return the same value.	
	 *	Returns NullString if "pScan" is NullScannable, or if none
	 *	of these tokens have been scanned after the last call to
	 *	scanUseString or "pScan" was opened.
	 *
	 *  AppStatePtr
	 *  scanUseState(pScan)
	 *  ScannablePtr	pScan;
	 *	Returns the AppStatePtr associated with the most recently
	 *	scanned GEOMETRY.   The returned AppState is not static
	 *	and should be destroyed with asDestroy when it is no longer
	 *	needed.   Like scanUseString, scanUseState clears the
	 *	"most recent geometry" field, so subsequent calls will not
	 *	return the same value.   Returns NullAppState if "pScan" is
	 *	NullScannable, or if no GEOMETRY has been scanned since the
	 *	last call to scanUseState or "pScan" was opened.
	 *
	 *  OTHER USEFUL FUNCTIONS --
	 *  int
	 *  scanLineNum(pScan)
	 *  ScannablePtr	pScan;
	 *	Returns the current line number in "pScan," of -1 if pScan
	 *	is NullScannable;
	\*/

#include "profile.h"

#define	TOK_END		 0
#define	TOK_APPLICATION	 1
#define	TOK_ROOM	 2
#define	TOK_NAMES	 3

#define	TOK_LBRACE	10
#define	TOK_RBRACE	11
#define	TOK_LBRACKET	12
#define	TOK_RBRACKET	13
#define	TOK_LPAREN	14
#define	TOK_RPAREN	15

#define	TOK_BOLT	20
#define	TOK_QUESTION	21
#define	TOK_EQUALS	22
#define	TOK_GETS	23
#define	TOK_PLUS	24
#define	TOK_SEMI	25
#define	TOK_COLON	26

#define	TOK_DEFAULT	30
#define	TOK_ICONIC	31
#define	TOK_NORMAL	32

#define	TOK_IF		200
#define	TOK_ELSE	201
#define	TOK_SWITCH	202
#define	TOK_CASE	203
#define	TOK_USE		204
#define	TOK_CLASS	205
#define	TOK_IGNORE	206

#define	TOK_VISIBLE	40
#define	TOK_HIDDEN	41
#define	TOK_ALWAYS	42
#define	TOK_WHEN_ACTIVE	43
#define	TOK_WHEN_NONEMPTY	44

#define	TOK_TRANSIENT	50
#define	TOK_PERMANENT	51

#define	TOK_PROPERTY	100
#define	TOK_STRING	101
#define	TOK_IDENT	102
#define	TOK_GEOMETRY	103
#define	TOK_COMMENT	104

typedef	struct _ScannableRec 	*ScannablePtr;
#define	NullScannable	((ScannablePtr)NULL)

#define	scannableFile		((unsigned)(1<<0))
#define	scannableFileName	((unsigned)(1<<1))
#define	scannableString		((unsigned)(1<<2))

#define	scannableDone		((unsigned)(1<<8))
#define	scannableEnd		((unsigned)(1<<9))

#define	DEBUG_TOKENS	0x08
extern	unsigned	scanDebug;

extern	ScannablePtr	scannableOpen(/* type, source */);
extern	void		scannableClose(/* pScan */);

extern	int		scanPeek(/* pScan */);
extern	int		scanToken(/* pScan */);

extern	char		*scanUseString(/* pScan */);
extern	AppStatePtr	 scanUseState(/* pScan */);

extern	int		 scanLineNum(/* pScan */);

#endif /* PROFSCAN_H */
