
  /*\
   *  $Header: room.c,v 5.2 90/01/11 13:38:53 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define	DEBUG_VAR	roomDebug
#include "utils.h"
#include "app.h"
#include "roomstr.h"

unsigned	roomDefaultAppSlots=	ROOM_DEFAULT_APP_SLOTS;
unsigned	roomDefaultVisibility=	ROOM_DEFAULT_VISIBILITY;
Boolean		roomIconifyFirst=	True;

/***====================================================================***/
/***                       DEBUGGING FUNCTIONS                          ***/
/***====================================================================***/

char *
roomText(pRoom)
RoomPtr	pRoom;
{
    if ((pRoom==NullRoom)||(pRoom->name==NullStringToken))	
		return("<NullRoom>");
    else	return(stText(pRoom->name));
}

/***====================================================================***/

void
roomDebugPrint(pRoom)
RoomPtr	pRoom;
{
Opaque	lstate;
GenKey	key;
GenData	data;
RoomAppInfoPtr	pAppInfo;

    if (pRoom!=NullRoom) {
	uDebug("room %s (0x%x)\n",roomText(pRoom),pRoom);
	uDebugIndent(1);
	uDebug("name=		%s\n",stText(pRoom->name));
	uDebug("active=		%s\n",booleanText(pRoom->active));
	uDebug("visibility=	0x%x\n",pRoom->visibility);
	uDebug("numApps=	%d\n",pRoom->numApps);
	uDebug("numLocalApps=	%d\n",pRoom->numLocalApps);
	uDebug("priv=		0x%x\n",pRoom->priv);
	uDebug("feedback=       0x%x\n",pRoom->feedback);
	uDebug("contents:\n");
	uDebugIndent(1);
	lIterator(pRoom->contents,key,data,lstate) {
	    pAppInfo=	(RoomAppInfoPtr)data;
	    uDebug("app %s (0x%x):\n",appText(pAppInfo->pApp),pAppInfo->pApp);
	    uDebugIndent(1);
	    uDebug("local=      %s (0x%x)\n",asText(pAppInfo->pLocal),
							pAppInfo->pLocal);
	    uDebug("default=    %s (0x%x)\n",asText(pAppInfo->pDefault),
							pAppInfo->pDefault);
	    uDebug("current=    %s (0x%x)\n",asText(pAppInfo->pReal),
							pAppInfo->pReal);
	    uDebugIndent(-1);
	}
	lEndIterator(pRoom->contents,lstate);
	uDebugIndent(-2);
    }
    return;
}

/***====================================================================***/

	/*\
	 * Private debuggging function.  returns a string for
	 * a RoomAppInfo.  Never returns NullString.
	\*/

static char *
roomAppInfoText(pAppInfo)
RoomAppInfoPtr	pAppInfo;
{
    if ((pAppInfo==NullRoomAppInfo)||(pAppInfo->pApp==NullApp))
		return("<NullRoomAppInfoInfo>");
    else	return(appText(pAppInfo->pApp));
}

/***====================================================================***/
/***              FUNCTIONS TO CREATE AND DESTROY A ROOM                ***/
/***====================================================================***/

RoomPtr
roomCreate(name,priv,feedback)
StringToken	name;
Opaque		priv;
RoomFeedbackFunc feedback;
{
RoomPtr	pRoom;

    uENTRY3("roomCreate(%s,0x%x,0x%x)\n",stText(name),priv,feedback);
    if (name==NullStringToken) {
	uRETURN(NullRoom);
    }
    pRoom=	uTypedAlloc(RoomRec);
    if (pRoom!=NullRoom) {
	pRoom->name=		name;
	pRoom->active=		False;
	pRoom->visibility=	roomDefaultVisibility;
	if (_Always(pRoom))	_SetVisible(pRoom);
	pRoom->contents= lCreate(roomDefaultAppSlots,NullListSortFunc,
						      genIntegerKeyType,
						      genFreedPointerDataType);
	if (pRoom->contents==NullList) {
	    (void)roomDestroy(pRoom);
	    uRETURN(NullRoom);
	}
	pRoom->numApps=		0;
	pRoom->numLocalApps=	0;
	pRoom->priv=		(Opaque)priv;
	pRoom->feedback=	(RoomFeedbackFunc)feedback;
	_Feedback(pRoom,roomFBCreated,NullApp);
	if (_IsVisible(pRoom))
	    _Feedback(pRoom,roomFBVisible,NullApp);
    }
    uRETURN(pRoom);
}

/***====================================================================***/

void
roomDestroy(pRoom)
RoomPtr	pRoom;
{
    uENTRY1("roomDestroy(%s)\n",roomText(pRoom));
    if (pRoom!=NullRoom) {
	if (_IsVisible(pRoom)) {
	    _SetHidden(pRoom);
	    _Feedback(pRoom,roomFBHidden,NullApp);
	}
	_Feedback(pRoom,roomFBDestroyed,NullApp);
	if (pRoom->contents!=NullList)
	    (void)lDestroy(pRoom->contents);
	pRoom->name=		NullStringToken;
	pRoom->contents=	NullList;
	pRoom->priv=		(Opaque)NULL;
	pRoom->numApps=		0;
	pRoom->numLocalApps=	0;
	pRoom->feedback=	(RoomFeedbackFunc)NULL;
	(void)uFree((Opaque)pRoom);
    }
    uVOIDRETURN;
}

/***====================================================================***/
/***   FUNCTIONS TO CHANGE OR QUERY THE SET OF MANAGED APPS IN A ROOM   ***/
/***====================================================================***/

Boolean
roomManageApp(pRoom,pApp)
RoomPtr	pRoom;
AppPtr	pApp;
{
RoomAppInfoPtr	pAppInfo;

    uENTRY2("roomManageApp(%s,%s)\n",roomText(pRoom),appText(pApp));
    if ((pRoom==NullRoom)||(pApp==NullApp)) {
	uRETURN(False);
    }
    if (lLookup(pRoom->contents,(GenKey)appName(pApp),(GenData *)&pAppInfo)) {
	uRETURN(False);	/* already managed by this room */
    }
    pAppInfo=	uTypedAlloc(RoomAppInfoRec);
    if (pAppInfo==NullRoomAppInfo) {
	uRETURN(False);
    }
    pAppInfo->pApp=	pApp;
    pAppInfo->pDefault=	appGetDefault(pApp);
    pAppInfo->pReal=	appGetCurrent(pApp);
    pAppInfo->pLocal=	appGetRoomState(pApp,pRoom->name);
    if ((pAppInfo->pDefault==NullAppState)||
	(pAppInfo->pReal==NullAppState)) {
	uFree((Opaque)pAppInfo);
	uRETURN(False);
    }
    if (_HasLocalState(pRoom,pAppInfo)) {
	if (asFullySame(pAppInfo->pLocal,pAppInfo->pDefault)) {
	    if (appRemoveRoomState(pApp,pRoom->name)) {
		pAppInfo->pLocal= NullAppState;
	    }
	}
    }
    (void)lStore(pRoom->contents,(GenKey)appName(pApp),(GenData)pAppInfo);
    if (_HasLocalState(pRoom,pAppInfo)) {
	_LocalAppAdded(pRoom);
    }
    pRoom->numApps++;
    _CheckRealState(pRoom,pAppInfo);
    _Feedback(pRoom,roomFBAppAdded,pApp);
    uRETURN(True);
}

/***====================================================================***/

Boolean
roomUnmanageApp(pRoom, pApp)
RoomPtr	pRoom;
AppPtr	pApp;
{
RoomAppInfoPtr	pAppInfo;

    uENTRY2("roomUnmanageApp(%s, %s)\n",roomText(pRoom),appText(pApp));
    if ((pRoom==NullRoom)||(pApp==NullApp)) {
	uRETURN(False);
    }
    (void)lLookup(pRoom->contents,(GenKey)appName(pApp),(GenData *)&pAppInfo);
    if (pAppInfo!=NullRoomAppInfo) {
	_Feedback(pRoom,roomFBAppRemoved,pApp);
	if (_HasLocalState(pRoom,pAppInfo))  {
	    _LocalAppGone(pRoom);
	}
	pRoom->numApps--;
	(void)lRemove(pRoom->contents,(GenKey)appName(pApp));
	uRETURN(True);
    }
    uRETURN(False);
}
	
/***====================================================================***/

AppPtr
roomLookupManagedApp(pRoom, appName)
RoomPtr	 	pRoom;
StringToken	appName;
{
RoomAppInfoPtr	pAppInfo;

    uENTRY2("roomLookupManagedApp(%s,%s)\n",roomText(pRoom),stText(appName));
    if ((pRoom==NullRoom)||(appName==NullStringToken)) {
	uRETURN(NullApp);
    }
    if (lLookup(pRoom->contents,(GenKey)appName,(GenData *)&pAppInfo)) {
	uRETURN(pAppInfo->pApp);
    }
    else {
	uRETURN(NullApp);
    }
}

/***====================================================================***/
/*** FUNCTIONS TO CHANGE, CHECK, OR QUERY THE STATE OF AN APP IN A ROOM ***/
/***====================================================================***/

Boolean
roomAppRealStateChanged(pRoom, pApp, pReal)
RoomPtr		pRoom;
AppPtr		pApp;
AppStatePtr	pReal;
{
RoomAppInfoPtr	pAppInfo;
AppStatePtr	pVisible;

    uENTRY3("roomAppRealStateChanged(%s,%s,%s)\n",roomText(pRoom),appText(pApp),
								asText(pReal));
    if ((pRoom==NullRoom)||(pApp==NullApp)||(!asLegalReal(pReal))) {
	uRETURN(False);
    }
    if (!lLookup(pRoom->contents,(GenKey)appName(pApp),(GenData *)&pAppInfo)) {
	uRETURN(False);
    }
    uASSERT("roomAppRealStateChanged", pApp==pAppInfo->pApp );

    pVisible= _GetVisibleState(pRoom,pAppInfo);
    if (!asProfileCorrectWRTReal(pVisible,pReal)) {
	if (!_HasLocalState(pRoom,pAppInfo)) {
	    pAppInfo->pLocal=	appSetRoomState(pApp,pRoom->name,
							pAppInfo->pDefault,
							False);
	    if (pAppInfo->pLocal==NullAppState) {
		uRETURN(False);
	    }
	    asCopyFully(pReal,pAppInfo->pLocal);
	    _LocalAppAdded(pRoom);
	}
	else {
	    pAppInfo->pLocal=	appSetRoomState(pApp,pRoom->name,pReal,True);
	}
    }
    _CheckRealState(pRoom,pAppInfo);
    if (_HasLocalState(pRoom,pAppInfo)) {
	if (asFullySame(pAppInfo->pLocal,pAppInfo->pDefault)) {
	    if (appRemoveRoomState(pApp,pRoom->name)) {
		pAppInfo->pLocal=	NullAppState;
		_LocalAppGone(pRoom);
	    }
	    else {
		uWarning("room %s and app %s disagree about local state\n",
					roomText(pRoom),appText(pApp));
	    }
	}
    }
    uRETURN(True);
}

/***====================================================================***/

Boolean
roomChangeAppLocalState(pRoom, pApp, pProf)
RoomPtr		pRoom;
AppPtr		pApp;
AppStatePtr	pProf;
{
RoomAppInfoPtr	pAppInfo;

    uENTRY3("roomChangeAppLocalState(%s,%s,%s)\n",roomText(pRoom),appText(pApp),
								asText(pProf));
    if ((pRoom==NullRoom)||(pApp==NullApp)||(!asLegalProfile(pProf))) {
	uRETURN(False);
    }
    if (!lLookup(pRoom->contents,(GenKey)appName(pApp),(GenData *)&pAppInfo)) {
	uRETURN(False);
    }
    uASSERT("roomChangeAppLocalState", pApp==pAppInfo->pApp );

    if (!_HasLocalState(pRoom,pAppInfo)) {
	pAppInfo->pLocal=	appSetRoomState(pApp,pRoom->name,pProf,False);
	if (!_HasLocalState(pRoom,pAppInfo)) {
	    uRETURN(False);
	}
	_LocalAppAdded(pRoom);
    }
    else {
	pAppInfo->pLocal=	appSetRoomState(pApp,pRoom->name,pProf,False);
    }
    _CheckRealState(pRoom,pAppInfo);
    uRETURN(True);
}

/***====================================================================***/

Boolean
roomCheckAppStates(pRoom,pApp)
RoomPtr	pRoom;
AppPtr	pApp;
{
RoomAppInfoPtr	pAppInfo;
Boolean		hadLocal,hasLocal;

    uENTRY2("roomCheckAppStates(%s,%s)\n",roomText(pRoom),appText(pApp));
    if ((pRoom==NullRoom)||(pApp==NullApp)) {
	uRETURN(False);
    }
    if (lLookup(pRoom->contents,(GenKey)appName(pApp),(GenData *)&pAppInfo)) {
	hadLocal= 		_HasLocalState(pRoom,pAppInfo);
	pAppInfo->pLocal=	appGetRoomState(pApp,pRoom->name);
	pAppInfo->pDefault=	appGetDefault(pApp);
	pAppInfo->pReal=	appGetCurrent(pApp);
	if (_HasLocalState(pRoom,pAppInfo)) {
	    if (asFullySame(pAppInfo->pLocal,pAppInfo->pDefault)) {
		if (appRemoveRoomState(pApp,pRoom->name)) {
		    pAppInfo->pLocal= NullAppState;
		}
	    }
	}
	hasLocal=		_HasLocalState(pRoom,pAppInfo);
	if (hadLocal!=hasLocal) {
	    if (hadLocal)	{ _LocalAppGone(pRoom); }
	    else		{ _LocalAppAdded(pRoom); }
	}
	_CheckRealState(pRoom,pAppInfo);
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

AppStatePtr
roomGetVisibleState(pRoom, pApp)
RoomPtr	 	pRoom;
AppPtr		pApp;
{
RoomAppInfoPtr	pAppInfo;
AppStatePtr	pVisible;

    uENTRY2("roomGetVisibleState(%s,%s)\n",roomText(pRoom),appText(pApp));
    if ((pRoom==NullRoom)||(pApp==NullApp)) {
	uRETURN(NullAppState);
    }
    if (lLookup(pRoom->contents,(GenKey)appName(pApp),(GenData *)&pAppInfo)) {
	pVisible= _GetVisibleState(pRoom,pAppInfo);
	uRETURN(pVisible);
    }
    uRETURN(NullAppState);
}

/***====================================================================***/
/***               FUNCTIONS TO CHANGE/QUERY ROOM VISIBILITY            ***/
/***====================================================================***/

Boolean
roomIsVisible(pRoom)
RoomPtr	pRoom;
{
    uENTRY1("roomIsVisible(%s)\n",roomText(pRoom));
    uRETURN(_IsVisible(pRoom));
}

/***====================================================================***/

Boolean
roomSetVisibility(pRoom,visibility,check)
RoomPtr		pRoom;
unsigned	visibility;
Boolean		check;
{
Boolean		wasVisible;

    uENTRY3("roomSetVisibility(%s,0x%x,%s)\n",roomText(pRoom),visibility,
							booleanText(check));
    if (pRoom!=NullRoom) {
	wasVisible=		_IsVisible(pRoom);
	pRoom->visibility=	visibility;
	if (check&&(_Always(pRoom)||_WhenNonEmpty(pRoom)||_WhenActive(pRoom))) {
	    if (_Always(pRoom))	_SetVisible(pRoom);
	    else {
		_SetHidden(pRoom);
		if (_WhenNonEmpty(pRoom)&&_HasLocalApps(pRoom)) {
		    _SetVisible(pRoom);
		}
		else if (_WhenActive(pRoom)&&_IsActive(pRoom)) {
		    _SetVisible(pRoom);
		}
	    }
	}
	if (_IsVisible(pRoom)!=wasVisible) {
	    _Feedback(pRoom,(wasVisible?roomFBHidden:roomFBVisible),NullApp);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
roomCheckVisibility(pRoom)
RoomPtr		pRoom;
{
Boolean	shouldBeVisible;

    uENTRY1("roomCheckVisibility(%s)\n",pRoom);
    if (pRoom!=NullRoom) {
	if (_Always(pRoom)||_WhenActive(pRoom)||_WhenNonEmpty(pRoom)) {
	    if (_Always(pRoom))	shouldBeVisible=	True;
	    else {
		shouldBeVisible= ((_WhenActive(pRoom)&&_IsActive(pRoom))||
				  (_WhenNonEmpty(pRoom)&&_HasLocalApps(pRoom)));
	    }
	    if (shouldBeVisible&&_IsHidden(pRoom)) {
		_SetVisible(pRoom);
		_Feedback(pRoom,roomFBVisible,NullApp);
	    }
	    else if ((!shouldBeVisible)&&_IsVisible(pRoom)) {
		_SetHidden(pRoom);
		_Feedback(pRoom,roomFBHidden,NullApp);
	    }
	}
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

unsigned
roomGetVisibility(pRoom)
RoomPtr		pRoom;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"roomGetVisibility(%s)\n",roomText(pRoom));
    if (pRoom!=NullRoom) {
	uFLAG_RETURN(pRoom->visibility);
    }
    uFLAG_RETURN(roomHidden);
}

/***====================================================================***/
/***           FUNCTIONS TO ACTIVATE OR DEACTIVATE A ROOM               ***/
/***====================================================================***/

Boolean
roomActivate(pRoom)
RoomPtr	pRoom;
{
Opaque		lstate;
GenKey		key;
GenData		data;
RoomAppInfoPtr	pAppInfo;
AppStatePtr	pState;
WinState	firstPassState;

    uENTRY1("roomActivate(%s)\n",roomText(pRoom));
    if (pRoom==NullRoom) {
	uRETURN(False);
    }
    if (!_IsActive(pRoom)) {
	pRoom->active=	True;
	if (_WhenActive(pRoom)&&(!_IsVisible(pRoom))) {
	    _SetVisible(pRoom);
	    _Feedback(pRoom,roomFBVisible,NullApp);
	}
	_Feedback(pRoom,roomFBActivated,NullApp);

	if (roomIconifyFirst)	firstPassState=	asIconic;
	else			firstPassState=	asNormal;

	lIterator(pRoom->contents,key,data,lstate) {
	    pAppInfo=	(RoomAppInfoPtr)data;
	    pState=	_GetVisibleState(pRoom,pAppInfo);
	    if (asWinState(pState)==firstPassState) {
		_CheckRealState(pRoom,pAppInfo);
	    }
	}
	lEndIterator(pRoom->contents,lstate);
	lIterator(pRoom->contents,key,data,lstate) {
	    pAppInfo=	(RoomAppInfoPtr)data;
	    pState=	_GetVisibleState(pRoom,pAppInfo);
	    if (asWinState(pState)!=firstPassState) {
		_CheckRealState(pRoom,pAppInfo);
	    }
	}
	lEndIterator(pRoom->contents,lstate);
    }
    uRETURN(True);
}

/***====================================================================***/

Boolean
roomDeactivate(pRoom)
RoomPtr	pRoom;
{
    uENTRY1("roomDeactivate(%s)\n",roomText(pRoom));
    if (pRoom!=NullRoom) {
	if (_IsActive(pRoom)) {
	    pRoom->active= False;
	    _Feedback(pRoom,roomFBDeactivated,NullApp);
	    if ((!_Always(pRoom))&&_WhenActive(pRoom)&&_IsVisible(pRoom)) {
		if ((!_WhenNonEmpty(pRoom))||_NoLocalApps(pRoom)) {
		    _SetHidden(pRoom);
		    _Feedback(pRoom,roomFBHidden,NullApp);
		}
	    }
	}
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/
/***                   MISCELLANEOUS ACCESS FUNCTIONS                   ***/
/***====================================================================***/

StringToken
roomName(pRoom)
RoomPtr	pRoom;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"roomName(%s)\n",roomText(pRoom));
    if (pRoom==NullRoom) {
	uFLAG_RETURN(NullStringToken);
    }
    uFLAG_RETURN(pRoom->name);
}

/***====================================================================***/

unsigned
roomNumApps(pRoom)
RoomPtr	pRoom;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"roomNumApps(%s)\n",roomText(pRoom));
    uFLAG_RETURN(pRoom->numApps);
}

/***====================================================================***/

unsigned
roomNumLocalApps(pRoom)
RoomPtr	pRoom;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"roomNumLocalApps(%s)\n",roomText(pRoom));
    uFLAG_RETURN(pRoom->numLocalApps);
}

/***====================================================================***/

void
roomSetFeedback(pRoom,func)
RoomPtr			pRoom;
RoomFeedbackFunc	func;
{
    uFLAG_ENTRY2(LOW_ENTRY_BIT,"roomSetFeedback(%s,0x%x)\n",roomText(pRoom),
								func);
    if (pRoom!=NullRoom) {
	pRoom->feedback=	func;
    }
    uFLAG_VOIDRETURN;
}

/***====================================================================***/

RoomFeedbackFunc
roomGetFeedback(pRoom)
RoomPtr	pRoom;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"roomSetFeedback(%s)\n",roomText(pRoom));
    if (pRoom!=NullRoom) {
	uFLAG_RETURN(pRoom->feedback);
    }
    uFLAG_RETURN((RoomFeedbackFunc)NULL);
}

/***====================================================================***/

void
roomSetPrivate(pRoom,priv)
RoomPtr	pRoom;
Opaque	priv;
{

    uFLAG_ENTRY2(LOW_ENTRY_BIT,"roomSetPrivate(%s,0x%x)\n",roomText(pRoom),
								priv);
    if (pRoom!=NullRoom) {
	pRoom->priv=	priv;
    }
    uFLAG_VOIDRETURN;
}

/***====================================================================***/

Opaque
roomGetPrivate(pRoom)
RoomPtr	pRoom;
{
    uFLAG_ENTRY1(LOW_ENTRY_BIT,"roomGetPriv(%s)\n",roomText(pRoom));
    if (pRoom!=NullRoom) {
	uFLAG_RETURN(pRoom->priv);
    }
    uFLAG_RETURN((Opaque)NULL);
}

/***====================================================================***/

struct roomIterArg {
    Boolean		localOnly;
    RoomPtr		pRoom;
    RoomIterFunc	pFunc;
    Opaque		arg;
};

static Boolean
_roomIterator(keyIn,dataIn,argIn)
GenKey		keyIn;
GenData	dataIn;
Opaque		argIn;
{
StringToken	 key=		(StringToken)keyIn;
RoomAppInfoPtr	 pAppInfo=	(RoomAppInfoPtr)dataIn;
struct roomIterArg	*pArg=	(struct roomIterArg *)argIn;

    uENTRY3("_roomIterator(%s,%s,0x%x)\n",stText(key),
					  roomAppInfoText(pAppInfo),pArg);
    if ((!pArg->localOnly)||(_HasLocalState(pArg->pRoom,pAppInfo))) {
	uRETURN((*pArg->pFunc)(pAppInfo->pApp,pAppInfo->pLocal,
					 pAppInfo->pDefault,
					 pAppInfo->pReal,pArg->arg));
    }
    uRETURN(True);
}

void
roomIterateApps(pRoom,pFunc,arg)
RoomPtr		pRoom;
RoomIterFunc	pFunc;
Opaque		arg;
{
struct	roomIterArg	iArg;

    uENTRY3("roomIterateApps(%s,0x%x,0x%x)\n",roomText(pRoom),pFunc,arg);
    if ((pRoom!=NullRoom)&&(pFunc!=NULL)) {
	iArg.localOnly=	False;
	iArg.pRoom=	pRoom;
	iArg.pFunc=	pFunc;
	iArg.arg=	arg;
	(void)lIterate(pRoom->contents,_roomIterator,(Opaque)&iArg);
    }
    uVOIDRETURN;
}

void
roomIterateLocalApps(pRoom,pFunc,arg)
RoomPtr		pRoom;
RoomIterFunc	pFunc;
Opaque		arg;
{
struct	roomIterArg	iArg;

    uENTRY3("roomIterateApps(%s,0x%x,0x%x)\n",roomText(pRoom),pFunc,arg);
    if ((pRoom!=NullRoom)&&(pFunc!=NULL)) {
	iArg.localOnly=	True;
	iArg.pRoom=	pRoom;
	iArg.pFunc=	pFunc;
	iArg.arg=	arg;
	(void)lIterate(pRoom->contents,_roomIterator,(Opaque)&iArg);
    }
    uVOIDRETURN;
}
