
  /*\
   *  $Header: rooms.c,v 5.0 90/01/10 06:54:24 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define	DEBUG_VAR	roomsDebug
#include "utils.h"
#include "strtbl.h"
#include "hash.h"
#include "apps.h"
#include "rooms.h"
#include "profile.h"

static	HashTablePtr		pAllRooms=	NullHashTable;
static	RoomPtr			CurrentRoom=	NullRoom;
static	RoomFeedbackFunc	DefaultFeedback=	NULL;
static	RoomFinderFunc		DefaultFinder=		NULL;
static	int			nRooms=	0;

/***====================================================================***/

void
DebugPrintRooms(name)
char	*name;
{
RoomPtr	pRoom;
Opaque	rstate;

    uENTRY1("DebugPrintRooms(%s)\n",uStringText(name));
    if (pAllRooms!=NullHashTable) {
	if (name==NullString) {
	    uDebug("All rooms:\n");
	    uDebugIndent(1);
	    RoomsIterator(pRoom,rstate) {
	 	roomDebugPrint(pRoom);
	    }
	    RoomsEndIterator(rstate);
	    uDebugIndent(-1);
	}
	else {
	    StringToken	token;
	    if (stGetTokenIfExists(name,&token)) {
		RoomPtr  pRoom= (RoomPtr)htLookup(pAllRooms,(GenKey)token);
		if (pRoom!=NullRoom) {
		    roomDebugPrint(pRoom);
		}
		else uDebug("Room %s doesn't exist\n",stText(token));
	    }
	    else uDebug("token %s undefined\n",uStringText(name));
	}
    }
    else uDebug("no rooms defined\n");
    uVOIDRETURN;
}

/***====================================================================***/

RoomPtr
LookupRoom(name)
StringToken	name;
{
RoomPtr	pRoom=	NullRoom;

    uENTRY1("LookupRoom(%s)\n",stText(name));
    if (pAllRooms!=NullHashTable) {
	pRoom= (RoomPtr)htLookup(pAllRooms,(GenKey)name);
    }
    uRETURN(pRoom);
}

/***====================================================================***/

int
NumberOfRooms()
{
   uENTRY("NumberOfRooms()\n");
   uRETURN(nRooms);
}

/***====================================================================***/

RoomPtr
CreateRoom(name)
StringToken		name;
{
RoomPtr		pRoom=	NullRoom;

    uENTRY1("CreateRoom(%s)\n",stText(name));
    if (pAllRooms!=NullHashTable) {
	pRoom=	(RoomPtr)htLookup(pAllRooms,(GenKey)name);
	if (pRoom!=NullRoom) {
	    uRETURN(NullRoom);
	}
    }
    else {
	pAllRooms= htCreate((unsigned)13,genIntegerKeyType,genPointerDataType);
	if (pAllRooms==NullHashTable) {
	    uRETURN(NullRoom);
	}
    }

    pRoom= roomCreate(name,(Opaque)NULL,DefaultFeedback);
    if (pRoom!=NullRoom) { /* realize all active apps in the new room */
	(void)htAdd(pAllRooms,(GenKey)name,(GenData)pRoom);
	(void)ActiveAppsIterate((AppsIterFunc)roomManageApp,(Opaque)pRoom);
	(void)profileForRoom(name);
    }
    nRooms++;
    uRETURN(pRoom);
}

/***====================================================================***/

Boolean
DestroyRoom(pRoom)
RoomPtr	pRoom;
{
RoomPtr		pStoredRoom;

    uENTRY1("DestroyRoom(%s)\n",roomText(pRoom));
    if ((pRoom!=NullRoom)&&(pAllRooms!=NullHashTable)) {
	if (pRoom==CurrentRoom) {
	    CurrentRoom=	NullRoom;
	}
	pStoredRoom=	(RoomPtr)htRemove(pAllRooms,(GenKey)roomName(pRoom));
	if (pStoredRoom==pRoom) {
	    roomDestroy(pRoom);
	    nRooms--;
	    uRETURN(True);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

typedef struct _RoomsIterArgRec {
	RoomsIterFunc	func;
	Opaque		arg;
} RoomsIterArgRec,*RoomsIterArgPtr;

static Boolean
_RoomsIterator(row,keyIn,dataIn,argIn)
int		row;
GenKey		keyIn;
GenData	dataIn;
Opaque		argIn;
{
StringToken	key=	(StringToken)keyIn;
RoomPtr		pRoom=	(RoomPtr)dataIn;
RoomsIterArgPtr	pArg=	(RoomsIterArgPtr)argIn;
Boolean	ok;

    uENTRY4("_RoomsIterator(%d,%s,%s,0x%x)\n",row,stText(key),roomText(pRoom),
								pArg);
    ok= (*pArg->func)(pRoom,pArg->arg);
    uRETURN(ok);
}

void
RoomsIterate(func,arg)
RoomsIterFunc	func;
Opaque		arg;
{
RoomsIterArgRec	iarg;

    uENTRY2("RoomsIterate(0x%x,0x%x)\n",func,arg);
    if ((func==NULL)||(pAllRooms==NullHashTable)) {
	uVOIDRETURN;
    }
    iarg.func=	func;
    iarg.arg=	arg;
    (void)htIterate(pAllRooms,_RoomsIterator,(Opaque)&iarg);
    uVOIDRETURN;
}

/***====================================================================***/

Boolean
SwitchToRoom(pRoom)
RoomPtr	pRoom;
{
    uENTRY1("SwitchToRoom(%s)\n",roomText(pRoom));
    if (pRoom==CurrentRoom) {
	uRETURN(True);
    }
    if (CurrentRoom!=NullRoom) {
	roomDeactivate(CurrentRoom);
    }
    CurrentRoom=	pRoom;
    if (pRoom!=NullRoom) {
	uRETURN(roomActivate(pRoom));
    }
    uRETURN(True);
}

/***====================================================================***/

RoomPtr
GetCurrentRoom()
{
RoomPtr	pRoom;

    uENTRY("GetCurrentRoom()\n");
    if (((CurrentRoom==NullRoom)||(!roomIsVisible(CurrentRoom)))&&
	 (DefaultFinder!=NULL)) {
	pRoom= (*DefaultFinder)();
	(void)SwitchToRoom(pRoom);
    }
    uRETURN(CurrentRoom);
}

/***====================================================================***/

void
SetDefaultRoomFinder(pFinder)
RoomFinderFunc	pFinder;
{
    uENTRY1("SetDefaultRoomFinder(0x%x)\n",pFinder);
    DefaultFinder=	pFinder;
    uVOIDRETURN;
}

/***====================================================================***/

void
SetDefaultRoomFeedback(pFunc)
RoomFeedbackFunc	pFunc;
{
    uENTRY1("SetDefaultRoomFeedback(0x%x)\n",pFunc);
    DefaultFeedback=	pFunc;
    uVOIDRETURN;
}

/***====================================================================***/

Boolean	
_RoomsInitIter(pState)
Opaque		*pState;
{
Boolean	ok=	False;

    uENTRY1("_RoomsInitIter(0x%x)\n",pState);
    if (pAllRooms!=NullHashTable) {
	ok= _htInitIter(pAllRooms,pState);
    }
    else *pState=	(Opaque)NULL;
    uRETURN(ok);
}

/***====================================================================***/

Boolean	
_RoomsIterNext(pState, ppRoom)
Opaque		 pState;
RoomPtr		*ppRoom;
{
Boolean	ok= False;
GenKey	key;
GenData	data;

    uENTRY2("_RoomsIterNext(0x%x,0x%x)\n",pState,ppRoom);
    if ((pAllRooms!=NullHashTable)&&(pState!=(Opaque)NULL)) {
	ok=	_htIterNext(pAllRooms,pState,&key,&data);
	if (ok) {
	    *ppRoom= (RoomPtr)data;
	}
	else *ppRoom=	NullRoom;
    }
    uRETURN(ok);
}

/***====================================================================***/

Boolean	
_RoomsIterNextVisible(pState, ppRoom)
Opaque		 pState;
RoomPtr		*ppRoom;
{
Boolean	ok= False;
GenKey	key;
GenData	data;

    uENTRY2("_RoomsIterNextVisible(0x%x,0x%x)\n",pState,ppRoom);
    if ((pAllRooms!=NullHashTable)&&(pState!=(Opaque)NULL)) {
	do {
	    ok=	_htIterNext(pAllRooms,pState,&key,&data);
	    if (ok) {
		*ppRoom= (RoomPtr)data;
	    }
	    else *ppRoom=	NullRoom;
	} while ((ok)&&(!roomIsVisible(*ppRoom)));
    }
    uRETURN(ok);
}

/***====================================================================***/

void
RoomsEndIterator(state)
Opaque	state;
{
    uENTRY1("RoomsEndIterator(0x%x)\n",state);
    if (state!=(Opaque)NULL)
	htEndIterator(pAllRooms,state);
    uVOIDRETURN;
}
