
  /*\
   *  $Header: list.c,v 5.0 90/01/10 06:50:54 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define	DEBUG_VAR	listDebug
#include "utils.h"
#include "liststr.h"


#define	_lCompare(pL,k1,k2)	((pL)->pSort?(*(pL)->pSort)(k1,k2):\
				     _genCompareKey((pL)->keyType,k1,k2))

/***====================================================================***/

ListPtr
lCreate(size, sortFunc, keyType, dataType)
unsigned		size;
CompareKeyFunc		sortFunc;
GenKeyDescPtr		keyType;
GenDataDescPtr		dataType;
{
ListPtr	pList;

    uENTRY4("lCreate(%d,0x%x,0x%x,0x%x)\n",size,sortFunc,keyType,dataType);
    if (size<1) {
	uRETURN(NullList);
    }
    if (keyType==NullGenKeyDesc)	keyType=	genDefaultKeyType;
    if (dataType==NullGenDataDesc)	dataType=	genDefaultDataType;
    pList=	uTypedAlloc(ListRec);
    if (pList!=NullList) {
	pList->pCells=	uTypedCalloc(size,ListCellRec);
	if (pList->pCells!=NullListCell) {
	    pList->nStored=	0;
	    pList->nCells=	size;
	    pList->pSort=	sortFunc;
	    pList->keyType=	keyType;
	    pList->dataType=	dataType;
	}
	else {
	    uFree((Opaque)pList);
	    uRETURN(NullList);
	}
    }
    uRETURN(pList);
}

/***====================================================================***/

void
lDestroy(pList)
ListPtr	pList;
{
int	i;
    uENTRY1("lDestroy(0x%x)\n",pList);
    if (pList!=NullList) {
	if (pList->pCells!=NullListCell) {
	    for (i=0;i<pList->nStored;i++) {
		_genRemoveKey(pList->keyType,pList->pCells[i].key);
		_genRemoveData(pList->dataType,pList->pCells[i].data);
	    }
	    uFree((Opaque)pList->pCells);
	    pList->pCells=0;
	}
	pList->nStored=		pList->nCells=	0;
	pList->pSort=		NullListSortFunc;
	pList->keyType=		NullGenKeyDesc;
	pList->dataType=	NullGenDataDesc;
	uFree((Opaque)pList);
    }
    uVOIDRETURN;
}

/***====================================================================***/

int
lNItems(pList)
ListPtr	pList;
{
    uENTRY1("lNItems(0x%x)\n",pList);
    if (pList!=NullList) {
	uRETURN(pList->nStored);
    }
    uRETURN(0);
}

/***====================================================================***/

static Boolean
_lGrowCells(pList)
ListPtr	pList;
{
    uENTRY1("_lGrowCells(0x%x)\n",pList);
    if ((pList!=NullList)&&(pList->nCells>0)) {
	pList->nCells*= 2;
	pList->pCells=	(ListCellPtr)uRealloc((Opaque)pList->pCells,
				 (unsigned)pList->nCells*sizeof(ListCellRec));
	if (pList->pCells!=NullListCell) {
	    uRETURN(True);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

static Boolean
_lDeleteCell(pList,where)
ListPtr		pList;
unsigned	where;
{
unsigned	i;

    uENTRY2("_lDeleteCell(0x%x,%d)\n",pList,where);
    if ((pList!=NullList)&&(where<pList->nStored)) {
	_genRemoveKey(pList->keyType,pList->pCells[where].key);
	_genRemoveData(pList->dataType,pList->pCells[where].data);
	for (i=where;i<pList->nStored;i++) {
	    pList->pCells[i]= pList->pCells[i+1];
	}
	pList->nStored--;
	uRETURN(True);
    }
    uRETURN(False);;
}

/***====================================================================***/

static Boolean
_lInsertCell(pList,where,key,data)
ListPtr		pList;
unsigned	where;
GenKey		key;
GenData		data;
{
unsigned	i;

    uENTRY4("_lInsertCell(0x%x,%d,0x%x,0x%x)\n",pList,where,key,data);
    if ((pList!=NullList)&&(where<=pList->nStored)) {
	if ((pList->nStored==pList->nCells)&&(!_lGrowCells(pList))) {
	    uInternalError("REALLOC FAILED List 0x%x may be corrupted\n",
								pList);
	    uAction("restart ASAP!!\n");
	    uRETURN(False);
	}
	    
	for (i=pList->nStored;i>where;i--) {
	    pList->pCells[i]= pList->pCells[i-1];
	}
	pList->pCells[where].key=	key;
	pList->pCells[where].data=	data;
	pList->nStored++;
	uRETURN(True);
    }
    uRETURN(False);;
}

/***====================================================================***/

static Comparison
_lFindSortedSpot(pList,key,pIndex)
ListPtr		pList;
GenKey		key;
unsigned	*pIndex;
{
int		i;
Comparison	compare= Greater;

    uENTRY3("_lFindSortedSpot(0x%x,0x%x,0x%x)\n",pList,key,pIndex);
    if (pList!=NullList) {
	for (i=0;i<pList->nStored;i++) {
	    compare= _lCompare(pList,pList->pCells[i].key,key);
	    if (compare!=Less) {
		*pIndex=	i;
		uRETURN(compare);
	    }
	}
	*pIndex=	i;
	uRETURN(Greater);
    }
    uRETURN(CannotCompare);
}

/***====================================================================***/

Boolean
lStore(pList, key, data)
ListPtr		pList;
GenKey		key;
GenData		data;
{
unsigned	index;
Comparison	compare;

    uENTRY3("lStore(0x%x,0x%x,0x%x)\n",pList,key,data);
    if (pList!=NullList) {
	compare= _lFindSortedSpot(pList,key,&index);
	if (compare==Greater) {
	    _lInsertCell(pList,index,key,data);
	    uRETURN(True);
	}
	else if (compare==Equal) {
	    pList->pCells[index].data=	_genReplaceData(pList->dataType,data,
						&pList->pCells[index].data);
	    uRETURN(True);
	}
    }
    uInternalError("storing 0x%x into name list 0x%x\n",data,pList);
    uRETURN(False);
}

/***====================================================================***/

Boolean
lRemove(pList, key)
ListPtr		pList;
GenKey		key;
{
unsigned	index;
Comparison	compare;

    uENTRY2("lRemove(0x%x,0x%x)\n",pList,key);
    if (pList!=NullList) {
	compare= _lFindSortedSpot(pList,key,&index);
	if (compare==Equal) {
	    _lDeleteCell(pList,index);
	    uRETURN(True);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
lRemoveAll(pList)
ListPtr		pList;
{
unsigned	i;

    uENTRY1("lRemoveAll(0x%x)\n",pList);
    if (pList!=NullList) {
	for (i=0;i<pList->nStored;i++) {
	    _genRemoveKey(pList->keyType,pList->pCells[i].key);
	    _genRemoveData(pList->dataType,pList->pCells[i].data);
	}
	pList->nStored=	0;
	uRETURN(True);
    }
    uRETURN(False);
}
/***====================================================================***/

Boolean
lLookup(pList, key, pData)
ListPtr	 	 pList;
GenKey		 key;
GenData		*pData;
{
unsigned	index;
Comparison	compare;

    uENTRY2("lLookup(0x%x,0x%x)\n",pList,key);
    if (pList!=NullList) {
	compare= _lFindSortedSpot(pList,key,&index);
	if (compare==Equal) {
	    if (pData!=(GenData *)NULL)
		*pData=	pList->pCells[index].data;
	    uRETURN(True);
	}
	else if (pData!=(GenData *)NULL)
	    *pData=	_genNullData(pList->dataType);
    }
    uRETURN(False);
}

/***====================================================================***/

void
lIterate(pList, pFunc, arg)
ListPtr		pList;
GenIterFunc	pFunc;
Opaque		arg;
{
int	i;

    uENTRY3("lIterate(0x%x,0x%x,0x%x)\n",pList,pFunc,arg);
    if ((pList!=NullList)&&(pFunc!=(GenIterFunc)NULL)) {
	for (i=0;i<pList->nStored;i++) {
	    if (!(*pFunc)(pList->pCells[i].key,pList->pCells[i].data,arg))
		break;
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

Boolean
_lInitIter(pList, pState)
ListPtr		  pList;
Opaque		 *pState;
{
    uENTRY2("_lInitIter(0x%x,0x%x)\n",pList,pState);
    *pState=	(Opaque)0;
    if ((pList!=NullList)&&(pList->nStored>0)) {
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
_lIterNext(pList, pState, pKey, pData)
ListPtr		  pList;
Opaque		 *pState;
GenKey 		 *pKey;
GenData		 *pData;
{
unsigned	index= (unsigned)*pState;

    uENTRY4("_lIterNext(0x%x,0x%x,0x%x,0x%x)\n",pList,pState,pKey,pData);
    if ((pList!=NullList)&&(index<pList->nStored)) {
	*pKey=	pList->pCells[index].key;
	*pData=	pList->pCells[index].data;
	index++;
	*pState=	(Opaque)index;
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

void
lEndIterator(pList, state)
ListPtr	pList;
Opaque	state;
{
    uENTRY2("lEndIterator(0x%x,%d)\n",pList,state);
    uVOIDRETURN;
}
