
  /*\
   *  $Header: regex.c,v 5.0 90/01/10 06:51:07 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define	DEBUG_VAR	regexDebug

#include "utils.h"
#include "regex.h"

extern	Boolean	rxMatch();

/***====================================================================***/

static	Boolean
rxStarMatch(ptrn,ptrnLength,str,indices,matchFunc,arg)
char	*ptrn;
int	ptrnLength;
char	*str;
char	**indices;
Boolean	(*matchFunc)();
Opaque	arg;
{
int	i;

    uENTRY5("rxStarMatch(%s,%d,%s,0x%x,0x%x)\n",ptrn,ptrnLength,str,indices,
								matchFunc);
    if (((*str)!=NUL)&&(*matchFunc)(ptrn,str,arg)&&
	(rxStarMatch(ptrn,ptrnLength,str+1,indices,matchFunc,arg))) {
	for (i=0;i<ptrnLength;i++) {
	    indices[i]=	str;
	}
	uRETURN(True);
    }
    else {
	for (i=0;i<ptrnLength;i++) {
	    indices[i]=	NULL;
	}
	uRETURN(rxMatch(ptrn+ptrnLength,str,indices+ptrnLength));
    }
}

/***====================================================================***/

static	Boolean
rxQuestionMatch(ptrn,ptrnLength,str,indices,matchFunc,arg)
char	*ptrn;
int	ptrnLength;
char	*str;
char	**indices;
Boolean	(*matchFunc)();
Opaque	arg;
{
int	i;

    uENTRY5("rxQuestionMatch(%s,%d,%s,0x%x,0x%x)\n",ptrn,ptrnLength,str,indices,
								matchFunc);
    if ((*matchFunc)(ptrn,str,arg)&&
	 	(rxMatch(ptrn+ptrnLength,str+1,indices+ptrnLength)))	{
	for (i=0;i<ptrnLength;i++) {
	    indices[i]=	str;
	}
	uRETURN(True);
    }
    else {
	for (i=0;i<ptrnLength;i++) {
	    indices[i]=	NULL;
	}
	uRETURN(rxMatch(ptrn+ptrnLength,str,indices+ptrnLength));
    }
}

/***====================================================================***/

/* ARGSUSED */
static Boolean
rxExactMatch(ptrn,str,arg)
char	*ptrn;
char	*str;
Opaque	arg;
{
    uENTRY3("rxExactMatch(%s,%s,0x%x)\n",ptrn,str,arg);
    if (ptrn[0]==str[0])
	uRETURN(True);
    uRETURN(False);
}

/***====================================================================***/

/* ARGSUSED */
static Boolean
rxDotMatch(ptrn,str,arg)
char	*ptrn;
char	*str;
Opaque	 arg;
{
    uENTRY3("rxDotMatch(%s,%s,0x%x)\n",ptrn,str,arg);
    if ((ptrn[0]=='.')&&(str[0]!=NUL))
	uRETURN(True);
    uRETURN(False);
}

/***====================================================================***/

/* ARGSUSED */
static Boolean
rxSlashMatch(ptrn,str,arg)
char	*ptrn;
char	*str;
Opaque	 arg;
{
    uENTRY3("rxSlashMatch(%s,%s,0x%x)\n",ptrn,str,arg);
    if ((ptrn[0]=='\\')&&(ptrn[1]==str[0]))
	uRETURN(True);
    uRETURN(False);
}

/***====================================================================***/

/* ARGSUSED */
static	Boolean
rxBracketMatchOnce(ptrn,str,bracketChars)
char	*ptrn;
char	*str;
char	bracketChars[256];
{
    uENTRY3("rxBracketMatchOnce(%s,%s,0x%x)\n",ptrn,str,bracketChars);
    uRETURN(bracketChars[*str]!=False);
}

/***====================================================================***/

static	Boolean
rxBracketMatch(ptrn,str,indices)
char	*ptrn;
char	*str;
char	**indices;
{
char	matchChars[256],*ptrnStart;
Boolean	defaultMatch=	False;
int	i;

    uENTRY3("rxBracketMatch(%s,%s,0x%x)\n",ptrn,str,indices);
    if (*ptrn!='[')	uRETURN(False);
    ptrnStart=	ptrn++;

    if		(*ptrn=='^')	{ defaultMatch=	True; ptrn++; }

    matchChars[0]=	False;
    for (i=1;i<256;i++) {
	matchChars[i]=	defaultMatch;
    }
    while ((*ptrn)&&(*ptrn!=']')) {
	if (*ptrn=='\\') {
	    ptrn++;
	    if (*ptrn==NUL)	break;
	}
	matchChars[*ptrn]=	!defaultMatch;
	ptrn++;
    }
    if (*ptrn!=']')	uRETURN(False);
    ptrn++;
    if (*ptrn=='*') {
	ptrn++;
	uRETURN(rxStarMatch(ptrnStart,ptrn-ptrnStart,str,indices,
					rxBracketMatchOnce,(Opaque)matchChars));
    }
    else if (*ptrn=='?') {
	ptrn++;
	uRETURN(rxQuestionMatch(ptrnStart,ptrn-ptrnStart,str,indices,
					rxBracketMatchOnce,(Opaque)matchChars));
    }
    if (matchChars[*str]) {
	int i;
	for (i=ptrn-ptrnStart;i>0;i--) {
	    *indices++=	str;
	}
	uRETURN(rxMatch(ptrn,str+1,indices));
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
rxMatch(ptrn,str,indices)
char	*ptrn;
char	*str;
char	**indices;
{

    uENTRY3("rxMatch(%s,%s,0x%x)\n",ptrn,str,indices);
    if (ptrn[0]==NUL) {
	uRETURN((Boolean)str[0]==NUL);
    }

    if (ptrn[0]=='[')
	uRETURN(rxBracketMatch(ptrn,str,indices));
    if (ptrn[0]=='.') {
	if (ptrn[1]=='*') {
	    uRETURN(rxStarMatch(ptrn,2,str,indices,rxDotMatch,NULL));
	}
	else if (ptrn[1]=='?') {
	    uRETURN(rxQuestionMatch(ptrn,2,str,indices,rxDotMatch,NULL));
	}
	else if (str[0]!=NUL) {
	    indices[0]=	str;
	    uRETURN(rxMatch(ptrn+1,str+1,indices+1));
	}
	else uRETURN(False);
    }
    if (ptrn[0]=='\\') {
	indices[0]= str;
	if ((ptrn[1]=='(')||(ptrn[1]==')')) {
	    indices[1]=	str;
	    uRETURN(rxMatch(ptrn+2,str,indices+2));
	}
	if (ptrn[1]==NUL) {
	    uRETURN(str[0]==NUL);
	}
	if  (ptrn[2]=='*') {
	    uRETURN(rxStarMatch(ptrn,3,str,indices,rxSlashMatch,NULL));
	}
	else if  (ptrn[2]=='?') {
	    uRETURN(rxQuestionMatch(ptrn,3,str,indices,rxSlashMatch,NULL));
	}
	if (ptrn[1]==str[0]) {
	    indices[1]= str;
	    uRETURN(rxMatch(ptrn+2,str+1,indices+2));
	}
    }
    if (ptrn[1]=='*') 
	uRETURN(rxStarMatch(ptrn,2,str,indices,rxExactMatch,NULL));
    if (ptrn[1]=='?') 
	uRETURN(rxQuestionMatch(ptrn,2,str,indices,rxExactMatch,NULL));
    if (ptrn[0]==str[0]) {
	indices[0]= str;
	uRETURN(rxMatch(ptrn+1,str+1,indices+1));
    }
    uRETURN(False);
}

/***====================================================================***/

static void
rxFindMatches(wholePtrn,indices,pM)
char		 *wholePtrn;
char		**indices;
RXMatchesPtr	  pM;
{
char	*ptrn;
int	nMatches,openSave;

    uENTRY3("rxFindMatches(%s,0x%x,0x%x)\n",wholePtrn,indices,pM);
    nMatches=	0;
    ptrn=	wholePtrn;
    while (*ptrn!=NUL) {
	if ((*ptrn=='\\')&&(*(ptrn+1)=='('))	{ nMatches++; ptrn++; }
	ptrn++;
    }
    pM->nMatches=	0;
    if (nMatches<1)
	uVOIDRETURN;
    if (nMatches>REGEX_MAX_MATCHES)
	nMatches= REGEX_MAX_MATCHES;

    ptrn= 	wholePtrn;
    openSave=	False;
    while ((*ptrn!=NUL)&&(pM->nMatches<nMatches)) {
	if ((*ptrn=='\\')&&(*(ptrn+1)=='(')) {
	    if (openSave) 
		goto BAILOUT;
	    openSave=	True;
	    pM->matchStrings[pM->nMatches]=	indices[ptrn-wholePtrn];
	}
	if ((*ptrn=='\\')&&(*(ptrn+1)==')')) {
	    if (!openSave)
		goto BAILOUT;
	    openSave=	False;
	    pM->matchLengths[pM->nMatches]=	indices[ptrn-wholePtrn]-
						pM->matchStrings[pM->nMatches];
	    pM->nMatches++;
	}
	ptrn++;
    }
    uVOIDRETURN;
BAILOUT:
    pM->nMatches=	0;
    uVOIDRETURN;
}

/***====================================================================***/

Boolean
regexMatch(wholePtrn,wholeStr,pMatches)
char		*wholePtrn;
char		*wholeStr;
RXMatchesPtr	 pMatches;
{
char		**matchIndices;

    uENTRY3("regexMatch(%s,%s,0x%x)\n",wholePtrn,wholeStr,pMatches);
    matchIndices=	(char **)uTmpAlloc(strlen(wholePtrn)*sizeof(char *));
    if (matchIndices==NULL)
	uRETURN(False);
    if (!rxMatch(wholePtrn,wholeStr,matchIndices)) {
	uTmpFree(matchIndices);
	uRETURN(False);
    }

    if (pMatches!=NULL) {
	rxFindMatches(wholePtrn,matchIndices,pMatches);
    }
    uTmpFree(matchIndices);
    uRETURN(True);
}
