
  /*\
   *  $Header: xrApp.c,v 5.2 90/01/11 14:08:41 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>

#define	DEBUG_VAR	xrAppDebug
#include "utils.h"
#include "hash.h"
#include "regex.h"

#include "appstate.h"
#include "apps.h"
#include "nameprof.h"

#include "xrDebug.h"
#include "xrXUtils.h"
#include "xrApp.h"
#include "xrooms.h"

#include "xrGen.h"

Boolean		xraNonICCCM;
Boolean		xraUseWindowNames;
Boolean		xraDfltAppsTransient;
Boolean		xraUseGeometries;
Boolean		xraReopenForGeom;
Boolean		xraUnmapForState,xraUnmapForGeom;
Boolean		xraManageTransients;
Boolean		xraManageWithdrawn;

HashTablePtr	xraXApps;

/***====================================================================***/

void
xraDebugPrint()
{
Opaque	istate;
GenKey	key;
GenData	data;
AppPtr	pApp;
XAppInfoPtr	 pXInfo;

    if (xraXApps!=NullHashTable) {
	htIterator(xraXApps,key,data,istate) {
	    pApp=	(AppPtr)data;
	    pXInfo=	(XAppInfoPtr)appGetOutputPriv(pApp);
	    uDebug("0x%x=	%s\n",data,appText(pApp));
	    if (data==(GenData)pXInfo->client) {
		uDebug("    parent=0x%x,client=0x%x\n",pXInfo->parent,
							pXInfo->client);
	    }
	}
	htEndIterator(istate);
    }
    return;
    
}

/***====================================================================***/

AppPtr
xraLookupWindowApp(win)
Window	win;
{
AppPtr	pApp;
    uENTRY1("xraLookupWindowApp(0x%x)\n",win);
    pApp= (AppPtr)htLookup(xraXApps,(GenKey)win);
    uRETURN(pApp);
}

/***====================================================================***/

static Bool
xraFindMissingEvents(qDpy,pEvent,pXInfo)
Display	 	*qDpy;
XEvent		*pEvent;
XAppInfoPtr	 pXInfo;
{
Boolean		handled= False;

    uENTRY3("xraFindMissingEvents(0x%x,0x%x,0x%x)\n",qDpy,pEvent,pXInfo);
    switch (pEvent->type) {
	case ConfigureNotify:
	    if (pEvent->xconfigure.window==pXInfo->client) {
		if ((pEvent->xconfigure.send_event)&&
			WantsX(pXInfo,pEvent->xconfigure.x)&&
			WantsY(pXInfo,pEvent->xconfigure.y)) {
		    GotX(pXInfo);
		    GotY(pXInfo);
		    uDEBUG2(WATCH_EVENTS,"sneaking away config pos=(%d,%d)\n",
				pEvent->xconfigure.x,pEvent->xconfigure.y);
		    handled= True;
		}
		else if (WantsWidth(pXInfo,pEvent->xconfigure.width)&&
			 WantsHeight(pXInfo,pEvent->xconfigure.height)) {
		    GotWidth(pXInfo);
		    GotHeight(pXInfo);
		    uDEBUG2(WATCH_EVENTS,"sneaking away config size=(%d,%d)\n",
			pEvent->xconfigure.width,pEvent->xconfigure.height);
		    handled= True;
		}
	    }
	    break;
	case PropertyNotify:
	    if ((pEvent->xproperty.window==pXInfo->client)&&
		    (pEvent->xproperty.atom == WM_STATE)) {
		WinState winState= xruGetState(pXInfo->client,False);
		if (WantsState(pXInfo,winState)) {
		    GotState(pXInfo);
		    handled=	True;
		}
	    }
	    break;
	case ClientMessage:
	    if ((pEvent->xclient.window==pXInfo->client)&&
		    WantsX(pXInfo,pEvent->xclient.data.s[0])&&
		    WantsY(pXInfo,pEvent->xclient.data.s[1])) {
		GotX(pXInfo);
		GotY(pXInfo);
		uDEBUG2(WATCH_EVENTS,"sneaking away msg pos=(%d,%d)\n",
			pEvent->xclient.data.s[0],pEvent->xclient.data.s[1]);
		handled= True;
	    }
	    break;
    }
    uRETURN(handled);
}

#define	MAX_TIMEOUT	2

static Boolean
xraTryToClear(pXInfo)
XAppInfoPtr	pXInfo;
{
XEvent	event;
int	timeout= 	0;

    uENTRY1("xraTryToClear(0x%x)\n",pXInfo);
    while (WantsAny(pXInfo)&&(timeout<MAX_TIMEOUT)) {
	if (!XCheckIfEvent(xroomsDpy,&event,xraFindMissingEvents,pXInfo)) {
	    timeout++;
	    sleep(1);
	}
    }
    uRETURN(!WantsAny(pXInfo));
}

/***====================================================================***/

/* ARGSUSED */
Boolean
xraNonICCCMAppUpdate(pApp,pNewState,priv)
AppPtr		pApp;
AppStatePtr	pNewState;
Opaque		priv;
{
XAppInfoPtr	pXInfo;
AppStatePtr	pOldState;
Boolean		geomChange;
Boolean		reopenWin= False;

    uENTRY3("xraNonICCCMAppUpdate(%s,%s,0x%x)\n",appText(pApp),
						asText(pNewState),priv);

    uDEBUG2(WATCH_APPS,"Reconfiguring %s\n   to   %s\n",appText(pApp),
							asText(pNewState));
    pXInfo=	(XAppInfoPtr)priv;

#ifdef NOTDEF
    if (WantsAny(pXInfo)&&(!xraTryToClear(pXInfo))) {
	uWarning("app %s waiting for a change (0x%x)\n",appText(pApp),
						pXInfo->wants.flags);
    }
#endif
    pOldState=	pXInfo->pRealState;
    uDEBUG1(WATCH_APPS,"   from %s\n",asText(pOldState));

    geomChange=	((xraUseGeometries)&&
		 (!asSelectedSame(pOldState,pNewState,asGeomValues)));
    reopenWin= xraReopenForGeom&&geomChange&&
		 (asWinState(pNewState)==asNormal);

    if ((reopenWin)||
	(asWinState(pOldState)==asNormal)&&(asWinState(pNewState)==asIconic)) {
	XWMHints	hints;
	hints.flags=	StateHint;

	uDEBUG(WATCH_APPS,"   iconifying\n");
	hints.initial_state=	IconicState;
	if (xraUnmapForState)
	    XUnmapWindow(xroomsDpy,pXInfo->client);
	XSetWMHints(xroomsDpy,pXInfo->client,&hints);
	if (xraUnmapForState)
	    XMapWindow(xroomsDpy,pXInfo->client);
	XSync(xroomsDpy,False);
	SetWantsState(pXInfo,asIconic);
    }
    if (geomChange) {
	XSizeHints	hints;
	int		x,y,oldX,oldY;
	unsigned	width,height,oldWidth,oldHeight;

	asValues(pOldState,(WinState *)NULL,&oldX,&oldY,&oldWidth,&oldHeight);
	asValues(pNewState,(WinState *)NULL,&x,&y,&width,&height);
	hints.flags=	0;
	if ((oldX!=x)||(oldY!=y)) {
	    hints.flags|=	PPosition;
	    hints.x=	x;
	    hints.y=	y;
	    uDEBUG(WATCH_APPS,"   position changed.\n");
	    SetWantsPos(pXInfo,x,y);
	}
	if ((oldWidth!=width)||(oldHeight!=height)) {
	    hints.flags|=	PSize;
	    hints.width=	width;
	    hints.height=	height;
	    uDEBUG(WATCH_APPS,"   size changed.\n");
	    SetWantsSize(pXInfo,width,height);
	}
	if (hints.flags) {
	    uDEBUG(WATCH_APPS,"   telling X\n");

	    if ((xraUnmapForGeom)&&(asWinState(pNewState)!=asWithdrawn)) {
		XGrabServer(xroomsDpy);
		XUnmapWindow(xroomsDpy,pXInfo->client);
		XSetNormalHints(xroomsDpy, pXInfo->client, &hints);
		XMoveResizeWindow(xroomsDpy,pXInfo->client,x,y,width,height);
		XMapWindow(xroomsDpy,pXInfo->client);
		XUngrabServer(xroomsDpy);
	    }
	    else {
		XSetNormalHints(xroomsDpy, pXInfo->client, &hints);
		XMoveResizeWindow(xroomsDpy,pXInfo->client,x,y,width,height);
	    }
	}
    }
    if (reopenWin||((asWinState(pOldState)==asIconic)&&
		 (asWinState(pNewState)==asNormal))) {
	XWMHints	hints;
	hints.flags=	StateHint;
	uDEBUG(WATCH_APPS,"   deiconifying\n");
	hints.initial_state=	NormalState;
	if (xraUnmapForState)
	    XUnmapWindow(xroomsDpy,pXInfo->client);
	XSetWMHints(xroomsDpy,pXInfo->client,&hints);
	if (xraUnmapForState)
	    XMapWindow(xroomsDpy,pXInfo->client);
	XSync(xroomsDpy,False);
	SetWantsState(pXInfo,asNormal);
    }
    uRETURN(True);
}

/***====================================================================***/

/* ARGSUSED */
Boolean
xraAppUpdate(pApp,pNewState,priv)
AppPtr		pApp;
AppStatePtr	pNewState;
Opaque		priv;
{
XAppInfoPtr	pXInfo;
AppStatePtr	pOldState;

    uENTRY3("xraAppUpdate(%s,%s,0x%x)\n",appText(pApp),asText(pNewState),priv);

    uDEBUG2(WATCH_APPS,"Reconfiguring %s\n   to   %s\n",appText(pApp),
							asText(pNewState));
    pXInfo=	(XAppInfoPtr)priv;

    if (WantsAny(pXInfo)&&(!xraTryToClear(pXInfo))) {
	uWarning("app %s waiting for a change (0x%x)\n",appText(pApp),
						pXInfo->wants.flags);
    }
    pOldState=	pXInfo->pRealState;
    uDEBUG1(WATCH_APPS,"   from %s\n",asText(pOldState));

    if ((asWinState(pOldState)==asNormal)&&(asWinState(pNewState)==asIconic)) {
	XEvent	event;
	uDEBUG(WATCH_APPS,"   iconifying\n");
	event.xclient.type=		ClientMessage;
	event.xclient.display=		xroomsDpy;
	event.xclient.window=		pXInfo->client;
	event.xclient.message_type=	WM_CHANGE_STATE;
	event.xclient.format=		32;
	event.xclient.data.l[0]=	IconicState;
	XSendEvent(xroomsDpy,xroomsRealRoot,False,
			SubstructureRedirectMask|SubstructureNotifyMask,
			&event);
	SetWantsState(pXInfo,asIconic);
    }
    if (xraUseGeometries&&(!asSelectedSame(pOldState,pNewState,asGeomValues))) {
	XSizeHints	hints;
	int		x,y,oldX,oldY;
	unsigned	width,height,oldWidth,oldHeight;

	asValues(pOldState,(WinState *)NULL,&oldX,&oldY,&oldWidth,&oldHeight);
	asValues(pNewState,(WinState *)NULL,&x,&y,&width,&height);
	hints.flags=	0;
	if ((oldX!=x)||(oldY!=y)) {
	    hints.flags|=	PPosition;
	    hints.x=	x;
	    hints.y=	y;
	    uDEBUG(WATCH_APPS,"   position changed.\n");
	    SetWantsPos(pXInfo,x,y);
	}
	if ((oldWidth!=width)||(oldHeight!=height)) {
	    hints.flags|=	PSize;
	    hints.width=	width;
	    hints.height=	height;
	    uDEBUG(WATCH_APPS,"   size changed.\n");
	    SetWantsSize(pXInfo,width,height);
	}
	if (hints.flags) {
	    uDEBUG(WATCH_APPS,"   telling X\n");
	    XSetNormalHints(xroomsDpy, pXInfo->client, &hints);
	    XMoveResizeWindow(xroomsDpy,pXInfo->client,x,y,width,height);
	}
    }
    if ((asWinState(pOldState)==asIconic)&&(asWinState(pNewState)==asNormal)) {
	uDEBUG(WATCH_APPS,"   deiconifying\n");
	XMapWindow(xroomsDpy,pXInfo->client);
	SetWantsState(pXInfo,asNormal);
    }
    XFlush(xroomsDpy);
    uRETURN(True);
}

/***====================================================================***/

void
xraSetupModeApp(name,pApp,pRealState,win)
char		*name;
AppPtr		 pApp;
AppStatePtr	 pRealState;
Window		 win;
{
AppStatePtr	pState;

    uENTRY4("xraSetupModeApp(%s,%s,%s,0x%x)\n",uStringText(name),appText(pApp),
							asText(pRealState),win);
    if ((pApp!=NullApp)&&(!appIsPermanent(pApp))) {
	pState=	appGetDefault(pApp);
	if (asWinState(pRealState)==asIconic) {
	    appRemoveAllRoomStates(pApp);
	    asCopyFully(pRealState,pState);
	    asChangeSelected(pState,asWinStateValue,asIconic,0,0,(unsigned)0,
							    (unsigned)0);
	    appSetPermanence(pApp,True);
	}
	else if ((win==xroomsMainWindow)||
	    regexMatch(".*[xX]?rooms*",name,NULL)||
	    regexMatch(".*[cC]lock.*",name,NULL)||
	    regexMatch(".*[lL]oad.*",name,NULL)||
	    regexMatch(".*[bB]iff.*",name,NULL)||
	    regexMatch(".*[iI][cC][oO][nN][ 	]*[bB][oO][xX].*",name,NULL)||
	    regexMatch(".*[iI][cC][oO][nN][sS].*",name,NULL)) {
	    
	    if (pState!=NullAppState) {
		asCopyFully(pRealState,pState);
		appRemoveAllRoomStates(pApp);
		appSetPermanence(pApp,True);
	    }
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

void 
xraAddApp(window)
Window	window;
{
Window		 client;
int		 border_width, depth;
Window		 winreturn;
AppPtr		 pApp;
char		*name,*class;
StringToken	 nameToken;
WinState	 winState;
AppStatePtr	 pState= NullAppState;
int		 x,y;
unsigned	 width,height;
XAppInfoPtr	 pXInfo;

    uENTRY1("xraAddApp(0x%x)\n",window);
    pApp= (AppPtr)htLookup(xraXApps,(GenKey)window);
    if (pApp!=NullApp) {
	uVOIDRETURN;
    }

    if ((client=xruFindClientWindow(window, &winState))==NULL) {
	uVOIDRETURN;
    }
    if ((!xraManageTransients)&&(xruIsTransient(client))) {
	uVOIDRETURN;
    }

    if ((pApp= (AppPtr)htLookup(xraXApps,(GenKey)client))!=NullApp) {
	pXInfo=	(XAppInfoPtr)appGetOutputPriv(pApp);
	if (pXInfo->client==client) {
	    if (pXInfo->parent!=client)
		htRemove(xraXApps,(GenKey)pXInfo->parent);
	    pXInfo->parent=	window;
	    htAdd(xraXApps,(GenKey)window,(GenData)pApp);
	    uVOIDRETURN;
	}
	else pApp=	NullApp;
    }
    else if (winState==asWithdrawn) {
	uVOIDRETURN;
    }

    xrnGetName(client,&name,&class);
    uDEBUG3(WATCH_APPS,"got name %s (class %s) for window 0x%x\n",
				uStringText(name),uStringText(class),client);
    if (name==NullString) {
	if (class==NullString) {
	    uVOIDRETURN;
	}
	else  {
	    name= 	class;
	    class=	NullString;
	}
    }
    nameToken=		stGetToken(name);

    XGetGeometry(xroomsDpy, client, &winreturn, &x, &y, &width, &height,
		   				&border_width, &depth);
    XTranslateCoordinates(xroomsDpy, client, xroomsRoot,
			    -border_width, -border_width, 
			    &x, &y, &winreturn);

    pState=	asCreate(asAllValues,winState,x,y,width,height);
    if (pState==NullAppState) {
	uVOIDRETURN;
    }
    pXInfo=	uTypedAlloc(XAppInfoRec);
    if (pXInfo==NullXAppInfo) {
	asDestroy(pState);
	uVOIDRETURN;
    }

    pApp=	GetApp(nameToken,True);

    if (xroomsSetupMode) {
	if (winState!=asWithdrawn)
	    xraSetupModeApp(name,pApp,pState,client);
    }

    if (pApp!=NullApp) {
	uDEBUG4(WATCH_APPS,"New app \"%s\" %x (%x): (%s)\n", 
			appText(pApp), window, client, asText(pState));
	SetWantsNone(pXInfo);
	pXInfo->parent=		window;
	pXInfo->client=		client;
	pXInfo->pRealState=	pState;
	XSelectInput(xroomsDpy, client, PropertyChangeMask|StructureNotifyMask);
	if (xraNonICCCM) (void)appSetUpdateFunc(pApp,xraNonICCCMAppUpdate);
	else		 (void)appSetUpdateFunc(pApp,xraAppUpdate);
	(void)appSetOutputPriv(pApp,(Opaque)pXInfo);
	(void)htAdd(xraXApps,(GenKey)client,(GenData)pApp);
	if (window!=client)
	    (void)htAdd(xraXApps,(GenKey)window,(GenData)pApp);
	if (!AppActivated(pApp,pState)) {
	    uWarning("couldn't activate app \"%s\"\n",appText(pApp));
	}
	else {
	    xrnNoteAppName(client,appText(pApp));
	    if (xroomsSetupMode) {
		appSetPermanence(pApp,True);
	    }
	}
    }
    else {
	(void)uFree((Opaque)pXInfo);
	uWarning("couldn't create app \"%s\"\n",stText(nameToken));
    }
    if (name==NullString)	(void)uFree((Opaque)name);
    if (class==NullString)	(void)uFree((Opaque)class);

    uVOIDRETURN;
}

/***====================================================================***/

void
xraClientResized(client,newW,newH)
Window		client;
unsigned	newW,newH;
{
AppPtr		pApp;
XAppInfoPtr	pXInfo;
AppStatePtr	pState;
Boolean		realChange;
unsigned	oldW,oldH;

    uENTRY3("xraClientResized(0x%x,%d,%d)\n",client,newW,newH);
    pApp= (AppPtr)htLookup(xraXApps,(GenKey)client);

    uDEBUG3(WATCH_EVENTS,"Got a resize notify for app %s (=%dx%d)\n",
						appText(pApp),newW,newH);

    if (pApp==NullApp) {
	xraAddApp(client);
	pApp=	(AppPtr)htLookup(xraXApps,(GenKey)client);
    }

    if (pApp!=NullApp) {
	pXInfo=	(XAppInfoPtr)appGetOutputPriv(pApp);
	if (pXInfo==NullXAppInfo) {
	    uInternalError("NULL X info in xraClientResized\n");
	    uVOIDRETURN;
	}
	if (client!=pXInfo->client) {	/* don't do WM top level windows */
	    uVOIDRETURN;
	}

	pState=	pXInfo->pRealState;
	asValues(pState,(WinState *)NULL,NULL,NULL,&oldW,&oldH);
	if ((oldW!=newW)||(oldH!=newH)) {
	    asChangeSelected(pState,asWidthValue|asHeightValue,asWithdrawn,0,0,
								newW,newH);
	    uDEBUG4(WATCH_EVENTS,"App %s (%x,%x) changed:\n  %s [w,h,config]\n",
					appText(pApp),pXInfo->parent,client,
								asText(pState));
	    if (WantsWidth(pXInfo,newW)&&WantsHeight(pXInfo,newH)) {
		GotWidth(pXInfo);
		GotHeight(pXInfo);
		realChange=	False;
	    }
	    else realChange=	True;
	}
	else	realChange=	False;

	if (realChange) {
	    uDEBUG(WATCH_EVENTS,"   Looks like a real change\n");
	    (void)AppStateChanged(pApp,pState);
	}
	else {
	    uDEBUG(WATCH_EVENTS,"   I was looking for this event\n");
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
xraClientMoved(client,newX,newY)
Window		client;
int		newX,newY;
{
AppPtr		pApp;
XAppInfoPtr	pXInfo;
AppStatePtr	pState;

    uENTRY3("xraClientMoved(0x%x,%d,%d)\n",client,newX,newY);
    pApp= (AppPtr)htLookup(xraXApps,(GenKey)client);
    if (pApp==NullApp) {
	xraAddApp(client);
	pApp=	(AppPtr)htLookup(xraXApps,(GenKey)client);
    }
    if (pApp!=NullApp) {
	int	oldX,oldY;

	uDEBUG1(WATCH_EVENTS,"Got a client  move for app %s\n",appText(pApp));
	pXInfo=	(XAppInfoPtr)appGetOutputPriv(pApp);
	if (pXInfo==NullXAppInfo) {
	    uInternalError("NULL X info or geom in ClientMoved\n");
	    uVOIDRETURN;
	}
	if (client!=pXInfo->client) {
	    uVOIDRETURN;
	}
	pState=	pXInfo->pRealState;
	asValues(pState,(WinState *)NULL,&oldX,&oldY,NULL,NULL);
	if ((oldX!=newX)||(oldY!=newY)) {
	    asChangeSelected(pState,asXValue|asYValue,asInactive,
					newX,newY,(unsigned)0,(unsigned)0);
	    uDEBUG4(WATCH_EVENTS,"App %s (%x,%x) changed:\n   (%s) [x,y,msg]\n",
					appText(pApp),
					pXInfo->parent,pXInfo->client,
					asText(pState));
	}
	if (WantsX(pXInfo,newX)&&WantsY(pXInfo,newY)) {
	    GotX(pXInfo);
	    GotY(pXInfo);
	    uDEBUG(WATCH_EVENTS,"   I was looking for this event\n");
	}
	else {
	    uDEBUG(WATCH_EVENTS,"   Looks like a real event\n");
	    AppStateChanged(pApp,pState);
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
xraClientDestroyed(client)
Window	client;
{
AppPtr		pApp;
XAppInfoPtr	pXInfo;
Window		parent= None;

    uENTRY1("xraClientDestroyed(0x%x)\n",client);
    uDEBUG(WATCH_EVENTS,"Got a destroy client\n");
    pApp= (AppPtr)htLookup(xraXApps,(GenKey)client);
    if (pApp!=NullApp) {
	pXInfo=	(XAppInfoPtr)appGetOutputPriv(pApp);
	if (pXInfo!=NullXAppInfo) {
	    parent= pXInfo->parent;
	    (void)uFree((Opaque)pXInfo);
	}
	AppDeactivated(pApp);
	htRemove(xraXApps,(GenKey)client);
	htRemove(xraXApps,(GenKey)parent);
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
xraStateChanged(client)
Window	client;
{
AppPtr		pApp;
WinState	winState;
AppStatePtr	pState=	NullAppState;
XAppInfoPtr	pXInfo;
Boolean		myRequest= False;

    uENTRY1("xraStateChanged(0x%x)\n",client);
    pApp= (AppPtr)htLookup(xraXApps,(GenKey)client);

#ifdef PROPERTY_RACE_CONDITION
    if (pApp==NullApp) {
	uDEBUG1(WATCH_EVENTS,"Got a property notify for non-app 0x%x\n",client);
	xraAddApp(client);
	uVOIDRETURN;
    }
#endif
    uDEBUG1(WATCH_EVENTS,"Got a state change for app %s\n",appText(pApp));
    if (pApp!=NullApp) {
	pXInfo=		(XAppInfoPtr)appGetOutputPriv(pApp);
	if (client!=pXInfo->client) {
	    uVOIDRETURN;
	}
	winState=	xruGetState(client,False);
	if (WantsState(pXInfo,winState)) {
	    GotState(pXInfo);
	    uDEBUG(WATCH_EVENTS,"I was looking for this event\n");
	    myRequest=	True;
	}
	uDEBUG2(WATCH_APPS,"%s -> %s\n",appText(pApp),asWinStateText(winState));
	if ((winState==asInactive)||
	    ((winState==asWithdrawn)&&(!xraManageWithdrawn))) {
	    if (pXInfo!=NULL)
		uFree((Opaque)pXInfo);
	    AppDeactivated(pApp);
	}
	else if (!myRequest) {
	    int 	x,y,border_width, depth;
	    unsigned 	width,height;
	    Window 	winreturn;

	    XGetGeometry(xroomsDpy, client, &winreturn, &x, &y, &width, &height,
			       			  &border_width, &depth);
	    XTranslateCoordinates(xroomsDpy,client,xroomsRealRoot,
						-border_width, -border_width,
						&x, &y, &winreturn);
	    pState=	pXInfo->pRealState;
	    asChangeSelected(pState,asAllValues,winState,x,y,width,height);
	    uDEBUG2(WATCH_APPS,"new state for %s is %s\n",appText(pApp),
							  asText(pState));
	    AppStateChanged(pApp,pState);
	}
	else if ( asWinState(pXInfo->pRealState)!=asWithdrawn) {
	    asChangeSelected(pXInfo->pRealState,asWinStateValue,winState,
								0,0,0,0);
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
xraInit()
{
    uENTRY("xraInit()\n");
    xraXApps=	htCreate((unsigned)71,genIntegerKeyType,genPointerDataType);
    if (xraXApps==NullHashTable) {
	uFatalError("Couldn't create xraXApps\n");
	/* NOTREACHED */
    }
    uVOIDRETURN;
}
