
  /*\
   *  $Header: xrDoProto.c,v 5.2 90/01/11 14:11:26 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>

#define	DEBUG_VAR	xrdpDebug
#include "utils.h"
#include "hash.h"

#include "appstate.h"
#include "apps.h"
#include "rooms.h"

#include "xrDebug.h"
#include "xrXUtils.h"
#include "xrApp.h"
#include "xrGen.h"
#include "xrProtoArgs.h"
#include "xrProtoIter.h"
#include "xrooms.h"


/***====================================================================***/

char *
xrdpDoGetAppName(which,appName)
unsigned	which;
char		*appName;
{
AppPtr		pApp;
char		*name;

    uENTRY2("xrdpDoGetAppName(0x%x,%s)\n",which,uStringText(appName));
    pApp=	xrpaGetApp(which,appName);
    if (pApp!=NullApp) {
	name=	appText(pApp);
    }
    else name=	NullString;
    uRETURN(name);
}

/***====================================================================***/

Boolean
xrdpDoRemember(pArgs)
xrRememberArgsPtr	pArgs;
{
AppPtr		 pApp;
Boolean		 ok= True;

    uENTRY1("xrdpDoRemember(0x%x)\n",pArgs);
    pApp=	xrpaGetApp(pArgs->whichApp,pArgs->appName);
    if (pApp!=NullApp)
	 ok= SetAppPermanence(pApp,pArgs->remember);
    else ok= False;

    if (!ok) {
	uWarning("REMEMBER request failed\n");
    }
    uRETURN(ok);
}

/***====================================================================***/

Boolean
xrdpDoSetState(pArgs)
xrSetStateArgsPtr	pArgs;
{
AppStatePtr	 pState,pRealState;
AppPtr		 pApp;
RoomPtr		 pRoom;
Opaque		 riState,aiState,asiState;
Boolean		 ok= True;
Boolean		 checkAllRooms= False;
unsigned	 parts;

    uENTRY1("xrdpDoSetState(0x%x)\n",pArgs);

    if (pArgs->whichState&XR_DFLT_STATE) {
	pArgs->whichRoom=	XR_CURRENT_ROOM;
    }
    if (pArgs->whichState==XR_LOCAL_STATE) {
	pArgs->whichState|=	XR_CREATE_LOCAL_STATES;
    }
    if (pArgs->whichState&(XR_ALL_STATES)) {
	checkAllRooms=	True;
	if (pArgs->whichRoom&(XR_ALL_ROOMS|XR_VISIBLE_ROOMS)) {
	    pArgs->whichRoom=	XR_CURRENT_ROOM;
	}
	pArgs->whichState|=	XR_DESTROY_LOCAL_STATES;
    }

    xrpiAppsIterator(pArgs->whichApp,pApp,pArgs->appName,aiState) {
	pRealState=	appGetCurrent(pApp);
	if (pArgs->whichPart&XR_QUERY_PARTS)
	     parts= xrpaLastParts()|XR_WINSTATE_PART;
	else parts= pArgs->whichPart;
	xrpiRoomsIterator(pArgs->whichRoom,pRoom,pArgs->roomName,riState) {
	    xrpiAppStatesIterator(pApp,pRoom,pArgs->whichState,pState,asiState){
		switch (pArgs->doWhat) {
		    case 'M':	parts|=	asFlags(pState);
				break;
		    case 'C':	parts=	asFlags(pState)&(~parts);
		    default:	break;
		}
		asChangeProfileFlags(pRealState,pState,parts);
	    }
	    xrpiEndAppStatesIterator(pApp,asiState);
	    if (!checkAllRooms)
		roomCheckAppStates(pRoom,pApp);
	}
	xrpiEndRoomsIterator(riState);
	if (checkAllRooms) {
	    RoomsIterator(pRoom,riState) {
		roomCheckAppStates(pRoom,pApp);
	    }
	    RoomsEndIterator(riState);
	}
    }
    xrpiEndAppsIterator(aiState);
    if (!ok) {
	uWarning("SET STATE request failed\n");
    }
    uRETURN(ok);
}

/***====================================================================***/

Boolean
xrdpDoBolt(pArgs)
xrSetStateArgsPtr	pArgs;
{
AppStatePtr	 pState;
AppPtr		 pApp;
RoomPtr		 pRoom;
Opaque		 riState,aiState,asiState;
Boolean		 ok= True;
Boolean		 checkRoom= False;
Boolean		(*changeFunc)(/* pState, parts */);
unsigned	 parts;

    uENTRY1("xrdpDoBolt(0x%x)\n",pArgs);

    if (pArgs->whichState&(XR_ALL_STATES|XR_DFLT_STATE)) {
	pArgs->whichRoom=	XR_CURRENT_ROOM;
    }
    if (pArgs->whichState==XR_LOCAL_STATE) {
	checkRoom=	True;
	pArgs->whichState|=	XR_CREATE_LOCAL_STATES;
    }

    if (pArgs->toWhat=='B') {
	switch (pArgs->doWhat) {
	    case 'S':	changeFunc=	asSetBolts;	break;
	    case 'M':	changeFunc=	asBolt;		break;
	    case 'C':	changeFunc=	asUnbolt;	break;
	}
    }
    else {
	switch (pArgs->doWhat) {
	    case 'S':	changeFunc=	asSetLooseBolts;	break;
	    case 'M':	changeFunc=	asLoosenBolts;		break;
	    case 'C':	changeFunc=	asTightenBolts;		break;
	}
    }

    xrpiAppsIterator(pArgs->whichApp,pApp,pArgs->appName,aiState) {
	if (pArgs->whichPart&XR_QUERY_PARTS)
	     parts= xrpaLastParts();
	else parts= pArgs->whichPart;
	xrpiRoomsIterator(pArgs->whichRoom,pRoom,pArgs->roomName,riState) {
	    xrpiAppStatesIterator(pApp,pRoom,pArgs->whichState,pState,asiState){
		if (pArgs->whichPart&XR_QUERY_PARTS)
		    parts|=	(asBolts(pState)&asWinStateValue);
		(*changeFunc)(pState,parts);
	    }
	    xrpiEndAppStatesIterator(pApp,asiState);
	    if (checkRoom)
		roomCheckAppStates(pRoom,pApp);
	}
	xrpiEndRoomsIterator(riState);
    }
    xrpiEndAppsIterator(aiState);
    if (!ok) {
	uWarning("BOLT request failed\n");
    }
    uRETURN(ok);
}

/***====================================================================***/

Boolean
xrdpDoSetVisibility(pArgs)
xrVisibilityArgsPtr	pArgs;
{
Boolean	ok=	True;
RoomPtr	pRoom;
Opaque	riState;

    uENTRY1("xrdpDoSetVisibility(0x%x)\n",pArgs);
    if (pArgs==(xrVisibilityArgsPtr)NULL) {
	uRETURN(False);
    }

    if (pArgs->createRoom) {
	pArgs->whichRoom|= XR_CREATE_ROOM;
    }
    xrpiRoomsIterator(pArgs->whichRoom,pRoom,pArgs->roomName,riState) {
	ok= roomSetVisibility(pRoom,pArgs->visibility,pArgs->checkAuto)&&ok;
    }
    xrpiEndRoomsIterator(riState);

    if (!ok) {
	uWarning("VISIBILITY request failed\n");
    }
    uRETURN(ok);
}
