#ifndef XRGEN_H
#define	XRGEN_H 1

  /*\
   *  $Header: xrGen.h,v 5.0 90/01/10 07:00:53 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xlib.h>
#include <X11/Xatom.h>

typedef struct _xrRequestRec {
	unsigned	request;
	unsigned	size;
	Window		win;
} xrRequestRec,*xrRequestPtr;

#define	XR_RQ_NOOP		0
#define	XR_RQ_SET_STATE		1
#define	XR_RQ_REMEMBER		2
#define	XR_RQ_SET_VISIBILITY	3
#define	XR_RQ_LOAD_PROFILE	4
#define	XR_RQ_APP_NAME		9
#define	XR_RQ_DEBUG		10

	/*
	 * these should correspond to values in appstate.h
	 */
#define	XR_X_PART		((unsigned)(1<<0))
#define	XR_Y_PART		((unsigned)(1<<1))
#define	XR_POSITION_PART	(XR_X_PART|XR_Y_PART)
#define	XR_WIDTH_PART		((unsigned)(1<<2))
#define	XR_HEIGHT_PART		((unsigned)(1<<3))
#define	XR_SIZE_PART		(XR_WIDTH_PART|XR_HEIGHT_PART)
#define	XR_X_NEGATIVE		((unsigned)(1<<4))
#define	XR_Y_NEGATIVE		((unsigned)(1<<5))
#define	XR_GEOM_PARTS		(XR_POSITION_PART|XR_SIZE_PART)
#define	XR_WINSTATE_PART	((unsigned)(1<<8))
#define	XR_QUERY_PARTS		((unsigned)(1<<10))
#define	XR_ALL_PARTS		(XR_GEOM_PARTS|XR_WINSTATE_PART)

#define	XR_QUERY_STATE		((unsigned)(1<<0))
#define	XR_VISIBLE_STATE	((unsigned)(1<<1))
#define	XR_LOCAL_STATE		((unsigned)(1<<2))
#define	XR_DFLT_STATE		((unsigned)(1<<3))
#define	XR_ALL_STATES		((unsigned)(1<<4))
#define	XR_WHICH_MASK		((unsigned)0x1f)
#define	XR_CREATE_LOCAL_STATES	((unsigned)(1<<9))
#define	XR_DESTROY_LOCAL_STATES	((unsigned)(1<<10))

#define	XR_QUERY_APP		((unsigned)(1<<0))
#define	XR_QUERY_MULT_APPS	((unsigned)(1<<1))
#define	XR_NAMED_APP		((unsigned)(1<<2))
#define	XR_WINDOW_APP		((unsigned)(1<<3))
#define	XR_LAST_APP		((unsigned)(1<<4))
#define	XR_ALL_APPS		((unsigned)(1<<5))
#define	XR_ACTIVE_APPS		((unsigned)(1<<6))
#define	XR_NO_APP		((unsigned)(1<<7))

	/*
	 * these should correspond to values from room.h
	 */
#define	XR_ROOM_HIDDEN		((unsigned)0)
#define	XR_ROOM_VISIBLE		((unsigned)1<<0)
#define	XR_ROOM_WHEN_NON_EMPTY	((unsigned)1<<1)
#define	XR_ROOM_WHEN_ACTIVE	((unsigned)1<<2)
#define	XR_ROOM_ALWAYS		((unsigned)1<<3)

#define	XR_QUERY_ROOM		((unsigned)(1<<0))
#define	XR_QUERY_MULT_ROOMS	((unsigned)(1<<1))
#define	XR_NAMED_ROOM		((unsigned)(1<<2))
#define	XR_CURRENT_ROOM		((unsigned)(1<<3))
#define	XR_LAST_ROOM		((unsigned)(1<<4))
#define XR_ALL_ROOMS		((unsigned)(1<<5))
#define	XR_VISIBLE_ROOMS	((unsigned)(1<<6))
#define	XR_NO_ROOM		((unsigned)(1<<7))
#define	XR_CREATE_ROOM		((unsigned)(1<<9))

typedef	struct _xrSetStateArgsRec {
	char		doWhat;
	char		toWhat;
	unsigned	whichPart;
	unsigned	whichState;
	unsigned	whichApp;
	char		*appName;
	unsigned	whichRoom;
	char		*roomName;
} xrSetStateArgsRec,*xrSetStateArgsPtr;

typedef struct _xrRememberArgsRec {
	Boolean		 remember;
	unsigned	 whichApp;
	char		*appName;
} xrRememberArgsRec,*xrRememberArgsPtr;

typedef struct _xrVisibilityArgsRec {
	unsigned	 visibility;
	unsigned	 whichRoom;
	char		*roomName;
	Boolean		 checkAuto;
	Boolean		 createRoom;
} xrVisibilityArgsRec,*xrVisibilityArgsPtr;

/*	TYPE		SYNTAX						*/ /*	<StatePart>	[+!][+-!?][+-!?][+!?][+!?]			*/
/*			 ^use State/don't use State			*/
/*			      ^use positive X, negative X, don't	*/
/*				use X, query whether to use X		*/
/*				     ^use positive Y, negative Y, don't */
/*				      use Y, query whether to use Y	*/
/*				          ^ use Width, don't use Width, */
/*					    query whether to use Width	*/
/*						^ use Height, don't use */
/*						  Height, query whether */
/*						  to use Height		*/
/*	<States>	[ADLV?]						*/
/*			All|Default|Local|Visible|Query State		*/
/*	<Apps>		[AVL(N<name>)(W<id>)?*]				*/
/*			All|actiVe|Last used|Named|Window id|Query	*/
/*						* = Query Multiple	*/
/*	<Rooms>		[ALC(N<name>)V?]				*/
/*			All|Last used|Current|Named|all Visible|Query	*/

/*	REQUEST		ARGUMENTS					*/
/*	NOOP		string (ignored)				*/
/*	SET_STATE	[SMC][PBL]<StatePart><States><Apps><Rooms>	*/
/*			Set|Merge|Clear Parts|Bolts|Loose bolts		*/ 
/*	REMEMBER	[RF]<Apps>					*/
/*			Remember|Forget 				*/
/*	SET_VISIBILITY  [Aa][Nn][Aa][VH][+-][Cc]<Rooms>			*/
/*			Always/not always Non-empty/not non-empty	*/
/*			when Active/not when active Visible|Hidden	*/
/*			+ = check automatic visibility, - = don't check */
/*			Create/don't create room if necessary		*/
/*	LOAD		string -- format of an xrooms profile		*/
/*	DEBUG 		string (passed to xrDebugSettings)		*/

extern	Atom ROOMS_CURRENT, ROOMS_ALL; 
extern	Atom ROOMS_REQUEST, ROOMS_ARGUMENT, ROOMS_RETURN;

extern	void	xrgenInit(/* dpy */);
extern	void	xrgenCleanUp(/* dpy, root */);

#endif /* XRGEN_H */
