
  /*\
   *  $Header: xrMain.c,v 5.1 90/01/11 14:12:19 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <X11/Intrinsic.h>

#define	BOOLEAN_DEFINED
#define	OPAQUE_DEFINED
#define	DEBUG_VAR	xrMainDebug
#include "utils.h"
#include "hash.h"

#include "appstate.h"
#include "apps.h"
#include "room.h"

#include "xrDebug.h"
#include "xrXUtils.h"
#include "xrProfile.h"
#include "xrProto.h"
#include "xrApp.h"
#include "xrUI.h"
#include "xrooms.h"

#include "xrGen.h"

Atom	WM_STATE,WM_CHANGE_STATE,WM_MOVED;

/***====================================================================***/

Display		*xroomsDpy;
int		 xroomsScreenIndex;
Screen		*xroomsScrn;
Window		 xroomsRoot;
Window		 xroomsRealRoot;

Window		 xroomsMainWindow=	None;

#ifdef COMPILE_STAMP
char		*xroomsCompileStamp=	COMPILE_STAMP;
#else
char		*xroomsCompileStamp=	"no stamp";
#endif
unsigned	 xroomsMajorVersion=	1;	/* don't change this */
unsigned	 xroomsMinorVersion=	2;	/* don't change this */
unsigned	 xroomsLocalVersion=	0;	/* you can change this */

Boolean		 xroomsPrintWarnings=	True;

Boolean		 xroomsSetupMode=	True;

char		*debugSettings=	NULL;

/***====================================================================***/

void
HandleConfigureNotify(event)
XEvent	*event;
{

    uENTRY1("HandleConfigureNotify(0x%x)\n",event);
    if (!event->xconfigure.override_redirect) {
	if (event->xconfigure.send_event) {
	    xraClientMoved(event->xconfigure.window,event->xconfigure.x,
						event->xconfigure.y);
	} 
	else {
	    xraClientResized(event->xconfigure.window,
				(unsigned)event->xconfigure.width,
				(unsigned)event->xconfigure.height);
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
HandleClientMessage(event)
XEvent	*event;
{
    uENTRY1("HandleClientMessage(0x%x)\n",event);
    if (event->xclient.message_type != WM_MOVED) {
	uVOIDRETURN;
    }
    xraClientMoved(event->xclient.window,event->xclient.data.s[0],
					 event->xclient.data.s[1]);
    uVOIDRETURN;
}

/***====================================================================***/

void
HandlePropertyNotify(event)
XEvent	*event;
{
    uENTRY1("HandlePropertyNotify(0x%x)\n",event);
    uDEBUG3(WATCH_EVENTS,"property %d %s on 0x%x\n",event->xproperty.atom,
		(event->xproperty.state==PropertyNewValue?"changed":"deleted"),
		event->xproperty.window);

    if (event->xproperty.window==xroomsRealRoot) {
	if ((event->xproperty.atom == ROOMS_REQUEST)&&
	    (event->xproperty.state == PropertyNewValue)) {
	    (void)xrprHandleRequests(xroomsDpy,xroomsRealRoot);
	}
    }
    else if ((event->xproperty.atom == WM_STATE)&&
   	(event->xproperty.state == PropertyNewValue))  { 
	xraStateChanged(event->xproperty.window);
    }
    else if (xroomsPrintWarnings&&
	     (event->xproperty.state==PropertyNewValue)&&
	     ((event->xproperty.atom==XA_WM_NAME)||
	      (event->xproperty.atom==XA_WM_ICON_NAME))) {
	AppPtr	pApp;
	pApp=	xraLookupWindowApp(event->xproperty.window);
	if ((!xrnHaveProfile())&&(appIsPermanent(pApp))) {
	    uWarning("%s PROPERTY changed on permanent application \"%s\"\n",
			(event->xproperty.atom==XA_WM_NAME?"WM_NAME":
							"WM_ICON_NAME"),
			appText(pApp));
	    uAction("This application will probably be identified incorrectly\n");
	    uAction("in your .xroomsrc.  See xrooms(1) for a description of\n");
	    uAction("ways to use name profiles to circumvent this problem.\n");
	    uAction("To avoid this message in the future, invoke xrooms -nowarnings\n");
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
HandleDestroyNotify(event)
XEvent	*event;
{
    uENTRY1("HandleDestroyNotify(0x%x)\n",event);
    uDEBUG(WATCH_EVENTS,"Got a destroy notify\n");
    xraClientDestroyed(event->xdestroywindow.window);
    uVOIDRETURN;
}

/***====================================================================***/

void
DoExit(code,message)
int	 code;
char	*message;
{
    uENTRY2("DoExit(%d,%s)\n",code,uStringText(message));
    xrgenCleanUp(xroomsDpy,xroomsRealRoot);
    XSync(xroomsDpy, False);
    XCloseDisplay(xroomsDpy);
    if (code!=0) {/* abnormal exit -- print message */
	uError("%s\n",message);
    }
    exit(code);
    /* NOTREACHED */
}

/***====================================================================***/

main(argc, argv)
int	  argc;
char	**argv;
{
Window			*children;
int	 		 numchildren, i;
XEvent	 		 event;

    xrInitialize(argc,argv);

    if (xrpfLoadFile(NULL))
	xroomsSetupMode=	False;

    if (xruGetChildren(xroomsRoot, &children, &numchildren)) {
	for (i=0 ; i<numchildren ; i++) {
	    xraAddApp(children[i]);
	}
	if (numchildren)
	    XFree((char *) children);
    }

    xruiBecomeVisible();

    if (xroomsRoot!=xroomsRealRoot) {
	XSelectInput(xroomsDpy,xroomsRoot,SubstructureNotifyMask);
	XSelectInput(xroomsDpy,xroomsRealRoot,PropertyChangeMask);
    }
    else {
	XSelectInput(xroomsDpy,xroomsRoot,
				SubstructureNotifyMask|PropertyChangeMask);
    }

    for (;;) {
	XtNextEvent(&event);
	uDEBUG2(WATCH_EVENTS,"event of type %d (0x%x)\n",event.type,event.type);
	if ((event.type==MapNotify) && (event.xmap.event==xroomsRoot)) {
	    uDEBUG(WATCH_EVENTS,"Got a map notify\n");
	    xraAddApp(event.xmap.window);
	}
	else if (event.type == PropertyNotify)
	    HandlePropertyNotify(&event);
	else if (event.type == DestroyNotify)
	    HandleDestroyNotify(&event);
	else if (xraUseGeometries && (event.type == ConfigureNotify)) {
	    HandleConfigureNotify(&event);
	}
	else if (xraUseGeometries && (event.type == ClientMessage)) {
	    HandleClientMessage(&event);
	}
	XtDispatchEvent(&event);
    }
}
