
  /*\
   *  $Header: xrNames.c,v 5.1 90/01/11 14:12:56 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xlib.h>
#include <X11/Xatom.h>

#define	DEBUG_VAR	xrNamesDebug
#include "utils.h"
#include "strtbl.h"
#include "nameprof.h"
#include "rooms.h"

#include "xrDebug.h"
#include "xrXUtils.h"
#include "xrNames.h"
#include "xrooms.h"

/***====================================================================***/

Boolean		xrnUseAppNames;
Boolean		xrnUseWindowNames;
StringToken	xrnNameLookup;
Atom		xrnAppNameAtom=	None;

/***====================================================================***/

static	Boolean
xrnResolveNames(clientIn,what,name,ppOutValue)
Opaque	  clientIn;
char	 *name;
char	**ppOutValue;
{
Atom	  nameAtom;
Window	  client= (Window)clientIn;
Boolean	  ok;

    uENTRY4("XResolveNames(0x%x,0x%x,%s,0x%x,)\n",client,what,uStringText(name),
								ppOutValue);
    if (what==NPPropertyName) {
	nameAtom=	(Atom)XInternAtom(xroomsDpy,name,True);
	if (nameAtom==None) {
	    ok=	False;
	    *ppOutValue=	NullString;
	}
	else {
	    ok= xruGetPropertyString(xroomsDpy,client,nameAtom,ppOutValue);
	    uDEBUG2(WATCH_APPS,"PropFinder, ok= %s, string= %s\n",
							booleanText(ok),
			     uStringText((ppOutValue?*ppOutValue:NullString)));
	}
    }
    else if (what==NPIdentifier) {
	StringToken	nameToken;
	if (stGetTokenIfExists(name,&nameToken)) {
	    ok= nprGetName(nameToken,clientIn,ppOutValue,(char **)NULL);
	}
	else ok= False;
	if (!ok) {
	    if (uStringEqual(name,"currentroom")) {
		RoomPtr	pRoom=	GetCurrentRoom();
		if (pRoom!=NullRoom) {
		    *ppOutValue=	uStringDup(roomText(pRoom));
		    ok=	True;
		}
	    }
	}
    }
    uRETURN(ok);
}

/***====================================================================***/

static	Boolean
xrnChangeNamed(clientIn,what,name,value,append)
Opaque		  clientIn;
unsigned	  what;
char		 *name;
char		 *value;
Boolean		  append;
{
Window		 client= (Window)clientIn;

    uENTRY5("XRChangeNamed(0x%x,0x%x,%s,%s,0x%x)\n",client,what,
					uStringText(name),uStringText(value),
					booleanText(append));
    if ((name==NullString)||(strlen(name)<1)) {
	uRETURN(False);
    }
    if (what==NPPropertyName) {
	Atom		 nameAtom;
	unsigned	 mode;

	if (append)	mode=	PropModeAppend;
	else	mode=	PropModeReplace;

	nameAtom=	(Atom)XInternAtom(xroomsDpy,name,False);
	if ((nameAtom!=None)&&(value!=NullString)&&(strlen(value)>0)) {
	    XChangeProperty(xroomsDpy,client,nameAtom,XA_STRING,8,mode,
							value,strlen(value));
	    XFlush(xroomsDpy);
	    uDEBUG3(WATCH_APPS,"PropChange, setting <%s> to \"%s\" on 0x%x\n",
				uStringText(name),uStringText(value),client);
	}
	else if (nameAtom!=None) {
	    if (!append)
		XDeleteProperty(xroomsDpy,client,nameAtom);
	}
	else {
	    uWarning("Error interning atom for \"%s\"\n",uStringText(name));
	}
	uRETURN(True);
    }
    else if (what==NPIdentifier) {
	if (uStringEqual(name,"currentroom")) {
	    StringToken	nameToken=	stGetToken(value);
	    RoomPtr	pRoom=		LookupRoom(nameToken);

	    if (pRoom==NullRoom)	pRoom=	CreateRoom(nameToken);
	    if (pRoom!=NullRoom) {
		SwitchToRoom(pRoom);
		uRETURN(True);
	    }
	}
	uRETURN(False);
    }
    uRETURN(False);
}


/***====================================================================***/

Boolean
xrnGetName(win,ppName,ppClass)
Window	  win;
char	**ppName;
char	**ppClass;
{
Boolean	found;

    uENTRY3("xrnGetName(0x%x,0x%x,0x%x)\n",win,ppName,ppClass);
    *ppName=	NullString;
    *ppClass=	NullString;
    found= nprGetName(xrnNameLookup,(Opaque)win,ppName,ppClass);
    if (!found) {
        if (xrnUseAppNames&&(xrnAppNameAtom!=None)) {
            if (!xruGetPropertyString(xroomsDpy,win,xrnAppNameAtom,ppName))
                *ppName= NullString;
        }
        if (*ppName==NullString) {
            if (xrnUseWindowNames) {
                XFetchName(xroomsDpy, win, ppName);
                if (*ppName==NullString)
                    XGetIconName(xroomsDpy,win,ppName);
            }
            else {
                XGetIconName(xroomsDpy, win, ppName);
                if (*ppName==NullString)
                    XFetchName(xroomsDpy, win, ppName);
            }
        }
	if (*ppName!=NullString) 
	    found=	True;
    }
    uRETURN(found);
}

/***====================================================================***/

void
xrnNoteAppName(win,name)
Window	win;
char	*name;
{
    uENTRY2("xrnNoteAppName(0x%x,%s)\n",win,uStringText(name));

    if (xrnAppNameAtom!=None) {
	XChangeProperty(xroomsDpy,win,xrnAppNameAtom,XA_STRING,8,
					PropModeReplace,
					name,strlen(name));
        XFlush(xroomsDpy);
    }
    uVOIDRETURN;
}

/***====================================================================***/

Boolean
xrnHaveProfile()
{
    uENTRY("xrnHaveProfile()\n");
    if (nprLookupFunction(xrnNameLookup)!=NullNPStatement) {
	uRETURN(True);
    }
    uRETURN(False);
}

/***====================================================================***/

void
xrnInit()
{
    uENTRY("xrnInit()\n");
    nprSetResolver(xrnResolveNames);
    nprSetChanger(xrnChangeNamed);
    uVOIDRETURN;
}
