
  /*\
   *  $Header: xrProfile.c,v 5.0 90/01/10 07:01:06 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>

#define	DEBUG_VAR	xrProfileDebug
#include "utils.h"
#include "hash.h"
#include "appstate.h"
#include "profscan.h"

#include "xrDebug.h"
#include "xrooms.h"
#include "xrProfile.h"

/***====================================================================***/

Boolean		 xrpfAutoSaveNew=	True;
Boolean		 xrpfSaveRoomProfiles=	False;
Boolean		 xrpfBackupConfig=	False;
char		*xrpfConfigFile=	NULL;

/***====================================================================***/

static FILE *
xrpfFindFileToRead()
{
FILE	*file= NULL;
char	*home;
char	 buf[256];

    uENTRY("xrpfFindFileToRead()\n");
    if (xrpfConfigFile!=NullString) {
	file= fopen(xrpfConfigFile,"r");
	if (file==NULL) {
	    uWarning("Can't open \"%s\" to read defaults\n",xrpfConfigFile);
	}
	uRETURN(file);
    }
    if ((file==NULL)&&((home=(char *)getenv("HOME"))!=NULL)) {
	sprintf(buf,"%s/.xroomsrc",home);
	file= fopen(buf,"r");
	if (file==NULL) {
	    sprintf(buf,"%s/.xrooms.init",home);
	    file= fopen(buf,"r");
	    if (file!=NULL) {
		uWarning("Intializing from .xrooms.init, which is obsolete\n");
		uAction("defaults will be saved to your .xroomsrc\n");
	    }
	}
    }
    if (file==NULL) {
	file= fopen("/usr/lib/X11/system.xroomsrc","r");
	if (file!=NULL) {
	    uWarning("Loading from /usr/lib/X11/system.xroomsrc\n");
	    uAction("defaults will be saved to your .xroomsrc\n");
	}
    }
    if (file==NULL) {
	file= fopen("/usr/local/lib/X11/system.xroomsrc","r");
	if (file!=NULL) {
	    uWarning("Loading from /usr/local/lib/X11/system.xroomsrc\n");
	    uAction("defaults will be saved to your .xroomsrc\n");
	}
    }
    if (file==NULL) {
	uWarning("Couldn't find a config file anywhere\n");
	if (xroomsSetupMode)	
	     uAction("Trying some heuristics for defaults\n");
	else uAction("No defaults\n");
    }
    uRETURN(file);
}

/***====================================================================***/

Boolean
xrpfLoadString(str)
char	*str;
{
Boolean	ok=	True;

    uENTRY1("xrpfLoadString(%20s)\n",uStringText(str));
    if (str!=NullString) {
	ScannablePtr	pScan;
	
	pScan=	scannableOpen(scannableString,(Opaque)str);
	if (pScan!=NullScannable) {
	    if (!profileRead(pScan)) {
		uWarning("Error reading config file\n");
		ok= False;
	    }
	    scannableClose(pScan);
	}
	uRETURN(ok);
    }
    uRETURN(False);
}

/***====================================================================***/

Boolean
xrpfLoadFile(fileName)
char	*fileName;
{
FILE	*file= NULL;
Boolean	 ok= False;

    uENTRY1("xrpfLoadFile(%s)\n",uStringText(fileName));
    if (fileName==NullString) {
	file=	xrpfFindFileToRead();
    }
    else {
	file=	fopen(fileName,"r");
	if (file==NULL) {
	    uWarning("Couldn't load config file %s\n",fileName);
	    uAction("profile not loaded\n");
	}
    }
    if (file!=NULL) {
	ScannablePtr	pScan;
	
	pScan=	scannableOpen(scannableFile,(Opaque)file);
	if (pScan!=NullScannable) {
	    if (!profileRead(pScan)) {
		uWarning("Error reading config file\n");
	    }
	    else ok=	True;
	    scannableClose(pScan);
	}
	fclose(file);
    }
    uRETURN(ok);
}

/***====================================================================***/

static FILE *
xrpfFindFileToWrite()
{
FILE	*file;
char	*home;
char	buf[256];

    uENTRY("xrpfFindFileToWrite()\n");
    if (xrpfConfigFile==NullString) {
	home= (char *)getenv("HOME");
	sprintf(buf,"%s/.xroomsrc",home);
	xrpfConfigFile=	uStringDup(buf);
    }
    if (xrpfConfigFile!=NullString) {
	if (xrpfBackupConfig) {
	    sprintf(buf,"%s.bak",xrpfConfigFile);
	    unlink(buf);
	    rename(xrpfConfigFile,buf);
	}
	file= fopen(xrpfConfigFile,"w");
	if (xrpfBackupConfig) {
	    if (file==(FILE *)NULL) {
		rename(buf,xrpfConfigFile);
	    }
	    xrpfBackupConfig=	False;
	}
    }
    if (file==NULL) {
	char *user=	(char *)getenv("USER");
	if (user!=NULL)	sprintf(buf,"/usr/tmp/xroomsrc.%s",user);
	else		sprintf(buf,"/usr/tmp/xroomsrc.tmp");
	uWarning("Couldn't write to config file\n");
	file= fopen(buf,"w");
	if (file!=NULL)	uAction("saving profile to %s\n",buf);
	else		uAction("profile not saved\n");
    }
    uRETURN(file);
}

/***====================================================================***/

void
xrpfSaveProfiles(fileName)
char	*fileName;
{
FILE		*file;

    uENTRY1("xrpfSaveProfiles(%s)\n",uStringText(fileName));
    if (fileName==NULL) {
	file= xrpfFindFileToWrite();
    }
    else {
	file= fopen(fileName,"w");
	if (file==NULL) {
	    uWarning("Couldn't open %s to save profiles\n",fileName);
	    uAction("profiles not saved\n");
	}
    }
    if (file!=NULL) {
	(void)profileWrite(file,xrpfSaveRoomProfiles);
	fclose(file);
	xroomsSetupMode=	False;
    }
    uVOIDRETURN;
}

