  
  /*\
   *  $Header: xrProto.c,v 5.1 90/01/10 17:05:15 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/Xproto.h>
#include <X11/Xatom.h>
#define	OPAQUE_DEFINED
#define	BOOLEAN_DEFINED
#define	DEBUG_VAR	xrprDebug
#include "utils.h"
#include "appstate.h"
#include "rooms.h"
#include "apps.h"

#include "xrooms.h"
#include "xrProto.h"
#include "xrRoom.h"

/***====================================================================***/

unsigned	xrpDfltBoltState;

/***====================================================================***/

void
xrpSetCurrent(dpy, root, nameAtom)
Display	*dpy;
Window	 root;
Atom	 nameAtom;
{
    uENTRY3("xrpSetCurrent(0x%x,0x%x,0x%x)\n",dpy,root,nameAtom);
    (void)XChangeProperty(dpy,root,ROOMS_CURRENT,XA_ATOM, 32, 
					PropModeReplace,(char *)&nameAtom,1);
    uVOIDRETURN;
}

/***====================================================================***/

struct _xrappIterArgRec {
	int	 nNames;
	Atom	*pNames;
};

static Boolean
_xrpAddRoom(pRoom,pArg)
RoomPtr			 pRoom;
struct _xrappIterArgRec *pArg;
{
XRoomInfoPtr		 pRoomInfo;

    uENTRY2("_xrpAddRoom(%s,0x%x)\n",roomText(pRoom),pArg);
    if (pRoom!=NullRoom) {
	pRoomInfo=	(XRoomInfoPtr)roomGetPrivate(pRoom);
	if ((pRoomInfo!=NullXRoomInfo)&&(roomGetVisibility(pRoom)&roomVisible)){
	    pArg->pNames[pArg->nNames]=	pRoomInfo->nameAtom;
	    pArg->nNames++;
	}
    }
    uRETURN(True);
}

/***====================================================================***/

void
xrpUpdateAllRooms(dpy, root)
Display	*dpy;
Window	 root;
{
int			 nRooms;
Atom			*pNames;
struct _xrappIterArgRec	 iarg;

    uENTRY2("xrpUpdateAllRooms(0x%x,0x%x)\n",dpy,root);
    nRooms=	NumberOfRooms();
    if (nRooms<=0) {
	XDeleteProperty(dpy,root,ROOMS_ALL);
	uVOIDRETURN;
    }
    pNames=	uTypedCalloc(nRooms,Atom);
    if (pNames==(Atom *)NULL) {
	uVOIDRETURN;
    }
    iarg.nNames= 0;
    iarg.pNames= pNames;
    RoomsIterate(_xrpAddRoom,(Opaque)&iarg);
    if (iarg.nNames>0) {
	XChangeProperty(dpy,root,ROOMS_ALL,XA_ATOM,32,PropModeReplace,
				(unsigned char *)pNames,iarg.nNames);
				     
    }
    uFree((Opaque)pNames);
    uVOIDRETURN;
}

/***====================================================================***/

void
xrprReturnString(dpy,win,len,str)
Display	*dpy;
Window	 win;
int	 len;
char	*str;
{
    uENTRY4("xrprReturnString(0x%x,0x%x,%d,0x%x)\n",dpy,win,len,str);
    if (win!=None) {
	XChangeProperty(dpy,win,ROOMS_RETURN,XA_STRING,8,PropModeReplace,
						(unsigned char *)str,len);
	XFlush(dpy);
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
xrprReturnStatus(dpy,win,success)
Display	*dpy;
Window	 win;
Boolean	 success;
{
char	buf[4];

    uENTRY3("xrprReturnStatus(0x%x,0x%x,%s)\n",dpy,win,booleanText(success));
    if (win!=None) {
	buf[0]=	buf[1]= buf[2]= buf[3]= NUL;
	if (success)	buf[0]=	't';
	else		buf[0]=	'f';
	XChangeProperty(dpy,win,ROOMS_RETURN,XA_STRING,8,PropModeReplace,
								buf,1);
	XFlush(dpy);
    }
    uVOIDRETURN;
}

/***====================================================================***/

static int
_xrprParseVisibility(str,pVis)
char	 *str;
unsigned *pVis;
{
unsigned	vis=	0;
Boolean		ok= True;

    uENTRY2("_xrprParseVisibility(%s,0x%x)\n",uStringText(str),pVis);
    if (strlen(str)>=4) {
	if		(str[0]=='A')	vis|=	roomAlways;
	else if 	(str[0]=='a')	vis&=	(~roomAlways);
	else				ok=	False;

	if		(str[1]=='N')	vis|=	roomWhenNonEmpty;
	else if 	(str[1]=='n')	vis&=	(~roomWhenNonEmpty);
	else				ok=	False;

	if		(str[2]=='A')	vis|=	roomWhenActive;
	else if 	(str[2]=='a')	vis&=	(~roomWhenActive);
	else			ok=	False;

	if		(str[3]=='B')	vis|=	roomVisible;
	else if 	(str[3]=='H')	vis&=	(~roomVisible);
	else			ok=	False;
	if (ok&&(pVis!=(unsigned *)NULL)) {
	    *pVis=	vis;
	    uRETURN(4);
	}
    }
    uRETURN(0);
}

/***====================================================================***/

static int
_xrprParseWhichPart(str,pWhich)
char	 *str;
unsigned *pWhich;
{
unsigned	parts=	0;
Boolean		query=	False;

    uENTRY2("_xrprParseWhichPart(%s,0x%x)\n",uStringText(str),pWhich);
    if (strlen(str)<5) {
	uRETURN(0);
    }
    switch (str[0]) {
	case '+':	parts|=	asWinStateValue;	break;
	case '!':	parts&=	(~asWinStateValue);	break;
	default:	uRETURN(0);
    }
    switch (str[1]) {
	case '?':	query=	True;				break;
	case '+':	parts|=	asXValue;			break;
	case '-':	parts|= asXValue|asXNegative;		break;
	case '!':	parts&=	(~(asXValue|asXNegative));	break;
	default:	uRETURN(0);
    }
    switch (str[2]) {
	case '?':	query=	True;				break;
	case '+':	parts|=	asYValue;			break;
	case '-':	parts|= asYValue|asYNegative;		break;
	case '!':	parts&=	(~(asYValue|asYNegative));	break;
	default:	uRETURN(0);
    }
    switch (str[3]) {
	case '?':	query=	True;			break;
	case '+':	parts|=	asWidthValue;		break;
	case '!':	parts&=	(~asWidthValue);	break;
	default:	uRETURN(0);
    }
    switch (str[4]) {
	case '?':	query=	True;			break;
	case '+':	parts|=	asHeightValue;		break;
	case '!':	parts&=	(~asHeightValue);	break;
	default:	uRETURN(0);
    }
    if (query)	*pWhich=	XR_QUERY_PARTS;
    else	*pWhich=	parts;
    uRETURN(5);
}

/***====================================================================***/

static Boolean
_xrprParseWhichState(ch,pWhich)
char	  ch;
unsigned *pWhich;
{

    uENTRY2("_xrprParseWhichState(%c,0x%x)\n",ch,pWhich);
    switch (ch) {
	case 'A':	*pWhich=	XR_ALL_STATES;		break;
	case 'D':	*pWhich=	XR_DFLT_STATE;		break;
	case 'L':	*pWhich=	XR_LOCAL_STATE;		break;
	case 'V':	*pWhich=	XR_VISIBLE_STATE;	break;
	case '?':	*pWhich=	XR_QUERY_STATE;		break;
	default:	uRETURN(False);
    }
    uRETURN(True);
}

/***====================================================================***/

static Boolean
_xrprParseWhichApp(str,pLen,pWhich,ppName)
char	 *str;
int	 *pLen;
unsigned *pWhich;
char	**ppName;
{
int	len=	1;

    uENTRY4("_xrprParseWhichApp(%s,0x%x,0x%x,0x%x)\n",str,pLen,pWhich,ppName);
    *pWhich=	XR_QUERY_APP;
    *ppName=	NullString;
    switch (str[0]) {
	case 'A':	*pWhich=	XR_ALL_APPS;	break;
	case 'V':	*pWhich=	XR_ACTIVE_APPS;	break;
	case 'L':	*pWhich=	XR_LAST_APP;	break;
	case '?':	*pWhich=	XR_QUERY_APP;	break;
	case 'N':	
	case 'W':	
			if (str[1]==NUL) {
			    *pWhich=	XR_QUERY_APP;
			    len= 2;
			}
			else {
			    if (str[0]=='N')		*pWhich= XR_NAMED_APP;
			    else if (str[0]=='W')	*pWhich= XR_WINDOW_APP;
			    *ppName=	&str[1];
			    len=	1+strlen(*ppName)+1;
			}
			break;
	default:	uRETURN(False);
    }
    *pLen=	len;
    uRETURN(True);
}

/***====================================================================***/

static Boolean
_xrprParseWhichRoom(str,pLen,pWhich,ppName)
char	 *str;
int	 *pLen;
unsigned *pWhich;
char	**ppName;
{
int	len=	1;

    uENTRY4("_xrprParseWhichRoom(%s,0x%x,0x%x,0x%x)\n",str,pLen,pWhich,ppName);
    *pWhich=	XR_CURRENT_ROOM;
    *ppName=	NullString;
    switch (str[0]) {
	case 'A':	*pWhich=	XR_ALL_ROOMS;		break;
	case 'V':	*pWhich=	XR_VISIBLE_ROOMS;	break;
	case 'C':	*pWhich=	XR_CURRENT_ROOM;	break;
	case 'L':	*pWhich=	XR_LAST_ROOM;		break;
	case '?':	*pWhich=	XR_QUERY_ROOM;		break;
	case 'N':	if (str[1]==NUL) {
			    *pWhich=	XR_QUERY_ROOM;
			    len= 2;
			}
			else {
			    *pWhich=	XR_NAMED_ROOM;
			    *ppName=	&str[1];
			    len=	1+strlen(*ppName)+1;
			}
			break;
	default:	uRETURN(False);
    }
    *pLen=	len;
    uRETURN(True);
}

/***====================================================================***/

static Boolean
_xrprParseSetStateRequest(str,pArgs)
char			*str;
xrSetStateArgsPtr	 pArgs;
{
Boolean	ok= True;
int	roomI;

    uENTRY2("_xrprParseSetStateRequest(%s,0x%x)\n",uStringText(str),pArgs);
    if ((str!=NullString)&&(strlen(str)>=9)) {
	if ((str[0]=='S')||(str[0]=='M')||(str[0]=='C'))
	     pArgs->doWhat=	str[0];
	else ok=			False;
	    
	if ((str[1]=='P')||(str[1]=='B')||(str[1]=='L'))
	     pArgs->toWhat=		str[1];
	else ok=			False;

	ok=	ok&&(_xrprParseWhichPart(&str[2],&pArgs->whichPart)>0);
	ok=	ok&&_xrprParseWhichState(str[7],&pArgs->whichState);
	ok=	ok&&_xrprParseWhichApp(&str[8],&roomI,&pArgs->whichApp,
						   &pArgs->appName);
	roomI+=	8;

	ok=	ok&&_xrprParseWhichRoom(&str[roomI],&roomI,&pArgs->whichRoom,
						  &pArgs->roomName);
    }
    else ok= False;

    if (!ok) {
	uWarning("Bad SET STATE request (%s)\n",uStringText(str));
    }
    uRETURN(ok);
}

/***====================================================================***/

static Boolean
_xrprParseRememberRequest(str,pArgs)
char			*str;
xrRememberArgsPtr	 pArgs;
{
Boolean	ok= True;
int	len;

    uENTRY2("_xrprParseRememberRequest(%s,0x%x)\n",uStringText(str),pArgs);
    if ((str!=NullString)&&(strlen(str)>=2)) {
	if	(str[0]=='R')	pArgs->remember=	True;
	else if (str[0]=='F')	pArgs->remember=	False;
	else			ok=			False;

	ok=	_xrprParseWhichApp(&str[1],&len,&pArgs->whichApp,
						 &pArgs->appName)&&ok;
    }
    else ok= False;

    if (!ok) {
	uWarning("Bad REMEMBER request (%s)\n",uStringText(str));
    }
    uRETURN(ok);
}

/***====================================================================***/

static Boolean
_xrprParseVisibilityRequest(str,pArgs)
char			*str;
xrVisibilityArgsPtr	 pArgs;
{
Boolean	ok= True;
int	len;

    uENTRY2("_xrprParseVisibilityRequest(%s,0x%x)\n",uStringText(str),pArgs);
    if ((str!=NullString)&&(strlen(str)>=7)) {
	len= _xrprParseVisibility(str,&pArgs->visibility);
	if (len>0) {
	    if 		(str[len]=='+')	pArgs->checkAuto=	True;
	    else if	(str[len]=='-')	pArgs->checkAuto=	False;
	    else			ok=			False;
	    len++;

	    if 		(str[len]=='C')	pArgs->createRoom=	True;
	    else if	(str[len]=='c')	pArgs->createRoom=	False;
	    else			ok=			False;
	    len++;

	    ok= ok&&_xrprParseWhichRoom(&str[len],&len,&pArgs->whichRoom,
						  	&pArgs->roomName);
	}
    }
    else ok= False;

    if (!ok) {
	uWarning("Bad VISIBILITY request (%s)\n",uStringText(str));
    }
    uRETURN(ok);
}

/***====================================================================***/

static Boolean
_xrprParseGetAppNameRequest(str,pWhichApp,ppAppName)
char		 *str;
unsigned	 *pWhichApp;
char		**ppAppName;
{
Boolean	ok= True;
int	len;

    uENTRY3("_xrprParseGetAppNameRequest(%s,0x%x,0x%x)\n",uStringText(str),
							pWhichApp,ppAppName);
    if (str!=NullString) {
	ok=	_xrprParseWhichApp(&str[0],&len,pWhichApp,ppAppName);
    }
    else ok= False;

    if (!ok) {
	uWarning("Bad GET APP NAME request (%s)\n",uStringText(str));
    }
    uRETURN(ok);
}

/***====================================================================***/

Boolean
xrprDispatchRequest(dpy,root,pRequest,pArgStart)
Display		*dpy;
Window	 	 root;
xrRequestPtr	 pRequest;
int		*pArgStart;
{
Atom	actualType;
int	actualFormat,nItems,bytesAfter;
char	*pString= NullString;
Boolean	ok=	False;

    uENTRY4("xrprDispatchRequest(0x%x,0x%x,0x%x,0x%x)\n",dpy,root,pRequest,
							 pArgStart);
    if (pRequest->size>0) {
	    if (XGetWindowProperty(dpy,pRequest->win,ROOMS_ARGUMENT,
				*pArgStart,pRequest->size*4,True,XA_STRING,
				&actualType,&actualFormat,&nItems,&bytesAfter,
				&pString)==Success) {
	    if ((actualType==XA_STRING)&&(nItems==(pRequest->size)*4)&&
		(actualFormat==8)) {
		if (bytesAfter==0)	*pArgStart=	0;
		else			*pArgStart+=	pRequest->size;
		ok=	True;
	    }
	    else {
		uError("bogus _XROOMS_ARGUMENT property (%d!=%d||%d!=%d)\n",
					actualType,XA_STRING,
					nItems,pRequest->size*4);
		uAction("removing properties. requests may be lost\n");
		XDeleteProperty(dpy,root,ROOMS_REQUEST);
		XDeleteProperty(dpy,pRequest->win,ROOMS_ARGUMENT);
	    }
	}
    }
    else {
	pString=	NullString;
	ok=		True;
    }

    if (ok) {
	uDEBUG1(1,"REQUEST = \"%s\"\n",uStringText(pString));
	if (pRequest->request==XR_RQ_NOOP) {
	    uDEBUG1(1,"NOOP = \"%s\"\n",uStringText(pString));
	}
	else if (pRequest->request==XR_RQ_DEBUG) {
	    extern void xrDebugSettings(/*settings*/);
	    xrDebugSettings(pString);
	    xrprReturnStatus(dpy,pRequest->win,True);
	}
	else if (pRequest->request==XR_RQ_SET_STATE) {
	    xrSetStateArgsRec	stateArgs;
	    if (_xrprParseSetStateRequest(pString,&stateArgs)) {
		switch (stateArgs.toWhat) {
		    case 'P':
			ok= xrdpDoSetState(&stateArgs);
			break;
		    case 'B':	case 'L':
			ok= xrdpDoBolt(&stateArgs);
			break;
		    default:
			ok= False;
			break;
		}
	    }
	    else ok=	False;
	    xrprReturnStatus(dpy,pRequest->win,ok);
	}
	else if (pRequest->request==XR_RQ_REMEMBER) {
	    xrRememberArgsRec	args;
	    if (_xrprParseRememberRequest(pString,&args)) {
		ok= xrdpDoRemember(&args);
	    }
	    else ok= False;
	    xrprReturnStatus(dpy,pRequest->win,ok);
	}
	else if (pRequest->request==XR_RQ_SET_VISIBILITY) {
	    xrVisibilityArgsRec	 args;
	    if (_xrprParseVisibilityRequest(pString,&args)) {
		ok= xrdpDoSetVisibility(&args);
	    }
	    else ok= False;
	    xrprReturnStatus(dpy,pRequest->win,ok);
	}
	else if (pRequest->request==XR_RQ_LOAD_PROFILE) {
	    ok=	xrpfLoadString(pString);
	    xrprReturnStatus(dpy,pRequest->win,ok);
	}
	else if (pRequest->request==XR_RQ_APP_NAME) {
	    unsigned	which;
	    char	*appName=	NullString;
	    char	buf[258];
	    if (_xrprParseGetAppNameRequest(pString,&which,&appName)) {
		appName= xrdpDoGetAppName(which,appName);
		if (appName!=NullString) {
		    sprintf(buf,"t%s",appName);
		    appName=	buf;
		}
	    }
	    else ok= False;
	    if ((ok)&&(appName!=NullString))
		 xrprReturnString(dpy,pRequest->win,strlen(appName),appName);
	    else xrprReturnStatus(dpy,pRequest->win,False);
	}
	else {
	    xrprReturnStatus(dpy,pRequest->win,False);
	}
	if (pString!=NullString)
	    (void)XFree(pString);
	uRETURN(bytesAfter==0);
    }
    else {
	xrprReturnStatus(dpy,pRequest->win,False);
    }
    uRETURN(True);
}

/***====================================================================***/

void
xrprHandleRequests(dpy,root)
Display	*dpy;
Window	 root;
{
long	reqStart;
Atom	actualType;
int	actualFormat,nItems,bytesAfter;
int	argStart= 0;
Boolean	done=	False;
xrRequestPtr	pRequest;

    uENTRY2("xrprHandleRequests(0x%x,0x%x)\n",dpy,root);
    reqStart= 0;
    while (!done) {
	if (XGetWindowProperty(dpy,root,ROOMS_REQUEST,reqStart,3,True,
				XA_INTEGER,&actualType,&actualFormat,&nItems,
				&bytesAfter,&pRequest)==Success) {
	    if ((actualType==XA_INTEGER)&&(actualFormat==32)&&(nItems==3)) {
		reqStart+= 3;
		switch (pRequest->request) {
		    default:
			done= xrprDispatchRequest(dpy,root,pRequest,&argStart);
			break;
		}
		done=	done&&(bytesAfter==0);
		if (bytesAfter==0)
		    reqStart=	0;
		if (pRequest!=NULL) 
		    (void)XFree(pRequest);
	    }
	    else {
		if (actualType!=None)
		    uError("bogus _XROOMS_REQUEST property\n");
		    uAction("removing properties. requests may be lost\n");
		    XDeleteProperty(dpy,root,ROOMS_REQUEST);
		    XDeleteProperty(dpy,pRequest->win,ROOMS_ARGUMENT);
		    done=	True;
		}
	}
	else done= True;
    }
    uVOIDRETURN;
}
