
  /*\
   *  $Header: xrProtoArgs.c,v 5.2 90/01/11 15:17:44 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>

#define	DEBUG_VAR	xrpaDebug
#include "utils.h"
#include "hash.h"

#include "appstate.h"
#include "apps.h"
#include "rooms.h"

#include "xrDebug.h"
#include "xrXUtils.h"
#include "xrApp.h"
#include "xrGen.h"
#include "xrProtoArgs.h"
#include "xrooms.h"


/***====================================================================***/

static	xrpaSelectInfoRec	xrpaLastPress= { NullApp,0,0,0,0,0 };
static	RoomPtr			xrpaLastRoom=	NullRoom;

static RoomPtr
xrpaGetLastRoom()
{
    uENTRY("xrpaGetLastRoom()\n");
    uRETURN(xrpaLastRoom);
}

static void
xrpaSetLastRoom(pRoom)
RoomPtr	pRoom;
{
    uENTRY1("xrpaSetLastRoom(%s)\n",roomText(pRoom));
    xrpaLastRoom=	pRoom;
    uVOIDRETURN;
}

AppPtr
xrpaGetLastApp()
{
    uENTRY("xrpaGetLastApp()\n");
    uRETURN(xrpaLastPress.pApp);
}

static void
xrpaSetLastApp(pApp)
AppPtr	pApp;
{
    uENTRY1("xrpaSetLastApp(%s)\n",appText(pApp));
    xrpaLastPress.pApp=	pApp;
    xrpaLastPress.x=		xrpaLastPress.y=	1;
    xrpaLastPress.button=	1;
    xrpaLastPress.dx=		xrpaLastPress.dy=	0;
    uVOIDRETURN;
}

/***====================================================================***/

static void
xrpaFindSectorAndDelta(win,x1,y1,x2,y2,pPressInfo)
Window		win;
int		x1,y1;
int		x2,y2;
xrpaSelectInfoPtr pPressInfo;
{
Window		rootRtrn,kidRtrn;
int		realX,realY;
int		realWidth,realHeight;
unsigned	borderRtrn,depthRtrn;
int		xSector=	1;
int		ySector=	1;
int		dx=		0;
int		dy=		0;

    uENTRY6("xrpaFindSectorAndDelta(0x%x,%d,%d,%d,%d,0x%x)\n",win,
						x1,y1,x2,y2,pPressInfo);
    
    if (XGetGeometry(xroomsDpy,win,&rootRtrn,&realX,&realY,
					&realWidth,&realHeight,
					&borderRtrn,&depthRtrn)) {
	XTranslateCoordinates(xroomsDpy,rootRtrn,win,x1,y1,&realX,&realY,
								&kidRtrn);
	x1=	realX;	y1=	realY;
	XTranslateCoordinates(xroomsDpy,rootRtrn,win,x2,y2,&realX,&realY,
								&kidRtrn);
	x2=	realX;	y2=	realY;
	uDEBUG4(WATCH_EVENTS,"translated coordinates: (%d,%d) -> (%d,%d)\n",
								x1,y1,x2,y2);
	uDEBUG2(WATCH_EVENTS,"window is %dx%d\n",realWidth,realHeight);
	realWidth/=3;	realHeight/= 3;
	if 	(x1<=realWidth)		xSector=	1;
	else if (x1<=(realWidth*2))	xSector=	0;
	else 				xSector=	-1;
	if 	(y1<=realHeight)	ySector=	1;
	else if (y1<=(realHeight*2))	ySector=	0;
	else 				ySector=	-1;

	dx=	x1-x2;	
	if 	(dx>realWidth)		dx=	1;
	else if	(dx<(-realWidth))	dx=	-1;
	else				dx=	0;

	dy=	y1-y2;
	if 	(dy>realHeight)		dy=	1;
	else if	(dy<(-realHeight))	dy=	-1;
	else				dy=	0;
    }
    if (pPressInfo!=NullSelectInfo) {
	pPressInfo->x=	xSector;
	pPressInfo->y=	ySector;
	pPressInfo->dx=	dx;
	pPressInfo->dy=	dy;
    }
    uVOIDRETURN;
}

/***====================================================================***/

/*
 * Routine to let user select a window using the mouse, lifted from dsimple.c
 * with stuff to locate client window added.
 */

AppPtr 
xrpaSelectApp(pInfo)
xrpaSelectInfoPtr	pInfo;
{
int	status;
Cursor	cursor;
XEvent	event;
Window	targetWin= None;
int	buttons= 0;
int	button,x1,y1,x2,y2;

    uENTRY1("xrpaSelectApp(0x%x)\n",pInfo);
    /* Make the target cursor */
    cursor = XCreateFontCursor(xroomsDpy, XC_crosshair);

    do {
	/* Grab the pointer using target cursor, letting it roam all over */
	status = XGrabPointer(xroomsDpy, xroomsRoot, False,
			ButtonPressMask|ButtonReleaseMask, GrabModeSync,
			GrabModeAsync, xroomsRoot, cursor, CurrentTime);
    } while (status!=GrabSuccess);

    /* Let the user select a window... */
    while ((targetWin == None) || (buttons != 0)) {
	/* allow one more event */
	XAllowEvents(xroomsDpy,SyncPointer, CurrentTime);
	XWindowEvent(xroomsDpy,xroomsRoot,ButtonPressMask|ButtonReleaseMask,
								&event);
	switch (event.type) {
	    case ButtonPress:
		if (targetWin == None) {
		    targetWin = event.xbutton.subwindow; /* window selected */
		    button=	event.xbutton.button;
		    x1=		event.xbutton.x;
		    y1=		event.xbutton.y;
		    if ((targetWin == None)||(buttons>1))
			targetWin = xroomsRoot;
		}
		buttons++;
		break;
	    case ButtonRelease:
		   /* there may have been some down before we started */
		if (buttons > 0) 
		    buttons--;
		if (event.xbutton.button==button) {
		    x2=	event.xbutton.x;
		    y2=	event.xbutton.y;
		}
		break;
	}
    } 

    XUngrabPointer(xroomsDpy, CurrentTime);      /* Done with pointer */

    if (targetWin!=xroomsRoot) {
	Window	tmpWin= targetWin;

	tmpWin=		xruFindClientWindowAbove(targetWin,(WinState *)NULL);
	if (tmpWin==None) 
	    tmpWin=	xruFindClientWindow(targetWin,(WinState *)NULL);
	if (tmpWin!=None) {
	    AppPtr	pApp=	xraLookupWindowApp(tmpWin);
	    if ((pInfo!=NullSelectInfo)&&(pApp!=NullApp)) {
		pInfo->pApp=	pApp;
		pInfo->button=	button;
		xrpaFindSectorAndDelta(tmpWin,x1,y1,x2,y2,pInfo);
	    }
	    uRETURN(pApp);
	}
    }
    uRETURN(NullApp);
}


/***====================================================================***/

unsigned
xrpaLastParts()
{
unsigned	parts= 0;

    uENTRY("xrpaLastParts()\n");
    if		(xrpaLastPress.x<0)	parts|= XR_X_PART|XR_X_NEGATIVE;
    else if	(xrpaLastPress.x>0)	parts|= XR_X_PART;

    if		(xrpaLastPress.y<0)	parts|= XR_Y_PART|XR_Y_NEGATIVE;
    else if	(xrpaLastPress.y>0)	parts|= XR_Y_PART;

    if (xrpaLastPress.dx!=0)
	parts|=	XR_WIDTH_PART;
    if (xrpaLastPress.dy!=0)
	parts|=	XR_HEIGHT_PART;
    uDEBUG4(WATCH_EVENTS,"press at (%d,%d), delta was (%d,%d)\n",
				xrpaLastPress.x,xrpaLastPress.y,
				xrpaLastPress.dx,xrpaLastPress.dy);
    uDEBUG1(WATCH_EVENTS,"parts= 0x%x\n",parts);
    uRETURN(parts);
}

/***====================================================================***/

AppPtr
xrpaGetApp(which,name)
unsigned	 which;
char		*name;
{
StringToken	 nameToken;
AppPtr		 pApp= NullApp;

    uENTRY2("xrpaGetApp(0x%x,%s)\n",which,uStringText(name));
    if ((which==XR_QUERY_APP)||(which==XR_QUERY_MULT_APPS))
	pApp=	xrpaSelectApp(&xrpaLastPress);
    else if (which==XR_LAST_APP)	pApp=	xrpaGetLastApp();
    else if (which==XR_NAMED_APP) {
	if (stGetTokenIfExists(name,&nameToken)) {
	    pApp= LookupApp(nameToken);
	}
	xrpaSetLastApp(pApp);
    }
    else if (which==XR_WINDOW_APP) {
	Window	winId;
	if (sscanf(name,"0x%x",&winId)==1) {
	    winId=	xruFindClientWindow(winId,(WinState *)NULL);
	    if (winId!=None) {
		pApp=	xraLookupWindowApp(winId);
		if (pApp!=NullApp)
		    xrpaSetLastApp(pApp);
	    }
	}
	else pApp=	NullApp;
    }
    else if (which==XR_ALL_APPS) {
	pApp=	NullApp;
    }
    uRETURN(pApp);
}

/***====================================================================***/

RoomPtr
xrpaGetRoom(which,name)
unsigned	 which;
char		*name;
{
StringToken	nameToken;
RoomPtr		pRoom= NullRoom;

    uENTRY2("xrpaGetRoom(0x%x,%s)\n",which,uStringText(name));
    if (which&XR_QUERY_ROOM) {
	/* not implemented yet */
	pRoom=	GetCurrentRoom();
	xrpaSetLastRoom(pRoom);
    }
    else if (which&XR_CURRENT_ROOM)  {
	pRoom=	GetCurrentRoom();
	xrpaSetLastRoom(pRoom);
    }
    else if (which&XR_LAST_ROOM)
	pRoom=	xrpaGetLastRoom();
    else if (which&XR_NAMED_ROOM) {
	if (stGetTokenIfExists(name,&nameToken)) {
	    pRoom= LookupRoom(nameToken);
	}
	if ((pRoom==NullRoom)&&(which&XR_CREATE_ROOM)) {
	    nameToken=	stGetToken(name);
	    pRoom=	CreateRoom(nameToken);
	}
	xrpaSetLastRoom(pRoom);
    }
    else if (which&XR_ALL_ROOMS) {
	pRoom=	NullRoom;
    }
    uRETURN(pRoom);
}

/***====================================================================***/

AppStatePtr
xrpaGetState(which,pApp,pRoom)
unsigned	which;
AppPtr		pApp;
RoomPtr		pRoom;
{
AppStatePtr	pState=	NullAppState;

    uENTRY3("xrpaGetState(0x%x,%s,%s)\n",which,appText(pApp),
						roomText(pRoom));

    if (pApp!=NullApp) {
	if (which&XR_QUERY_STATE) {
	    which=	xrpDfltBoltState;
	}
	if (which&(XR_DFLT_STATE|XR_ALL_STATES)) 
	     pState=	appGetDefault(pApp);
	else if ((which&XR_LOCAL_STATE)&&(pRoom!=NullRoom)) {
	    pState= appSetRoomState(pApp,roomName(pRoom),appGetDefault(pApp),
									False);
	    if (appIsActive(pApp))
		roomCheckAppStates(pRoom,pApp);
	}
	else if ((which&XR_VISIBLE_STATE)&&(pRoom!=NullRoom)) {
	    if (appIsActive(pApp))
		pState=	roomGetVisibleState(pRoom,pApp);
	    else {
		pState= appGetRoomState(pApp,roomName(pRoom));
		if (pState==NullAppState) {
		    pState=	appGetDefault(pApp);
		}
	    }
	}
	else pState=	NullAppState;
    }
    uRETURN(pState);
}

