
  /*\
   *  $Header: xrProtoIter.c,v 5.1 90/01/11 15:18:07 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xlib.h>
#define	DEBUG_VAR	xrpiDebug
#include "utils.h"

#include "appstate.h"
#include "apps.h"
#include "rooms.h"

#include "xrGen.h"
#include "xrProtoArgs.h"
#include "xrProtoIter.h"

/***====================================================================***/
/***          ITERATE ACROSS SELECTED STATES OF AN APPLICATION          ***/
/***====================================================================***/

typedef struct	_xrpiIterState	{
	unsigned	flags;
	unsigned	which;
	Opaque		pending;
	Opaque		pIState;
} xrpiIterStateRec,*xrpiIterStatePtr;
#define	NullIterState	((xrpiIterStatePtr)NULL)

#define	xrpiFreeMe		((unsigned)(1<<1))
#define	xrpiInUse		((unsigned)(1<<2))
#define	xrpiSinglePending	((unsigned)(1<<3))
#define	xrpiIterInProgress	((unsigned)(1<<4))

static	xrpiIterStateRec	dfltASIterState;

/***====================================================================***/

Boolean
_xrpiInitAppStatesIter(pApp, pRoom, whichStates, ppIState)
AppPtr		 pApp;
RoomPtr		 pRoom;
unsigned	 whichStates;
Opaque		*ppIState;
{
Boolean	ok=	True;
xrpiIterStatePtr	pIState;
StringToken	rmName;

    uENTRY4("_xrpiInitAppStatesIter(%s,%s,0x%x,0x%x)\n",appText(pApp),
					roomText(pRoom),whichStates,ppIState);
    if ((pApp==NullApp)||(ppIState==(Opaque *)NULL)) {
	uRETURN(False);
    }
    if (pRoom==NullRoom) {
	pRoom=	GetCurrentRoom();
	if (pRoom==NullRoom) {
	    uRETURN(False);
	}
    }
    rmName=	roomName(pRoom);
    if (!(dfltASIterState.flags&xrpiInUse)) {
	pIState=	&dfltASIterState;
	pIState->flags=	xrpiInUse;
    }
    else {
	pIState=	uTypedAlloc(xrpiIterStateRec);
	if (pIState==NullIterState) {
	    uRETURN(False);
	}
	pIState->flags=	xrpiInUse|xrpiFreeMe;
    }
    pIState->pIState=	(Opaque)NULL;
    pIState->pending=	(Opaque)NULL;
    if (whichStates&XR_ALL_STATES) {
	if (whichStates&XR_DESTROY_LOCAL_STATES) {
	    AppStatePtr	pState=	roomGetVisibleState(pRoom,pApp);
	    if (pState!=NullAppState) {
		appSetDefault(pApp,pState,False);
	    }
	    appRemoveAllRoomStates(pApp);
	}
	else {
	    if (_appInitIter(pApp,&pIState->pIState))
		pIState->flags|=	xrpiIterInProgress;
	}
    }

    switch (whichStates&XR_WHICH_MASK) {
	case XR_QUERY_STATE:	/* not implemented */
	case XR_VISIBLE_STATE:
		pIState->pending= (Opaque)roomGetVisibleState(pRoom,pApp);
		if (pIState->pending!=(Opaque)NULL)
		     pIState->flags|=	xrpiSinglePending;
		else ok= False;
		break;
	case XR_LOCAL_STATE:
		pIState->pending=	(Opaque)appGetRoomState(pApp,rmName);
		if ((pIState->pending==(Opaque)NULL)&&
		    (whichStates&XR_CREATE_LOCAL_STATES)) {
		    pIState->pending=	(Opaque)appGetDefault(pApp);
		    pIState->pending=	(Opaque)appSetRoomState(pApp,rmName,
						(AppStatePtr)pIState->pending,
						False);
		}
		if (pIState->pending==(Opaque)NULL)
			pIState->flags|=	xrpiSinglePending;
		else	ok= False;
		break;
	case XR_ALL_STATES:
	case XR_DFLT_STATE:
		pIState->pending=	(Opaque)appGetDefault(pApp);
		pIState->flags|=	xrpiSinglePending;
		break;
	default:
		ok= False;
		break;
    }
    *ppIState=	(Opaque)pIState;
    uRETURN(ok);
}

/***====================================================================***/

Boolean
_xrpiAppStatesIterNext( inState, pApp, ppState )
Opaque		 inState;
AppPtr		 pApp;
AppStatePtr	*ppState;
{
xrpiIterStatePtr pIState= (xrpiIterStatePtr)inState;
Boolean		 ok;
StringToken	 name;

    uENTRY3("_xrpiAppStatesIterNext(0x%x,%s,0x%x)\n",pIState,appText(pApp),
								ppState);
    if (pIState->flags&xrpiSinglePending) {
	*ppState=	(AppStatePtr)pIState->pending;
	pIState->flags&= (~xrpiSinglePending);
	uRETURN(True);
    }
    if (pIState->flags&xrpiIterInProgress) {
	ok= _appIterNext(pApp,&pIState->pIState,&name,ppState);
	if (!ok) {
	    pIState->flags&= (~xrpiIterInProgress);
	}
	uRETURN(ok);
    }
    uRETURN(False);
}

/***====================================================================***/

void
xrpiEndAppStatesIterator( pApp, inState )
AppPtr	pApp;
Opaque	inState;
{
xrpiIterStatePtr pIState= (xrpiIterStatePtr)inState;

    uENTRY1("xrpiEndAppStatesIterator( 0x%x )\n",pIState);
    if ( pIState!= NullIterState ) {
	pIState->flags&=	(~xrpiSinglePending);
	pIState->pending=	(Opaque)NULL;
	if (pIState->pIState!=(Opaque)NULL) {
	    pIState->flags&=	(~xrpiIterInProgress);
	    appEndIterator(pApp,pIState->pIState);
	    pIState->pIState=	(Opaque)NULL;
	}
	pIState->flags&=	(~xrpiInUse);
	if (pIState->flags&xrpiFreeMe) {
	    uFree((Opaque)pIState);
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

static	xrpiIterStateRec	dfltAppIterState;

Boolean
_xrpiInitAppsIter( whichApps, nameOrId, ppIState )
unsigned	 whichApps;
char		*nameOrId;
Opaque		*ppIState;
{
Boolean	ok=	True;
xrpiIterStatePtr	pIState;

    uENTRY3("_xrpiInitAppsIter(0x%x,0x%x,%s)\n",whichApps,uStringText(nameOrId),
								ppIState);
    if (!(dfltAppIterState.flags&xrpiInUse)) {
	pIState=	&dfltAppIterState;
	pIState->flags=	xrpiInUse;
    }
    else {
	pIState=	uTypedAlloc(xrpiIterStateRec);
	if (pIState==NullIterState) {
	    uRETURN(False);
	}
	pIState->flags=	xrpiInUse|xrpiFreeMe;
    }
    pIState->pIState=	(Opaque)NULL;
    pIState->which=	whichApps;
    pIState->pending=	(Opaque)NULL;

    if (whichApps==XR_ALL_APPS) {
	ok= _AppsInitIter(AllApps,&pIState->pIState);
	if (ok)	pIState->flags|=	xrpiIterInProgress;
    }
    else if (whichApps==XR_ACTIVE_APPS) {
	ok= _AppsInitIter(ActiveApps,&pIState->pIState);
	if (ok)	pIState->flags|=	xrpiIterInProgress;
    }
    else {
	pIState->pending=	(Opaque)xrpaGetApp(whichApps,nameOrId);
	if (pIState->pending!=(Opaque)NULL)
	     pIState->flags|=	xrpiSinglePending;
	else ok= 		False;
    }
    *ppIState=	(Opaque)pIState;
    uRETURN(ok);
}

/***====================================================================***/

Boolean
_xrpiAppsIterNext( inState, ppApp )
Opaque		 inState;
AppPtr		*ppApp;
{
xrpiIterStatePtr pIState= (xrpiIterStatePtr)inState;
Boolean		 ok;

    uENTRY2("_xrpiAppsIterNext(0x%x,0x%x)\n",pIState,ppApp);
    if (pIState->flags&xrpiSinglePending) {
	*ppApp=	(AppPtr)pIState->pending;
	if (pIState->which==XR_QUERY_MULT_APPS) {
	    pIState->pending=(Opaque)xrpaGetApp(XR_QUERY_MULT_APPS,NullString);
	    if (pIState->pending==(Opaque)NULL) {
		pIState->flags&= (~xrpiSinglePending);
	    }
	}
	else pIState->flags&= (~xrpiSinglePending);
	uRETURN(True);
    }
    if (pIState->flags&xrpiIterInProgress) {
	ok= _AppsIterNext(pIState->pIState,ppApp);
	if (!ok) {
	    pIState->flags&= (~xrpiIterInProgress);
	}
	uRETURN(ok);
    }
    uRETURN(False);
}

/***====================================================================***/

void
xrpiEndAppsIterator( inState )
Opaque	inState;
{
xrpiIterStatePtr pIState= (xrpiIterStatePtr)inState;

    uENTRY1("xrpiEndAppsIterator( 0x%x )\n",pIState);
    if ( pIState!= NullIterState ) {
	pIState->flags&=	(~xrpiSinglePending);
	pIState->pending=	(Opaque)NULL;
	if (pIState->pIState!=(Opaque)NULL) {
	    pIState->flags&=	(~xrpiIterInProgress);
	    AppsEndIterator(pIState->pIState);
	    pIState->pIState=	(Opaque)NULL;
	}
	pIState->flags&=	(~xrpiInUse);
	if (pIState->flags&xrpiFreeMe) {
	    uFree((Opaque)pIState);
	}
    }
}

/***====================================================================***/

static	xrpiIterStateRec	dfltRoomsIterState;

Boolean
_xrpiInitRoomsIter( whichRooms, name, ppIState)
unsigned	 whichRooms;
char		*name;
Opaque		*ppIState;
{
Boolean	ok=	True;
xrpiIterStatePtr	pIState;

    uENTRY3("_xrpiInitRoomsIter(0x%x,%s,0x%x)\n",whichRooms,uStringText(name),
							ppIState);
    if (!(dfltRoomsIterState.flags&xrpiInUse)) {
	pIState=	&dfltRoomsIterState;
	pIState->flags=	xrpiInUse;
    }
    else {
	pIState=	uTypedAlloc(xrpiIterStateRec);
	if (pIState==NullIterState) {
	    uRETURN(False);
	}
	pIState->flags=	xrpiInUse|xrpiFreeMe;
    }
    pIState->pIState=	(Opaque)NULL;
    pIState->pending=	(Opaque)NULL;

    if ((whichRooms==XR_ALL_ROOMS)||(whichRooms==XR_VISIBLE_ROOMS)) {
	ok= _RoomsInitIter(&pIState->pIState);
	if (ok)	pIState->flags|=	xrpiIterInProgress;
	pIState->which=	whichRooms;
    }
    else {
	pIState->pending=	(Opaque)xrpaGetRoom(whichRooms,name);
	if (pIState->pending!=(Opaque)NULL)
	     pIState->flags|=	xrpiSinglePending;
	else ok= 		False;
    }
    *ppIState=	(Opaque)pIState;
    uRETURN(ok);
}

/***====================================================================***/

Boolean
_xrpiRoomsIterNext( inState, ppRoom )
Opaque		 inState;
RoomPtr		*ppRoom;
{
xrpiIterStatePtr pIState= (xrpiIterStatePtr)inState;
Boolean		 ok;

    uENTRY2("_xrpiRoomsIterNext(0x%x,0x%x)\n",pIState,ppRoom);
    if (pIState->flags&xrpiSinglePending) {
	*ppRoom=	(RoomPtr)pIState->pending;
	pIState->flags&= (~xrpiSinglePending);
	uRETURN(True);
    }
    if (pIState->flags&xrpiIterInProgress) {
	if (pIState->which==XR_ALL_ROOMS)
		ok= _RoomsIterNext(pIState->pIState,ppRoom);
	else	ok= _RoomsIterNextVisible(pIState->pIState,ppRoom);
	if (!ok) {
	    pIState->flags&= (~xrpiIterInProgress);
	}
	uRETURN(ok);
    }
    uRETURN(False);
}

/***====================================================================***/

void
xrpiEndRoomsIterator( inState )
Opaque	inState;
{
xrpiIterStatePtr pIState= (xrpiIterStatePtr)inState;

    uENTRY1("xrpiEndRoomsIterator( 0x%x )\n",pIState);
    if ( pIState!= NullIterState ) {
	pIState->flags&=	(~xrpiSinglePending);
	pIState->pending=	(Opaque)NULL;
	if (pIState->pIState!=(Opaque)NULL) { 
	    pIState->flags&=	(~xrpiIterInProgress);
	    RoomsEndIterator(pIState->pIState);
	    pIState->pIState=	(Opaque)NULL;
	}
	pIState->flags&=	(~xrpiInUse);
	if (pIState->flags&xrpiFreeMe) {
	    uFree((Opaque)pIState);
	}
    }
}

