
  /*\
   *  $Header: xrSetup.c,v 5.2 90/01/11 15:18:32 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#define	OPAQUE_DEFINED
#define	BOOLEAN_DEFINED
#define	DEBUG_VAR	xrSetupDebug
#include "utils.h"
#include "strtbl.h"
#include "rooms.h"
#include "nameprof.h"
#include "xrDebug.h"
#include "xrApp.h"
#include "xrProfile.h"
#include "xrNames.h"
#include "xrXUtils.h"
#include "xrUI.h"
#include "xrGen.h"
#include "xrooms.h"

/***====================================================================***/

static void
xrUsage()
{
    uInformation("no usage line yet\n");
    exit(0);
}

/***====================================================================***/

static XrmOptionDescRec commandOptions[] = {
    {"-allrooms",	"allRooms",	XrmoptionNoArg,  "on" },
    {"-appstransient",	"appsTransient",XrmoptionNoArg,	 "on" },
    {"-appspermanent",	"appsTransient",XrmoptionNoArg,	 "off"},
    {"-autorooms",	"allRooms",	XrmoptionNoArg,  "off"},
    {"-backup",		"backupConfig", XrmoptionNoArg,  "on" },
    {"-config",		"config",	XrmoptionSepArg, (caddr_t) NULL},
    {"-confirm",	"confirmBox",	XrmoptionNoArg,	 "on" },
    {"-debug",		"debugFlags",	XrmoptionSepArg, (caddr_t)NULL},
    {"-debugxrooms",	"debugXrooms",	XrmoptionSepArg, (caddr_t)0},
    {"-dfltroom",	"defaultRoom",	XrmoptionSepArg, (caddr_t) NULL},
    {"-dxwm",		"useDxwm",	XrmoptionNoArg,  "on"},
    {"-fakeroot",	"findFakeRoot",	XrmoptionNoArg,  "on"},
    {"-fakestate",	"findFakeState",XrmoptionNoArg,  "on"},
    {"-icccm",		"nonICCCM",	XrmoptionNoArg,	 "off"},
    {"-iconifyfirst",	"iconifyFirst", XrmoptionNoArg,"on"},
    {"-iconifylast",	"iconifyLast",  XrmoptionNoArg,"off"},
    {"-ignoreappnames",	"useAppNames",	XrmoptionNoArg,  "off"},
    {"-namelookup",	"nameLookup",	XrmoptionSepArg, (caddr_t)NULL},
    {"-setupmode",	"allowSetupMode",XrmoptionNoArg, "on"},
    {"-noconfirm",	"confirmBox",	XrmoptionNoArg,	 "off" },
    {"-nogeometries",	"useGeometries",XrmoptionNoArg,  "off"},
    {"-nonicccm",	"nonICCCM",	XrmoptionNoArg,	 "on" },
    {"-noappnameatom",	"appNameAtom",	XrmoptionNoArg,  (caddr_t)NULL},
    {"-nosetupmode",	"allowSetupMode",XrmoptionNoArg,  "off"},
    {"-notransients",	"manageTransients",XrmoptionNoArg, "off" },
    {"-noversion",	"printVersion",  XrmoptionNoArg,  "off" },
    {"-nowarnings",	"printWarnings", XrmoptionNoArg,  "off" },
    {"-nowithdrawn",	"manageWithdrawn", XrmoptionNoArg, "off" },
    {"-realroot",	"findFakeRoot",	XrmoptionNoArg,	 "off" },
    {"-reopenforgeom",	"reopenForGeom",XrmoptionNoArg, "on"},
    {"-appnameatom",	"appNameAtom",	XrmoptionSepArg, (caddr_t)NULL},
    {"-saverooms",	"saveRooms",	XrmoptionNoArg,  "on" },
    {"-saveapps",	"saveRooms",	XrmoptionNoArg,  "off" },
    {"-transients",	"manageTransients", XrmoptionNoArg, "on" },
    {"-unmapforgeom",	"unmapForGeom",	XrmoptionNoArg,	 "on" },
    {"-unmapforstate",	"unmapForState",XrmoptionNoArg,	 "on" },
    {"-useappnames",	"useAppNames",	XrmoptionNoArg,  "on" },
    {"-useiconnames",	"useWindowNames",XrmoptionNoArg,  "off" },
    {"-usewinnames",	"useWindowNames",XrmoptionNoArg,  "on" },
    {"-version",	"printVersion",  XrmoptionNoArg,  "on" },
    {"-warnings",	"printWarnings", XrmoptionNoArg,  "on" },
    {"-withdrawn",	"manageWithdrawn", XrmoptionNoArg, "on" },
#ifdef UI_ARGS
    UI_ARGS
#endif
};

/***====================================================================***/

typedef struct xrLegalArgValues {
	String		debugSettings;
	Boolean		printVersion;
	Boolean		printWarnings;
	Boolean		allowSetupMode;

	Boolean		showAllRooms;
	String		defaultRoom;
	Boolean		confirmBox;

	Boolean		iconifyFirst;

	Boolean		dxwm;
	Boolean		dfltAppsTransient;
	Boolean		geometries;
	Boolean		nonICCCM;
	Boolean		reopenForGeom;
	Boolean		unmapForGeom;
	Boolean		unmapForState;
	Boolean		manageTransients;
	Boolean		manageWithdrawn;

	Boolean		useAppNames;
	Boolean		useWindowNames;
	char		*appNameAtom;

	Boolean		backupConfig;
	String		configFile;
	Boolean		saveRoomProfiles;

	String		nameLookupFunc;

	Boolean		findFakeRoot;
	Boolean		findFakeState;
#ifdef UI_FIELDS
	UI_FIELDS
#endif
} xrLegalArgValuesRec,*xrLegalArgValuesPtr;

#define	xrOffset(f)	XtOffset(xrLegalArgValuesPtr,f)

/***====================================================================***/

static XtResource resources[] = {
    {"allowSetupMode",   "AllowSetupMode",  XtRBoolean, sizeof(Boolean),
	 xrOffset(allowSetupMode),  XtRImmediate, (caddr_t)True},
    {"allRooms", "AllRooms", XtRBoolean, sizeof(Boolean),
	xrOffset(showAllRooms), XtRImmediate, (caddr_t)True},
    {"appNameAtom", "AppNameAtom", XtRString, sizeof(char *),
	 xrOffset(appNameAtom), XtRString, (caddr_t) "_XROOMS_APP_NAME" },
    {"appsTransient", "AppsTransient", XtRBoolean, sizeof(Boolean),
	xrOffset(dfltAppsTransient), XtRImmediate, (caddr_t)True},
    {"backupConfig", "BackupConfigFile", XtRBoolean, sizeof(Boolean),
	xrOffset(backupConfig), XtRImmediate, (caddr_t)False},
    {"confirmBox", "ConfirmBox", XtRBoolean, sizeof(Boolean),
	xrOffset(confirmBox), XtRImmediate, (caddr_t) True},
    {"config", "ConfigFile", XtRString, sizeof (char *),
        xrOffset(configFile), XtRString, (caddr_t) NULL},
    {"debugFlags", "DebugFlags", XtRString, sizeof(char *),
	xrOffset(debugSettings), XtRString, (caddr_t) NULL},
    {"defaultRoom", "DefaultRoom", XtRString, sizeof(char *),
	xrOffset(defaultRoom), XtRString, (caddr_t) "Main"},
    {"findFakeRoot", "FindFakeRoot", XtRBoolean, sizeof(Boolean),
	xrOffset(findFakeRoot), XtRImmediate, (caddr_t) False},
    {"findFakeState", "FindFakeState", XtRBoolean, sizeof(Boolean),
	 xrOffset(findFakeState), XtRImmediate, (caddr_t) False},
    {"iconifyFirst",  "IconifyFirst",  XtRBoolean, sizeof(Boolean),
	xrOffset(iconifyFirst), XtRImmediate, (caddr_t) True},
    {"manageTransients", "ManageTransients", XtRBoolean, sizeof(Boolean),
	 xrOffset(manageTransients), XtRImmediate, (caddr_t) False},
    {"manageWithdrawn", "ManageWithdrawn", XtRBoolean, sizeof(Boolean),
	 xrOffset(manageWithdrawn), XtRImmediate, (caddr_t) False},
    {"nameLookup", "NameLookup", XtRString, sizeof(char *),
	 xrOffset(nameLookupFunc), XtRString, (caddr_t) "ChooseAppName" },
    {"nonICCCM",    "NonICCCM",  XtRBoolean, sizeof(Boolean),
	 xrOffset(nonICCCM),  XtRImmediate, (caddr_t)False},
    {"printVersion",  "PrintVersion",  XtRBoolean, sizeof(Boolean),
	 xrOffset(printVersion),  XtRImmediate, (caddr_t)False},
    {"printWarnings",  "PrintWarnings",  XtRBoolean, sizeof(Boolean),
	 xrOffset(printWarnings),  XtRImmediate, (caddr_t)False},
    {"reopenForGeom", "ReopenForGeom", XtRBoolean, sizeof(Boolean),
	 xrOffset(reopenForGeom), XtRImmediate, (caddr_t) False},
    {"saveRooms",   "SaveRooms", XtRBoolean, sizeof(Boolean),
	 xrOffset(saveRoomProfiles), XtRImmediate,(caddr_t)True},
    {"unmapForGeom", "UnmapForGeom", XtRBoolean, sizeof(Boolean),
	 xrOffset(unmapForGeom), XtRImmediate, (caddr_t) False},
    {"unmapForState", "UnmapForState", XtRBoolean, sizeof(Boolean),
	 xrOffset(unmapForState), XtRImmediate, (caddr_t) False},
    {"useAppNames", "UseAppNames", XtRBoolean, sizeof(Boolean),
	 xrOffset(useAppNames), XtRImmediate, (caddr_t) True},
    {"useDxwm",	"UseDXWM", XtRBoolean, sizeof(Boolean),
	 xrOffset(dxwm), XtRImmediate, (caddr_t) False},
    {"useGeometries", "UseGeometries", XtRBoolean, sizeof(Boolean),
	 xrOffset(geometries), XtRImmediate, (caddr_t) True},
    {"useWindowNames", "UseWindowNames", XtRBoolean, sizeof(Boolean),
	 xrOffset(useWindowNames), XtRImmediate, (caddr_t) False},
#ifdef UI_RESOURCES
    UI_RESOURCES(xrLegalArgValuesPtr)
#endif
};

/***====================================================================***/

xrInitializePackages(pRes)
xrLegalArgValuesPtr	pRes;
{
    uENTRY1("xrInitializePackages(0x%x)\n",pRes);

    stInit((unsigned)750,(unsigned)20,False,(Opaque)NULL);
    asSetScreenSize((unsigned)WidthOfScreen(xroomsScrn),
				(unsigned)HeightOfScreen(xroomsScrn));
    SetDefaultRoomFinder(xruiFindDefaultRoom);

    xraInit();
    xrnInit();

    if (pRes->debugSettings!=NullString)
	xrDebugSettings(pRes->debugSettings);

    roomIconifyFirst=		pRes->iconifyFirst;
    xroomsSetupMode=		pRes->allowSetupMode;
    if (pRes->showAllRooms)
	 roomDefaultVisibility= roomAlways|roomVisible;
    else roomDefaultVisibility= roomWhenActive|roomWhenNonEmpty;
    xroomsPrintWarnings=	pRes->printWarnings;

    if (pRes->printVersion) {
	uInformation("xrooms version %d.%02d.%02d  [%s]\n",
			xroomsMajorVersion,xroomsMinorVersion,
			xroomsLocalVersion,xroomsCompileStamp);
    }
    
    xruiDefaultRoom=		pRes->defaultRoom;
    xruiConfirmExit=		pRes->confirmBox;

    xraDfltAppsTransient=	pRes->dfltAppsTransient;
    xraUseGeometries=		pRes->geometries;
    xraNonICCCM=		pRes->nonICCCM;
    xraReopenForGeom=		pRes->reopenForGeom;
    xraUnmapForGeom=		pRes->unmapForGeom;
    xraUnmapForState=		pRes->unmapForState;
    xraManageTransients=	pRes->manageTransients;
    xraManageWithdrawn=		pRes->manageWithdrawn;

    xrpfBackupConfig=		pRes->backupConfig;
    xrpfConfigFile=		pRes->configFile;
    xrpfSaveRoomProfiles=	pRes->saveRoomProfiles;

    xrnUseAppNames=		pRes->useAppNames;
    xrnUseWindowNames=		pRes->useWindowNames;
    if ((pRes->appNameAtom!=NullString)&&(strlen(pRes->appNameAtom)>0))
	 xrnAppNameAtom=	XInternAtom(xroomsDpy,pRes->appNameAtom,False);
    else xrnAppNameAtom=	None;

    if (pRes->nameLookupFunc!=NullString) {
	nprDefaultName=		stGetToken(pRes->nameLookupFunc);
	xrnNameLookup=		nprDefaultName;
    }

    xruFindFakeRoot=		pRes->findFakeRoot;
    xruFindFakeState=		pRes->findFakeState;

    if (pRes->dxwm) {
	xraNonICCCM=		True;
	xruFindFakeRoot=	True;
	xruFindFakeState=	True;
	xraReopenForGeom=	True;
	roomIconifyFirst=	False;
    }

    UI_SETTINGS(pRes);
    uVOIDRETURN;
}

/***====================================================================***/

static xrLegalArgValuesRec	xrResources;

Boolean
xrInitialize(argc, argv)
int	  argc;
char	**argv;
{
Widget			 topLevel;

    uENTRY2("xrInitialize(%d,0x%x)\n",argc,argv);
    if ((argc==2)&&(uStringEqual(argv[1],"-help"))) {
	xrUsage();
	/* NOTREACHED */
    }
    topLevel = XtInitialize("xrooms", "XRooms",
			    commandOptions, XtNumber(commandOptions),
			    &argc, argv);
    if (topLevel==(Widget)NULL) {
	uRETURN(False);
    }
    XtGetApplicationResources(topLevel,&xrResources,
						resources,XtNumber(resources),
						NULL, (Cardinal) 0);
    xroomsDpy=	XtDisplay(topLevel);
    xroomsScrn=	XtScreen(topLevel);

    WM_STATE=		XInternAtom(xroomsDpy, "WM_STATE", False);
    WM_CHANGE_STATE=	XInternAtom(xroomsDpy, "WM_CHANGE_STATE", False);
    WM_MOVED=		XInternAtom(xroomsDpy, "WM_MOVED", False);
    XSetErrorHandler(xruIgnoreErrors);

    xrInitializePackages(&xrResources);

    xroomsRealRoot=	RootWindowOfScreen(xroomsScrn);
    xroomsRoot=		xruFindRoot();

    xrgenInit(xroomsDpy);
    xruiInit(topLevel);
    uRETURN(True);
}

