
  /*\
   *  $Header: xrXUtils.c,v 5.0 90/01/10 07:01:42 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xproto.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#define	DEBUG_VAR xruDebug
#include "utils.h"
#include "appstate.h"

#include "xrDebug.h"
#include "xrooms.h"
#include "xrXUtils.h"

/***====================================================================***/

Boolean	xruFindFakeRoot;
Boolean	xruFindFakeState;

/***====================================================================***/

Boolean
xruGetChildren(window, children, nChildren)
Window	  window;
Window	**children;
int	 *nChildren;
{
Window	  rtrnRoot,rtrnParent;

    uENTRY3("xruGetChildren(0x%x,0x%x,0x%x)\n",window,children,nChildren);
    if (XQueryTree(xroomsDpy, window, &rtrnRoot, &rtrnParent, children, 
								nChildren)) {
	uRETURN(True);
    }
    uRETURN(False);
}


/***====================================================================***/

Window
xruGetParent(window)
Window	  window;
{
Window	  rtrnRoot,rtrnParent;
Window	 *children= NULL;
int	  nChildren;

    uENTRY1("xruGetParent(0x%x)\n",window);
    if (XQueryTree(xroomsDpy,window,&rtrnRoot,&rtrnParent,&children,
							&nChildren)) {
	if ((nChildren>0)&&(children))
	    (void)XFree	(children);
	uRETURN(rtrnParent);
    }
    uRETURN(None);
}

/***====================================================================***/

/*
 * Find the window manager root window (the parent of the reparented
 * application windows).  For most window managers, this is simply
 * RootWindow(dpy, screen).  For dxwm, this is it's full-screen
 * pseudo-root.
 *
 * To find the window manager root window,
 *   1.  If DEC_WM_DECORATION_GEOMETRY is not an interned atom, return
 *       RootWindow(dpy, screen)
 *   2.  Look for a direct child of RootWindow(dpy, screen) that is
 *       full-screen and has override_redirect = True.  [=top]
 *   3.  If top does not exist, return RootWindow(dpy, screen).
 *   4.  If top does not have a single, full-screen child, return
 *       RootWindow(dpy, screen).  [=WmRoot]
 *   5.  Otherwise, return WmRoot.
 *
 */


Window
xruWMRoot(dpy, root, xdw, xdh)
Display *dpy;
Window root;
int xdw, xdh;
{
  Atom dec_wm;
  Window top, ig_win, *children;
  int numchildren, i;
  XWindowAttributes xwa;

  dec_wm = XInternAtom(dpy, "DEC_WM_DECORATION_GEOMETRY", True);
  if (dec_wm == None)
    return root;
  XQueryTree(dpy, root, &ig_win, &ig_win, &children, &numchildren);
  top = NULL;
  for (i=0; i<numchildren; i++) {
    XGetWindowAttributes(dpy, children[i], &xwa);
    if (xwa.x == 0 && xwa.y == 0 &&
	xwa.width == xdw &&
	xwa.height == xdh &&
	xwa.border_width == 0 &&
	xwa.override_redirect && (xwa.map_state==IsViewable)) {
      top = children[i];
      break;
    }
  }
  if (!top)
    return root;
  XQueryTree(dpy, top, &ig_win, &ig_win, &children, &numchildren);
  for (i=0; i<numchildren; i++) {
    XGetWindowAttributes(dpy, children[i], &xwa);
    if (xwa.x == 0 && xwa.y == 0 &&
	xwa.width == xdw &&
	xwa.height == xdh &&
	xwa.border_width == 0 &&
	!xwa.override_redirect) {
      return children[i];
    }
  }
  return root;
}

/***====================================================================***/

Window
xruWMRootWindow(dpy, screen)
Display *dpy;
int screen;
{
  return xruWMRoot(dpy, RootWindow(dpy, screen),
		XDisplayWidth(dpy, screen), XDisplayHeight(dpy, screen));
}

/***====================================================================***/

/*
 * Find the "root" window; the window in which the window manager puts
 * decorated windows.  Almost always the real root window (except for
 * things like dxwm.)
 */
Window
xruFindRoot()
{
    uENTRY("xruFindRoot()\n");
    if (xruFindFakeRoot) {
	uRETURN(xruWMRootWindow(xroomsDpy,xroomsScreenIndex));
    }
    uRETURN(XRootWindow(xroomsDpy,xroomsScreenIndex));
}

/***====================================================================***/

/*
 * Return the iconic state for the window.  If it doesn't have one,
 * return asInactive.
 */

WinState
xruGetState(window,lookForName)
Window	window;
Boolean	lookForName;
{
Atom		 actualType;
unsigned long	 actualFormat, nItems, bytesAfter;
char		*propRtrn;
WinState	 result;

    uENTRY2("xruGetState(0x%x,%s)\n",window,booleanText(lookForName));
    if (XGetWindowProperty(xroomsDpy, window, WM_STATE, 0, 1, False,
		       AnyPropertyType, &actualType, &actualFormat,
		       &nItems, &bytesAfter, &propRtrn)==Success) {
	if ((actualType != None)&&(nItems>0)) {
	    bcopy(propRtrn, (char *) &result, sizeof(int));
	    XFree(propRtrn);
	    uRETURN(result);
	}
    }

    if ((lookForName)&&(XGetWindowProperty(xroomsDpy,window,XA_WM_NAME,0,1,
			False,AnyPropertyType, &actualType, &actualFormat,
			&nItems, &bytesAfter, &propRtrn)==Success)) {

	if ((actualType != None)&&(nItems>0)) {
	    uDEBUG2(WATCH_APPS,"window 0x%x has name \"%s\"but no state\n",
						window,propRtrn);
	    if (propRtrn!=NullString)
		XFree(propRtrn);
	    if (xruFindFakeState) {
		XWindowAttributes xwa;
		XGetWindowAttributes(xroomsDpy,window,&xwa);
		if (!xwa.override_redirect) {
		   if (xwa.map_state==IsViewable)	result=	asNormal;
		   else 				result=	asIconic;
		}
		else result=	asInactive;
		uDEBUG1(WATCH_APPS,"faking %s state\n",asWinStateText(result));
	    }
	    else {
		XSelectInput(xroomsDpy, window, PropertyChangeMask);
		result= xruGetState(window,False);
		uDEBUG2(WATCH_APPS,"second attempt at 0x%x returns %d\n",
							window,result);
	    }
	    uRETURN(result);
	}
    }
    uRETURN(asInactive);
}

/***====================================================================***/


Boolean
xruIsTransient(window)
Window	window;
{
Atom		 actualType;
unsigned long	 actualFormat, nItems, bytesAfter;
char		*propRtrn;

    uENTRY1("xruIsTransient(0x%x)\n",window);
    if (XGetWindowProperty(xroomsDpy, window, XA_WM_TRANSIENT_FOR, 0, 1, False,
		       AnyPropertyType, &actualType, &actualFormat,
		       &nItems, &bytesAfter, &propRtrn)==Success) {
	if ((actualType != None)&&(nItems>0)) {
	    if (propRtrn!=NullString) XFree(propRtrn);
	    uRETURN(True);
	}
    }
    uRETURN(False);
}

/***====================================================================***/

Window 
xruFindClientWindowAbove(window, pState)
Window	 	 window;
WinState	*pState;
{
WinState	 state;

    uENTRY2("xruFindClientWindowAbove(0x%x,0x%x)\n",window,pState);
    do {
	state= xruGetState(window,True);
	if (state != asInactive) {
	    if (pState!=(WinState *)NULL)
	    *pState = state;
	    uRETURN(window);
	}
	window=	xruGetParent(window);
    } while ((window!=None)&&(window!=xroomsRoot)&&(window!=xroomsRealRoot));

    uRETURN(None);
}

/***====================================================================***/

Window 
xruFindClientWindow(window, pState)
Window	 	 window;
WinState	*pState;
{
WinState	 state;
Window		*children;
int	 	 i,nChildren;
Window	 	 result;

    uENTRY2("xruFindClientWindow(0x%x,0x%x)\n",window,pState);
    state= xruGetState(window,True);
    if (state != asInactive) {
	if (pState!=(WinState *)NULL)
	    *pState = state;
	uRETURN(window);
    }

    result=	NULL;
    if (xruGetChildren(window, &children, &nChildren)) {
	for (i=0;(i<nChildren)&&(!result); i++) {
	    result= xruFindClientWindow(children[i], pState);
	}

	if (nChildren)
	    XFree((char *) children);
    }
    uRETURN(result);
}

/***====================================================================***/

Boolean
xruGetPropertyString(dpy, window, name, pRtrn)
Display		 *dpy;
Window		  window;
Atom		  name;
char		**pRtrn;
{
Atom		 actualtype;
unsigned long	 actualformat, nitems, bytesafter;
char		*propreturn = NULL;
char		*ptr, buf[20];
XSizeHints	*pHints;
int		 i;

    if (pRtrn!=(char **)NULL)
	*pRtrn=	NullString;
    if (name == None)
	return False;
    if (XGetWindowProperty(dpy, window, name, 0, 99999, False,
			    AnyPropertyType, &actualtype, &actualformat,
			    &nitems, &bytesafter, &propreturn)==Success) {
	if (bytesafter > 0) {
	    uWarning("Far too big a property was stored.\n");
	    return False;
	}
	if ((actualtype == None)||(propreturn==NULL)) {
	    uDEBUG(WATCH_APPS,"property not found\n");
	    return False;
	}
    }
    else {
	uDEBUG(WATCH_APPS,"XGetWindowProperty failed\n");
	return False;
    }

    if (actualtype == XA_STRING) {
	for (i=nitems * actualformat / 8 , ptr=propreturn ; i > 0 ; i--, ptr++)
	    if (*ptr == 0) *ptr = ' ';
	if (pRtrn!=(char **)NULL)
	    *pRtrn=	propreturn;
	return True;
    }
    else if (actualtype == XA_WM_SIZE_HINTS) {
	pHints= (XSizeHints *)propreturn;
	if ((pHints->flags & (USSize | USPosition))&&(pRtrn!=(char **)NULL)) {
	    *pRtrn=	propreturn = (char *) malloc(30);
	    *propreturn = 0;
	    if (pHints->flags & USSize)
		sprintf(propreturn, "%dx%d", pHints->width, pHints->height);
	    if (pHints->flags & USPosition) {
		sprintf(buf, "+%d+%d", pHints->x, pHints->y);
		strcat(propreturn, buf);
	    }
	}
	(void)XFree(pHints);
	return True;
    }
    else if (actualtype == WM_STATE) {
	WinState	result;
	bcopy(propreturn, (char *) &result, sizeof(int));
	XFree(propreturn);
	if (pRtrn!=(char **)NULL) {
	    ptr=	asWinStateText(result);
	    *pRtrn = uStringDup(ptr);
	}
	return True;
    }
    else {
	XFree(propreturn);
	return(True);
    }
}

/***====================================================================***/

int 
xruIgnoreErrors(dpy, event)
Display		*dpy;
XErrorEvent 	*event;
{
    if ((event->request_code == X_GetProperty)||
	(event->request_code == X_MapWindow)||
	(event->request_code == X_QueryTree)) {
	if (DEBUG_VAR&WATCH_ERRORS)
	    _XPrintDefaultError(dpy, event, stderr);
	return 0;
    }
    _XPrintDefaultError(dpy, event, stderr);
    return 0;
}


