
  /*\
   *  $Header: xrXawRoom.c,v 5.2 90/01/11 17:09:51 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xos.h>
#include <X11/Intrinsic.h>
#include <X11/Xutil.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>

#ifdef X11R4
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#else
#include <X11/Command.h>
#include <X11/Form.h>
#endif

#define	BOOLEAN_DEFINED
#define	OPAQUE_DEFINED
#define	DEBUG_VAR	xrRoomDebug
#include "utils.h"
#include "hash.h"

#include "rooms.h"

#include "xrDebug.h"
#include "xrXUtils.h"
#include "xrRoom.h"
#include "xrooms.h"

#include "xrUI.h"
#include "xrGen.h"
#include "xrProto.h"

static	Widget 	xrrInverted=		NULL;
unsigned	xawButtonWidth;
unsigned	xawButtonCols;


/***====================================================================***/


static void
xrrFlipColors(widget)
Widget	widget;
{
    static Arg arglist[] = {
	{XtNforeground, NULL},
	{XtNbackground, NULL},
    };
    XtArgVal temp;

    uENTRY1("FlipColors(0x%x)\n",widget);
    arglist[0].value = arglist[1].value = NULL;
    XtGetValues(widget, arglist, XtNumber(arglist));
    temp = arglist[0].value;
    arglist[0].value = arglist[1].value;
    arglist[1].value = temp;
    XtSetValues(widget, arglist, XtNumber(arglist));
    uVOIDRETURN;
}

/***====================================================================***/

#define MAX_BUTTONS 100
static Widget buttons[MAX_BUTTONS];
static int num_buttons = 0;
static void
xrrAdjustButtons()
{
    Widget lasth = NULL, lastv = NULL;
    int each, count = 0;
    static Arg args[] = {
        {XtNfromHoriz, (XtArgVal) NULL},
        {XtNfromVert, (XtArgVal) NULL},
        {XtNhorizDistance, (XtArgVal) 3},
        {XtNvertDistance, (XtArgVal) 3},
    };

    for(each=0; each<num_buttons; each++) {
      if (XtIsManaged(buttons[each])) {
	args[0].value = (XtArgVal) lasth;
	args[1].value = (XtArgVal) lastv;
	XtSetValues(buttons[each], args, XtNumber(args));
	if ((++count % xawButtonCols) == 0) {
	  lasth = NULL;
	  lastv = buttons[each];
	} else {
	  lasth = buttons[each];
	}
      }
    }

    if (XtIsRealized(xruiScrollW)) {
      /* For some reason, this causes the layout changes to take effect */
      XtSetMappedWhenManaged(xruiMenu, False);
      XtManageChild(xruiMenu);
      XtUnmanageChild(xruiMenu);
      XtSetMappedWhenManaged(xruiMenu, True);
    }
}

/***====================================================================***/

static void
xrrManageRoom(pRoom)
RoomPtr	pRoom;
{
XRoomInfoPtr	pRoomInfo;

    uENTRY1("xrrManageRoom(%s)\n",roomText(pRoom));
    if (pRoom!=NullRoom) {
	pRoomInfo=	(XRoomInfoPtr)roomGetPrivate(pRoom);
	XtManageChild(pRoomInfo->widget);
	xrrAdjustButtons();
    }
    uVOIDRETURN;
}

/***====================================================================***/

static void
xrrInvertButton(widget)
Widget	widget;
{
    uENTRY1("InvertButton(0x%x)\n",widget);
    xrrInverted = widget;
    xrrFlipColors(widget);
    uVOIDRETURN;
}

/***====================================================================***/

static void
xrrUninvertButton()
{
    uENTRY("xrrUninvertButton()\n");
    if (xrrInverted) xrrFlipColors(xrrInverted);
    xrrInverted= NULL;
    uVOIDRETURN;
}

/***====================================================================***/

static void 
xrrDoButtonPress(widget, p, data)
Widget	widget;
Opaque	p, data;
{
RoomPtr pRoom=		(RoomPtr)p;

    uENTRY3("xrrDoButtonPress(0x%x,%s,0x%x)\n",widget,roomText(pRoom),data);
    SwitchToRoom(pRoom);
    uVOIDRETURN;
}

/***====================================================================***/

static void
xrrUnmanageRoom(pRoom)
RoomPtr	pRoom;
{
XRoomInfoPtr	pRoomInfo;

    uENTRY1("xrrUnmanageRoom(%s)\n",roomText(pRoom));
    if (pRoom!=NullRoom) {
	pRoomInfo=	(XRoomInfoPtr)roomGetPrivate(pRoom);
	if (!(roomGetVisibility(pRoom)&roomVisible)) {
	    if (pRoomInfo->widget==xrrInverted) {
		xrrUninvertButton();
	    }
	    XtUnmanageChild(pRoomInfo->widget);
	    xrrAdjustButtons();
	}
    }

    uVOIDRETURN;
}

/***====================================================================***/

extern void xruiHandlePopup();

static Widget 
xrrAddButtonForRoom(pRoom)
RoomPtr		pRoom;
{
Widget	 result= NULL;
char	*name;
XRoomInfoPtr	pRoomInfo;

    static Arg args[] = {
	{XtNlabel, (XtArgVal) NULL},
	{XtNwidth, (XtArgVal) 0},
	{XtNresizable, (XtArgVal) TRUE},
	{XtNtop, (XtArgVal) XtChainTop},
	{XtNbottom, (XtArgVal) XtChainTop},
	{XtNleft, (XtArgVal) XtChainLeft},
	{XtNright, (XtArgVal) XtChainLeft},
    };

    uENTRY1("xrrAddButtonForRoom(%s)\n",roomText(pRoom));
    if (pRoom!=NullRoom) {
	name=	stGetString(roomName(pRoom));
	args[0].value = (XtArgVal)name;
	args[1].value = (XtArgVal)xawButtonWidth;
	result = XtCreateWidget(name, commandWidgetClass, xruiScrollW,
				args, XtNumber(args));
	XtAddEventHandler(result,ButtonPressMask,False,xruiHandlePopup,NULL);
	buttons[num_buttons++] = result;
	XtAddCallback(result, XtNcallback, xrrDoButtonPress, pRoom);
	pRoomInfo=	uTypedAlloc(XRoomInfoRec);
	if (pRoomInfo!=NullXRoomInfo) {
	    pRoomInfo->widget=		result;
	    pRoomInfo->nameAtom=	XInternAtom(xroomsDpy,name,False);
	}
	roomSetPrivate(pRoom,(Opaque)pRoomInfo);
    }
    uRETURN(result);
}

/***====================================================================***/

static void
xrrButtonsRearranged()
{
    uENTRY("xrrButtonsRearranged()\n");
/*    SaveNeeded();*/
    uVOIDRETURN;
}

/***====================================================================***/

void
xrrRoomFeedback(pRoom,what,pApp)
RoomPtr		pRoom;
unsigned	what;
AppPtr		pApp;
{
XRoomInfoPtr	pRoomInfo;

    uENTRY3("xrrRoomFeedback(%s,%d,%s)\n",roomText(pRoom),what,appText(pApp));
    pRoomInfo=	(XRoomInfoPtr)roomGetPrivate(pRoom);
    switch (what) {
	case roomFBCreated:
		if (pRoomInfo==NullXRoomInfo) {
		    xrrAddButtonForRoom(pRoom);
		    xrpUpdateAllRooms(xroomsDpy,xroomsRealRoot);
		}
		break;
	case roomFBDestroyed:
		if (pRoomInfo!=NullXRoomInfo) {
		    xrrUnmanageRoom(pRoom);
		    uFree((Opaque)pRoomInfo);
		    xrpUpdateAllRooms(xroomsDpy,xroomsRealRoot);
		}
		break;
	case roomFBVisible:
		xrrManageRoom(pRoom);
		xrpUpdateAllRooms(xroomsDpy,xroomsRealRoot);
		break;
	case roomFBHidden:
		if (pRoom==GetCurrentRoom())
		    SwitchToRoom(xruiFindDefaultRoom());
		xrrUnmanageRoom(pRoom);
		xrpUpdateAllRooms(xroomsDpy,xroomsRealRoot);
		break;
	case roomFBActivated:
		if (xrrInverted!=NULL)	xrrUninvertButton();
		if (pRoomInfo!=NullXRoomInfo) {
		    if (roomIsVisible(pRoom)) {
			xrrInvertButton(pRoomInfo->widget);
		    }
		    xrpSetCurrent(xroomsDpy,xroomsRealRoot,pRoomInfo->nameAtom);
		}
		uDEBUG1(WATCH_EVENTS,"Switching to room %s\n",roomText(pRoom));
		break;
	case roomFBDeactivated:
		if (pRoomInfo!=NullXRoomInfo) {
		    if (roomIsVisible(pRoom)) {
			xrrUninvertButton();
		    }
		}
		break;
    }
    uVOIDRETURN;
}

/***====================================================================***/

Opaque
xrrInit(parent)
Widget	parent;
{
    uENTRY1("xrrInit(0x%x)\n",parent);
    SetDefaultRoomFeedback(xrrRoomFeedback);
    uRETURN((Opaque)parent);
}
