
  /*\
   *  $Header: xrXawUI.c,v 5.4 90/01/11 16:21:11 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <X11/Xos.h>
#include <X11/Intrinsic.h>
#include <X11/Xutil.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <X11/Xatom.h>

#include <X11/Shell.h>
#ifdef X11R4
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#else
#include <X11/Form.h>
#include <X11/Command.h>
#include <X11/Dialog.h>
#endif
#include <X11/IntrinsicP.h>

#define	BOOLEAN_DEFINED
#define	OPAQUE_DEFINED
#define	DEBUG_VAR	xruiDebug
#include "utils.h"
#include "hash.h"

#include "rooms.h"

#include "xrActions.h"
#include "xrDebug.h"
#include "xrXUtils.h"
#include "xrProfile.h"
#include "xrRoom.h"
#include "xrooms.h"

#include "xrUI.h"

/***====================================================================***/

Widget	 xruiTopLevel,xruiMenu,xruiWinMenu;
Widget	 xruiSetupModeItem;
Widget	 xruiPromptWidget = NULL;
Widget	 xruiPromptText = NULL;

Widget	 xruiScrollW;

char	*xruiDefaultRoom;
Boolean	 xruiConfirmExit;

/***====================================================================***/

static void 
xruiDestroyPromptWindow()
{
    uENTRY("xruiDestroyPromptWindow()\n");
    if (xruiPromptWidget != NULL) {
	XtDestroyWidget(xruiPromptWidget);
	xruiPromptWidget = NULL;
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiPopDownWinMenu()
{
  XtPopdown(xruiWinMenu);
}

/***====================================================================***/

static	Boolean	xruiNeedString;

static void 
xruiDoPromptNo(widget,clientData,callData)
Widget	widget;
Opaque	clientData;
Opaque	callData;
{
    uENTRY3("xruiDoPromptNo(0x%x,0x%x,0x%x)\n",widget,clientData,callData);
    xruiDestroyPromptWindow();
    uVOIDRETURN;
}

/***====================================================================***/

void	(*xruiCallback)();

void 
xruiDoPromptYes(widget,clientData,callData)
Widget	widget;
Opaque	clientData;
Opaque	callData;
{
void	(*callback)(/* str */);
char	*ptr;

    uENTRY3("xruiDoPromptYes(0x%x,0x%x,0x%x)\n",widget,clientData,callData);
    callback= xruiCallback;
    if (xruiPromptWidget != NULL) {
	if (xruiNeedString) {
	    Arg args[1];

	    XtSetArg(args[0], XtNstring, &ptr);
	    XtGetValues(xruiPromptText, args, 1);
	    if (ptr && !strlen(ptr)) {
	      ptr = NullString;
	    }
	    xruiDestroyPromptWindow();
	    if ((ptr!=NullString)&&(index(ptr,';')!=NullString)) {
		*(index(ptr,';'))=	NUL;
	    }
	    if (callback!=NULL) {
		(*callback)(ptr);
	    }
	    if (ptr!=NullString)
		XtFree(ptr);
	}
	else if (callback!=NULL)
	    (*callback)(NullString);
    }
    uVOIDRETURN;
}


/***====================================================================***/

void
xruiRaiseMenu(widget, subwidget, event)
Widget	 widget;
Widget	 subwidget;
XEvent  *event;
{
    XRaiseWindow(XtDisplay(widget), XtWindow(widget));
}

/***====================================================================***/

void
xruiFocusPopup(widget, subwidget, event)
Widget	 widget;
Widget	 subwidget;
XEvent  *event;
{
    if (event->type == Expose || event->type == MapNotify) {
      XSetInputFocus(XtDisplay(subwidget), XtWindow(subwidget),
		     RevertToPointerRoot, CurrentTime);
      XtRemoveEventHandler(widget, XtAllEvents, True, xruiFocusPopup, 0);
    }
}

/***====================================================================***/

void
xruiCenterWidget(w)
Widget w;
{
  Window ignorew;
  int x, y, ignorei;
  unsigned int ignoreu;
  Dimension width, height;
  Arg args[5];
  int count;

  XQueryPointer(XtDisplay(w), XtWindow(w), &ignorew, &ignorew, &x, &y,
		&ignorei, &ignorei, &ignoreu);

  count = 0;
  XtSetArg(args[count], XtNwidth, &width); count++;
  XtSetArg(args[count], XtNheight, &height); count++;
  XtGetValues(w, args, count);

  x = x - width/2;
  y = y - height/2;
  if (x+width > WidthOfScreen(XtScreen(w)))
    x = WidthOfScreen(XtScreen(w)) - width;
  if (y+height > HeightOfScreen(XtScreen(w)))
    y = HeightOfScreen(XtScreen(w)) - height;
  if (x < 0) x = 0;
  if (y < 0) y = 0;

  count = 0;
  XtSetArg(args[count], XtNx, x); count++;
  XtSetArg(args[count], XtNy, y); count++;
  XtSetValues(w, args, count);
}


/***====================================================================***/

void
xruiTopWidget(w)
Widget w;
{
  Window ignorew;
  int x, y, ignorei;
  unsigned int ignoreu;
  Dimension width, height;
  Arg args[5];
  int count;

  XQueryPointer(XtDisplay(w), XtWindow(w), &ignorew, &ignorew, &x, &y,
		&ignorei, &ignorei, &ignoreu);

  count = 0;
  XtSetArg(args[count], XtNwidth, &width); count++;
  XtSetArg(args[count], XtNheight, &height); count++;
  XtGetValues(w, args, count);

  x = x - width/2;
  if (x+width > WidthOfScreen(XtScreen(w)))
    x = WidthOfScreen(XtScreen(w)) - width;
  if (y+height > HeightOfScreen(XtScreen(w)))
    y = HeightOfScreen(XtScreen(w)) - height;
  if (x < 0) x = 0;
  if (y < 0) y = 0;

  count = 0;
  XtSetArg(args[count], XtNx, x); count++;
  XtSetArg(args[count], XtNy, y); count++;
  XtSetValues(w, args, count);
}


/***====================================================================***/

void
xruiDoPrompt(prompt,needString,callback)
char	 *prompt;
Boolean	  needString;
void	(*callback)();
{

    Arg dargs[10];
    int cnt = 0;
    static XtTranslations promptoverride = NULL;
    static Arg shellArgs[] = {
	{XtNallowShellResize, (XtArgVal) True},
	{XtNinput, (XtArgVal) True},
    };
    Widget button;
    DialogWidget dw;

    uENTRY3("xruiDoPrompt(%s,%s,0x%x)\n",uStringText(prompt),
					booleanText(needString),callback);
    xruiDestroyPromptWindow();

    xruiPromptWidget = XtCreatePopupShell("popup",
	transientShellWidgetClass, xruiTopLevel, shellArgs,
					  XtNumber(shellArgs));

    XtSetArg(dargs[cnt], XtNvalue, ""); cnt++;
    XtSetArg(dargs[cnt], XtNlabel, prompt); cnt++;
    XtSetArg(dargs[cnt], XtNinput, True); cnt++;
    dw = (DialogWidget) XtCreateManagedWidget("dialog", dialogWidgetClass,
					      xruiPromptWidget,
					      dargs, cnt);

    button = XtCreateManagedWidget("Yes", commandWidgetClass, dw, NULL, 0);
    XtAddCallback(button, XtNcallback, xruiDoPromptYes, NULL);
    button = XtCreateManagedWidget("No", commandWidgetClass, dw, NULL, 0);
    XtAddCallback(button, XtNcallback, xruiDoPromptNo, NULL);

    xruiPromptText = XtNameToWidget(dw, "value");
    if (needString && xruiPromptText) {
      XtSetKeyboardFocus(xruiTopLevel, None);
      XtSetKeyboardFocus(dw, xruiPromptText);
      XtAddEventHandler(dw, ExposureMask, False,
			xruiFocusPopup, xruiPromptText);
    }

    xruiCallback = callback;
    if (needString) {
	if (promptoverride == NULL)
	    promptoverride = 
		XtParseTranslationTable("<Key>0xff0d: do-prompt-yes()");
	XtOverrideTranslations(xruiPromptText, promptoverride);	
    } else {
      XtUnmanageChild(xruiPromptText);
    }
    xruiNeedString=	needString;

    XtRealizeWidget(xruiPromptWidget);
    xruiCenterWidget(xruiPromptWidget);
    XtPopup(xruiPromptWidget, XtGrabExclusive);
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiHandlePopup(widget, closure, event)
Widget	 widget;
Opaque	 closure;
XEvent	*event;
{
Arg arg;

    uENTRY3("xruiHandlePopup(0x%x,0x%x,0x%x)\n",widget,closure,event);
    if (event->type != ButtonPress || event->xbutton.button != 2) {
	uVOIDRETURN;
    }
    if (xroomsSetupMode) {
	XtSetArg(arg, XtNlabel, (XtArgVal)"Disable Set-up Mode");
	XtSetValues(xruiSetupModeItem, &arg, 1);
    }
    else {
	XtSetArg(arg, XtNlabel, (XtArgVal)"Enable Set-up Mode");
	XtSetValues(xruiSetupModeItem, &arg, 1);
    }

    XtRealizeWidget(xruiMenu);
    xruiTopWidget(xruiMenu);
    XtPopup(xruiMenu, XtGrabExclusive);
    uVOIDRETURN;
}


/***====================================================================***/

void
xruiDoSaveProfile()
{
    uENTRY("xruiDoSave()\n");
    XtPopdown(xruiMenu);
    xruiDoPrompt("Enter file to write (<cr> for default):",
						True,xrpfSaveProfiles);
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiDoLoadProfile()
{
    uENTRY("xruiDoSave()\n");
    XtPopdown(xruiMenu);
    xruiDoPrompt("Enter file to load (<cr> for default):",True,xrpfLoadFile);
    uVOIDRETURN;
}

/***====================================================================***/

static void
xruiReallyCreateRoom(roomName)
char	*roomName;
{
StringToken	token;
RoomPtr		pRoom;

    uENTRY1("xruiReallyCreateRoom(%s)\n",uStringText(roomName));
    if (roomName!=NullString) {
	token=	stGetToken(roomName);
	pRoom=	LookupRoom(token);
	if (pRoom==NullRoom)
	    pRoom= CreateRoom(token);
	if (pRoom!=NullRoom) {
	    roomSetVisibility(pRoom,roomGetVisibility(pRoom)|roomVisible,False);
	}
    }
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiDoCreateRoom()
{
char		*roomName= NullString;

    uENTRY("xruiDoCreateRoom()\n");
    XtPopdown(xruiMenu);
    xruiDoPrompt("Enter room to create/show:",True,xruiReallyCreateRoom);
    uVOIDRETURN;
}

/***====================================================================***/

static void
xruiReallyDestroyRoom(roomName)
char		*roomName;
{
StringToken	 token;
RoomPtr		 pRoom;

    uENTRY1("xruiReallyDestroyRoom(%s)\n",uStringText(roomName));
    if (roomName!=NullString) {
	token=	stGetToken(roomName);
	if (token!=NullStringToken) {
	    pRoom=	LookupRoom(token);
	}
    }
    else pRoom=	GetCurrentRoom();
    if (pRoom!=NullRoom)
	DestroyRoom(pRoom);
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiDoDestroyRoom()
{
    uENTRY("xruiDoDestroyRoom()\n");
    XtPopdown(xruiMenu);
    xruiDoPrompt("Enter room to destroy (<cr> for current):",True,
					xruiReallyDestroyRoom);
    uVOIDRETURN;
}

/***====================================================================***/

static void
xruiReallyHideRoom(roomName)
char		*roomName;
{
StringToken	 token;
RoomPtr		 pRoom;

    uENTRY1("xruiReallyHideRoom(%s)\n",uStringText(roomName));
    if (roomName!=NullString) {
	token=	stGetToken(roomName);
	if (token!=NullStringToken) {
	    pRoom=	LookupRoom(token);
	}
    }
    else pRoom=	GetCurrentRoom();
    if (pRoom!=NullRoom)
	roomSetVisibility(pRoom,roomGetVisibility(pRoom)&(~roomVisible),False);
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiDoHideRoom()
{
    uENTRY("xruiDoHideRoom()\n");
    XtPopdown(xruiMenu);
    xruiDoPrompt("Enter room to hide (<cr> for current):",True,
					xruiReallyDestroyRoom);
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiDoWindowStates()
{
    uENTRY("xruiDoWindowStates()\n");
    XtPopdown(xruiMenu);
    XtRealizeWidget(xruiWinMenu);
    xruiTopWidget(xruiWinMenu);
    XtPopup(xruiWinMenu, XtGrabExclusive);
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiCloseMainMenu()
{
    uENTRY("xruiCloseMainMenu()\n");
    XtPopdown(xruiMenu);
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiCloseWinMenu()
{
    uENTRY("xruiCloseWinMenu()\n");
    XtPopdown(xruiWinMenu);
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiFlipSetupMode()
{
    uENTRY("xruiFlipSetupMode()\n");
    XtPopdown(xruiMenu);
    xroomsSetupMode=	!xroomsSetupMode;
    uVOIDRETURN;
}

/***====================================================================***/

/* ARGSUSED */
static void
xruiReallyExit(ignoreMe)
char	*ignoreMe;
{
    uENTRY1("xruiReallyExit(%s)\n",uStringText(ignoreMe));
    DoExit(0,"Exit menu item selected");
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiDoExit()
{
    uENTRY("xruiDoExit()\n");
    XtPopdown(xruiMenu);
    if (xruiConfirmExit)
	xruiDoPrompt("Do you really want to exit xrooms?",False,xruiReallyExit);
    else DoExit(0,"Exit menu item selected");
    uVOIDRETURN;
}


/***====================================================================***/

static RoomPtr
xruiCreateDefaultRoom()
{
RoomPtr	pRoom;

    uENTRY("xruiCreateDefaultRoom()\n");
    pRoom=	LookupRoom(stGetToken(xruiDefaultRoom));
    if (pRoom==NullRoom) 
	pRoom= CreateRoom(stGetToken(xruiDefaultRoom));
    if (pRoom!=NullRoom)
	roomSetVisibility(pRoom,roomVisible,True);
    uRETURN(pRoom);
}

/***====================================================================***/

RoomPtr
xruiFindDefaultRoom()
{
RoomPtr		pRoom;

    uENTRY("xruiFindDefaultRoom()\n");
    pRoom=	LookupRoom(stGetToken(xruiDefaultRoom));
    if (pRoom!=NullRoom) {
	if (!(roomGetVisibility(pRoom)&roomVisible)) {
	    roomSetVisibility(pRoom,roomVisible,True);
	}
	uRETURN(pRoom);
    }
    uRETURN(xruiCreateDefaultRoom());
}

/***====================================================================***/

static void
xruiInitMenus(parent)
Widget	parent;
{
    static Arg buttonArgs[] = {
      {XtNfromVert, (XtArgVal) NULL},
      {XtNlabel, (XtArgVal) NULL},
      {XtNvertDistance, (XtArgVal) 0},
      {XtNfromHoriz, (XtArgVal) NULL},
      {XtNhorizDistance, (XtArgVal) 0},
      {XtNresize, (XtArgVal) False},
    };
    static Arg fargs[] = {
      {XtNheight, 1},
      {XtNwidth, 1},
    };
    Widget fw, w[20];
    int i, count = 0;
    Dimension max, wid;
    Arg arg;

    max = 0;
    XtSetArg(arg, XtNwidth, &wid);
    uENTRY1("xruiInitMenus(0x%x)\n",parent);
    XtAddEventHandler(xruiTopLevel,ButtonPressMask,False,xruiHandlePopup,NULL);
    XtAddEventHandler(xruiScrollW,ButtonPressMask,False,xruiHandlePopup,NULL);

    xruiMenu= XtCreatePopupShell("menu",
				 overrideShellWidgetClass, parent,
				 NULL, 0);
    fw = XtCreateManagedWidget("form", formWidgetClass, xruiMenu,
			       fargs, XtNumber(fargs));

    buttonArgs[0].value = (XtArgVal) NULL;
    buttonArgs[1].value = (XtArgVal) "Create Room";
    w[count] = XtCreateManagedWidget("Create Room", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xruiDoCreateRoom, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Remove Room";
    w[count] = XtCreateManagedWidget("Remove Room", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xruiDoDestroyRoom, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Hide Room";
    w[count] = XtCreateManagedWidget("Hide Room", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xruiDoHideRoom, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Load Profile";
    w[count] = XtCreateManagedWidget("Load Profile", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xruiDoLoadProfile, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Save Profile";
    w[count] = XtCreateManagedWidget("Save Profile", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xruiDoSaveProfile, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }

    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Change States ...";
    w[count] = XtCreateManagedWidget("Change State In Profile",
			commandWidgetClass,fw,buttonArgs,XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xruiDoWindowStates, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }

    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Disable Set-up Mode";
    w[count] = XtCreateManagedWidget("Set-up Mode", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    xruiSetupModeItem= w[count];
    XtAddCallback(w[count], XtNcallback, xruiFlipSetupMode, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }

    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Close Menu";
    w[count] = XtCreateManagedWidget("Close Menu", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xruiCloseMainMenu, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }

    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Exit xrooms";
    w[count] = XtCreateManagedWidget("Exit xrooms", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xruiDoExit, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }

    XtSetArg(arg, XtNwidth, max);
    for (i=0; i<count; i++) {
      XtSetValues(w[i], &arg, 1);
    }


    xruiWinMenu= XtCreatePopupShell("winmenu",
				 overrideShellWidgetClass, parent,
				 NULL, 0);
    fw = XtCreateManagedWidget("form", formWidgetClass, xruiWinMenu,
			       fargs, XtNumber(fargs));
    XtSetArg(arg, XtNwidth, &wid);

    count = 0;
    buttonArgs[0].value = (XtArgVal) NULL;
    buttonArgs[1].value = (XtArgVal) "Open In All Rooms";
    w[count] = XtCreateManagedWidget("Open In All Rooms",commandWidgetClass,fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xractDoSetAll, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Open Here Only";
    w[count] = XtCreateManagedWidget("Open Here Only", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xractDoSetLocalOnly, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Make Permanent";
    w[count] = XtCreateManagedWidget("Make Permanent", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xractDoMakePermanent, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Make Transient";
    w[count] = XtCreateManagedWidget("Make Transient", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xractDoMakeTransient, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Bolt Geometry";
    w[count] = XtCreateManagedWidget("Bolt Geometry", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xractDoBoltGeom, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Unbolt Geometry";
    w[count] = XtCreateManagedWidget("Unbolt Geometry", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xractDoUnboltGeom, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Bolt Window Open";
    w[count] = XtCreateManagedWidget("Bolt Window Open",commandWidgetClass,fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xractDoBoltState, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Unbolt Window State";
    w[count]= XtCreateManagedWidget("Unbolt Window State",commandWidgetClass,fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xractDoUnboltState, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }
    buttonArgs[0].value = (XtArgVal) w[count-1];
    buttonArgs[1].value = (XtArgVal) "Close Menu";
    w[count] = XtCreateManagedWidget("Close Menu", commandWidgetClass, fw,
				     buttonArgs, XtNumber(buttonArgs));
    XtAddCallback(w[count], XtNcallback, xruiCloseWinMenu, NULL);
    XtGetValues(w[count++], &arg, 1);
    if (wid > max) {
      max = wid;
    }

    XtSetArg(arg, XtNwidth, max);
    for (i=0; i<count; i++) {
      XtSetValues(w[i], &arg, 1);
    }
    uVOIDRETURN;
}

/***====================================================================***/

static	XtActionsRec	actions[] = {
	{ "do-prompt-yes",	xruiDoPromptYes }
};

void
xruiInit(parent)
Widget	parent;
{
    static Arg fargs[] = {
      {XtNheight, 150},
      {XtNwidth, 35},
    };

    uENTRY1("xruiInit(0x%x)\n",parent);
    xruiTopLevel=	parent;
    xroomsMainWindow=	XtWindow(xruiTopLevel);
    xruiScrollW= XtCreateWidget("main", formWidgetClass, parent,
				fargs, XtNumber(fargs));
    (void)xrrInit(xruiScrollW);
    XtManageChild(xruiScrollW);
    XtAddActions(actions, XtNumber(actions));
    XSync(xroomsDpy,False);
    uVOIDRETURN;
}

/***====================================================================***/

void
xruiBecomeVisible()
{
    uENTRY("xruiBecomeVisible()\n");
    XtRealizeWidget(xruiTopLevel);
    xruiInitMenus(xruiScrollW);
    (void)GetCurrentRoom();
    XSync(xroomsDpy,False);
    uVOIDRETURN;
}

