
  /*\
   *  $Header: xrReq.c,v 5.0 90/01/10 07:02:47 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#define	DEBUG_VAR	xrreqDebug
#include "utils.h"
#include "strtbl.h"
#include <X11/Xproto.h>
#include <X11/Xatom.h>
#undef	True
#undef	False
#include "xrReq.h"


int	xrreqTimeout=	5;

/***====================================================================***/

StringToken
xrreqGetCurrent(dpy, root)
Display	*dpy;
Window	 root;
{
StringToken	token;
char	*str;
Atom	actualType,*pRtrn;
int	actualFormat,nItems,bytesAfter;

    uENTRY2("xrreqGetCurrent(0x%x,0x%x)\n",dpy,root);
    if (XGetWindowProperty(dpy,root,ROOMS_CURRENT,0,1,False,XA_ATOM,
			&actualType,&actualFormat,&nItems,&bytesAfter,
			(char *)&pRtrn)==Success) {
	if ((actualType==XA_ATOM)&&(nItems==1)) {
	    str= XGetAtomName(dpy,*pRtrn);
	    XFree(pRtrn);
	    token= stGetToken(str);
	    XFree(str);
	    uRETURN(token);
	}
    }
    uRETURN(NullStringToken);
}

/***====================================================================***/

Boolean
xrreqGetAllRooms(dpy, root, pNRooms, pRooms)
Display		*dpy;
Window	 	 root;
int		*pNRooms;
StringToken	*pRooms;
{

    uENTRY4("xrreqGetAllRooms(0x%x,0x%x,0x%x,0x%x)\n",dpy,root,pNRooms,pRooms);
    uWarning("xrreqGetAllRooms not implemented yet\n");
    uRETURN(False);
}

/***==================================================================***/


static void
noPropertyChange()
{
    uFatalError("Xrooms does not appear to be running (timeout)\n");
}

char *
xrreqGetReply(dpy,root,win)
Display	*dpy;
Window	 root;
Window	 win;
{
XEvent	 event;
char	*str=	NullString;
Boolean	 done= False;
Atom	 actualType,actualFormat,nItems,bytesAfter;

    uENTRY3("xrreqGetReply(0x%x,0x%x,0x%x)\n",dpy,root,win);
#ifndef NO_TIMEOUTS
    if (xrreqTimeout>0)
	uSetTimeout(xrreqTimeout,0,noPropertyChange);
#endif
    while (!done) {
	XWindowEvent(dpy,win,PropertyChangeMask,&event);
	if ((event.type==PropertyNotify)&&
	    (event.xproperty.state==PropertyNewValue)&&
	    (event.xproperty.atom==ROOMS_RETURN)) {
#ifndef NO_TIMEOUTS
	    if (xrreqTimeout>0)
		uClearTimeout();
#endif
	    XGetWindowProperty(dpy,win,ROOMS_RETURN,0,10000,True,XA_STRING,
			&actualType,&actualFormat,&nItems,&bytesAfter,&str);
	    XDestroyWindow(dpy,win);
	    done=	True;
	}
    }
    uRETURN(str);
}

/***==================================================================***/

Window
xrreqGetArgWindow(dpy,root)
Display	*dpy;
Window	 root;
{
Window	win;
XSetWindowAttributes	xswa;

    uENTRY2("xrreqGetArgWindow(0x%x,0x%x)\n",dpy,root);
    xswa.override_redirect=	True;
    xswa.event_mask=		PropertyChangeMask;
    win= XCreateWindow(dpy,root,0,0,1,1,0,CopyFromParent,InputOutput,
			CopyFromParent,CWOverrideRedirect|CWEventMask,&xswa);
    if (win==None) {
	uFatalError("Couldn't create argument window\n");
	/* NOTREACHED */
    }
    uRETURN(win);
}

/***====================================================================***/

char *
xrreqStringRequest(dpy,root,req,len,arg)
Display		*dpy;
Window 	 	 root;
unsigned	 req;
int		 len;
char		*arg;
{
xrRequestRec	 xreq;
char		*argstr;
char		*reply;

    uENTRY5("xrreqStringRequest(0x%x,0x%x,0x%x,%d,%s)\n",dpy,root,req,len,arg);
    if (arg!=NullString) {
	xreq.request=	req;
	xreq.size=	(len+3)/4;
	xreq.win=	xrreqGetArgWindow(dpy,root);
	argstr=	(char *)uAlloc(xreq.size);
	if (argstr!=NullString) {
	    bcopy(arg,argstr,len);
	    XGrabServer(dpy);
	    if (XChangeProperty(dpy,root,ROOMS_REQUEST,XA_INTEGER,32,
				PropModeAppend,&xreq,3)==Success) {
		XChangeProperty(dpy,xreq.win,ROOMS_ARGUMENT,XA_STRING,8,
				PropModeAppend,argstr,xreq.size*4);
	    }
	    XUngrabServer(dpy);
	    XFlush(dpy);
	    uFree(argstr);
	}
	reply=	xrreqGetReply(dpy,root,xreq.win);
	uRETURN(reply);
    }
    uRETURN(NullString);
}

/***==================================================================***/

Boolean
xrreqIssueSetState(dpy,root,pArgs)
Display			*dpy;
Window	 	 	 root;
xrSetStateArgsPtr	 pArgs;
{
Boolean	ok;
int	len;
char	buf[1024],*rtrn;

    uENTRY3("xrreqIssueSetState(0x%x,0x%x,0x%x)\n",dpy,root,pArgs);
    if (pArgs->whichPart==0) {
	if (pArgs->whichApp==XR_QUERY_APP)
	     pArgs->whichPart=	XR_QUERY_PARTS;
	else if (pArgs->toWhat=='P')
	     pArgs->whichPart=	XR_WINSTATE_PART|XR_POSITION_PART;
	else pArgs->whichPart=	XR_GEOM_PARTS;
    }

    buf[0]=	pArgs->doWhat;
    buf[1]=	pArgs->toWhat;

    if (pArgs->whichPart&XR_WINSTATE_PART)	buf[2]=	'+';
    else					buf[2]=	'!';
    if (pArgs->whichPart&XR_X_PART) {
	if (pArgs->whichPart&XR_X_NEGATIVE)	buf[3]=	'-';
	else					buf[3]= '+';
    }
    else if (pArgs->whichPart&XR_QUERY_PARTS)	buf[3]= '?';
    else					buf[3]=	'!';

    if (pArgs->whichPart&XR_Y_PART) {
	if (pArgs->whichPart&XR_Y_NEGATIVE)	buf[4]=	'-';
	else					buf[4]= '+';
    }
    else if (pArgs->whichPart&XR_QUERY_PARTS)	buf[4]= '?';
    else					buf[4]=	'!';

    if (pArgs->whichPart&XR_WIDTH_PART)		buf[5]=	'+';
    else if (pArgs->whichPart&XR_QUERY_PARTS)	buf[5]= '?';
    else					buf[5]=	'!';

    if (pArgs->whichPart&XR_HEIGHT_PART)	buf[6]=	'+';
    else if (pArgs->whichPart&XR_QUERY_PARTS)	buf[6]= '?';
    else					buf[6]=	'!';

    switch (pArgs->whichState) {
	case	XR_QUERY_STATE:		buf[7]=	'?';	break;
	case	XR_LOCAL_STATE:		buf[7]=	'L';	break;
	case	XR_DFLT_STATE:		buf[7]=	'D';	break;
	case	XR_ALL_STATES:		buf[7]=	'A';	break;
	case	XR_VISIBLE_STATE:	
	default:			buf[7]=	'V';	break;
    }

    if ((pArgs->whichApp==XR_NAMED_APP)&&(pArgs->appName==NullString))
	pArgs->whichApp= XR_QUERY_APP;
    len=	9;
    switch (pArgs->whichApp) {
	case	XR_LAST_APP:		buf[8]=	'L';	break;
	case	XR_ALL_APPS:		buf[8]=	'A';	break;
	case	XR_ACTIVE_APPS:		buf[8]=	'V';	break;
	case	XR_NAMED_APP:		buf[8]=	'N';
					strcpy(&buf[9],pArgs->appName);
					len+=	strlen(pArgs->appName)+1;
					break;
	case	XR_QUERY_APP:	
	default:
					xrreqTimeout=	30; /* XXX Hack */
					buf[8]=	'?';	break;
    }

    if ((pArgs->whichRoom==XR_NAMED_ROOM)&&(pArgs->roomName==NullString))
	pArgs->whichRoom= XR_QUERY_ROOM;
    switch (pArgs->whichRoom) {
	case	XR_QUERY_ROOM:		buf[len++]=	'?';	break;
	case	XR_LAST_ROOM:		buf[len++]=	'L';	break;
	case	XR_ALL_ROOMS:		buf[len++]=	'A';	break;
	case	XR_VISIBLE_ROOMS:	buf[len++]=	'V';	break;
	case	XR_NAMED_ROOM:		buf[len++]=	'N';
					strcpy(&buf[len],pArgs->roomName);
					len+=	strlen(pArgs->roomName)+1;
					break;
	default:
	case	XR_CURRENT_ROOM:	buf[len++]=	'C';	break;
    }
    rtrn= xrreqStringRequest(dpy,root,XR_RQ_SET_STATE,len,buf);
    ok=	((rtrn!=NullString)&&(rtrn[0]=='t'));
    uRETURN(ok);
}

/***====================================================================***/

Boolean
xrreqIssueRemember(dpy,root,pArgs)
Display		*dpy;
Window	 	 root;
xrRememberArgsPtr	 pArgs;
{
Boolean	ok;
char	buf[264],*rtrn;

    uENTRY3("xrreqIssueRemember(0x%x,0x%x,0x%x)\n",dpy,root,pArgs);
    if (pArgs->remember)	buf[0]=	'R';
    else			buf[0]= 'F';

    switch (pArgs->whichApp) {
	case	XR_LAST_APP:		buf[1]=	'L';	buf[2]=	NUL;	break;
	case	XR_ALL_APPS:		buf[1]=	'A';	buf[2]=	NUL;	break;
	case	XR_ACTIVE_APPS:		buf[1]=	'V';	buf[2]=	NUL;	break;
	case	XR_NAMED_APP:		buf[1]=	'N';
					strcpy(&buf[2],pArgs->appName);
					break;
	case	XR_QUERY_APP:	
	default:			buf[1]=	'?';	buf[2]=	NUL;	break;
    }
    rtrn= xrreqStringRequest(dpy,root,XR_RQ_REMEMBER,strlen(buf),buf);
    ok=	((rtrn!=NullString)&&(rtrn[0]=='t'));
    uRETURN(ok);
}

/***====================================================================***/

Boolean
xrreqIssueVisibility(dpy,root,pArgs)
Display		*dpy;
Window	 	 root;
xrVisibilityArgsPtr	 pArgs;
{
Boolean	ok;
char	buf[264],*rtrn;

    uENTRY3("xrreqIssueVisibility(0x%x,0x%x,0x%x)\n",dpy,root,pArgs);
    if (pArgs->visibility&XR_ROOM_ALWAYS)		buf[0]=	'A';
    else						buf[0]= 'a';
    if (pArgs->visibility&XR_ROOM_WHEN_NON_EMPTY)	buf[1]=	'N';
    else						buf[1]= 'n';
    if (pArgs->visibility&XR_ROOM_WHEN_ACTIVE)		buf[2]=	'A';
    else						buf[2]= 'a';
    if (pArgs->visibility&XR_ROOM_VISIBLE)		buf[3]=	'V';
    else						buf[3]= 'H';
    if (pArgs->checkAuto)				buf[4]= '+';
    else						buf[4]= '-';
    if (pArgs->createRoom)				buf[5]= 'C';
    else						buf[5]= 'c';

    switch (pArgs->whichRoom) {
	case	XR_LAST_ROOM:		buf[6]=	'L';	buf[7]=	NUL;	break;
	case	XR_ALL_ROOMS:		buf[6]=	'A';	buf[7]=	NUL;	break;
	case	XR_VISIBLE_ROOMS:	buf[6]=	'V';	buf[7]=	NUL;	break;
	case	XR_NAMED_ROOM:		buf[6]=	'N';
					strcpy(&buf[7],pArgs->roomName);
					break;
	case	XR_CURRENT_ROOM:
	case	XR_QUERY_ROOM:	
	default:			buf[6]=	'C';	buf[7]=	NUL;	break;
    }
    rtrn= xrreqStringRequest(dpy,root,XR_RQ_SET_VISIBILITY,strlen(buf),buf);
    ok=	((rtrn!=NullString)&&(rtrn[0]=='t'));
    uRETURN(ok);
}

/***====================================================================***/

Boolean
xrreqIssueLoad(dpy,root,fileName)
Display		*dpy;
Window	 	 root;
char		*fileName;
{
Boolean	ok;
char	*rtrn,*buf;
int	szBuf,bufIndex,ch;
FILE	*file;

    uENTRY3("xrreqIssueLoad(0x%x,0x%x,%s)\n",dpy,root,uStringText(fileName));
    if (fileName!=NullString) {
	 file= fopen(fileName,"r");
	 if (file==NULL) {
	    uWarning("Can't open \"%s\" to read\n",uStringText(fileName));
	    uRETURN(False);
	 }
    }
    else file=	stdin;

    szBuf=	500;
    bufIndex=	0;
    buf=	(char *)uAlloc(szBuf);
    while (((ch=getc(file))!=EOF)&&(buf!=NullString)) {
	if (bufIndex==szBuf) {
	    szBuf*=	2;
	    buf=	(char *)uRealloc(buf,szBuf);
	}
	if (buf!=NullString) {
	    buf[bufIndex++]=	ch;
	}
    }
    if (buf!=NullString) {
	buf[bufIndex++]= NUL;
	if (bufIndex>1) {
	    rtrn= xrreqStringRequest(dpy,root,XR_RQ_LOAD_PROFILE,bufIndex,buf);
	    ok=	((rtrn!=NullString)&&(rtrn[0]=='t'));
	}
	else {
	    uError("Profile is too short\n");
	    ok=	False;
	}
	uFree(buf);
    }
    else { 
	uError("allocation failure in xrreqIssueLoad\n");
    }
    uRETURN(ok);
}

/***====================================================================***/

char *
xrreqIssueGetAppName(dpy,root,which,name)
Display		*dpy;
Window	 	 root;
unsigned	 which;
char		*name;
{
Boolean	ok;
char	buf[264];
int	len=1;
char	*str,*appName;

    uENTRY4("xrreqIssueGetAppName(0x%x,0x%x,0x%x,%s)\n",dpy,root,which,
							uStringText(name));
    if ((which==XR_NAMED_APP)&&(name==NullString))
	which= XR_QUERY_APP;
    switch (which) {
	case	XR_LAST_APP:		buf[0]=	'L';	break;
	case	XR_ALL_APPS:		buf[0]=	'A';	break;
	case	XR_ACTIVE_APPS:		buf[0]=	'V';	break;
	case	XR_NAMED_APP:		buf[0]=	'N';
					strcpy(&buf[1],name);
					len+=	strlen(name)+1;
					break;
	case	XR_QUERY_APP:	
	default:			buf[0]=	'?';	break;
    }
    str=  xrreqStringRequest(dpy,root,XR_RQ_APP_NAME,len,buf);
    if (str!=NullString) {
	if (str[0]=='t') {
	    appName=	uStringDup(&str[1]);
	}
	else appName=	NullString;
	uFree(str);
    }
    else appName=	NullString;
    uRETURN(appName);
}

/***====================================================================***/

Boolean
xrreqIssueDebug(dpy,root,str)
Display		*dpy;
Window	 	 root;
char		*str;
{
Boolean	ok;
char	*rtrn;

    uENTRY3("xrreqIssueDebug(0x%x,0x%x,%s)\n",dpy,root,uStringText(str));
    rtrn= xrreqStringRequest(dpy,root,XR_RQ_DEBUG,strlen(str),str);
    ok=	((rtrn!=NullString)&&(rtrn[0]=='t'));
    uRETURN(ok);
}

