
  /*\
   *  $Header: xrset.c,v 5.0 90/01/10 07:02:55 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

#include <stdio.h>
#include "X11/Xlib.h"
#include "X11/Xatom.h"
#include "X11/Xutil.h"
#include "X11/Xmd.h"

#define DEBUG_VAR xrreqDebug
#include "utils.h"
#include "xrGen.h"
#include "xrReq.h"

/***==================================================================***/

extern	char		*getenv();

char	*dpyName=	NullString;
Display *argDpy=	NULL;
int	 argScrn=	0;
Window	 argRoot=	None;

/***==================================================================***/

#define	DB	uDebug
void
Usage(name)
char	*name;
{
DB("usage: %s ([ args ] [command [cmdargs]])*\n",name);
DB("where recognized args are:\n");
DB("  -display      <name>        use named display\n");
DB("  -debug        <string>      set debugging in xrooms\n");
DB("  -verbose     [<level>]      set amount of output\n");
DB("  -set         [state flags]  set the managed parts of application state(s)\n");
DB("  -merge       [state flags]  add to the managed parts of application state(s)\n");
DB("  -clear       [state flags]  clear the managed parts of application state(s)\n");
DB("  -bolt        [state flags]  bolt parts of application state(s)\n");
DB("  -unbolt      [state flags]  unbolt parts of application state(s)\n");
DB("  -tighten     [state flags]  tighten bolts in application state(s)\n");
DB("  -loosen      [state flags]  loosen bolts in application state(s)\n");
DB("  -setbolts    [state flags]  set bolts in application state(s)\n");
DB("  legal state flags are:\n");
DB("    -local                    set local state (default)\n");
DB("    -default or -dflt         set default state\n");
DB("    -allstates or -as         set all states\n");
DB("    -visible                  set visible state\n");
DB("    -winstate or -state       set window state\n");
DB("    -x or -right              set right edge of window\n");
DB("    +x or -left               set left edge of window\n");
DB("    -y or -bottom             set bottom edge of window\n");
DB("    +y or -top                set top edge of window\n");
DB("    -position                 set top-right corner of window\n");
DB("    -width                    set width of window\n");
DB("    -height                   set height of window\n");
DB("    -size                     set width and height of window\n");
DB("    -geometry                 set entire window geometry\n");
DB("    -everything               set both geometry and application state\n");
DB("    -allapps or -aa           set all apps\n");
DB("    -activeapps               set all currently active apps\n");
DB("    -queryapp or -qa          query user for application (default)\n");
DB("    -lastapp or -la           re-use last application\n");
DB("    <string>                  name of application to set\n");
DB("    -room <string>            set state only in named room\n");
DB("    -current                  set state only in current room (default)\n");
DB("    -queryroom or -qr         query user for room (not implemented)\n");
DB("    -allrooms or -ar          set state in all rooms\n");
DB("    -visrooms                 set state in all visible rooms\n");
DB("  -remember  [args]           remember an application forever\n");
DB("  -forget    [args]           forget an application after it exits\n");
DB("  legal remember flags are:\n");
DB("    -query or -select         query application (default)\n");
DB("    -last                     re-use last selected application\n");
DB("    -all                      remember/forget all applications\n");
DB("    <string>                  name of application\n");
DB("  -show  [ show flags ]       make room (conditionally) visible\n");
DB("  -hide  [ hide flags ]       make room invisible\n");
DB("  legal flags for both -hide and -show are:\n");
DB("    -room <string>            name of room to change\n");
DB("    <string>                  name of room to change\n");
DB("    -current                  change visibility of current room\n");
DB("    -queryroom or -qr         query user for room to change (not implemented)\n");
DB("    -allrooms or -ar          change visibility of all rooms\n");
DB("    -visrooms                 change visibility of all visible rooms\n");
DB("    -create                   create the room if it doesn't exist\n");
DB("  legal flags for -show are:\n");
DB("    -always                   room is always visible\n");
DB("    -[when]active             room is visible when active\n");
DB("    -[when]nonempty           room is visible when non-empty\n");
DB("    -now                      room is visible now\n");
DB("  -load  [<file>]             load a profile file\n");
DB("  -appname [-id <integer>]    query name of application\n");
exit(0);
/* NOTREACHED */
return;
}

/***==================================================================***/

Boolean
OpenConnection()
{
    uENTRY("OpenConnection()\n");
    if (argDpy==NULL) {
	if (!(argDpy = XOpenDisplay(dpyName))) {
	    if (dpyName==NullString)	dpyName=	getenv("DISPLAY");
	    if (dpyName==NullString)	dpyName=	"unix:0";
	    uFatalError("Cannot open display %s\n",dpyName);
	    /*NOTREACHED */
	}

	argScrn= XDefaultScreen(argDpy);
	argRoot= XDefaultRootWindow(argDpy);
	xrgenInit(argDpy);

	uDEBUG1(1,"Opened %s \n",(dpyName?dpyName:"default display"));
    }
    uRETURN(True);
}

/***==================================================================***/

Boolean
CloseConnection()
{
    uENTRY("CloseConnection()\n");
/*    xrgenCleanUp(argDpy);*/
    XCloseDisplay(argDpy);
    argDpy=	NULL;
    argScrn=	0;
    argRoot=	None;
    uDEBUG1(1,"Closed %s \n",(dpyName?dpyName:"default display"));
    uRETURN(True);
}

/***==================================================================***/

int
GenArg(argc,argv,i)
int	argc;
char	*argv[];
{
    uENTRY3("GenArg(%d,0x%x,%d)\n",argc,argv,i);
    if (argc>i+1) {
	if (uStringPrefix(argv[i],"-display")) {
	     if ((dpyName!=NullString)&&(!uStringEqual(dpyName,argv[i]))&&
		 (argDpy!=NULL)) {
		CloseConnection();
	     }
	     if (!uStringEqual(dpyName,argv[i])) {
		dpyName=	argv[i+1];
	     }
	     uRETURN(2);
	}
	else if (uStringPrefix(argv[i],"-verbose")) {
	     int	level;
	     if ((sscanf(argv[i+1],"%d",&level)!=1)&&
		 (sscanf(argv[i+1],"0x%x",&level)!=1)) {
		 uDEBUG(1,"Setting verbose (1)\n");
		 xrreqDebug=	1;
		 uRETURN(1);
	     }
	     else {
		 uDEBUG1(1,"Setting verbose (%d)\n",level);
		 xrreqDebug=	level;
		 uRETURN(2);
	     }
	}
    }
    if (uStringPrefix(argv[i],"-help")) {
	Usage(argv[0]);
	/* NOTREACHED */
    }
    uRETURN(0);
}

/***====================================================================***/

int
GetStringArg(argc,argv,i,ppStr)
int	 argc;
char	*argv[];
int	 i;
char	**ppStr;
{
char	*arg;

    uENTRY4("GetStringArg(%d,0x%x,%d,0x%x)\n",argc,argv,i,ppStr);
    arg= argv[i];
    if (uStringEqual(arg,"--")) {
	i++;
	if (i<argc) {
	    *ppStr=	argv[i];
	    uRETURN(2);
	}
	else {
	    uError("Missing string after \"--\"\n");
	    *ppStr=	NullString;
	    uRETURN(1);
	}
    }
    else if (arg[0]=='-') {
	*ppStr=	NullString;
	uRETURN(0);
    }

    *ppStr=	argv[i];
    uRETURN(1);
}

/***====================================================================***/

/* ARGSUSED */
int
DebugArgs(argc,argv,i,ignored)
int	 argc;
char	*argv[];
int	 i;
char	*ignored;
{
Boolean ok= True;
int	first=i;
int	nUsed;
char	*arg;

    uENTRY4("handleDebug(0x%x,0x%x,%s,%s)\n",argc,argv,i,uStringText(ignored));
    while ((i<argc)&&(ok)) {
	nUsed=	GetStringArg(argc,argv,i,&arg);
	if ((nUsed>0)&&(arg!=NullString)&&(OpenConnection())) {
	    ok= xrreqIssueDebug(argDpy,argRoot,arg);
	    i+=	nUsed;
	}
	else {
	    nUsed=	GenArg(argc,argv,i);
	    if (nUsed)	i+= nUsed;
	    else	ok= False;
	}
    }
    uRETURN(i-first);
}

/***====================================================================***/

int
WhichAppArg(argc,argv,i,pWhich,ppName)
int	argc;
char	*argv[];
int	i;
unsigned	*pWhich;
char		**ppName;
{
int	nUsed=	1;
char	*arg,*tmp;

    uENTRY5("WhichAppArg(%d,0x%x,%d,0x%x,0x%x)\n",argc,argv,i,pWhich,ppName);
    if (i<argc) {
	arg=	argv[i];
	if (uStringPrefix(arg,"-allapps")||uStringPrefix(arg,"-aa")) {
	    *pWhich=	XR_ALL_APPS;
	    *ppName=	NullString;
	}
	else if (uStringPrefix(arg,"-activeapps")) {
	    *pWhich=	XR_ACTIVE_APPS;
	    *ppName=	NullString;
	}
	else if (uStringPrefix(arg,"-queryapp")||uStringPrefix(arg,"-qa"))  {
	    *pWhich=	XR_QUERY_APP;
	    *ppName=	NullString;
	}
	else if (uStringPrefix(arg,"-lastapp")||uStringPrefix(arg,"-la")) {
	    *pWhich=	XR_LAST_APP;
	    *ppName=	NullString;
	}
	else if (uStringPrefix(arg,"-application")) {
	    i++;
	    if (i<argc) {
		nUsed=	2;
		*pWhich=	XR_NAMED_APP;
		*ppName=	argv[i];
	    }
	    else nUsed=	0;
	}
	else if (uStringPrefix(arg,"-id")||uStringPrefix(arg,"-window")) {
	    int	winId;
	    i++;
	    if ((i<argc)&&
		((sscanf(argv[i],"0x%x",&winId)==1)||
		 (sscanf(argv[i],"%d",&winId)==1))) {
		*ppName=	(char *)uAlloc(strlen(argv[i])+3);
		if (*ppName!=NullString) {
		    nUsed=	2;
		    sprintf(*ppName,"0x%x",winId);
		    *pWhich=	XR_WINDOW_APP;
		}
		else nUsed=	0;
	    }
	    else nUsed=	0;
	}
	else if ((nUsed=GetStringArg(argc,argv,i,&tmp))>0) {
	    *pWhich=	XR_NAMED_APP;
	    *ppName=	tmp;
	}
	else {
	    nUsed=	0;
	    *pWhich=	XR_QUERY_APP;
	    *ppName=	tmp;
	}
    }
    uRETURN(nUsed);
}

/***====================================================================***/

int
WhichRoomArg(argc,argv,i,pWhich,ppName)
int	argc;
char	*argv[];
int	i;
unsigned	*pWhich;
char		**ppName;
{
int	nUsed=	1;
char	*arg,*tmp;

    uENTRY5("WhichRoomArg(%d,0x%x,%d,0x%x,0x%x)\n",argc,argv,i,pWhich,ppName);
    if (i<argc) {
	arg=	argv[i];
	if (uStringPrefix(arg,"-current")) {
	    *pWhich=	XR_CURRENT_ROOM;
	    *ppName=	NullString;
	}
	else if (uStringPrefix(arg,"-lastroom")||uStringPrefix(arg,"-lr")) { 
	    *pWhich=	XR_LAST_ROOM;
	    *ppName=	NullString;
	}
	else if (uStringPrefix(arg,"-queryroom")||uStringPrefix(arg,"-qr")) {
	    *pWhich=	XR_QUERY_ROOM;
	    *ppName=	NullString;
	}
	else if (uStringPrefix(arg,"-visrooms")||uStringPrefix(arg,"-visible")){
	    *pWhich=	XR_VISIBLE_ROOMS;
	    *ppName=	NullString;
	}
	else if (uStringPrefix(arg,"-allrooms")||uStringPrefix(arg,"-ar")) {
	    *pWhich=	XR_ALL_ROOMS;
	    *ppName=	NullString;
	}
	else if (uStringPrefix(arg,"-room")) {
	    i++;
	    if (i<argc) {
		nUsed=	2;
		*pWhich=	XR_NAMED_APP;
		*ppName=	argv[i];
	    }
	    else nUsed=	0;
	}
	else if ((nUsed=GetStringArg(argc,argv,i,&tmp))>0) {
	    *pWhich=	XR_NAMED_ROOM;
	    *ppName=	tmp;
	}
	else {
	    nUsed=	0;
	    *pWhich=	XR_CURRENT_ROOM;
	    *ppName=	NullString;
	}
    }
    uRETURN(nUsed);
}

/***====================================================================***/

int
WhichStateArg(argc,argv,i,pWhich)
int	argc;
char	*argv[];
int	i;
unsigned	*pWhich;
{
int	nUsed=	1;
char	*arg;

    uENTRY4("WhichStateArg(%d,0x%x,%d,0x%x)\n",argc,argv,i,pWhich);
    if (i<argc) {
	arg=	argv[i];
	if (uStringPrefix(arg,"-local")) {
	    *pWhich=	XR_LOCAL_STATE;
	}
	else if (uStringPrefix(arg,"-default")||uStringPrefix(arg,"-dflt")) {
	    *pWhich=	XR_DFLT_STATE;
	}
	else if (uStringPrefix(arg,"-allstates")||uStringPrefix(arg,"-as")) {
	    *pWhich= 	XR_ALL_STATES;
	}
	else if (uStringPrefix(argv[i],"-visible")) {
	    *pWhich=	XR_VISIBLE_STATE;
	}
	else {
	    nUsed=	0;
	    *pWhich=	XR_VISIBLE_STATE;
	}
    }
    uRETURN(nUsed);
}

/***====================================================================***/

int
WhichPartArg(argc,argv,i,pWhich)
int	argc;
char	*argv[];
int	i;
unsigned	*pWhich;
{
int	nUsed=	1;
char	*arg;

    uENTRY4("WhichPartArg(%d,0x%x,%d,0x%x)\n",argc,argv,i,pWhich);
    if (i<argc) {
	arg=	argv[i];
	if (uStringPrefix(arg,"-winstate")||uStringPrefix(arg,"-state"))
	    *pWhich=	XR_WINSTATE_PART;
	else if (uStringEqual(arg,"-x")||uStringPrefix(arg,"-right"))
	    *pWhich=	XR_X_PART|XR_X_NEGATIVE;
	else if (uStringEqual(arg,"+x")||uStringPrefix(arg,"-left"))
	    *pWhich=	XR_X_PART;
	else if (uStringEqual(arg,"-y")||uStringPrefix(arg,"-bottom"))
	    *pWhich=	XR_Y_PART|XR_Y_NEGATIVE;
	else if (uStringEqual(arg,"+y")||uStringPrefix(arg,"-top"))
	    *pWhich=	XR_Y_PART;
	else if (uStringPrefix(arg,"-position"))
	    *pWhich=	XR_POSITION_PART;
	else if (uStringPrefix(arg,"-width"))
	    *pWhich=	XR_WIDTH_PART;
	else if (uStringPrefix(arg,"-height"))
	    *pWhich=	XR_HEIGHT_PART;
	else if (uStringPrefix(arg,"-size"))
	    *pWhich=	XR_SIZE_PART;
	else if (uStringPrefix(arg,"-geometry"))
	    *pWhich=	XR_GEOM_PARTS;
	else if (uStringPrefix(arg,"-everything"))
	    *pWhich=	XR_ALL_PARTS;
	else {
	    nUsed=	0;
	    *pWhich=	XR_GEOM_PARTS;
	}
    }
    uRETURN(nUsed);
}

/***====================================================================***/

int
SetStateArgs(argc,argv,i,what)
int	 argc;
char	*argv[];
int	 i;
char	*what;
{
Boolean	ok=	True;
int	first=	 i;
Boolean	partSpec=	False;
Boolean	appSpec=	False;
Boolean	roomSpec=	False;
Boolean	stateSpec=	False;
unsigned		 tmpInt;
char			*tmpStr;
xrSetStateArgsRec	 args;
int			 nUsed;

    uENTRY4("SetStateArgs(%d,0x%x,%d,%s)\n",argc,argv,i,uStringText(what));
    args.doWhat=	what[0];
    args.toWhat=	what[1];
    args.whichPart=	0;
    args.whichState=	XR_VISIBLE_STATE;
    args.whichApp=	XR_QUERY_APP;
    args.appName=	NullString;
    args.whichRoom=	XR_CURRENT_ROOM;
    args.roomName=	NullString;

    while ((i<argc)&&(ok)) {
	if ((nUsed= WhichPartArg(argc,argv,i,&tmpInt))>0) {
	    if (partSpec)	args.whichPart|=	tmpInt;
	    else		args.whichPart=		tmpInt;
	    partSpec=	True;
	}
	else if ((nUsed= WhichStateArg(argc,argv,i,&tmpInt))>0) {
	    if (stateSpec&&OpenConnection()) {
		ok=	xrreqIssueSetState(argDpy,argRoot,&args);
	    }
	    args.whichState=	tmpInt;
	    stateSpec=		True;
	}
	else if ((nUsed= WhichAppArg(argc,argv,i,&tmpInt,&tmpStr))>0) {
	    if (appSpec&&OpenConnection()) {
		ok=	xrreqIssueSetState(argDpy,argRoot,&args);
	    }
	    args.whichApp=	tmpInt;
	    args.appName=	tmpStr;
	    appSpec=		True;
	}
	else if ((nUsed= WhichRoomArg(argc,argv,i,&tmpInt,&tmpStr))>0) {
	    if (roomSpec&&OpenConnection()) {
		ok=	xrreqIssueSetState(argDpy,argRoot,&args);
	    }
	    args.whichRoom=	tmpInt;
	    args.roomName=	tmpStr;
	    roomSpec=		True;
	}
	else {
	    nUsed=		GenArg(argc,argv,i);
	    if (nUsed>0)	i+= nUsed;
	    else		ok= False;
	    continue;
	}
	i+=	nUsed;;
    }
    if (OpenConnection()) {
	ok= xrreqIssueSetState(argDpy,argRoot,&args);
    }
    if (!ok) {
	uInformation("Set state request failed\n");
    }
    uRETURN(i-first);
}

/***==================================================================***/

int
RememberArgs(argc,argv,i,remember)
int	argc;
char	*argv[];
int	i;
char	*remember;
{
int			first=	 i;
int			nUsed;
Boolean			ok=	True;
Boolean			appSpec=	False;
unsigned		tmpInt;
char			*tmpStr;
xrRememberArgsRec	args;

    uENTRY4("RememberArgs(%d,0x%x,%d,%s)\n",argc,argv,i,uStringText(remember));

    args.remember=	(remember[0]=='T');
    args.whichApp=	XR_QUERY_APP;

    while ((i<argc)&&(ok)) {
	if ((nUsed= WhichAppArg(argc,argv,i,&tmpInt,&tmpStr))>0) {
	    if (appSpec&&OpenConnection()) {
		ok=	xrreqIssueRemember(argDpy,argRoot,&args);
	    }
	    args.whichApp=	tmpInt;
	    args.appName=	tmpStr;
	    appSpec=		True;
	}
	else {
	    nUsed=		GenArg(argc,argv,i);
	    if (nUsed>0)	i+= nUsed;
	    else		ok= False;
	}
	i+= nUsed;
    }
    if (OpenConnection()) {
	ok= xrreqIssueRemember(argDpy,argRoot,&args);
    }
    if (!ok) {
	uInformation("Remember request failed\n");
    }
    uRETURN(i-first);
}

/***==================================================================***/

/* ARGSUSED */
int
HideArgs(argc,argv,i,ignored)
int	argc;
char	*argv[];
int	i;
Opaque	ignored;
{
Boolean	ok=	True;
int	first=	 i;
Boolean	roomSpec=	False;
unsigned		 tmpInt;
char			*tmpStr;
xrVisibilityArgsRec	 args;
int			 nUsed;

    uENTRY4("HideArgs(%d,0x%x,%d,0x%x)\n",argc,argv,i,ignored);
    args.visibility=	XR_ROOM_HIDDEN;
    args.whichRoom=	XR_CURRENT_ROOM;
    args.roomName=	NullString;
    args.checkAuto=	False;
    args.createRoom=	False;

    while ((i<argc)&&(ok)) {
	if ((nUsed= WhichRoomArg(argc,argv,i,&tmpInt,&tmpStr))>0) {
	    if (roomSpec&&OpenConnection()) {
		ok=	xrreqIssueVisibility(argDpy,argRoot,&args);
	    }
	    args.whichRoom=	tmpInt;
	    args.roomName=	tmpStr;
	    roomSpec=		True;
	}
	else if ((nUsed=GetStringArg(argc,argv,i,&tmpStr))>0) {
	    if ((roomSpec)&&(OpenConnection())) {
		ok= xrreqIssueVisibility(argDpy,argRoot,&args);
	    }
	    args.whichRoom=	XR_NAMED_ROOM;
	    args.roomName=	tmpStr;
	    roomSpec=		True;
	}
	else if (uStringPrefix(argv[i],"-create")) {
	    args.createRoom=	True;
	}
	else {
	    nUsed=		GenArg(argc,argv,i);
	    if (nUsed>0)	i+= nUsed;
	    else		ok= False;
	    continue;
	}
	i+=	nUsed;;
    }
    if (OpenConnection()) {
	ok= xrreqIssueVisibility(argDpy,argRoot,&args);
    }
    if (!ok) {
	uInformation("HIDE request failed\n");
    }
    uRETURN(i-first);
}

/***==================================================================***/

/* ARGSUSED */
int
ShowArgs(argc,argv,i,ignored)
int	argc;
char	*argv[];
int	i;
Opaque	ignored;
{
Boolean	ok=	True;
int	first=	 i;
Boolean	roomSpec=	False;
unsigned		 tmpInt;
char			*tmpStr;
xrVisibilityArgsRec	 args;
int			 nUsed;

    uENTRY4("ShowArgs(%d,0x%x,%d,0x%x)\n",argc,argv,i,ignored);
    args.visibility=	XR_ROOM_VISIBLE;
    args.whichRoom=	XR_CURRENT_ROOM;
    args.roomName=	NullString;
    args.checkAuto=	True;
    args.createRoom=	False;

    while ((i<argc)&&(ok)) {
	if ((nUsed= WhichRoomArg(argc,argv,i,&tmpInt,&tmpStr))>0) {
	    if (roomSpec&&OpenConnection()) {
		ok=	xrreqIssueVisibility(argDpy,argRoot,&args);
	    }
	    args.whichRoom=	tmpInt;
	    args.roomName=	tmpStr;
	    roomSpec=		True;
	}
	else if ((nUsed=GetStringArg(argc,argv,i,&tmpStr))>0) {
	    if ((roomSpec)&&(OpenConnection())) {
		ok= xrreqIssueVisibility(argDpy,argRoot,&args);
	    }
	    args.whichRoom=	XR_NAMED_ROOM;
	    args.roomName=	tmpStr;
	    roomSpec=		True;
	}
	else if (uStringPrefix(argv[i],"-create")) {
	    args.createRoom=	True;
	}
	else if (uStringPrefix(argv[i],"-always")) {
	    args.visibility|=	XR_ROOM_ALWAYS;
	}
	else if (uStringPrefix(argv[i],"-active")||
		 uStringPrefix(argv[i],"-whenactive")) {
	    args.visibility|=	XR_ROOM_WHEN_ACTIVE;
	}
	else if (uStringPrefix(argv[i],"-nonempty")||
		 uStringPrefix(argv[i],"-whennonempty")) {
	    args.visibility|=	XR_ROOM_WHEN_NON_EMPTY;
	}
	else if (uStringPrefix(argv[i],"-now")) {
	    args.checkAuto=	False;	
	}
	else {
	    nUsed=		GenArg(argc,argv,i);
	    if (nUsed>0)	i+= nUsed;
	    else		ok= False;
	    continue;
	}
	i+=	nUsed;;
    }
    if (OpenConnection()) {
	ok= xrreqIssueVisibility(argDpy,argRoot,&args);
    }
    if (!ok) {
	uInformation("SHOW request failed\n");
    }
    uRETURN(i-first);
}

/***==================================================================***/

/* ARGSUSED */
int
LoadArgs(argc,argv,i,ignored)
int	argc;
char	*argv[];
int	i;
Opaque	ignored;
{
int	 first=	 i;
int	 nUsed;
char	*fileName= NullString,*tmp;
Boolean	 fileSpec;
Boolean	 ok=	True;

    uENTRY4("LoadArgs(%d,0x%x,%d,0x%x)\n",argc,argv,i,ignored);
    while ((i<argc)&&(ok)) {
	if ((nUsed=GetStringArg(argc,argv,i,&tmp))>0) {
	    if ((fileSpec)&&(OpenConnection())) {
		ok= xrreqIssueLoad(argDpy,argRoot,fileName);
	    }
	    fileName=	tmp;
	    fileSpec=	True;
	}
	else {
	    nUsed=		GenArg(argc,argv,i);
	    if (nUsed>0)	i+= nUsed;
	    else		ok= False;
	    continue;
	}
	i+=	nUsed;
    }
    if (OpenConnection()) {
	ok= xrreqIssueLoad(argDpy,argRoot,fileName);
    }
    if (!ok) {
	uInformation("Load request failed\n");
    }
    uRETURN(i-first);
}

/***==================================================================***/

/* ARGSUSED */
int
GetAppNameArgs(argc,argv,i,ignored)
int	argc;
char	*argv[];
int	i;
Opaque	ignored;
{
int			first=	 i;
int			nUsed;
Boolean			ok=	True;
Boolean			appSpec=	False;
unsigned		tmpInt;
char			*tmpStr;
unsigned		whichApp;
char			*name;

    uENTRY4("GetAppNameArgs(%d,0x%x,%d,0x%x)\n",argc,argv,i,ignored);
    whichApp=	XR_QUERY_APP;
    name=	NullString;

    while ((i<argc)&&(ok)) {
	if ((nUsed= WhichAppArg(argc,argv,i,&tmpInt,&tmpStr))>0) {
	    if (appSpec&&OpenConnection()) {
		char *rtrn= xrreqIssueGetAppName(argDpy,argRoot,whichApp,name);
		fprintf(stderr,"%s\n",uStringText(rtrn));
	    }
	    whichApp=	tmpInt;
	    name=	tmpStr;
	    appSpec=		True;
	}
	else {
	    nUsed=		GenArg(argc,argv,i);
	    if (nUsed>0)	i+= nUsed;
	    else		ok= False;
	}
	i+= nUsed;
    }
    if (OpenConnection()) {
	char *rtrn= xrreqIssueGetAppName(argDpy,argRoot,whichApp,name);
	fprintf(stderr,"%s\n",uStringText(rtrn));
    }
    uRETURN(i-first);
}

/***==================================================================***/

typedef struct _CommandArgRec {
	char	*cmdLineArg;
	int	(*argFunc)();
	char	*funcArg;
} CommandArgRec,*CommandArgPtr;

static	CommandArgRec	cmdArgs[] = {
	{ "-debug",	DebugArgs,	""	},
	{ "-set",	SetStateArgs,	"SP"	},
	{ "-merge",	SetStateArgs,	"MP"	},
	{ "-clear",	SetStateArgs,	"CP"	},
	{ "-bolt",	SetStateArgs,	"MB"	},
	{ "-unbolt",	SetStateArgs,	"CB"	},
	{ "-tighten",	SetStateArgs,	"CL"	},
	{ "-loosen",	SetStateArgs,	"ML"	},
	{ "-setbolts",	SetStateArgs,	"SB"	},
	{ "-remember",	RememberArgs,	"T"	},
	{ "-forget",	RememberArgs,	"F"	},
	{ "-hide",      HideArgs,	""	},
	{ "-show",      ShowArgs,	""	},
	{ "-load",	LoadArgs,	""	},
	{ "-appname",	GetAppNameArgs,	""	},
	{ NULL,		NULL,		NULL	}
};

int
CommandArgs(argc,argv,i)
int	argc;
char	*argv[];
{
CommandArgPtr	pCmd;

    uENTRY3("CommandArgs(%d,0x%x,%d)\n",argc,argv,i);
    pCmd=	cmdArgs;
    while (pCmd->cmdLineArg!=NullString) {
	if (uStringPrefix(argv[i],pCmd->cmdLineArg)) {
	    uRETURN(1+(*pCmd->argFunc)(argc,argv,i+1,pCmd->funcArg));
	}
	pCmd++;
    }
    uRETURN(0);
}

/***==================================================================***/

Boolean
HandleArgs(argc, argv)
int	argc;
char	*argv[];
{
int	i,nUsed;

    uENTRY2("HandleArgs(%d,0x%x)\n",argc,argv);
    i= 1;
    if (argc==1) {
	Usage(argv[0]);
	/* NOTREACHED */
    }
    while (i<argc) {
	nUsed=	GenArg(argc,argv,i);
	if (nUsed==0)
	    nUsed= CommandArgs(argc,argv,i);
	if (nUsed==0) {
	    uError("Unknown argument \"%s\"\n",argv[i]);
	    Usage(argv[0]);
	    /* NOTREACHED */
	}
	i+= nUsed;
    }
    uRETURN(True);
}

/***====================================================================***/

main(argc, argv)
int argc;
char **argv;
{
Boolean	ok;

    (void)stInit(100,10,False,False);
    ok= HandleArgs(argc, argv);
    ok= CloseConnection()&&ok;
    if (ok)	return(0);
    else	return(1);
}

